package com.at.video.atlas.ui.atlas

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.view.marginTop
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.atlas.R
import com.at.video.atlas.entity.AtlasTypeInfo
import com.at.video.atlas.ui.atlas.binder.*
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.melancholy.utils.DensityUtils
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.*
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/10 17:07
 * @Description: 图集
 */

@AndroidEntryPoint
@Route(path = "/atlas/AtlasActivity")
class AtlasActivity : BaseMviActivity<CommonTitleBarRefreshListBinding, AtlasViewModel>(),
    OnRefreshLoadMoreListener, GridSpanSizeLookup, OnItemClickListener, OnItemChildClickListener {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ATLAS_HOME)

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<AtlasViewModel> {
        return AtlasViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        getViewModel().sendEvent(AtlasUiEvent.InitStyle(homeStyle))
        requireTitleBar().apply {
            homeStyle?.content?.let {
                if (it.isNotEmpty()) {
                    it[0]?.config?.titleColor?.let { it1 -> setTitleColor(it1.toArgb()) }
                    it[0]?.config?.background?.let { it1 -> setBackgroundColor(it1.toArgb()) }
                }
            }
            addBackButton(Color.BLACK)
            setTitle("图集")
            val searchBtn =
                addRightImageButton(StyleUtils.getGlobalSearchIcon(), android.R.id.title)
            searchBtn.setOnClickListener {
                Router.getInstance().build("/atlas/SearchActivity").navigation()
            }

            val layoutParams = searchBtn.layoutParams as? ViewGroup.MarginLayoutParams
            layoutParams?.apply {
                rightMargin = DensityUtils.dp2px(
                    this@AtlasActivity, resources.getDimension(com.at.common.R.dimen.dp_4)
                )
                searchBtn.layoutParams = this
            }
        }
        val gridStyle = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.CATEGORY_GIDE)
        var spanCount = 4
        var gridType = 1
        gridStyle?.config?.let {
            spanCount = it.cols
            gridType = it.style
        }
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@AtlasActivity)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            srlRefresh.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.ATLAS_HOME).toArgb()
            )
//            rvList.setPadding(0,AppInstance.getApplication().resources
//                .getDimensionPixelSize(com.at.common.R.dimen.dp_16),0,0)
            rvList.layoutManager = GridLayoutManager(this@AtlasActivity, 1)
//            rvList.addItemDecoration(AtlasDecoration())
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
        }
//        mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
//            override fun getSpanSize(gridLayoutManager: GridLayoutManager, viewType: Int,
//                                     position: Int): Int {
//                if (viewType == 1) {
//                    return 1
//                }
//                return  StyleUtils.getSpanCount(spanCount)
//            }
//
//        })
        mAdapter.addItemBinder(AtlasListTypeBinder(gridType, spanCount))
        mAdapter.addItemBinder(
            GapLineBinder(
                StyleUtils.getPageBackground(StyleConsts.ATLAS_HOME).toArgb()
            )
        )
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.SWIPER_ADV
                )
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                style = JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.BANNER_ADV
                ), bg = StyleUtils.getPageBackground(StyleConsts.ATLAS_HOME)
            )
        )
        mAdapter.addItemBinder(AdTextBinder())
        mAdapter.addItemBinder(AtlasListBinder())
        mAdapter.addItemBinder(NoDataBinder(label = "暂无数据"))
        mAdapter.addChildClickViewIds(R.id.mb_switch, R.id.tv_more, R.id.mb_more)
        mAdapter.setOnItemChildClickListener(this)
        mAdapter.setOnItemClickListener(this)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(AtlasUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(AtlasUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is AtlasUiEffect.NoChangeDataEffect -> ToastUtils.show(it.msg)
                    is AtlasUiEffect.ChangeDataEffect -> {
                        mAdapter.notifyItemChanged(
                            it.position, it.data
                        )
                    }
                    else -> {}
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(AtlasUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(AtlasUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(AtlasUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(AtlasUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(AtlasUiEvent.LoadDataMore)
    }

    override fun getSpanSize(
        gridLayoutManager: GridLayoutManager, viewType: Int, position: Int
    ): Int {
        return when (viewType) {
            1 -> 1
            3, 4 -> 2
            else -> gridLayoutManager.spanCount
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        when (val data: Any = adapter.getItem(position)!!) {
            is AtlasFreeBinder.Free -> {
                Router.getInstance().build("/atlas/DetailActivity")
                    .withString("atlasId", data.info.id!!).navigation()
            }
            is AtlasPayBinder.Pay -> {
                Router.getInstance().build("/atlas/DetailActivity")
                    .withString("atlasId", data.info.id!!).navigation()
            }
            is AtlasTypeInfo -> {
                Router.getInstance().build("/atlas/ClassifyActivity")
                    .withString("mCategoryId", if (data.id != "-1") data.id.toString() else "")
                    .navigation()
            }
            is String -> {
                Router.getInstance().build("/atlas/AtlasMoreActivity").withString("parameter", data)
                    .navigation()
            }
//            is AdImageBinder.Data ->  Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//            is AdTextBinder.Data ->   Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
        }
    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: AtlasListBinder.Data = (adapter.getItem(position) as AtlasListBinder.Data?)!!
        when (view.id) {
            R.id.mb_switch -> getViewModel().sendEvent(
                AtlasUiEvent.Switch(data, position)
            )
            R.id.tv_more, R.id.mb_more -> {
                Router.getInstance().build("/atlas/AtlasMoreActivity")
                    .withString("parameter", data.title).navigation()

            }
        }
    }
}