package com.at.video.atlas.ui.atlas

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.R
import com.at.video.atlas.entity.*
import com.at.video.atlas.http.ApiService
import com.at.video.atlas.ui.atlas.binder.AtlasFreeBinder
import com.at.video.atlas.ui.atlas.binder.AtlasListBinder
import com.at.video.atlas.ui.atlas.binder.AtlasListTypeBinder
import com.at.video.atlas.ui.atlas.binder.AtlasPayBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.*
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.AdvertisingInfo
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.DataUtils.getTestAdv
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/28 09:40
 * @Description:
 */

data class AtlasUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class AtlasUiEvent : IUiEvent {
    object InitLoadData : AtlasUiEvent()
    object RefreshData : AtlasUiEvent()
    object LoadDataMore : AtlasUiEvent()
    data class InitStyle(val style: HomeStyle?) : AtlasUiEvent()
    data class Switch(val data: AtlasListBinder.Data, val position: Int) : AtlasUiEvent()

}

sealed class AtlasUiEffect : IUiEffect {
    data class ChangeDataEffect(val data: AtlasListBinder.Data, val position: Int) : AtlasUiEffect()
    data class NoChangeDataEffect(val msg: String) : AtlasUiEffect()
}

class AtlasViewModel(application: Application) :
    BaseAndroidViewModel<AtlasUiState, AtlasUiEvent, AtlasUiEffect>(application) {
    private var style: HomeStyle? = null
    override fun providerInitialState(): AtlasUiState {
        return AtlasUiState()
    }

    override fun handleEvent(event: AtlasUiEvent) {
        when (event) {
            is AtlasUiEvent.InitLoadData -> initLoadData()
            is AtlasUiEvent.RefreshData -> initLoadData(true)
            is AtlasUiEvent.LoadDataMore -> loadDataMore()
            is AtlasUiEvent.InitStyle -> style = event.style
            is AtlasUiEvent.Switch -> switch(event.data, event.position)
        }
    }


    @Synchronized
    private fun switch(data: AtlasListBinder.Data, position: Int) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["order"] = "random"
            map["limit"] = if (data.type == 1) 4 else 6
            when (data.dataType) {
                1 -> {
                    map["price_type"] = "1"
                }
                2 -> {
                    map["sort"] = "popularity"
                }
            }
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getPictureList(map)
            if (bean.success() && bean.list?.isNotEmpty() == true) {
                val videoData = mutableListOf<Any>()
                bean.list.forEach {
                    changeData(videoData, it)
                }
                setEffect {
                    AtlasUiEffect.ChangeDataEffect(
                        AtlasListBinder.Data(
                            videoData,
                            data.type,
                            data.title,
                            data.titleStyle,
                            data.dataType,
                        ), position
                    )
                }
            } else {
                setEffect {
                    AtlasUiEffect.NoChangeDataEffect("没有更多数据啦!")
                }
            }
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        viewModelScope.launch {
            getPicData().onStart {
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect {
                if (it == null) {
                    val datas: MutableList<Any> = mutableListOf()
                    style?.apply {
                        content?.forEach {
                            when (it!!.type) {
                                StyleConsts.SWIPER_ADV -> {
                                    val carourseAdvs = JsonUtils.getAdvListData<AdvertisingBean>(
                                        GlobalConsts.CAROUSEL_ADV
                                    )?.let { it1 ->
                                        DataUtils.filterAdvPosition(
                                            it1, AdvPosition.AtlasHome
                                        )
                                    }
                                    carourseAdvs?.let { advs ->
                                        datas.add(GapLineBinder.Data(""))
                                        datas.add(BannerBinder.BannerInfo(advs))
                                    }
                                }
                                StyleConsts.BANNER_ADV -> {
                                    val bannerAdvs = JsonUtils.getAdvListData<AdvertisingBean>(
                                        GlobalConsts.BANNER_ADV
                                    )?.let { it1 ->
                                        DataUtils.filterAdvPosition(
                                            it1, AdvPosition.AtlasHome
                                        )
                                    }
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    bannerAdvs?.let { advs ->
                                        advs.forEach { item ->
                                            run {
                                                adBinderData.add(GapLineBinder.Data(""))
                                                if (item.image_type != 3) {
                                                    adBinderData.add(AdImageBinder.Data(item))
                                                } else {
                                                    adBinderData.add(AdTextBinder.Data(item))
                                                }
                                            }
                                        }
                                        datas.addAll(adBinderData)
                                    }
                                }
                            }
                        }
                    }
                    datas.add(NoDataBinder.Data("暂无数据"))
                    if (isRefresh) {
                        setState {
                            copy(
                                refreshStatus = RefreshStatus.RefreshSuccess,
                                data = datas,
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                            )
                        }
                    } else {
                        setState {
                            copy(
                                pageStatus = PageStatus.Success,
                                data = datas,
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                            )
                        }
                    }
                } else {
                    val homeAtlasDatas = it as RecommendBean
                    if (homeAtlasDatas.success()) {
                        val datas: MutableList<Any> = mutableListOf()
                        val freeList = mutableListOf<Any>()
                        val heatList = mutableListOf<Any>()
                        val categorys = homeAtlasDatas.category_list?.list
                        homeAtlasDatas.free_list?.list?.forEach { it1 ->
                            changeData(freeList, it1)
                        }
                        homeAtlasDatas.hot_list?.list?.forEach { it1 ->
                            changeData(heatList, it1)
                        }
                        if (freeList.isEmpty() && heatList.isEmpty()) {
                            datas.add(NoDataBinder.Data("图集数据不存在!"))
                        } else {
                            style?.apply {
                                content?.forEach {
                                    when (it!!.type) {
                                        StyleConsts.SWIPER_ADV -> {
                                            val carourseAdvs =
                                                JsonUtils.getAdvListData<AdvertisingBean>(
                                                    GlobalConsts.CAROUSEL_ADV
                                                )?.let { it1 ->
                                                    DataUtils.filterAdvPosition(
                                                        it1, AdvPosition.AtlasHome
                                                    )
                                                }
                                            carourseAdvs?.let { advs ->
                                                datas.add(GapLineBinder.Data(""))
                                                datas.add(BannerBinder.BannerInfo(advs))
                                            }
                                        }
                                        StyleConsts.BANNER_ADV -> {
                                            val bannerAdvs =
                                                JsonUtils.getAdvListData<AdvertisingBean>(
                                                    GlobalConsts.BANNER_ADV
                                                )?.let { it1 ->
                                                    DataUtils.filterAdvPosition(
                                                        it1, AdvPosition.AtlasHome
                                                    )
                                                }
                                            val adBinderData: MutableList<Any> = mutableListOf()
                                            bannerAdvs?.let { advs ->
                                                advs.forEach { item ->
                                                    run {
                                                        adBinderData.add(GapLineBinder.Data(""))
                                                        if (item.image_type != 3) {
                                                            adBinderData.add(AdImageBinder.Data(item))
                                                        } else {
                                                            adBinderData.add(AdTextBinder.Data(item))
                                                        }
                                                    }
                                                }
                                                datas.addAll(adBinderData)
                                            }
                                        }
                                        StyleConsts.CATEGORY_GIDE -> {
                                            it.config?.apply {
                                                val list: MutableList<Any> = mutableListOf()
                                                when (style) {
                                                    1, 2 -> {
                                                        for (i in categorys!!.indices) {
                                                            val item = categorys[i] as Category
                                                            list.add(
                                                                AtlasTypeInfo(
                                                                    name = item.name!!,
                                                                    id = item.id!!,
                                                                    cover = item.cover!!
                                                                )
                                                            )
                                                        }
                                                        list.add(
                                                            AtlasTypeInfo(
                                                                name = "更多",
                                                                id = "-1",
                                                                cover = R.drawable.atlas_ic_more_bg
                                                            )
                                                        )
                                                    }
                                                    else -> {
                                                        for (i in categorys!!.indices) {
                                                            val item = categorys[i] as Category
                                                            list.add(
                                                                AtlasTypeInfo(
                                                                    name = item.name!!,
                                                                    id = item.id!!,
                                                                    cover = StyleUtils.getCategoryColor(
                                                                        i
                                                                    )
                                                                )
                                                            )
                                                        }
                                                        list.add(
                                                            AtlasTypeInfo(
                                                                name = "更多",
                                                                id = "-1",
                                                                cover = StyleUtils.getCategoryColor(
                                                                    -1
                                                                )
                                                            )
                                                        )
                                                    }
                                                }
//                                            datas.addAll(list)
                                                datas.add(AtlasListTypeBinder.Data(list))
                                            }
                                        }
                                        StyleConsts.FREE_LIST -> {
                                            val style = it.config?.layout?.style ?: 1
                                            var pageSize = if (style == 1) 4 else 6
                                            if (freeList.size < pageSize) {
                                                pageSize = freeList.size
                                            }
                                            datas.add(GapLineBinder.Data(""))
                                            datas.add(
                                                AtlasListBinder.Data(
                                                    freeList.subList(
                                                        0, pageSize
                                                    ), style, "免费阅读", it.config?.title, 1
                                                )
                                            )
                                        }
                                        StyleConsts.HOT_LIST -> {
                                            val style = it.config?.layout?.style ?: 1
                                            var pageSize = if (style == 1) 4 else 6
                                            if (freeList.size < pageSize) {
                                                pageSize = heatList.size
                                            }
                                            datas.add(GapLineBinder.Data(""))
                                            datas.add(
                                                AtlasListBinder.Data(
                                                    heatList.subList(
                                                        0, pageSize
                                                    ), style, "热门推荐", it.config?.title, 2
                                                )
                                            )
                                        }
                                    }
                                }
                            }
                        }

                        if (isRefresh) {
                            setState {
                                copy(
                                    refreshStatus = RefreshStatus.RefreshSuccess,
                                    data = datas,
                                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                                )
                            }
                        } else {
                            setState {
                                copy(
                                    pageStatus = PageStatus.Success,
                                    data = datas,
                                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                                )
                            }
                        }
                    } else {
                        setState {
                            copy(
                                pageStatus = PageStatus.Error(Throwable(message = "数据请求异常!")),
                                refreshStatus = RefreshStatus.RefreshSuccess
                            )
                        }
                    }
                }
            }
        }

    }

    private fun changeData(datas: MutableList<Any>, item: PictureBean) {
        if (item.price_type == 1) {
            datas.add(AtlasFreeBinder.Free(item))
        } else {
            datas.add(AtlasPayBinder.Pay(item))
        }
    }

    private fun loadDataMore() {

    }

    private fun getPicData(): Flow<Any?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            val bean =
                ApiFactory.getInstance().create(ApiService::class.java).getHomePictureList(map)
            if (bean.success() || (bean.message != null && bean.message.contains("该资源站点未配置"))) {
                emit(bean)
            } else {
                emit(null)
            }
        }
    }

}