package com.at.video.atlas.ui.atlas.binder

import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import com.at.video.atlas.R
import com.at.video.atlas.entity.PictureBean
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.CommonStringUtils
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/10 17:59
 * @Description:
 */

class AtlasFreeBinder(val type: Int = 1, val isHideTag: Boolean = false) :
    BaseItemBinder<AtlasFreeBinder.Free, BaseViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.atlas_atlas_free_view, parent, false)
        )
    }

    override fun convert(holder: BaseViewHolder, data: Free) {
        val root = holder.getView<FrameLayout>(R.id.fl_root)
        root.layoutParams = getRootParamsByStyleType()
        val image = holder.getView<DiversityImageView>(R.id.div_image)
        val params = image.layoutParams
        params.height = StyleUtils.getDpByDimen(getImageParamsByType())
        if (StyleUtils.tagEnable) {
            changeTagStyle(holder.getView(R.id.stv_tag))
        } else {
            holder.setGone(R.id.stv_tag, true)
        }
        holder.getView<ImageView>(R.id.div_image).loadByGlide(
                GlideImageConfig(
                    data.info.cover_vertical.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                ), WeakReference(context)
            )
        holder.setText(R.id.tv_name, data.info.name).setText(
                R.id.tv_desc, CommonStringUtils.splicingObliqueLineStrings(
                    data.info.categ_name, data.info.subcateg_name
                )
            ).setGone(R.id.tv_desc, isHideTag)
    }

    private fun getRootParamsByStyleType(): FrameLayout.LayoutParams {
        val params = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_137)
        )

        when (type) {
            1 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_266)
            }
            2 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_180)
            }
            else -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_150)
                params.width = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_80)
            }
        }
        return params
    }


    private fun getImageParamsByType() = when (type) {
        1 -> {
            com.at.common.R.dimen.dp_221
        }
        2 -> {
            com.at.common.R.dimen.dp_140
        }
        else -> {
            com.at.common.R.dimen.dp_118
        }
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    data class Free(val info: PictureBean)
}
