package com.at.video.atlas.ui.atlas.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.ViewModel
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.video.atlas.R
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.google.android.material.button.MaterialButton
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.SupportTextView
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.TextConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor
import com.video.module_widget.widget.HorizontalIntervalDecoration
import com.video.module_widget.widget.IntervalDecoration

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/8/29 上午10:27
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/8/29 上午10:27
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class AtlasListBinder(): BaseItemBinder<AtlasListBinder.Data,BaseViewHolder>(),OnItemClickListener {

    data class Data(val data:MutableList<Any>,val type:Int = 3,val title:String,val titleStyle:TextConfig?,val dataType:Int = 1)
//
//    init {
//        mAdapter.addItemBinder(CommonFreeBinder())
//        mAdapter.addItemBinder(CommonPayBinder())
//    }
   private val intervalDecoration = IntervalDecoration()
    private val horizontalIntervalDecoration = HorizontalIntervalDecoration()

    override fun convert(holder: BaseViewHolder, data: Data) {
        val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
        holder.setText(R.id.stv_title,data.title)
        val title = holder.getView<SupportTextView>(R.id.stv_title)
        data.titleStyle?.let {
            it.color?.let { it1 -> title.setTextColor(it1.toArgb()) }
            title.textSize = it.size.toFloat()
            title.letterSpacing = it.letterSpacing.toFloat()
        }
        mAdapter.addItemBinder(AtlasFreeBinder(data.type))
        mAdapter.addItemBinder(AtlasPayBinder(data.type))
            holder.getView<RecyclerView>(R.id.rv_data).apply {
           adapter = mAdapter
            when(data.type){
                1->{
                   layoutManager = GridLayoutManager(context, 2)
                   removeItemDecoration(intervalDecoration)
                   addItemDecoration(intervalDecoration)
                }
                2->{
                    layoutManager = GridLayoutManager(context, 3)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
                3->{
                    layoutManager = LinearLayoutManager(context,RecyclerView.HORIZONTAL,false)
                    removeItemDecoration(horizontalIntervalDecoration)
                    addItemDecoration(horizontalIntervalDecoration)
                }
            }
      }
        mAdapter.setNewInstance(data.data)
        mAdapter.setOnItemClickListener(this)


        val change = StyleUtils.change
        if(change!=null){
            val switch = holder.getView<MaterialButton>(R.id.mb_switch)
            holder.setGone(R.id.mb_switch,!change.enable)
            holder.setGone(R.id.mb_more,true)
            holder.setGone(R.id.tv_more,false)
            when(change.style){
                1->{
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_START
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFF8F8F8),context.toColor(R.color.color_FFF8F8F8))
                }
                2->{
                    switch.iconTint = StyleUtils.createColorStateList(context.toColor(R.color.color_FF4B5363),context.toColor(R.color.color_FF4B5363))
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFF8F8F8),context.toColor(R.color.color_FFF8F8F8))
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_END
                }
                else->{
                    holder.setGone(R.id.mb_more,false)
                    holder.setGone(R.id.tv_more,true)
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_END
                    switch.iconTint = StyleUtils.createColorStateList(context.toColor(R.color.color_FFFF613F),context.toColor(R.color.color_FFFF613F))
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFFFF6F4),context.toColor(R.color.color_FFFFF6F4))
                }
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder  = BaseViewHolder(
        LayoutInflater.from(parent.context)
         .inflate(R.layout.atlas_list_data, parent, false))

    override fun convert(holder: BaseViewHolder, data: Data, payloads: List<Any>) {
        val adapter1 = holder.getView<RecyclerView>(R.id.rv_data).adapter as BaseBinderAdapter

        if(payloads.isNotEmpty()){
            adapter1.setNewInstance((payloads[0] as Data).data)
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: Any = adapter.getItem(position)!!
        when(data){
            is AtlasFreeBinder.Free -> {
                Router.getInstance().build("/atlas/DetailActivity").withString("atlasId",data.info.id!!).navigation()
            }
            is AtlasPayBinder.Pay -> {
                Router.getInstance().build("/atlas/DetailActivity").withString("atlasId",data.info.id!!).navigation()
            }
            else ->{

            }
        }
    }
//
//    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
//        when(viewModel){
//            is ClassifyViewModel ->
//                viewModel.sendEvent(ClassifyUiEvent.Collect(adapter.getItem(position) as Any, position))
//            is MoreViewModel ->
//                viewModel.sendEvent(MoreUiEvent.Collect(adapter.getItem(position)!!, position))
//        }
//
//    }
}


