package com.at.video.atlas.ui.classify

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.ui.classify.binder.ClassifyFreeBinder
import com.at.video.atlas.ui.classify.binder.ClassifyPayBinder
import com.at.video.atlas.ui.classify.binder.ConditionsBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.*
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView

/**
 * @Create_time: 2022/5/10 20:20
 * @Description: 图集分类
 */

@Route(path = "/atlas/ClassifyActivity")
class ClassifyActivity :
    BaseMviActivity<CommonTitleBarRefreshListBinding, ClassifyViewModel>(),
    OnItemClickListener,
    OnRefreshLoadMoreListener,
    GridSpanSizeLookup {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ATLAS_CATEGORY)

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ClassifyViewModel> {
        return ClassifyViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        val mCategoryId = intent.getStringExtra("mCategoryId")
        getViewModel().sendEvent(ClassifyUiEvent.InitStyle(homeStyle))
        mCategoryId?.apply {
            getViewModel().sendEvent(ClassifyUiEvent.InitCategoryId(this))
        }

        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitle("分类")
        }

        val listStyle: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.RESULT)
        val spanCount = listStyle?.config?.layout?.style ?: 2
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@ClassifyActivity)
            val footer = LodeMoreView(this@ClassifyActivity)
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(this@ClassifyActivity, spanCount + 1)
            val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
            pool.setMaxRecycledViews(0, 30)
            rvList.setRecycledViewPool(pool)
            rvList.addItemDecoration(ClassifyDecoration())
            rvList.setBackgroundColor(Color.WHITE)
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
//            homeStyle?.background?.let { rvList.setBackgroundColor(it.toArgb()) }
        }

        mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
            override fun getSpanSize(
                gridLayoutManager: GridLayoutManager,
                viewType: Int,
                position: Int
            ): Int {
                if (viewType == 1 || viewType == 2 || viewType == 3 || viewType == 4 || viewType == 5 || viewType == 6) {
                    return spanCount + 1
                }
                return 1
            }
        })

        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 10)
        mAdapter.addItemBinder(ConditionsBinder(pool, getViewModel(), JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.FILTER_BAR), homeStyle?.background!!))
        mAdapter.addItemBinder(BannerBinder(JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.SWIPER_ADV)))
        mAdapter.addItemBinder(AdImageBinder(showMarginVertical = false, showMarginHorizontal = false, style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV), bg = StyleUtils.getPageBackground(StyleConsts.ATLAS_CATEGORY)))
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(GapLineBinder(StyleUtils.getPageBackground(StyleConsts.ATLAS_CATEGORY).toArgb()))
        mAdapter.addItemBinder(NoDataBinder())
        mAdapter.addItemBinder(ClassifyFreeBinder(spanCount))
        mAdapter.addItemBinder(ClassifyPayBinder(spanCount))
        mAdapter.setOnItemClickListener(this)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(ClassifyUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(ClassifyUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(ClassifyUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        when (val data: Any = adapter.getItem(position)!!) {
            is ClassifyFreeBinder.Free -> {
                Router.getInstance().build("/atlas/DetailActivity").withString("atlasId", data.info.id!!).navigation()
            }
            is ClassifyPayBinder.Pay -> {
                Router.getInstance().build("/atlas/DetailActivity").withString("atlasId", data.info.id!!).navigation()
            }
//            is AdImageBinder.Data -> Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//            is AdTextBinder.Data ->  Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
        }
    }

    override fun getSpanSize(
        gridLayoutManager: GridLayoutManager,
        viewType: Int,
        position: Int
    ): Int {
        return if (viewType == 1) gridLayoutManager.spanCount else 1
    }
}
