package com.at.video.atlas.ui.detail

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Color
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import android.widget.ImageButton
import android.widget.TextView
import androidx.activity.result.ActivityResult
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.immersiveStatusBar
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.atlas.R
import com.at.video.atlas.entity.PictureDetail
import com.at.video.atlas.ui.AtlasDialog
import com.at.video.atlas.widget.ImageViewer
import com.at.video.atlas.widget.PreCacheLayoutManager
import com.at.video.atlas.widget.loader.CoilLoader
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Autowired
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.TitleBar
import com.melancholy.widget.decoration.DefaultDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.dialog.ShareDialog
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.CommonStringUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/10 21:38
 * @Description:
 */

// @Route(path = "/atlas/DetailActivity")
@Route(path = "/atlas/DetailActivity", extras = 10086)
@AndroidEntryPoint
class DetailActivity :
    BaseMviActivity<CommonTitleBarRefreshListBinding, DetailViewModel>(),
    View.OnClickListener,
    OnRefreshLoadMoreListener {
    private val mAdapter: DetailAdapter = DetailAdapter()

    //    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ATLAS_DETAIL)

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    private var thumbUp: ImageButton? = null
    private var collect: ImageButton? = null
    private var shareUrl: ImageButton? = null
    private var atlasTitle: TextView? = null
    private val TAG = "DetailActivity"
    private var isInitData = false

    @JvmField
    @Autowired
    var atlasId: String = ""

    private val mStartForResult: ActivityResultLauncher<Intent> =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result: ActivityResult ->
            val data = result.data
            if (result.resultCode == RESULT_OK && data != null) {
                val pos = data.getIntExtra("pos", -1)
                if (pos >= 0 && curPos != pos) {
                    requireBinding().includedStatusRefreshList.includedRefreshList.rvList.scrollToPosition(
                        pos
                    )
                }
            }
        }

    override fun initialize(savedInstanceState: Bundle?) {
        Router.getInstance().inject(this)
        getViewModel().sendEvent(DetailUiEvent.InitStyle(homeStyle))
        val topBar: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.TOP_BAR)
        topIconStyle = topBar?.config?.style ?: 1
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            atlasTitle = TextView(this@DetailActivity)
            atlasTitle!!.text = ""
            val width = DensityUtils.getDisplayMetrics(this@DetailActivity).widthPixels
            val params: TitleBar.LayoutParams = TitleBar.LayoutParams(width / 2, WRAP_CONTENT)
            addLeftView(atlasTitle!!, android.R.id.title, params)
            atlasTitle!!.setSingleLine()
            atlasTitle!!.paint.isFakeBoldText = true
            atlasTitle!!.ellipsize = TextUtils.TruncateAt.END
            atlasTitle!!.textSize = DensityUtils.px2sp(
                this@DetailActivity,
                resources.getDimension(com.at.common.R.dimen.sp_16).toInt()
            )
            atlasTitle!!.setTextColor(
                ContextCompat.getColor(
                    this@DetailActivity,
                    R.color.color_FF484A4D
                )
            )
            // atlasTitle = setTitle("诛仙·御剑行")
            shareUrl = addRightImageButton(
                getDrawableResId(3),
                android.R.id.button3
            )
            collect = addRightImageButton(getDrawableResId(2), android.R.id.button2)
            thumbUp = addRightImageButton(getDrawableResId(1), android.R.id.button1)
            shareUrl?.apply {
                val params = layoutParams as ViewGroup.MarginLayoutParams
                params.rightMargin = DensityUtils.dp2px(
                    this@DetailActivity,
                    resources.getDimensionPixelSize(com.at.common.R.dimen.dp_8).toFloat()
                )
                layoutParams = params
            }
            shareUrl?.setOnClickListener(this@DetailActivity)
            thumbUp?.setOnClickListener(this@DetailActivity)
            collect?.setOnClickListener(this@DetailActivity)
        }
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            val linearLayout = PreCacheLayoutManager(this@DetailActivity)
            linearLayout.setExtraSpace(2)
            linearLayout.orientation = LinearLayoutManager.VERTICAL
            rvList.layoutManager = linearLayout
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            srlRefresh.setOnRefreshLoadMoreListener(this@DetailActivity)
            rvList.setItemViewCacheSize(2)
            rvList.itemAnimator = null
            rvList.visibility = View.INVISIBLE
            rvList.adapter = mAdapter
            val padding: Int = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_16)
            rvList.addItemDecoration(
                DefaultDecoration.create(
                    padding,
                    padding,
                    padding,
                    padding,
                    padding,
                    padding,
                    isIncludeEdge = true
                )
            )
            rvList.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.ATLAS_DETAIL).toArgb()
            )

            rvList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
                override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                    // 获取RecyclerView当前顶部显示的第一个条目对应的索引
                    val manager = recyclerView.layoutManager as LinearLayoutManager
                    val firstPos: Int = manager.findFirstVisibleItemPosition()
                    val lastPos: Int = manager.findLastVisibleItemPosition()
                    // //根据索引来获取对应的itemView
//                     val firstVisiableChildView: View? = manager.findViewByPosition(position)
                    //
                    // //获取当前显示条目的高度
                    // val itemHeight = firstVisiableChildView?.height?:0
                    //
                    // //获取当前Recyclerview 偏移量
                    // val flag = position * itemHeight - firstVisiableChildView?.top!!

                    val pos = lastPos
                    Log.e(TAG, "onScrolled: pos:$pos,curPos:$curPos")
                    if (curPos < pos + 1) { // 上报
                        curPos = pos
                        getViewModel().sendEvent(DetailUiEvent.UpdateBrowser(curPos + 1))
                        // Log.i("TAG", "onScrolled: pos: $pos")
                    }
                    super.onScrolled(recyclerView, dx, dy)
                }
            })
        }
//        mAdapter.addItemBinder(BannerBinder())
//        mAdapter.addItemBinder(AdImageBinder(showMarginHorizontal =false))
//        mAdapter.addItemBinder(AdTextBinder(showMarginHorizontal = false))
//        mAdapter.addItemBinder(ImageBinder())
        mAdapter.setOnItemClickListener { adapter, view, position ->
            // data 可以多张图片List或单张图片，支持的类型可以是{@link Uri}, {@code url}, {@code path},{@link File}, {@link DrawableRes resId}…等
            // 图片查看器
            // data 可以多张图片List或单张图片，支持的类型可以是{@link Uri}, {@code url}, {@code path},{@link File}, {@link DrawableRes resId}…等
            ImageViewer.load(adapter.data) // 要加载的图片数据，单张或多张
                .selection(position) // 当前选中位置
                .indicator(true) // 是否显示指示器，默认不显示
                .imageLoader(CoilLoader()) // 加载器，*必须配置，目前内置的有GlideImageLoader或PicassoImageLoader，也可以自己实现
                // .imageLoader(new PicassoImageLoader())
                .theme(R.style.ImageViewerTheme) // 设置主题风格，默认：R.style.ImageViewerTheme
                .orientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) // 设置屏幕方向,默认：ActivityInfo.SCREEN_ORIENTATION_BEHIND
                .start(this@DetailActivity, view, mStartForResult)
        }
    }

    private fun getDrawableResId(type: Int = 1): Int {
        var praiseDrawable = com.video.module_widget.R.drawable.vector_drawable_raise_normal1
        var collectDrawable = com.video.module_widget.R.drawable.vector_drawable_collect_normal1
        var shareDrawable = com.video.module_widget.R.drawable.vector_drawable_share1
        when (topIconStyle) {
            2 -> {
                praiseDrawable = com.video.module_widget.R.drawable.vector_drawable_raise_normal2
                collectDrawable = com.video.module_widget.R.drawable.vector_drawable_collect_normal2
                shareDrawable = com.video.module_widget.R.drawable.vector_drawable_share2
            }
            3 -> {
                praiseDrawable = com.video.module_widget.R.drawable.vector_drawable_raise_normal3
                collectDrawable = com.video.module_widget.R.drawable.vector_drawable_collect_normal3
                shareDrawable = com.video.module_widget.R.drawable.vector_drawable_share3
            }
        }
        return when (type) {
            1 -> praiseDrawable
            2 -> collectDrawable
            else -> shareDrawable
        }
    }

    var topIconStyle: Int = 1
    private fun getPraiseResId(isSelected: Boolean): Int {
        var praiseDrawable =
            if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected1 else com.video.module_widget.R.drawable.vector_drawable_raise_normal1
        when (topIconStyle) {
            2 -> {
                praiseDrawable =
                    if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected2 else com.video.module_widget.R.drawable.vector_drawable_raise_normal2
            }
            3 -> {
                praiseDrawable =
                    if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected3 else com.video.module_widget.R.drawable.vector_drawable_raise_normal3
            }
        }
        return praiseDrawable
    }

    private fun getCollectResId(isSelected: Boolean): Int {
        var collectDrawable =
            if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected1 else com.video.module_widget.R.drawable.vector_drawable_collect_normal1
        when (topIconStyle) {
            2 -> {
                collectDrawable =
                    if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected2 else com.video.module_widget.R.drawable.vector_drawable_collect_normal2
            }
            3 -> {
                collectDrawable =
                    if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected3 else com.video.module_widget.R.drawable.vector_drawable_collect_normal3
            }
        }
        return collectDrawable
    }

    private var curPos: Int = -1

    override fun loadData() {
        if (atlasId == "") {
            ToastUtils.show("获取图集失败,请稍后重试")
            onBack()
        }
    }

    override fun onResume() {
        super.onResume()
        getViewModel().sendEvent(DetailUiEvent.AtlasInfo(atlasId))
    }

    private var dialog: Dialog? = null
    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is DetailUiEffect.Loading -> LoadingUtil.showLoading(
                        this@DetailActivity,
                        it.msg
                    )
                    is DetailUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is DetailUiEffect.Toast -> {
                        ToastUtils.show(it.toast)
                        if (it.finish) {
                            finish()
                        }
                    }
                    is DetailUiEffect.Success -> {
                        refreshData(it.atlas)
                    }
                    is DetailUiEffect.CollectChange -> {
                        refreshData(it.atlas)
                    }
                    is DetailUiEffect.ThumbUpChange -> {
                        refreshData(it.atlas)
                    }
                    is DetailUiEffect.BuyAtlasState -> {
                        if (!it.state) { // 购买不成功情况展示
                            showGoldDialog()
                        } else {
                            hideDialog()
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::pos) {
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::datas) {
                if (it.isNullOrEmpty()) {
                    return@collectState
                }
                val datas = it as MutableList<String>
                mAdapter.setNewInstance(datas)
                dealWatchHistory()
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
    }

    private fun dealWatchHistory() {
        if (!isInitData) {
            val pos = getViewModel().state.value.pos - 1
            if (pos > 1) {
                requireBinding().includedStatusRefreshList.includedRefreshList.rvList.postDelayed(
                    {
                        requireBinding().includedStatusRefreshList.includedRefreshList.rvList.scrollToPosition(
                            pos
                        )
                        requireBinding().includedStatusRefreshList.includedRefreshList.rvList.visibility =
                            View.VISIBLE
                        isInitData = true
                    },
                    500
                )
            } else {
                requireBinding().includedStatusRefreshList.includedRefreshList.rvList.visibility =
                    View.VISIBLE
            }
        }
    }

    private fun hideDialog() {
        ToastUtils.show("购买成功")
        dialog?.dismiss()
    }

    private var goldDialog: Dialog? = null
    private fun showGoldDialog() {
        if (goldDialog == null) {
            goldDialog = AtlasDialog.createNoGoldDialog(
                this,
                object : AtlasDialog.BuyGoldListener {
                    override fun buyGold() {
                        goldDialog?.dismiss()
                        Router.getInstance().build("/member/GoldActivity").navigation()
                    }
                }
            )
        } else {
            goldDialog?.show()
        }
    }

    private fun showMyDialog(gold: Int) {
        if (dialog == null) {
            dialog =
                AtlasDialog.createPermissionDialog(
                    gold,
                    this,
                    object : AtlasDialog.AtlasListener {
                        override fun byGold() {
                            getViewModel().sendEvent(DetailUiEvent.BuyByGold)
                        }

                        override fun byMember() {
                            Router.getInstance().build("/member/RechargeActivity").navigation()
                        }
                    }
                )
        } else {
            dialog?.show()
        }
    }

    override fun onDestroy() {
        dialog?.apply {
            if (isShowing) {
                dismiss()
            }
        }
        super.onDestroy()
    }

    // 刷新数据
    @SuppressLint("SetTextI18n")
    private fun refreshData(atlas: PictureDetail?) {
        if (atlas == null || atlas.picture_chapter_list?.list.isNullOrEmpty()) {
            ToastUtils.show("获取图集失败,请稍后重试")
            finish()
        }
        // 添加更新播放
//        getViewModel().sendEvent(DetailUiEvent.UpdateBrowser(atlas?.watchHistory?.index ?: 0))

        // 获取成功图集列表
        atlas?.apply {
//            imagesList?.let {
//                mAdapter.setNewInstance(it.toMutableList())
//            }
            atlasTitle?.text = CommonStringUtils.splicingCommaStrings(
                atlas.picture?.name,
                atlas.picture?.categ_name,
                atlas.picture?.subcateg_name
            )
            thumbUp?.setImageResource(getPraiseResId(picture!!.is_star))
            collect?.setImageResource(getCollectResId(picture!!.is_collect))
//            if (useAuth == 2) {//无权限展示dialog购买
//                showMyDialog(chargingCoin)
//            }
        }
    }

    override fun immersiveBar() {
        immersiveStatusBar()
        setLightStatusBar(true)
    }

    override fun onClick(p0: View) {
        when (p0.id) {
            android.R.id.button1 -> {
                getViewModel().sendEvent(DetailUiEvent.ThumbUp)
            }
            android.R.id.button2 -> {
                getViewModel().sendEvent(DetailUiEvent.Collect)
            }
            android.R.id.button3 -> {
//                ClipboardUtils.copyText(
//                    this@DetailActivity,
//                    IMainService.getMainService().getSysConfig()
//                        .getConfigShareAddr() + "/atlas-detail/${atlasId}"
//                )
                if (GlobalConsts.SHARE_URL.isNullOrEmpty()) {
                    ToastUtils.show("功能暂未开放")
                } else {
                    ShareDialog().show(supportFragmentManager, "share_address")
                    getViewModel().sendEvent(DetailUiEvent.Share)
                }
            }
        }
    }

    override fun getViewModelClass(): Class<DetailViewModel> = DetailViewModel::class.java
    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DetailUiEvent.Refresh)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        refreshLayout.finishLoadMoreWithNoMoreData()
    }
}
