package com.at.video.atlas.ui.more

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.immersiveStatusBar
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.ui.atlas.AtlasDecoration
import com.at.video.atlas.ui.atlas.AtlasUiEvent
import com.at.video.atlas.ui.atlas.AtlasViewModel
import com.at.video.atlas.ui.atlas.binder.AtlasFreeBinder
import com.at.video.atlas.ui.atlas.binder.AtlasPayBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Autowired
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.GapLineBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/23 下午2:33
 */
@AndroidEntryPoint
@Route(path = "/atlas/AtlasMoreActivity")
class AtlasMoreActivity :
    BaseMviActivity<CommonTitleBarRefreshListBinding, AtlasMoreViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener {
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ATLAS_MORE)

    @JvmField
    @Autowired
    var parameter: String = ""
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()

    override fun initialize(savedInstanceState: Bundle?) {
        Router.getInstance().inject(this)
        requireTitleBar().apply {
            setTitle(parameter)
            addBackButton(Color.BLACK)
        }
        getViewModel().sendEvent(MoreUiEvent.InitStyle(homeStyle))
        val listStyle: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.ATLAS_RESULT)
        val spanCount = listStyle?.config?.layout?.style ?: 2
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@AtlasMoreActivity)
            val footer = LodeMoreView(this@AtlasMoreActivity)
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(this@AtlasMoreActivity, spanCount + 1)
            rvList.addItemDecoration(AtlasMoreDecoration())
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
        }

        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle,
                    StyleConsts.SWIPER_ADV
                )
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginHorizontal = false,
                showMarginVertical = false,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV)
            )
        )
        mAdapter.addItemBinder(
            AdTextBinder(
                showMarginHorizontal = false,
                showMarginVertical = false
            )
        )
        mAdapter.addItemBinder(
            GapLineBinder(
                StyleUtils.getPageBackground(StyleConsts.ATLAS_MORE).toArgb()
            )
        )

        mAdapter.addItemBinder(AtlasFreeBinder(spanCount, true))
        mAdapter.addItemBinder(AtlasPayBinder(spanCount, true))

        mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
            override fun getSpanSize(
                gridLayoutManager: GridLayoutManager, viewType: Int,
                position: Int
            ): Int {
                if (viewType == 1 || viewType == 2 || viewType == 3 || viewType == 4) {
                    return gridLayoutManager.spanCount
                }
                return 1
            }

        })

        mAdapter.setOnItemClickListener(this@AtlasMoreActivity)
    }

    override fun immersiveBar() {
        immersiveStatusBar()
        setLightStatusBar(true)
    }

    override fun loadData() {
        //首次获取数据
        getViewModel().sendEvent(MoreUiEvent.SetParameter(parameter))
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
    }

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<AtlasMoreViewModel> {
        return AtlasMoreViewModel::class.java
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoreUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoreUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        when (val data: Any = adapter.getItem(position)!!) {
            is AtlasFreeBinder.Free -> {
                Router.getInstance().build("/atlas/DetailActivity")
                    .withString("atlasId", data.info.id!!).navigation()
            }
            is AtlasPayBinder.Pay -> {
                Router.getInstance().build("/atlas/DetailActivity")
                    .withString("atlasId", data.info.id!!).navigation()
            }
        }
    }
}