package com.at.video.atlas.ui.search.result

import android.annotation.SuppressLint
import android.widget.FrameLayout
import android.widget.ImageView
import com.at.video.atlas.R
import com.at.video.atlas.entity.PictureBean
import com.at.video.atlas.ui.search.stringToHighLight
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.shape.view.ShapeMaterialButton
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeMaterialButtonExt
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/12 13:22
 * @Description:
 */

class SearchResultAdapter: BaseMultiItemQuickAdapter<PictureBean, BaseViewHolder>() {
    private var mKeyword: String = ""

    init {
        addItemType(0, R.layout.atlas_item_free_view)
        addItemType(1, R.layout.atlas_item_pay_view)
    }

    override fun convert(holder: BaseViewHolder, data: PictureBean) {
        val category = data.categ_name?:""
        holder.setText(R.id.tv_title, data.name!!.stringToHighLight(mKeyword))
            .setText(R.id.tv_category,category)
        holder.getView<ImageView>(R.id.div_img)
            .loadByGlide(
                GlideImageConfig(data.cover_vertical.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default),
                WeakReference(context)
            )
        if(data.price_type != 1) {
            changeTagStyle(holder.getView<ShapeMaterialButtonExt>(R.id.sb_pay_money))
            holder.setText(R.id.sb_pay_money, data.price.toString())
            holder.setGone(R.id.sb_pay_money,!StyleUtils.tagEnable)
        }else{
            changeTagStyle(holder.getView<ShapeTextViewExt>(R.id.stv_tag))
            holder.setGone(R.id.stv_tag,!StyleUtils.tagEnable)
        }
        holder.getView<ShapeMaterialButton>(R.id.smb_browser).setOnClickListener {
            holder.itemView.performClick()
        }
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt){
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when(StyleUtils.tagStyle){
            2->{
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3->{
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10,com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }
    private fun changeTagStyle(tagView: ShapeMaterialButtonExt){
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when(StyleUtils.tagStyle){
            2->{
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3->{
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10,com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setKeyword(keyword: String) {
        mKeyword = keyword
        notifyDataSetChanged()
    }
    fun getKeyword():String{
        return mKeyword
    }
}