package com.at.video.game.ui.game.binder // ktlint-disable package-name

import android.graphics.Typeface
import android.view.Gravity
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.global.Constants
import com.at.common.utils.JsonUtils
import com.at.video.game.R
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.melancholy.utils.DensityUtils
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.WordStyle
import com.video.module_widget.utils.toRgb

/**
 * @Create_time: 2022/6/23 19:44
 * @Description:
 */

class AdTextBinder(private val showMarginVertical: Boolean = false, private val showMarginHorizontal: Boolean = true) : BaseItemBinder<AdTextBinder.Data, BaseViewHolder>() {
    private val mPadding: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_16)
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.game_ad_text_view, parent, false)
        )
    }

    override fun convert(holder: BaseViewHolder, data: Data) {
        val cardView: CardView = holder.itemView as CardView
        val params = cardView.layoutParams
        val frameLayout = holder.getView<FrameLayout>(R.id.ad_root)
        val param = frameLayout.layoutParams as GridLayoutManager.LayoutParams
        if (showMarginVertical && showMarginHorizontal) {
            param.setMargins(mPadding, mPadding, mPadding, 0)
        }
        if (!showMarginHorizontal && !showMarginVertical) {
            param.setMargins(0, 0, 0, 0)
        }

        if (!showMarginHorizontal && showMarginVertical) {
            param.setMargins(0, mPadding, 0, 0)
        }
        val text: TextView = holder.getView(R.id.tv_content)
        val textParams: FrameLayout.LayoutParams = text.layoutParams as FrameLayout.LayoutParams
        text.text = data.data.image_url
        val wordStyle = data.data.word_style?.let { parseStyle(it) }
        if (wordStyle != null) {
            wordStyle.background?.let { cardView.setCardBackgroundColor(it.toRgb()) }
            wordStyle.height?.let {
                params.height = DensityUtils.dp2px(context, it.replace("px", "").toFloat())
            }
            wordStyle.textAlign?.let {
                textParams.gravity = when (it) {
                    "left" -> Gravity.LEFT or Gravity.CENTER_VERTICAL
                    "right" -> Gravity.RIGHT or Gravity.CENTER_VERTICAL
                    else -> Gravity.CENTER
                }
            }
            wordStyle.fontWeight?.let {
                text.typeface = if (it != "normal") Typeface.DEFAULT_BOLD else Typeface.DEFAULT
            }
            wordStyle.fontSize?.let {
                text.textSize = it.replace("px", "").toFloat()
            }
            wordStyle.color?.let {
                text.setTextColor(it.toRgb())
            }
        }
        holder.itemView.layoutParams = params
        text.layoutParams = textParams
        holder.itemView.setOnClickListener {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!)
                .navigation()
        }
    }

    data class Data(val data: AdvertisingBean)
    private fun parseStyle(json: String): WordStyle? {
        if (json.isEmpty()) {
            return null
        }
        return JsonUtils.fromJson(json)
    }
}
