package com.at.video.home.service

import android.app.Application
import android.app.Service
import android.content.Intent
import android.os.*
import android.util.Log
import com.at.module_export_main.IMainService
import com.at.video.export.mine.IMineService
import com.at.video.home.http.ApiService
import com.melancholy.network.ApiFactory
import com.melancholy.utils.AppInstance
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.dialog.SystemDialog
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.PhoneUtils
import kotlinx.coroutines.*

/**
 * @Create_time: 2022/6/20 17:43
 * @Description:
 */

class HeartbeatService : Service() {
    private var mHeartbeatJob: Job? = null

    companion object {
        @JvmStatic
        fun start() {
            val application: Application = AppInstance.getApplication()
            application.startService(Intent(application, HeartbeatService::class.java))
        }

        @JvmStatic
        fun stop() {
            val application: Application = AppInstance.getApplication()
            application.stopService(Intent(application, HeartbeatService::class.java))
        }
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        startHeartbeat()
        return START_NOT_STICKY
    }

    private fun startHeartbeat() {
        mHeartbeatJob = GlobalScope.launch(Dispatchers.IO) {
            while (true) {
                Log.d("TAG123456", "startHeartbeat")
                try {
                    val bean = ApiFactory.getInstance().create(ApiService::class.java).heartbeat(
                        systemVersion = "Android${PhoneUtils.getDeviceAndroidVersion()}",
                        deviceName = "${PhoneUtils.getDeviceBrand()} ${PhoneUtils.getDeviceModel()}"
                    )
                    when (bean.code) {
                        403 -> {
                            Log.d("TAG123456", "startHeartbeat${bean.message}")
                            synchronized(HeartbeatService::class.java) {
                                val intent = Intent(this@HeartbeatService, SystemDialog::class.java)
                                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                                intent.putExtra("Message", bean.message)
                                startActivity(intent)
                            }
                            return@launch
                        }
                        401, 200 -> delay(10_000)
                        else -> {}
                    }
                    val map: MutableMap<String, Any> = mutableMapOf()
                    val activeRecord = DataUtils.dataReport(2).active_record!!
                    activeRecord.user_id = IMineService.getMineService().getUser().value?.getUserId()
                    map["active_record"] = activeRecord
                    try {
                        val res = ApiFactory.getInstance().create(CommonApiService::class.java).dataReport(
                            GlobalConsts.DATA_REPORT + "/v1/basic_record/create",
                            map
                        )
                        if (res.success()) {
                            Log.d("DATA_REPORT", "上报成功")
                        } else {
                            Log.d("DATA_REPORT", "上报失败")
                        }
                    } catch (e: Exception) {
                        Log.e("Exception", e.message.toString())
                    }
                } catch (e: Exception) {
//                synchronized(HeartbeatService::class.java){
//                    val intent = Intent(this@HeartbeatService, SystemDialog::class.java)
//                    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
//                    intent.putExtra("Message","服务器异常，请稍后再试!")
//                    startActivity(intent)
//                    return@launch
//                }
                    delay(10_000)
                }
            }
        }
    }

    override fun onDestroy() {
        mHeartbeatJob?.cancel()
        super.onDestroy()
    }
}
