package com.at.video.home.ui.classify

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.home.R
import com.at.video.home.ui.classify.binder.ClassifyFreeBinder
import com.at.video.home.ui.classify.binder.ClassifyPayBinder
import com.at.video.home.ui.classify.binder.ConditionsBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DensityUtils
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.*
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView

/**
 * @Create_time: 2022/5/10 20:20
 * @Description: 视频分类
 */

@Route(path = "/home/ClassifyActivity")
class ClassifyActivity : BaseMviActivity<CommonTitleBarRefreshListBinding, ClassifyViewModel>(),
    OnItemClickListener, OnRefreshLoadMoreListener, OnItemChildClickListener {
    private val mAdapter = BaseBinderAdapter()

    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.VIDEO_FILTER)

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ClassifyViewModel> {
        return ClassifyViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        val reType: Int = intent.getIntExtra(Constants.KEY_DATA_1, -1)
        val id: String = intent.getStringExtra(Constants.KEY_DATA_2)!!
        getViewModel().sendEvent(ClassifyUiEvent.HeaderData(id, reType, homeStyle))
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitle(intent.getStringExtra(Constants.KEY_DATA))
            val searchBtn =
                addRightImageButton(R.drawable.home_ic_search_title_bar_search, android.R.id.title)
            searchBtn.setOnClickListener {
                Router.getInstance().build("/home/SearchActivity").navigation()
            }

            val layoutParams = searchBtn.layoutParams as? ViewGroup.MarginLayoutParams
            layoutParams?.apply {
                rightMargin = DensityUtils.dp2px(
                    this@ClassifyActivity, resources.getDimension(com.at.common.R.dimen.dp_4)
                )
                searchBtn.layoutParams = this
            }
        }
        val listStyle: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.RESULT)
        val spanCount = listStyle?.config?.layout?.style ?: 2
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            val footer = LodeMoreView(this@ClassifyActivity)
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setOnRefreshLoadMoreListener(this@ClassifyActivity)
            srlRefresh.setBackgroundColor(Color.WHITE)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(this@ClassifyActivity, spanCount + 1)
            val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
            pool.setMaxRecycledViews(0, 30)
            rvList.setRecycledViewPool(pool)
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
            rvList.addItemDecoration(ClassifyDecoration())
            mAdapter.setOnItemClickListener(this@ClassifyActivity)
            mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
                override fun getSpanSize(
                    gridLayoutManager: GridLayoutManager, viewType: Int, position: Int
                ): Int {
                    if (viewType == 1 || viewType == 2 || viewType == 3 || viewType == 4 || viewType == 5 || viewType == 6) {
                        return spanCount + 1
                    }
                    return 1
                }
            })
            mAdapter.addChildClickViewIds(R.id.iv_collect)
            mAdapter.setOnItemChildClickListener(this@ClassifyActivity)
        }


        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 40)
        mAdapter.addItemBinder(
            ConditionsBinder(
                pool,
                getViewModel(),
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.CONDITION)
            ), ConditionsBinder.ConditionsCallback()
        )
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.SWIPER_ADV
                )
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginVertical = false,
                showMarginHorizontal = false,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV),
                bg = StyleUtils.getPageBackground(StyleConsts.VIDEO_FILTER)
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(NoDataBinder())
        mAdapter.addItemBinder(
            GapLineBinder(
                StyleUtils.getPageBackground(StyleConsts.VIDEO_FILTER).toArgb()
            )
        )
//        mAdapter.addItemBinder(ClassifyVideoListBinder(this))
        mAdapter.addItemBinder(ClassifyFreeBinder(spanCount))
        mAdapter.addItemBinder(ClassifyPayBinder(spanCount))
        mAdapter.setOnItemClickListener(this@ClassifyActivity)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::data) {
                mAdapter.setDiffNewData(it.toMutableList())
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ClassifyUiState::collectStatus) {
                if (it.childPos == -1) {
                    return@collectState
                }
                try {
                    if (mAdapter.data[it.childPos] is ClassifyFreeBinder.Free) {
                        (mAdapter.data[it.childPos] as ClassifyFreeBinder.Free).info.apply {
                            is_collect = it.isCollect
                        }
                    } else if (mAdapter.data[it.childPos] is ClassifyPayBinder.Pay) {
                        (mAdapter.data[it.childPos] as ClassifyPayBinder.Pay).info.apply {
                            is_collect = it.isCollect
                        }
                    }
                    mAdapter.notifyItemChanged(it.childPos, ClassifyFreeBinder.REFRESH_COLLECT)
                } catch (e: java.lang.IndexOutOfBoundsException) {

                }

            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is ClassifyUiEffect.Toast -> ToastUtils.show(it.toast)
                    else -> {}
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(ClassifyUiEvent.InitLoadData)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: Any = adapter.getItem(position)!!
        if (data is AdImageBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        } else if (data is AdTextBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        } else if (data is ClassifyFreeBinder.Free || data is ClassifyPayBinder.Pay) {
            val id = when (val data = adapter.getItem(position)) {
                is ClassifyFreeBinder.Free -> data.info.id
                is ClassifyPayBinder.Pay -> data.info.id
                else -> "-1"
            }
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, id!!).navigation()
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(ClassifyUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(ClassifyUiEvent.LoadDataMore)
    }


    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        getViewModel().sendEvent(
            ClassifyUiEvent.Collect(
                adapter.getItem(position) as Any, position
            )
        )
    }
}