package com.at.video.home.ui.classify.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.video.home.R
import com.at.video.home.ui.home.IntervalDecoration
import com.at.video.home.ui.home.binder.CommonSingleBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/8/29 上午10:27
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/8/29 上午10:27
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class ClassifyVideoListBinder(private val onChild: OnItemChildLsViewClickListener) :
    BaseItemBinder<ClassifyVideoListBinder.Data, BaseViewHolder>(), OnItemClickListener {

    data class Data(val data: MutableList<Any>, val type: Int = 1, val isMore: Boolean = false)

    companion object {
        const val REFRESH_COLLECT = "collect_refresh"
    }

    private val intervalDecoration = IntervalDecoration()
    override fun convert(holder: BaseViewHolder, data: Data) {
        val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
        if (data.isMore) {
            mAdapter.addItemBinder(CommonSingleBinder())
        }
        if (!data.isMore || data.type == 1) {
            mAdapter.addItemBinder(ClassifyFreeBinder(data.type))
            mAdapter.addItemBinder(ClassifyPayBinder(data.type))
        }
        holder.getView<RecyclerView>(R.id.rv_data).apply {
            adapter = mAdapter
            when (data.type) {
                1 -> {
                    layoutManager = GridLayoutManager(context, 2)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
                2 -> {
                    if (data.isMore) {
                        layoutManager = LinearLayoutManager(context, RecyclerView.HORIZONTAL, false)
                        removeItemDecoration(intervalDecoration)
                        addItemDecoration(intervalDecoration)
                    } else {
                        layoutManager = GridLayoutManager(context, 3)
                        removeItemDecoration(intervalDecoration)
                        addItemDecoration(intervalDecoration)
                    }
                }
            }
        }
        mAdapter.setNewInstance(data.data)
        mAdapter.setOnItemClickListener(this)
        mAdapter.addChildClickViewIds(R.id.iv_collect)
        mAdapter.notifyItemChanged(1, "sss")
        mAdapter.setOnItemChildClickListener { adapter, view, position ->
            onChild.onlistViewClick(holder.layoutPosition, adapter, view, position)
        }
    }

    override fun convert(holder: BaseViewHolder, data: Data, payloads: List<Any>) {
        payloads.forEach { it ->
            when (it) {
                REFRESH_COLLECT -> {
                    try {
                        (holder.getView<RecyclerView>(R.id.rv_data).adapter as BaseBinderAdapter).apply {
                            setNewInstance(data.data)
                            notifyDataSetChanged()
                        }
                    } catch (e: Exception) {

                    }

                }
            }
        }

    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder =
        BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.home_video_list_binder, parent, false)
        )

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: Any = adapter.getItem(position)!!
        if (data is ClassifyFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        } else if (data is ClassifyPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        }
    }

    /**
     * 子RecylerView点击事件传递接口
     */
    interface OnItemChildLsViewClickListener {
        fun onlistViewClick(
            parentPos: Int, adapter: BaseQuickAdapter<*, *>, view: View, position: Int
        )
    }


}


