package com.at.video.home.ui.classify.binder

import android.graphics.Color
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.video.home.entity.ConditionsInfo
import com.at.video.home.ui.classify.ClassifyTypeAdapter
import com.at.video.home.ui.classify.ClassifyUiEvent
import com.at.video.home.ui.classify.ClassifyViewModel
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.decoration.DefaultDecoration
import com.video.module_widget.entity.Content

/**
 * @Create_time: 2022/5/27 10:31
 * @Description:
 */

class ConditionsBinder constructor(private val pool: RecyclerView.RecycledViewPool,
                                   private val mViewModel: ClassifyViewModel,val style:Content? = null):
    BaseItemBinder<ConditionsBinder.Conditions, BaseViewHolder>(), OnItemClickListener {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        Log.d("onCreateViewHolder", "onCreateViewHolder")
        val adapter = ClassifyTypeAdapter(style)
        val recyclerView = RecyclerView(parent.context)
        recyclerView.layoutManager = LinearLayoutManager(parent.context, RecyclerView.HORIZONTAL,
            false)
        val params = ViewGroup.MarginLayoutParams(MATCH_PARENT,
            WRAP_CONTENT)
        val padding: Int = parent.resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_8)
        recyclerView.setPadding(0, padding, 0, padding)
        recyclerView.layoutParams = params
        recyclerView.setHasFixedSize(false)
        recyclerView.setBackgroundColor(Color.WHITE)
        //不复用
        recyclerView.recycledViewPool.setMaxRecycledViews(viewType,0)
        // recyclerView.setRecycledViewPool(pool)
        recyclerView.itemAnimator = null
        recyclerView.setBackgroundColor(Color.WHITE)
        val hPadding: Int = parent.resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_16)
        recyclerView.addItemDecoration(DefaultDecoration.create(hPadding, 0, hPadding,
                0, 0, 0))
        recyclerView.adapter = adapter
        adapter.setOnItemClickListener(this)
        return BaseViewHolder(recyclerView)
    }

    override fun convert(holder: BaseViewHolder, data: Conditions) {
        val newAdapter = ((holder.itemView as RecyclerView).adapter as ClassifyTypeAdapter)
//        Log.d("convert", "${holder.bindingAdapterPosition} ${data.list[1].name}")
        newAdapter.setConditions(data)
    }

    override fun convert(holder: BaseViewHolder, data: Conditions, payloads: List<Any>) {
        val newAdapter = ((holder.itemView as RecyclerView).adapter as ClassifyTypeAdapter)
//        Log.d("convert payloads", "${holder.bindingAdapterPosition} ${data.list[1].name}")
        newAdapter.setConditions(data)
    }

    //type 0区域 1分类 2上映年代 3费用类型 4排序方式
    data class Conditions(val list: List<ConditionsInfo>, val key: String, var index: Int = 0)

    override fun onItemClick(baseAdapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        Log.d("convert click", (baseAdapter.getItem(position) as ConditionsInfo).name)
        mViewModel.sendEvent(ClassifyUiEvent
            .Switch((baseAdapter as ClassifyTypeAdapter).getConditions(), position))
    }

    class ConditionsCallback: DiffUtil.ItemCallback<Conditions>() {

        override fun areItemsTheSame(oldItem: Conditions, newItem: Conditions): Boolean {
            return oldItem.key == newItem.key
        }

        override fun areContentsTheSame(oldItem: Conditions, newItem: Conditions): Boolean {
            return oldItem == newItem
        }

        override fun getChangePayload(oldItem: Conditions, newItem: Conditions): Any? {
            return if(oldItem.index != newItem.index) {
                "notify"
            } else {
                null
            }
        }
    }
}