package com.at.video.home.ui.home

import android.content.res.ColorStateList
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.lifecycle.ViewModelStoreOwner
import androidx.lifecycle.lifecycleScope
import androidx.viewpager2.widget.ViewPager2
import com.at.common.adapter.FragmentAdapter
import com.at.common.adapter.PagerData
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.PageStatus
import com.at.common.utils.JsonUtils
import com.at.video.home.R
import com.at.video.home.databinding.HomeFragmentHomeBinding
import com.at.video.home.entity.NoticeBean
import com.at.video.home.entity.TabInfo
import com.at.video.home.ui.home.dialog.AdDialog
import com.at.video.home.ui.home.dialog.NoticeDialog
import com.at.video.home.ui.widget.CustomTabText1
import com.at.video.home.ui.widget.TopNoticeView
import com.at.video.update.dialog.UpdateDialog
import com.google.android.flexbox.FlexDirection
import com.google.android.material.tabs.TabLayout
import com.kennyc.view.MultiStateView
import com.loper7.tab_expand.ext.buildIndicator
import com.loper7.tab_expand.ext.buildText
import com.loper7.tab_expand.indicator.LinearIndicator
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppUtils
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.DiversityImageView
import com.melancholy.widget.tabs.TabLayoutMediators
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.home.HomeTopBarConfig
import com.video.module_widget.entity.home.HomeTopContent
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.*
import com.video.module_widget.widget.ShapeTextViewExt
import dagger.hilt.android.AndroidEntryPoint
import org.json.JSONObject
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/4/27 12:20
 * @Description: 首页
 */

@AndroidEntryPoint
@Route(path = "/home/HomeFragment")
class HomeFragment :
    BaseMviFragment<HomeFragmentHomeBinding, HomeViewModel>(),
    TabLayoutMediators.TabConfigurationStrategy,
    View.OnClickListener {
    private lateinit var mIbCollect: ImageView
    private lateinit var logo: DiversityImageView
    private lateinit var mStvFilter: View
    private var mTabs: List<TabInfo>? = null
    private var mMediator: TabLayoutMediators? = null
    private lateinit var mViewPagerAdapter: FragmentAdapter
    private lateinit var homeTabBarStyles: List<HomeTopContent>
    private lateinit var titleBars: List<HomeTopContent>
    private lateinit var home: HomeTopBarConfig
    private lateinit var searchBar: ShapeTextViewExt
    private var tabBarFirst: Boolean = false

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): HomeFragmentHomeBinding {
        return HomeFragmentHomeBinding.inflate(inflater, container, false)
    }

    override fun getViewModelStoreOwner(): ViewModelStoreOwner = requireActivity()

    override fun getViewModelClass(): Class<HomeViewModel> {
        return HomeViewModel::class.java
    }

    private fun parseStyle() {
        home = StyleUtils.getStyleBean(StyleConsts.HOME)!!
        val topBar = home.topBar
        for (index in topBar.indices) {
            if (topBar[index].type == StyleConsts.HOME_TAB_BAR) {
                homeTabBarStyles = topBar[index].content!!
                tabBarFirst = index == 0
            } else {
                titleBars = topBar[index].content!!
            }
        }
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        parseStyle()
        requireBinding().apply {
            logo = requireViewById(R.id.div_image)
            searchBar = requireViewById(R.id.stv_search)
            mIbCollect = requireViewById(R.id.iv_collect)
            mStvFilter = requireViewById(R.id.stv_filter)
            mStvFilter.setOnClickListener(this@HomeFragment)
            mIbCollect.setOnClickListener(this@HomeFragment)
            searchBar.setOnClickListener(this@HomeFragment)
            changeSearchStyle()

            if (tabBarFirst) {
                topBg.flexDirection = FlexDirection.COLUMN_REVERSE
                topBg.setPadding(0, getStatusBarHeight(), 0, 0)
            }
            val tbTab: TabLayout = requireViewById(com.at.common.R.id.tb_tab)
            val params: ViewGroup.LayoutParams = tbTab.layoutParams
            params.height = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_48)
            tbTab.layoutParams = params
            tbTab.tabMode = TabLayout.MODE_SCROLLABLE
            mViewPagerAdapter = FragmentAdapter(childFragmentManager, viewLifecycleOwner.lifecycle)
            vpViewPager.adapter = mViewPagerAdapter
            refreshLayout.setViewPager(vpViewPager)
            vpViewPager.registerOnPageChangeCallback(ViewPagerCallback())
            mMediator = TabLayoutMediators(
                tbTab,
                vpViewPager,
                this@HomeFragment,
                true
            )
            mMediator!!.attach()
        }
    }

    private fun getStatusBarHeight(): Int {
        val resourceId: Int =
            resources.getIdentifier("status_bar_height", "dimen", "android")
        return if (resourceId > 0) {
            resources.getDimensionPixelSize(resourceId)
        } else 0
    }

    private fun getPageStyleIndexByTab(index: Int = 0): Int {
        var defaultIndex = index
        if (!mTabs.isNullOrEmpty()) {
            val tab = mTabs!![index]
            defaultIndex = when (tab.id) {
                "recommend" -> {
                    0
                }
                "created_at" -> {
                    1
                }
                "popularity" -> {
                    2
                }
                else -> {
                    when (tab.reType) {
                        1 -> {
                            3
                        }
                        102 -> {
                            5
                        }
                        103 -> {
                            4
                        }
                        else -> 0
                    }
                }
            }
        }
        return defaultIndex
    }

    // 设置titleBar样式
    private fun changeSearchStyle(index: Int = 0) {
        val searchStyles = titleBars
        val searchIndex = getPageStyleIndexByTab(index)
        val content = searchStyles[searchIndex]
        content?.let { c ->
            c.config?.let {
                searchBar.apply {
                    text = it.search!!.placeholder
                    compoundDrawableTintList =
                        ColorStateList.valueOf(it.search!!.iconColor!!.toArgb())
                    setTextColor(it.search!!.color!!.toArgb())
                    setBackgroundColor(it.search!!.fill!!)
                    setBorderRadius(it.search!!.border!!.radius)
                    setBorderWidth(1)
                    setBorderColor(it.search!!.border!!.color!!)
                    setAlign(it.search!!.align!!)
                    reBuilder()
                }
                it.logo?.apply {
                    if (mode == 1) {
                        when (style) {
                            1 -> {
                                logo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_1)
                            }
                            2 -> {
                                logo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_2)
                            }
                            3 -> {
                                logo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_3)
                            }
                        }
                    } else {
                        logo.loadByGlide(
                            GlideImageConfig(
                                image.imageUrl(),
                                placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                                errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                            ),
                            WeakReference(context)
                        )
                        image?.let { it1 -> Log.e("ICON", it1) }
                    }
                }
                it.collection?.apply {
                    val img = when (style) {
                        1 -> R.drawable.home_search_collect1
                        2 -> R.drawable.home_search_collect2
                        else -> R.drawable.home_search_collect3
                    }
                    mIbCollect.setImageDrawable(resources.getDrawable(img))
                }
            }
        }
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is HomeUiEffect.Notice ->{
                        NoticeDialog.showDialog(
                            childFragmentManager,
                            it.notices
                        )
                    }
                    is HomeUiEffect.DialogAd -> if (home.dialogAdv?.show == true) {
                        AdDialog.showDialog(
                            childFragmentManager,
                            it.list.toCollection(ArrayList()),
                            home.dialogAdv
                        )
                    }
                    is HomeUiEffect.CheckUpdate ->{
                        getViewModel().sendEvent(HomeUiEvent.CheckUpdate(AppUtils.getVersionCode(requireContext())))
                    }
                    is HomeUiEffect.UpdateDialog -> {
                        it.appVersionInfo?.let { it1 ->
//                            AppUtils.getVersionName(requireContext())
                            val title = "${it.appVersionInfo.appName}(最新版本:${it1.packageName})"
                            UpdateDialog.showDialog(
                                childFragmentManager,
                                it1.downUrl!!.imageUrl()!!,
                                it1.size!!,
                                "是否更新到最新版$title?",
                                it1.updateLog!!,
                                it1.forciblyUpdate
                            )
                        }
                    }
                    else -> {}
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(HomeUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading ->
                            msvView.viewState =
                                MultiStateView.ViewState.LOADING
                        is PageStatus.Empty -> msvView.viewState = MultiStateView.ViewState.EMPTY
                        is PageStatus.Error -> {
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                .text = it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Success ->
                            msvView.viewState =
                                MultiStateView.ViewState.CONTENT
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(HomeUiState::tabs) {
                if (it != null) {
                    mTabs = it.tabs
                    changeTabBarStyleByData(0)
                    changePageBackground(0)
                    for (tab: TabInfo in mTabs!!) {
                        val id: Long = "${tab.name}${tab.id}".hashCode().toLong()
                        val bundle = Bundle()
                        bundle.putString(Constants.KEY_DATA, tab.id)
                        if (!it.carouselAdv.isNullOrEmpty()) {
                            bundle.putParcelableArrayList(
                                Constants.KEY_DATA_1,
                                it.carouselAdv.toCollection(ArrayList())
                            )
                        }
                        if (!it.bannerAdv.isNullOrEmpty()) {
                            bundle.putParcelableArrayList(
                                Constants.KEY_DATA_2,
                                it.bannerAdv.toCollection(ArrayList())
                            )
                        }
                        CacheHelper.getInstance()
                            .putString(GlobalConsts.CAROUSEL_ADV, JsonUtils.toJson(it.carouselAdv))
                        CacheHelper.getInstance()
                            .putString(GlobalConsts.BANNER_ADV, JsonUtils.toJson(it.bannerAdv))
//                        if(!it.banAd.isNullOrEmpty()) {
//                            bundle.putParcelableArrayList(Constants.KEY_DATA_3, it.banAd.toCollection(ArrayList()))
//                        }
//                        if(!it.typeAd.isNullOrEmpty()) {
//                            bundle.putParcelableArrayList(Constants.KEY_DATA_4, it.typeAd.toCollection(ArrayList()))
//                        }
                        val path: String = when (tab.reType) {
                            100 -> "/home/RecommendFragment"
                            101 -> "/home/MostFragment"
                            102 -> "/home/StarFragment"
                            103 -> "/home/AlbumFragment"
                            else -> "/home/DynamicFragment"
                        }
                        mViewPagerAdapter.add(PagerData(path, id, bundle))
                    }
                    requireBinding().vpViewPager.offscreenPageLimit = 1
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(HomeUiState::notices) {
                if (it != null) {
                    createTopNotice(it)
                }
            }
        }
    }

    private lateinit var mTabsBg: String

    // 设置tabbar样式
    private fun changeTabBarStyleByData(position: Int = 0, isFirst: Boolean = true) {
        val tbTab: TabLayout = requireViewById(com.at.common.R.id.tb_tab)
        mTabsBg = ""
        var height = 0
        var normalColor = ""
        var selectedColor = ""
        var normalSize = 0f
        var selectedSize = 0f
        val indicatorColors = mutableListOf<Int>()
        fun addData(homeTabBarStyle: HomeTopContent) {
            homeTabBarStyle.config!!.apply {
                if (!background.isNullOrEmpty()) {
                    mTabsBg = background!!
                }
                indicator?.let {
                    if (it.height != 0) {
                        height = it.height
                        indicatorColors.add(it.color!!.toArgb())
                    }
                }

//                indicatorColors.add(indicatorColor!!.toArgb())
                text?.let {
                    normalSize = DensityUtils.px2sp(
                        requireContext(),
                        requireContext().getSp(it.size).toInt()
                    )
                    selectedSize = DensityUtils.px2sp(
                        requireContext(),
                        requireContext().getSp(it.selectedSize).toInt()
                    )
                    normalColor = it.color!!
                    selectedColor = it.selectedColor!!
                }
            }
        }

        val tab = mTabs!![position]
        when (tab.id) {
            "recommend" -> {
                addData(homeTabBarStyles[0])
            }
            "created_at" -> {
                addData(homeTabBarStyles[1])
            }
            "popularity" -> {
                addData(homeTabBarStyles[2])
            }
            else -> {
                when (tab.reType) {
                    1 -> {
                        addData(homeTabBarStyles[3])
                    }
                    102 -> {
                        addData(homeTabBarStyles[5])
                    }
                    103 -> {
                        addData(homeTabBarStyles[4])
                    }
                }
            }
        }
//        tbTab.buildIndicator<CustomIndicator>()
//            .setIndicatorColors(indicatorColors)
//            .setHeight(requireContext().resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_2))
//            .setColor(indicatorColors[0])
//            .bind()
        tbTab.buildIndicator<LinearIndicator>()
            .setHeight(height)
            .bind()
        // .setNormalTextSize(DensityUtils.px2sp(requireContext(), resources.getDimensionPixelOffset(com.at.common.R.dimen.sp_14)))
        tbTab.buildText<CustomTabText1>().setTabColor(normalColor, selectedColor)
            .setTabNormalSizes(normalSize)
            .setTabSelectedSizes(selectedSize)
            .setSelectTextBold(true)
            .setNormalTextBold(false)
            .bind()
    }

    private var noticeView: TopNoticeView? = null

    /**
     * 创建顶部公告
     */
    private fun createTopNotice(notices: List<NoticeBean>) {
        val style = home.noticeBar[0]
        noticeView = TopNoticeView(requireContext())
        val params: LinearLayout.LayoutParams = LinearLayout.LayoutParams(
            MATCH_PARENT,
            resources.getDimensionPixelSize(com.at.common.R.dimen.dp_36)
        )
        params.topMargin = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_6)
//        params.bottomMargin = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_6)
        noticeView?.layoutParams = params
        noticeView?.setOnItemClickListener(object : TopNoticeView.OnItemClickListener {
            override fun onItemClick(data: NoticeBean) {
                NoticeDialog.showDialog(childFragmentManager, listOf(data))
            }
        })
        noticeView?.changeStyle(style.config)
        noticeView?.setData(notices)
        requireBinding().llNotice.addView(noticeView, 0)
    }

    // 设置公告样式
    private fun changeNotice(index: Int = 0) {
        val defaultIndex = getPageStyleIndexByTab(index)
        val style = home.noticeBar[defaultIndex]
        noticeView?.changeStyle(style.config)
    }

    override fun loadData() {
        getViewModel().sendEvent(HomeUiEvent.InitLoadData)
    }

    override fun onConfigureTab(tab: TabLayout.Tab, position: Int) {
        tab.text = mTabs?.get(position)?.name
    }

    override fun onDestroyView() {
        mMediator?.detach()
        mMediator = null
        super.onDestroyView()
    }

    override fun onClick(view: View) {
        when (view.id) {
            // 我的收藏
            R.id.iv_collect -> {
                Router.getInstance().build("/mine/CollectActivity").navigation()
            }
            // 搜索
            R.id.stv_search -> {
                Router.getInstance().build("/home/SearchActivity").navigation()
            }
            // 筛选
            R.id.stv_filter -> {
                val tab: TabInfo = mTabs!![requireBinding().vpViewPager.currentItem]
                Router.getInstance().build("/home/ClassifyActivity")
                    .withString(Constants.KEY_DATA, tab.name)
                    .withInt(Constants.KEY_DATA_1, tab.reType)
                    .withString(Constants.KEY_DATA_2, tab.id)
                    .navigation()
            }
        }
    }

    private inner class ViewPagerCallback : ViewPager2.OnPageChangeCallback() {

        override fun onPageSelected(position: Int) {
            changeSearchStyle(position)
            changeNotice(position)
            changePageBackground(position)
            changeTabBarStyleByData(position, false)
            if (mTabsBg.isNotEmpty()) {
                requireBinding().apply {
                    topBg.setBackgroundColor(mTabsBg.toArgb())
                }
            }
            if (100 > mTabs?.get(position)?.reType!!) {
                mIbCollect.visibility = View.GONE
                mStvFilter.visibility = View.VISIBLE
            } else {
                mIbCollect.visibility = View.VISIBLE
                mStvFilter.visibility = View.GONE
            }
        }
    }

    private fun changePageBackground(position: Int = 0) {
        var background = ""
        var style: JSONObject
        if (!mTabs.isNullOrEmpty()) {
            val tab = mTabs!![position]
            background = when (tab.id) {
                "recommend" -> {
                    style =
                        JSONObject(
                            CacheHelper.getInstance().getString(StyleConsts.HOME_RECOMMEND, "")!!
                        )
                    style.get(StyleConsts.BACKGROUND).toString()
                }
                "created_at" -> {
                    style =
                        JSONObject(CacheHelper.getInstance().getString(StyleConsts.HOME_NEW, "")!!)
                    style.get(StyleConsts.BACKGROUND).toString()
                }
                "popularity" -> {
                    style =
                        JSONObject(CacheHelper.getInstance().getString(StyleConsts.HOME_HOST, "")!!)
                    style.get(StyleConsts.BACKGROUND).toString()
                }
                else -> {
                    when (tab.reType) {
                        1 -> {
                            style =
                                JSONObject(
                                    CacheHelper.getInstance()
                                        .getString(StyleConsts.HOME_CATEGORY, "")!!
                                )
                            style.get(StyleConsts.BACKGROUND).toString()
                        }
                        102 -> {
                            style =
                                JSONObject(
                                    CacheHelper.getInstance().getString(StyleConsts.HOME_HOST, "")!!
                                )
                            style.get(StyleConsts.BACKGROUND).toString()
                        }
                        103 -> {
                            style =
                                JSONObject(
                                    CacheHelper.getInstance()
                                        .getString(StyleConsts.HOME_ALBUM, "")!!
                                )
                            style.get(StyleConsts.BACKGROUND).toString()
                        }
                        else -> {
                            style =
                                JSONObject(
                                    CacheHelper.getInstance()
                                        .getString(StyleConsts.HOME_PEOPLE, "")!!
                                )
                            style.get(StyleConsts.BACKGROUND).toString()
                        }
                    }
                }
            }
        }
        requireBinding().llBody.setBackgroundColor(background.toArgb())
    }
}
