package com.at.video.home.ui.home.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import com.at.video.home.R
import com.at.video.home.entity.VideoBean
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.video.module_widget.imageUrl
import com.video.module_widget.strToInt
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeMaterialButtonExt
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/4/27 20:31
 * @Description:
 */

class CommonSingleBinder : BaseItemBinder<VideoBean, BaseViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.home_common_single_view, parent, false)
        )
    }

    override fun convert(holder: BaseViewHolder, data: VideoBean) {
        holder.getView<ImageView>(R.id.div_image)
            .loadByGlide(
                GlideImageConfig(
                    data.cover_horizontal.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
                ),
                WeakReference(context)
            )

        var bgImg = if (data.cover_horizontal.isNullOrEmpty()) data.cover_vertical.imageUrl() else data.cover_horizontal.imageUrl()
        holder.getView<ImageView>(R.id.div_bg_image)
            .loadByGlide(
                GlideImageConfig(
                    bgImg,
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
                ),
                WeakReference(context)
            )
        var coverImg = if (data.cover_vertical.isNullOrEmpty()) data.cover_horizontal.imageUrl() else data.cover_vertical.imageUrl()
        holder.getView<ImageView>(R.id.div_image)
            .loadByGlide(
                GlideImageConfig(
                    coverImg,
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
                ),
                WeakReference(context)
            )
        val fileName = data.name?.split("-")
        val actors = StringBuilder("${data.actors?.split(",")?.joinToString(" / ")}")
//        if (!data.area.isNullOrEmpty()) {
//            dateStr.append("·${data.area}")
//        }
//        if (!data.subcateg_name.isNullOrEmpty()) {
//            dateStr.append("·${data.subcateg_name}")
//        }
//        holder.setText(R.id.tv_desc,data.intro)
        if (actors.isNotEmpty() && !data.categ_name.isNullOrEmpty()) {
            actors.append(" / ")
        }
        actors.append("${data.categ_name}")
//        holder.setText(R.id.tv_date, dateStr.toString())
        holder.setText(R.id.tv_actors, actors)
        if (fileName != null && fileName.size > 1) {
            holder.getView<TextView>(R.id.tv_desc).visibility = View.VISIBLE
            holder.setText(R.id.tv_title, fileName[0])
                .setText(R.id.tv_desc, data.highlight)
        } else {
            holder.setText(R.id.tv_title, data.name)
            holder.getView<TextView>(R.id.tv_desc).visibility = View.GONE
        }
        val stvTag = holder.getView<ShapeTextViewExt>(R.id.stv_tag)
        val sbGold = holder.getView<ShapeMaterialButtonExt>(R.id.smb_glod)
        if (StyleUtils.tagEnable) {
            if ((data.price_type?.strToInt() ?: 1) == 1) {
                stvTag.visibility = View.VISIBLE
                sbGold.visibility = View.GONE
                changeTagStyle(stvTag)
            } else {
                stvTag.visibility = View.GONE
                sbGold.visibility = View.VISIBLE
                sbGold.text = data.price
                changeTagStyle(sbGold)
            }
        } else {
            sbGold.visibility = View.GONE
            stvTag.visibility = View.GONE
        }
    }

    private fun changeTagStyle(tagView: ShapeMaterialButtonExt) {
        val params = tagView.layoutParams as ConstraintLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt) {
        val params = tagView.layoutParams as ConstraintLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }
}
