package com.at.video.home.ui.home.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.DiffUtil
import com.at.video.home.R
import com.at.video.home.entity.VideoBean
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeMaterialButtonExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/27 11:06
 * @Description:
 */

class MorePayBinder(val type: Int = 1) :
    BaseItemBinder<MorePayBinder.Pay, BaseViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.home_classify_pay_view, parent, false)
        )
    }

    private fun getRootParamsByStyleType(): FrameLayout.LayoutParams {
        val params = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_137)
        )

        when (type) {
            1 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_266)
            }
            2 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_190)
            }
            else -> {

            }
        }
        return params
    }


    private fun getImageParamsByType() = when (type) {
        1 -> {
            com.at.common.R.dimen.dp_221
        }
        2 -> {
            com.at.common.R.dimen.dp_150
        }
        else -> {
            com.at.common.R.dimen.dp_221
        }
    }

    private fun changeTagStyle(tagView: ShapeMaterialButtonExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    override fun convert(holder: BaseViewHolder, data: Pay) {
        val root = holder.getView<FrameLayout>(R.id.fl_root)
        root.layoutParams = getRootParamsByStyleType()
        val image = holder.getView<DiversityImageView>(R.id.div_image)
        val params = image.layoutParams
        params.height = StyleUtils.getDpByDimen(getImageParamsByType())

        changeTagStyle(holder.getView(R.id.smb_glod))
        holder.setGone(R.id.stv_tag, !StyleUtils.tagEnable)

        val orientation = if (type == 1) 0 else 1
        var imageUrl:String? =
            if (data.info.cover_vertical?.isNotEmpty() == true) data.info.cover_vertical else data.info.cover_horizontal

        holder.getView<ImageView>(R.id.div_image).loadByGlide(
            GlideImageConfig(
                imageUrl.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
            ), WeakReference(context)
        )
        holder.setGone(R.id.iv_collect,true)
//        val update: String = if(data.info.publish_status?.toInt() == 1) {
//            "更新至${data.info.episode}集"
//        } else {
//            "全${data.info.episode_total}集"
//        }
        holder.getView<TextView>(R.id.tv_update).visibility =
            if ((data.info.episode?.toInt() ?: 0) > 0) View.VISIBLE else View.GONE
        holder.setText(R.id.tv_title, data.info.name).setText(R.id.smb_glod, data.info.price)
            .setText(R.id.stv_name, data.info.highlight).setText(R.id.tv_update, data.info.note)
            .setText(R.id.tv_score, data.info.score)
    }


    data class Pay(val info: VideoBean)

    class PayCallback : DiffUtil.ItemCallback<Pay>() {

        override fun areItemsTheSame(oldItem: Pay, newItem: Pay): Boolean {
            return oldItem.info.id == newItem.info.id
        }

        override fun areContentsTheSame(oldItem: Pay, newItem: Pay): Boolean {
            return oldItem.info == newItem.info
        }

        override fun getChangePayload(oldItem: Pay, newItem: Pay): Any? {
            return if (oldItem.info.is_collect != newItem.info.is_collect) {
                return "notify"
            } else {
                null
            }
        }
    }
}