package com.at.video.home.ui.home.binder

import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.video.home.R
import com.at.video.home.entity.CategoryInfo
import com.at.video.home.entity.VideoBean
import com.at.video.home.ui.home.IntervalDecoration
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.google.android.material.button.MaterialButton
import com.google.android.material.button.MaterialButton.ICON_GRAVITY_TEXT_END
import com.google.android.material.button.MaterialButton.ICON_GRAVITY_TEXT_START
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.entity.Content
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.getSp
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/8/29 上午10:27
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/8/29 上午10:27
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class VideoDataBinder: BaseItemBinder<VideoDataBinder.Data, BaseViewHolder>(),OnItemClickListener {

    data class Data(val data:MutableList<Any>, val title:String, val type:Int, val id:String? =null, val curPos:Int = 0, val dataType:Int = 0,
                    val style:Content? = null)
//
//    init {
//        mAdapter.addItemBinder(CommonFreeBinder())
//        mAdapter.addItemBinder(CommonPayBinder())
//    }
    private val intervalDecoration = IntervalDecoration()
    private val horizontalIntervalDecoration =HorizontalIntervalDecoration()

    override fun convert(holder: BaseViewHolder, data: Data) {
        holder.setText(R.id.stv_title,data.title)
        val title = holder.getView<TextView>(R.id.stv_title)
        data.style?.config?.title?.let {
            title.apply {
                it.color?.let { it1 ->
                    setTextColor(it1.toArgb())
                }
                setTextSize(TypedValue.COMPLEX_UNIT_PX,context.getSp(it!!.size))
            }
        }
        val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
        mAdapter.addItemBinder(CommonFreeBinder(data.type))
        mAdapter.addItemBinder(CommonPayBinder(data.type))

        holder.getView<RecyclerView>(R.id.rv_data).apply {
           adapter = mAdapter
            when(data.type){
                1->{
                    layoutManager = GridLayoutManager(context, 2)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
                2->{
                    layoutManager = GridLayoutManager(context, 3)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
                3->{
                    layoutManager = LinearLayoutManager(context,RecyclerView.HORIZONTAL,false)
                    removeItemDecoration(horizontalIntervalDecoration)
                    addItemDecoration(horizontalIntervalDecoration)
                }
            }
      }
        mAdapter.setOnItemClickListener(this)
        mAdapter.setNewInstance(data.data)
       val change = StyleUtils.change
        if(change!=null){
            val switch = holder.getView<MaterialButton>(R.id.mb_switch)
            holder.setGone(R.id.mb_switch,!change.enable)
            holder.setGone(R.id.mb_more,true)
            holder.setGone(R.id.tv_more,false)
            when(change.style){
                1->{
                    switch.iconGravity = ICON_GRAVITY_TEXT_START
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFF8F8F8),context.toColor(R.color.color_FFF8F8F8))
                }
                2->{
                    switch.iconTint = StyleUtils.createColorStateList(context.toColor(R.color.color_FF4B5363),context.toColor(R.color.color_FF4B5363))
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFF8F8F8),context.toColor(R.color.color_FFF8F8F8))
                     switch.iconGravity = ICON_GRAVITY_TEXT_END
                }
                else->{
                    holder.setGone(R.id.mb_more,false)
                    holder.setGone(R.id.tv_more,true)
                    switch.iconGravity = ICON_GRAVITY_TEXT_END
                    switch.iconTint = StyleUtils.createColorStateList(context.toColor(R.color.color_FFFF613F),context.toColor(R.color.color_FFFF613F))
                    switch.backgroundTintList = StyleUtils.createColorStateList(context.toColor(R.color.color_FFFFF6F4),context.toColor(R.color.color_FFFFF6F4))
                }
            }
        }
    }

    override fun convert(holder: BaseViewHolder, data: Data, payloads: List<Any>) {
        val adapter1 = holder.getView<RecyclerView>(R.id.rv_data).adapter as BaseBinderAdapter

        if(payloads.isNotEmpty()){
            adapter1.setNewInstance((payloads[0] as Data).data)
        }
    }
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder  = BaseViewHolder(
        LayoutInflater.from(parent.context)
         .inflate(R.layout.home_video_data_binder, parent, false))

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if(data is CommonFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        } else if(data is CommonPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        }
    }


}


