package com.at.video.home.ui.home.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.video.home.R
import com.at.video.home.ui.classify.binder.ClassifyFreeBinder
import com.at.video.home.ui.classify.binder.ClassifyPayBinder
import com.at.video.home.ui.home.IntervalDecoration
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.google.android.material.button.MaterialButton
import com.google.android.material.button.MaterialButton.ICON_GRAVITY_TEXT_END
import com.google.android.material.button.MaterialButton.ICON_GRAVITY_TEXT_START
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils

/**
 *
 * @Description: 更多列表
 * @CreateDate: 2022/8/29 上午10:27
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/8/29 上午10:27
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class VideoMoreListBinder() : BaseItemBinder<VideoMoreListBinder.Data, BaseViewHolder>(),
    OnItemClickListener {

    data class Data(val data: MutableList<Any>, val type: Int = 1, val isMore: Boolean)

    private val intervalDecoration = IntervalDecoration()
    private val horizontalIntervalDecoration = HorizontalIntervalDecoration()
    override fun convert(holder: BaseViewHolder, data: Data) {
        val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
        mAdapter.addItemBinder(CommonSingleBinder())
        mAdapter.addItemBinder(MoreFreeBinder(data.type))
        mAdapter.addItemBinder(MorePayBinder(data.type))


        holder.getView<RecyclerView>(R.id.rv_data).apply {
            adapter = mAdapter
            layoutManager = GridLayoutManager(context, 2)
            removeItemDecoration(intervalDecoration)
            addItemDecoration(intervalDecoration)
        }
        mAdapter.setNewInstance(data.data)
        mAdapter.setOnItemClickListener(this)

    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder =
        BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.home_video_list_binder, parent, false)
        )

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if (data is MoreFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        } else if (data is MorePayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        }
    }
}


