package com.at.video.home.ui.home.dialog

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.at.video.home.R
import com.at.video.home.databinding.HomeDialogAdBinding
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.home.Adv
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.*
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/12 19:39
 * @Description: 广告弹窗
 */

class AdDialog :
    BaseBindingDialogFragment<HomeDialogAdBinding>(),
    View.OnClickListener,
    OnItemClickListener {

    private var mAdapter: AdAdapter? = null
    private var style: Adv? = null
    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, imags: ArrayList<AdvertisingBean>, advStyle: Adv?) {
            val dialog = AdDialog()
            dialog.arguments = bundleOf(Constants.KEY_DATA to imags)
            dialog.style = advStyle
            dialog.show(manager, null)
        }
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): HomeDialogAdBinding {
        return HomeDialogAdBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            ivClose.setOnClickListener(this@AdDialog)
            mAdapter = AdAdapter(style)
            vpViewPager.adapter = mAdapter
            vpViewPager.isUserInputEnabled = false
            mAdapter?.setOnItemClickListener(this@AdDialog)
            mAdapter?.setNewInstance(requireArguments().getParcelableArrayList(Constants.KEY_DATA))
            sbNext.apply {
                setOnClickListener(this@AdDialog)
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            if (mAdapter?.itemCount == 1) {
                sbNext.text = "关闭"
            } else {
                sbNext.text = "下一个"
            }
            style?.let {
                slRoot.setBorderRadius(it.radius)
                slRoot.reBuilder()
            }
        }
    }

    override fun getWidth(): Int {
        var width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_335)
        context?.let {
            width = DensityUtils.getDisplayMetrics(context!!).widthPixels
        }
        return width
    }

    override fun getHeight(): Int {
        var height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_472)
        style?.let {
//            height = context?.getPx(it.height) ?: resources.getDimensionPixelSize(com.at.common.R.dimen.dp_472)
            height = (DensityUtils.getDisplayMetrics(context!!).heightPixels * 0.7).toInt()
        }
        Log.e("height", height.toString())
        return height
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {
    }

    override fun onClick(view: View) {
        if (view.id == R.id.iv_close) {
            dismissAllowingStateLoss()
        } else {
            if (mAdapter?.itemCount == 1 || requireBinding().sbNext.text == "关闭") {
                dismissAllowingStateLoss()
            } else {
                val currentItem: Int = requireBinding().vpViewPager.currentItem
                val next: Int = currentItem + 1
                requireBinding().vpViewPager.setCurrentItem(next, false)
                if (next == mAdapter!!.itemCount - 1) {
                    requireBinding().sbNext.text = "关闭"
                }
            }
        }
    }

    private class AdAdapter(val style: Adv?) : BaseQuickAdapter<AdvertisingBean,
        BaseViewHolder>(R.layout.home_item_dialog_ad) {

        override fun convert(holder: BaseViewHolder, item: AdvertisingBean) {
            holder.getView<DiversityImageView>(R.id.div_image)
                .loadByGlide(
                    GlideImageConfig(
                        item.image_url.imageUrl(),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_ad_default
                    ),
                    WeakReference(context)
                )
            style?.let {
                holder.getView<DiversityImageView>(R.id.div_image).setFullRadius(it.radius.toFloat())
            }
//            holder.getView<ImageView>(R.id.div_image)
//                .load(ImageConfig(item.image_url.imageUrl(), placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default, errorId = com.video.module_widget.R.drawable.common_ic_ad_default))

            /*holder.getView<ImageView>(R.id.div_image)
                .load(ImageConfig(item.addr.imageUrl(),
                    placeholderId = R.drawable.home_ic_ad_default,
                    errorId = R.drawable.home_ic_ad_default))*/
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: AdvertisingBean = adapter.getItem(position) as AdvertisingBean
        Router.getInstance().build("/webview/WebViewActivity")
            .withString(Constants.KEY_DATA, data.jump_addr!!)
            .navigation()
    }
}
