package com.at.video.home.ui.home.dialog

import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.at.video.home.R
import com.at.video.home.databinding.HomeDialogNoticeBinding
import com.at.video.home.entity.NoticeBean
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb

/**
 * @Create_time: 2022/5/12 16:45
 * @Description: 详情通知
 */

class NoticeDialog : BaseBindingDialogFragment<HomeDialogNoticeBinding>(), View.OnClickListener {
    private lateinit var mNotices: ArrayList<NoticeBean>
    private var mIndex: Int = 0

    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, msg: List<NoticeBean>) {
            val dialog = NoticeDialog()
            dialog.arguments = bundleOf(Constants.KEY_DATA to msg.toCollection(ArrayList()))
            dialog.show(manager, null)
        }
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): HomeDialogNoticeBinding {
        return HomeDialogNoticeBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            ivClose.setOnClickListener(this@NoticeDialog)
            mNotices = requireArguments().getParcelableArrayList(Constants.KEY_DATA)!!
            tvContent.text = mNotices[mIndex].content
            tvContent.movementMethod = ScrollingMovementMethod.getInstance()
            sbClose.apply {
                setOnClickListener(this@NoticeDialog)
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            if (mNotices.size == 1) {
                sbClose.text = "知道了"
            } else {
                sbClose.text = "下一条"
            }
        }
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_422)
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {
    }

    override fun onClick(view: View) {
        if (mIndex + 1 == mNotices.size || view.id == R.id.iv_close) {
            dismissAllowingStateLoss()
        } else {
            requireBinding().apply {
                ++mIndex
                if (mIndex + 1 == mNotices.size) {
                    sbClose.text = "知道了"
                }
                tvContent.text = mNotices[mIndex].content
            }
        }
    }
}
