package com.at.video.home.ui.home.dynamic

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.entity.CategoryInfo
import com.at.video.home.ui.home.binder.*
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/27 16:33
 * @Description: 动态获取 除了文章和图集
 */

@AndroidEntryPoint
@Route(path = "/home/DynamicFragment")
class DynamicFragment : BaseMviFragment<CommonStatusRefreshListBinding, DynamicViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener {

    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.HOME_CATEGORY)

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): CommonStatusRefreshListBinding {
        return CommonStatusRefreshListBinding.inflate(inflater, container, false)
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<DynamicViewModel> {
        return DynamicViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        getViewModel().sendEvent(
            DynamicUiEvent.HeaderData(
                requireArguments().getString(Constants.KEY_DATA)!!,
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_1),
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_2),
//            requireArguments().getParcelableArrayList(Constants.KEY_DATA_3),
//            requireArguments().getParcelableArrayList(Constants.KEY_DATA_4),
                homeStyle
            )
        )
        requireBinding().includedRefreshList.apply {
            val footer = LodeMoreView(requireContext())
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setOnRefreshLoadMoreListener(this@DynamicFragment)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(requireContext(), 1)
//            rvList.addItemDecoration(RecommendIntervalDecoration())
            rvList.itemAnimator = null
            rvList.setHasFixedSize(true)
            rvList.adapter = mAdapter
            srlRefresh.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.HOME_CATEGORY).toArgb()
            )
        }
//        mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
//            override fun getSpanSize(gridLayoutManager: GridLayoutManager, viewType: Int,
//                                     position: Int): Int {
//                if (viewType == 5 || viewType == 6) {
//                    return 1
//                }
//                return gridLayoutManager.spanCount
//            }
//
//        })
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.SWIPER_ADV
                ), showMarginVertical = true
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                style = JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.BANNER_ADV
                ), showMarginVertical = true, showMarginHorizontal = false
            )
        )
        mAdapter.addItemBinder(FunctionBinder())
        mAdapter.addItemBinder(AdTextBinder())
        mAdapter.addItemBinder(VideoListBinder())
//        mAdapter.addItemBinder(CommonFreeBinder())
//        mAdapter.addItemBinder(CommonPayBinder())
        mAdapter.addItemBinder(CommonSingleBinder())
        mAdapter.addItemBinder(VideoMoreListBinder())
        mAdapter.addItemBinder(NoDataBinder())
        mAdapter.setOnItemClickListener(this)
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DynamicUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DynamicUiState::refreshStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DynamicUiState::loadMoreStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DynamicUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(DynamicUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DynamicUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DynamicUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if (data is CommonFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        } else if (data is CommonPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        } else if (data is CategoryInfo) {
            Router.getInstance().build("/player/PlayerActivity")
                .withInt(Constants.KEY_DATA, data.id).navigation()
        } else if (data is AdImageBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        } else if (data is AdTextBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        if (!hidden) {
            getViewModel().sendEvent(DynamicUiEvent.RefreshData)
        }
    }
}