package com.at.video.home.ui.home.dynamic

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.entity.CategoryInfo
import com.at.video.home.entity.VideoBean
import com.at.video.home.http.ApiService
import com.at.video.home.ui.home.binder.*
import com.at.video.home.ui.home.most.MostUiEvent
import com.at.video.home.ui.home.most.MostUiState
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.utils.StyleUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject
import com.video.module_widget.entity.AdvertisingInfo
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.onStart

/**
 * @Create_time: 2022/4/27 16:39
 * @Description:
 */

data class DynamicUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class DynamicUiEvent : IUiEvent {
    data class HeaderData(
        val id: String,
        val carouselAdv: List<AdvertisingBean>?,
        val bannerAdv: List<AdvertisingBean>?,
//        val banAd: List<AdvertisingInfo>?,
//        val typeAd: List<AdvertisingInfo>?,
        val style: HomeStyle?
    ) : DynamicUiEvent()

    object InitLoadData : DynamicUiEvent()
    object RefreshData : DynamicUiEvent()
    object LoadDataMore : DynamicUiEvent()
}

sealed class DynamicUiEffect : IUiEffect

@HiltViewModel
class DynamicViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<DynamicUiState, DynamicUiEvent, DynamicUiEffect>(application) {
    private var mOffset: Int = 0
    private val mLimit: Int = 21
    private lateinit var mId: String
    private var carouselAdv: List<AdvertisingBean>? = null
    private var bannerAdv: List<AdvertisingBean>? = null

    //    private var mBanAd: List<AdvertisingInfo>? = null
//    private var mTypeAd: List<AdvertisingInfo>? = null
    private var style: HomeStyle? = null
    private var hasMore = false
    override fun providerInitialState(): DynamicUiState {
        return DynamicUiState()
    }

    override fun handleEvent(event: DynamicUiEvent) {
        when (event) {
            is DynamicUiEvent.HeaderData -> {
                mId = event.id
                carouselAdv = event.carouselAdv
                bannerAdv = event.bannerAdv
//                mBanAd = event.banAd
//                mTypeAd = event.typeAd
                style = event.style
            }
            is DynamicUiEvent.InitLoadData -> initLoadData()
            is DynamicUiEvent.RefreshData -> {
                mOffset = 0
                initLoadData(true)
            }
            is DynamicUiEvent.LoadDataMore -> loadDataMore()
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        viewModelScope.launch(Dispatchers.IO) {
            val datas: MutableList<Any> = mutableListOf()
            val functionBinderData: FunctionBinder.FunctionInfo = FunctionBinder.FunctionInfo(
                mutableListOf()
            )

            getMoviesMoreList().onStart {
                if (!isRefresh) {
                    setState { copy(pageStatus = PageStatus.Loading) }
                } else {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(Throwable("刷新失败"))) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect {
                val objData = it
                val list: MutableList<Any> = mutableListOf()
                val more: MutableList<Any> = mutableListOf()
                var remainData: MutableList<VideoBean> = mutableListOf()
                style?.apply {
                    content?.forEach { it1 ->
                        when (it1!!.type) {
                            StyleConsts.SWIPER_ADV -> {
                                val bannerAdvs = carouselAdv?.let { it2 ->
                                    DataUtils.filterAdvPosition(
                                        it2, AdvPosition.Categories
                                    )
                                }
                                bannerAdvs?.let { it1 ->
                                    datas.add(BannerBinder.BannerInfo(it1))
                                }
                            }
                            StyleConsts.QUICK_ENTRY -> if (StyleUtils.homeMenuEnable) datas.add(
                                functionBinderData
                            )
                            StyleConsts.BANNER_ADV -> {
                                val bannerAdvs = bannerAdv?.let { it2 ->
                                    DataUtils.filterAdvPosition(it2, AdvPosition.Categories)
                                }
                                bannerAdvs?.let { it3 ->
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    it3.forEach { it2 ->
                                        if (it2.image_type != 3) {
                                            adBinderData.add(AdImageBinder.Data(it2))
                                        } else {
                                            adBinderData.add(AdTextBinder.Data(it2))
                                        }
                                    }
                                    datas.addAll(adBinderData)
                                }
                            }
                            StyleConsts.DATA_FIXED -> {
                                val style = it1.config?.layout?.style
                                val listCount = StyleUtils.getGridCountByType(style ?: 1)
                                if (!objData.isNullOrEmpty()) {
                                    if (objData.size >= listCount) {
                                        remainData =
                                            objData.subList(listCount, objData.size).toMutableList()
                                    }
                                    val len = if (remainData.size > 0) listCount else objData.size
                                    for (i: Int in 0 until len) {
                                        addDataByCoin(list, objData[i])
                                    }
                                    datas.add(VideoListBinder.Data(list, style ?: 1, false))
                                }
                            }
                            StyleConsts.DATA_MORE -> {
                                val style = it1.config?.layout?.style ?: 1
                                val moreListCount = StyleUtils.getGridCountByType(style)
                                if (remainData.size > 0) {
                                    if (style == 1) { // 两列平铺 需要处理
                                        var lastData: MutableList<VideoBean> = mutableListOf()
                                        if (remainData.size >= moreListCount) {
                                            lastData =
                                                remainData.subList(moreListCount, remainData.size)
                                        }
                                        val len =
                                            if (lastData.size > 0) moreListCount else remainData.size
                                        for (i: Int in 0 until len) {
                                            addMoreDataByCoin(more, remainData[i])
                                        }
                                        datas.add(VideoMoreListBinder.Data(more, style, true))
                                        if (lastData.size > 0) {
                                            datas.addAll(lastData)
                                        }
                                    } else {//卡片布局不需要处理
                                        datas.addAll(remainData)
                                    }
                                }
                            }
                        }
                    }
                }
                if (it.isNullOrEmpty()) {
                    datas.add(NoDataBinder.Data("暂无数据"))
                }
                if (!isRefresh) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                } else {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshSuccess,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun addDataByCoin(datas: MutableList<Any>, item: VideoBean) {
        if (item.price_type?.toInt() == 1) {
            datas.add(CommonFreeBinder.Free(item))
        } else {
            datas.add(CommonPayBinder.Pay(item))
        }
    }

    private fun addMoreDataByCoin(datas: MutableList<Any>, item: VideoBean) {
        if (item.price_type?.toInt() == 1) {
            datas.add(MoreFreeBinder.Free(item))
        } else {
            datas.add(MorePayBinder.Pay(item))
        }
    }

    private fun loadDataMore() {
        viewModelScope.launch {
            getMoviesMoreList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    setState {
                        copy(
                            data = data + it,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    // 获取视频
    private fun getMoviesMoreList(): Flow<List<VideoBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["offset"] = mOffset
            map["limit"] = mLimit
            map["category_id"] = mId
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getFilmList(map)
            if (bean.success() || bean.code == 502) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    hasMore = bean.total!!.toInt() > mOffset + bean.list.size
                    emit(bean.list)
                }

            } else {
                throw Error(bean.msg())
            }
        }
    }
}