package com.at.video.home.ui.home.recommend

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.home.R
import com.at.video.home.ui.home.binder.CommonFreeBinder
import com.at.video.home.ui.home.binder.CommonPayBinder
import com.at.video.home.ui.home.binder.FunctionBinder
import com.at.video.home.ui.home.binder.VideoDataBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.*
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/27 16:33
 * @Description: 推荐
 */

@AndroidEntryPoint
@Route(path = "/home/RecommendFragment")
class RecommendFragment :
    BaseMviFragment<CommonStatusRefreshListBinding, RecommendViewModel>(),
    OnRefreshLoadMoreListener,
    OnItemChildClickListener,
    OnItemClickListener {

    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.HOME_RECOMMEND)
    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): CommonStatusRefreshListBinding {
        return CommonStatusRefreshListBinding.inflate(inflater, container, false)
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<RecommendViewModel> {
        return RecommendViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        getViewModel().sendEvent(
            RecommendUiEvent.HeaderData(
                requireArguments().getString(Constants.KEY_DATA)!!,
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_1),
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_2),
//            requireArguments().getParcelableArrayList(Constants.KEY_DATA_3),
                homeStyle
            )
        )
        requireBinding().includedRefreshList.apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@RecommendFragment)
            val footer = LodeMoreView(requireContext())
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(requireContext(), 1)
//            rvList.addItemDecoration(RecommendIntervalDecoration())
            rvList.itemAnimator = null
            val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
            pool.setMaxRecycledViews(5, 30)
            rvList.setRecycledViewPool(pool)
            rvList.setHasFixedSize(true)
            rvList.adapter = mAdapter
            srlRefresh.setBackgroundColor(StyleUtils.getPageBackground(StyleConsts.HOME_RECOMMEND).toArgb())
        }
        mAdapter.addItemBinder(FunctionBinder())
        mAdapter.addItemBinder(BannerBinder(JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.SWIPER_ADV), showMarginVertical = true))
        mAdapter.addItemBinder(AdImageBinder(showMarginVertical = true, style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV), showMarginHorizontal = false))
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(VideoDataBinder())
        mAdapter.addItemBinder(NoDataBinder())
//        mAdapter.addItemBinder(CommonFreeBinder())
//        mAdapter.addItemBinder(CommonPayBinder())
//        mAdapter.addItemBinder(HeaderBinder())
//        mAdapter.addItemBinder(FooterBinder())
        mAdapter.addChildClickViewIds(R.id.mb_switch, R.id.tv_more, R.id.mb_more)
        mAdapter.setOnItemClickListener(this)
        mAdapter.setOnItemChildClickListener(this)
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RecommendUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RecommendUiState::refreshStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RecommendUiState::loadMoreStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RecommendUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is RecommendUiEffect.NoChangeDataEffect -> ToastUtils.show(it.msg)
                    is RecommendUiEffect.ChangeDataEffect ->
                        mAdapter.notifyItemChanged(
                            it.pos,
                            it.data
                        )
                    else -> {}
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(RecommendUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(RecommendUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(RecommendUiEvent.LoadDataMore)
    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        when (view.id) {
            R.id.mb_switch -> getViewModel().sendEvent(
                RecommendUiEvent
                    .Switch(adapter.getItem(position) as VideoDataBinder.Data, position)
            )
            R.id.tv_more, R.id.mb_more -> {
//                GlobalScope.launch {
//                    "anc".substring(10)
//                }
                val data = adapter.getItem(position) as VideoDataBinder.Data
                Router.getInstance().build("/home/MoreActivity")
                    .withString(Constants.KEY_DATA, data.title)
                    .withString(Constants.KEY_DATA_1, data.id!!)
                    .withInt(Constants.KEY_DATA_2, data.dataType)
                    .navigation()
            }
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if (data is CommonFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        } else if (data is CommonPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.data.id!!).navigation()
        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        if (!hidden) {
            getViewModel().sendEvent(RecommendUiEvent.RefreshData)
        }
    }
}
