package com.at.video.home.ui.home.star

import android.graphics.Rect
import android.view.View
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.melancholy.utils.AppInstance

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/26 下午3:33
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/26 下午3:33
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class StarDecoration: RecyclerView.ItemDecoration() {

    private val mPadding: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_16)

    override fun getItemOffsets(outRect: Rect, view: View, parent: RecyclerView,
                                state: RecyclerView.State) {
        val itemPosition = parent.getChildAdapterPosition(view)
        val manager: GridLayoutManager = parent.layoutManager as GridLayoutManager
        val spanSizeLookup: GridLayoutManager.SpanSizeLookup = manager.spanSizeLookup
        val spanCount: Int = manager.spanCount
        // 返回该item占据份数的起始索引 0到spanCount之间
        val index = spanSizeLookup.getSpanIndex(itemPosition, spanCount)
        val groupIndex = spanSizeLookup.getSpanGroupIndex(itemPosition, spanCount)
        //返回该item所占据份数 0到spanCount之间
        val size = spanSizeLookup.getSpanSize(itemPosition)
        if(size != spanCount) {
            //内部均分
            //outRect.left = index * 48 / spanCount
            //outRect.top = 48
            //outRect.right = 48 - (index + 1) * 48 / spanCount
            //outRect.set(left, 48, right, 0)

            //左右全部均分
            // 每个span分配的间隔⼤⼩
            val spanSpace = mPadding * (spanCount + 1) / spanCount
            // 列左、右间隙
            outRect.left = mPadding * (index + 1) - spanSpace * index
            outRect.top = mPadding
            outRect.right = spanSpace * (index + 1) - mPadding * (index + 1)
        } else {
            outRect.set(mPadding, mPadding, mPadding, 0)
        }
    }
}