package com.at.video.home.ui.home.star

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.entity.StarInfo
import com.at.video.home.ui.home.binder.FunctionBinder
import com.at.video.home.ui.home.star.binder.StarTypeBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/27 16:33
 * @Description: 人物
 */

@AndroidEntryPoint
@Route(path = "/home/StarFragment")
class StarFragment :
    BaseMviFragment<CommonStatusRefreshListBinding, StarViewModel>(),
    OnRefreshLoadMoreListener,
    OnItemClickListener {

    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private lateinit var homeStyle: HomeStyle

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): CommonStatusRefreshListBinding {
        val binding = CommonStatusRefreshListBinding.inflate(inflater, container, false)
//        binding.root.setBackgroundColor(resources.getColor(R.color.color_FFF6F8FB))
        return binding
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<StarViewModel> {
        return StarViewModel::class.java
    }

    private val mPadding: Int =
        AppInstance.getApplication().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        homeStyle = StyleUtils.getHomePageStyle(StyleConsts.HOME_PEOPLE)
        getViewModel().sendEvent(
            StarUiEvent.HeaderData(
                requireArguments().getString(Constants.KEY_DATA)!!,
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_1),
                requireArguments().getParcelableArrayList(Constants.KEY_DATA_2),
//                requireArguments().getParcelableArrayList(Constants.KEY_DATA_3),
                homeStyle
            )
        )
        requireBinding().includedRefreshList.apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@StarFragment)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            val footer = LodeMoreView(requireContext())
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            rvList.layoutManager = GridLayoutManager(requireContext(), 1)
            rvList.itemAnimator = null
            rvList.setHasFixedSize(true)
            rvList.adapter = mAdapter
            srlRefresh.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.HOME_PEOPLE).toArgb()
            )
        }
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle,
                    StyleConsts.SWIPER_ADV
                ),
                showMarginVertical = true
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginVertical = true,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV)
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(FunctionBinder())
        mAdapter.addItemBinder(NoDataBinder())
        mAdapter.addItemBinder(
            StarTypeBinder(
                getViewModel(),
                StyleUtils.getHomeContentStyle(homeStyle, StyleConsts.PEOPLE_FILTER_BAR),
                StyleUtils.getHomeContentStyle(homeStyle, StyleConsts.PEOPLE_GRID)
            )
        )
//        mAdapter.addItemBinder(StarBinder())
        mAdapter.setOnItemClickListener(this)
    }

    private var scrollAble = false

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(StarUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(StarUiState::refreshStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                            srlRefresh.finishLoadMore()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(StarUiState::loadMoreStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.resetNoMoreData()
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(StarUiState::data) {
                mAdapter.setNewInstance(it)
//                if (scrollAble) {
//                    scrollAble = false
//                    for (i in 0..it.size) {
//                        if (i >= it.size) {
//                            break
//                        }
//                        if (it[i] is StarInfo) {
//                            requireBinding().includedRefreshList.rvList.scrollToPosition(i)
//                            break
//                        }
//                    }
//                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(StarUiState::starDatas) {
                if (it != null) {
                    mAdapter.notifyItemChanged(it.curPosition, it)
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(StarUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(StarUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(StarUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if (data is StarInfo) {
            Router.getInstance().build("/player/DetailActivity")
                .withInt(Constants.KEY_DATA, data.id).navigation()
        } else if (data is AdImageBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        } else if (data is AdTextBinder.Data) {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!).navigation()
        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)

        if (!hidden) {
            getViewModel().sendEvent(StarUiEvent.RefreshData)
        }
    }
}
