package com.at.video.home.ui.home.star.binder

import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.video.home.databinding.HomeStarTypeViewBinding
import com.at.video.home.entity.Area
import com.at.video.home.entity.StarBean
import com.at.video.home.ui.home.star.StarDecoration
import com.at.video.home.ui.home.star.StarTypeDecoration
import com.at.video.home.ui.home.star.StarUiEvent
import com.at.video.home.ui.home.star.StarViewModel
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.QuickViewBindingItemBinder
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.entity.Content

/**
 * @Create_time: 2022/4/27 20:49
 * @Description:
 */

class StarTypeBinder(private val viewModel:StarViewModel,private val content:Content? = null,private val listStyle:Content? = null): QuickViewBindingItemBinder<StarTypeBinder.StarType, HomeStarTypeViewBinding>(),OnItemClickListener {

    private val mPadding: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_16)


    override fun onCreateViewBinding(layoutInflater: LayoutInflater, parent: ViewGroup,
                                     viewType: Int): HomeStarTypeViewBinding {
        val binding = HomeStarTypeViewBinding.inflate(layoutInflater, parent, false)
//        binding.root.setBackgroundColor(Color.RED)
        binding.rvList.itemAnimator = null
        binding.rvData.itemAnimator = null
        binding.rvList.setHasFixedSize(true)
        binding.rvData.setHasFixedSize(true)
//        val params = binding.rvList.layoutParams
//        params.height = parent.context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_31)
//        binding.rvList.layoutParams = params
        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 10)
        binding.rvList.setRecycledViewPool(pool)
        binding.rvList.layoutManager = LinearLayoutManager(context,RecyclerView.HORIZONTAL,false)
        binding.rvData.setRecycledViewPool(pool)

        binding.rvList.addItemDecoration(StarTypeDecoration())
        val adapter = StartTypeAdapter(content)
        val startAdapter = BaseBinderAdapter()
        startAdapter.addItemBinder(StarBinder())
        startAdapter.addItemBinder(NoDataBinder())

        adapter.setOnItemClickListener(this)
        binding.rvList.adapter = adapter
        binding.rvData.adapter = startAdapter
        binding.rvData.setHasFixedSize(true)
        val spanCount = listStyle?.config?.layout?.style?:1
        binding.rvData.layoutManager = GridLayoutManager(context, spanCount +2)
        binding.rvData.addItemDecoration(StarDecoration())
        startAdapter.setOnItemClickListener(this)
        return binding
    }


    var curPosition:Int = 0
    override fun convert(holder: BinderVBHolder<HomeStarTypeViewBinding>, data: StarType) {
        updateData(holder,data)
    }

    override fun convert(
        holder: BinderVBHolder<HomeStarTypeViewBinding>,
        data: StarType,
        payloads: List<Any>
    ) {
        val starData = payloads[0] as StarType
       updateData(holder,starData)
    }

    private fun updateData(holder: BinderVBHolder<HomeStarTypeViewBinding>, data: StarType){
        val adapter =holder.viewBinding.rvList.adapter as StartTypeAdapter
        val startAdapter = holder.viewBinding.rvData.adapter as BaseBinderAdapter
        val spanCount = listStyle?.config?.layout?.style?:1
        if(data.starData[0] is NoDataBinder.Data){
            holder.viewBinding.rvData.layoutManager = GridLayoutManager(context, 1)
        }else{
            holder.viewBinding.rvData.layoutManager = GridLayoutManager(context, spanCount + 2)
        }
        adapter.setNewInstance(data.data)
        adapter.setSelectIndex(data.position)
        startAdapter.setNewInstance(data.starData)
        curPosition = data.curPosition
    }

    data class StarType(val data: MutableList<Area>, val starData:MutableList<Any>, var position: Int = 0, var curPosition:Int = 0)

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        if(adapter is BaseBinderAdapter){
            val data = adapter.getItem(position)
            if(data is StarBinder.Data){
                Router.getInstance().build("/player/DetailActivity")
                    .withString(Constants.KEY_DATA, data.data.id!!).navigation()
            }
        }else{
            val data = adapter.getItem(position) as Area
            viewModel.sendEvent(StarUiEvent.ChangeType(data,position,curPosition))
        }
    }
}