package com.at.video.home.ui.more

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.ui.classify.binder.ClassifyFreeBinder
import com.at.video.home.ui.classify.binder.ClassifyPayBinder
import com.at.video.home.ui.classify.binder.ClassifyVideoListBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/30 16:02
 * @Description:
 */

@AndroidEntryPoint
@Route(path = "/home/MoreActivity")
class MoreActivity :
    BaseMviActivity<CommonTitleBarRefreshListBinding, MoreViewModel>(),
    OnRefreshLoadMoreListener,
    OnItemClickListener,
    ClassifyVideoListBinder.OnItemChildLsViewClickListener {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.VIDEO_MORE)

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<MoreViewModel> {
        return MoreViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitle(intent.getStringExtra(Constants.KEY_DATA))
        }
        val id: String? = intent.getStringExtra(Constants.KEY_DATA_1)
        val type: Int = intent.getIntExtra(Constants.KEY_DATA_2, -1)
        getViewModel().sendEvent(MoreUiEvent.Header(id, type, homeStyle))
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            val footer = LodeMoreView(this@MoreActivity)
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            srlRefresh.setOnRefreshLoadMoreListener(this@MoreActivity)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            homeStyle?.let {
                srlRefresh.setBackgroundColor(it.background!!.toArgb())
            }
            rvList.layoutManager = GridLayoutManager(this@MoreActivity, 1)
            val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
            pool.setMaxRecycledViews(0, 30)
            rvList.setRecycledViewPool(pool)
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
//            mAdapter.addItemBinder(CommonSingleBinder())
//            mAdapter.addItemBinder(
//                BannerBinder(
//                    JsonUtils.getHomeContentStyle(
//                        homeStyle,
//                        StyleConsts.SWIPER_ADV
//                    )
//                )
//            )
//            mAdapter.addItemBinder(
//                AdImageBinder(
//                    showMarginVertical = true,
//                    style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV)
//                )
//            )
//            mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
//            mAdapter.addItemBinder(ClassifyFreeBinder())
//            mAdapter.addItemBinder(ClassifyPayBinder())
            mAdapter.addItemBinder(ClassifyVideoListBinder(this@MoreActivity))

//            mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
//                override fun getSpanSize(gridLayoutManager: GridLayoutManager, viewType: Int,
//                                         position: Int): Int {
//                    if (viewType == 1|| viewType == 2||viewType == 3) {
//                        return gridLayoutManager.spanCount
//                    }
//                    return 1
//                }
//
//            })

//            mAdapter.addItemBinder(ClassifyVideoListBinder(this@MoreActivity))
//            mAdapter.addChildClickViewIds(R.id.iv_collect)
            mAdapter.setOnItemClickListener(this@MoreActivity)
//            mAdapter.setOnItemChildClickListener(this@MoreActivity)
        }
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoreUiState::collectStatus) {
                try {
                    if (it.parentPos == -1) {
                        return@collectState
                    }
                    if (mAdapter.data[it.parentPos] is ClassifyVideoListBinder.Data) {
                        if ((mAdapter.data[it.parentPos] as ClassifyVideoListBinder.Data).data[it.childPos] is ClassifyFreeBinder.Free) {
                            ((mAdapter.data[it.parentPos] as ClassifyVideoListBinder.Data).data[it.childPos] as ClassifyFreeBinder.Free).info.apply {
                                is_collect = it.isCollect
                            }
                        } else if ((mAdapter.data[it.parentPos] as ClassifyVideoListBinder.Data).data[it.childPos] is ClassifyPayBinder.Pay) {
                            ((mAdapter.data[it.parentPos] as ClassifyVideoListBinder.Data).data[it.childPos] as ClassifyPayBinder.Pay).info.apply {
                                is_collect = it.isCollect
                            }
                        }
                        mAdapter.notifyItemChanged(
                            it.parentPos,
                            ClassifyVideoListBinder.REFRESH_COLLECT
                        )
                    }
                } catch (e: Exception) {
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(MoreUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoreUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoreUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val id = when (val data = adapter.getItem(position)) {
            is ClassifyFreeBinder.Free -> data.info.id
            is ClassifyPayBinder.Pay -> data.info.id
            else -> "-1"
        }
        Router.getInstance().build("/player/PlayerActivity").withString(Constants.KEY_DATA, id!!)
            .navigation()
    }

//    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
//        getViewModel().sendEvent(MoreUiEvent.Collect(adapter.getItem(position)!!, position))
//    }

    override fun onlistViewClick(
        parentPos: Int,
        adapter: BaseQuickAdapter<*, *>,
        view: View,
        position: Int
    ) {
        getViewModel().sendEvent(
            MoreUiEvent.Collect(
                parentPos,
                adapter.getItem(position)!!,
                position
            )
        )
    }
}
