package com.at.video.home.ui.more

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.home.entity.UpdateMoreCollectStatus
import com.at.video.home.entity.VideoBean
import com.at.video.home.http.ApiService
import com.at.video.home.ui.classify.binder.ClassifyFreeBinder
import com.at.video.home.ui.classify.binder.ClassifyPayBinder
import com.at.video.home.ui.classify.binder.ClassifyVideoListBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.onStart
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/30 16:03
 * @Description:
 */

data class MoreUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf(),
    val collectStatus: UpdateMoreCollectStatus = UpdateMoreCollectStatus(-1, -1, -1)
) : IUiState

sealed class MoreUiEvent : IUiEvent {
    data class Header(val id: String?, val type: Int, val style: HomeStyle?) : MoreUiEvent()
    object InitLoadData : MoreUiEvent()
    object RefreshData : MoreUiEvent()
    object LoadDataMore : MoreUiEvent()
    data class Collect(val parentPos: Int, val info: Any, val position: Int) : MoreUiEvent()
}

sealed class MoreUiEffect : IUiEffect

class MoreViewModel(application: Application) :
    BaseAndroidViewModel<MoreUiState, MoreUiEvent, MoreUiEffect>(application) {
    private var mId: String? = ""
    private var mType: Int = -1
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var style: HomeStyle? = null
    private var hasMore = false
    override fun providerInitialState(): MoreUiState {
        return MoreUiState()
    }

    override fun handleEvent(event: MoreUiEvent) {
        when (event) {
            is MoreUiEvent.Header -> {
                mId = event.id
                mType = event.type
                style = event.style
            }
            is MoreUiEvent.InitLoadData -> {
                initLoadData()
            }
            is MoreUiEvent.RefreshData -> initLoadData(true)
            is MoreUiEvent.LoadDataMore -> loadDataMore()
            is MoreUiEvent.Collect -> collect(event.parentPos, event.info, event.position)
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        mLimit = getLimit()
        viewModelScope.launch {
            getMoviesMoreList().onStart {
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(Throwable("刷新失败"))) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else {
                    val datas: MutableList<Any> = mutableListOf()
                    style?.let { it1 ->
                        it1.content?.forEach { item ->
                            when (item!!.type) {
//                                StyleConsts.SWIPER_ADV -> {
//                                    val carourseAdvs =
//                                        JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)
//                                    carourseAdvs?.let { advs ->
//                                        datas.add(BannerBinder.BannerInfo(advs))
//                                    }
//                                }
//                                StyleConsts.BANNER_ADV -> {
//                                    val bannerAdvs =
//                                        JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
//                                    val adBinderData: MutableList<Any> = mutableListOf()
//                                    bannerAdvs?.let { advs ->
//                                        advs.forEach { item ->
//                                            run {
//                                                if (item.image_type != 3) {
//                                                    adBinderData.add(AdImageBinder.Data(item))
//                                                } else {
//                                                    adBinderData.add(AdTextBinder.Data(item))
//                                                }
//                                            }
//                                        }
//                                        datas.addAll(adBinderData)
//                                    }
//                                }
                                StyleConsts.VIDEO_GRID -> {
                                    val style = item.config!!.layout!!.style
                                    val data: MutableList<Any> = mutableListOf()
                                    it.forEach { item ->
                                        if (item.price_type?.toInt() == 1) {
                                            data.add(ClassifyFreeBinder.Free(item))
                                        } else {
                                            data.add(ClassifyPayBinder.Pay(item))
                                        }
                                    }
                                    datas.add(ClassifyVideoListBinder.Data(data, style))
                                }
                            }
                        }
                    }
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = datas,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun getLimit(): Int {
        val num = if (style != null) {
            StyleUtils.getHomeContentStyle(style!!, StyleConsts.VIDEO_GRID)?.config?.layout?.limit
        } else 9
        return num ?: 9
    }

    private fun loadDataMore() {
        viewModelScope.launch {
            getMoviesMoreList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    setState {
                        copy(
                            data = data + it,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    // 获取更多视频
    private fun getMoviesMoreList(): Flow<List<VideoBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["offset"] = mOffset
            map["limit"] = mLimit
            when (mType) {
                1, 2 -> {
                    map["sort"] = mId!!
                }
                3 -> map["category_id"] = mId!!
                else -> map["album_id"] = mId!!
            }

            val bean = ApiFactory.getInstance().create(ApiService::class.java).getFilmList(map)
            if (bean.success()) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    hasMore = bean.total!!.toInt() > mOffset + bean.list?.size!!
                    emit(bean.list)
                }
            } else {
                throw Error(bean.msg())
            }
        }
    }

    /**
     * @desc 收藏
     * @param data Any 视频数据
     * @param position Int 界面展示集合位置
     */
    private fun collect(parentPos: Int, data: Any, position: Int) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = 1
            map["belong_app_id"] = Constants.APP_ID
            var isCollect: Boolean = false
            var resourceId: String = ""
            when (data) {
                is ClassifyFreeBinder.Free -> {
                    resourceId = data.info.id.toString()
                    isCollect = data.info.is_collect == 1
                }
                is ClassifyPayBinder.Pay -> {
                    resourceId = data.info.id.toString()
                    isCollect = data.info.is_collect == 1
                }
                else -> {
                    ToastUtils.show("未获取到正确的参数信息")
                    return@launch
                }
            }
            map["resource_id"] = resourceId
            val ids = mutableListOf<String>()
            ids.add(resourceId)
            map["resource_id_list"] = ids
            val bean = if (!isCollect) {
                ApiFactory.getInstance().create(CommonApiService::class.java).collect(map = map)
            } else {
                ApiFactory.getInstance().create(CommonApiService::class.java)
                    .deleteCollect(map = map)
            }
            if (bean.success()) {
                setState {
                    copy(
                        collectStatus = UpdateMoreCollectStatus(
                            parentPos,
                            position,
                            if (isCollect) {
                                2
                            } else {
                                1
                            }
                        )
                    )
                }
            }
        }
    }
}
