package com.at.video.home.ui.search

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.KeyEvent
import android.widget.TextView
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentTransaction
import androidx.lifecycle.lifecycleScope
import com.at.common.databinding.CommonTitleBarFrameBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.video.home.R
import com.at.video.home.databinding.HomeSearchTitleBarTopSearchBinding
import com.at.video.home.ui.search.history.SearchHistoryFragment
import com.at.video.home.ui.search.result.SearchResultFragment
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.ClearEditText
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/28 19:10
 * @Description: 搜索页面
 */

@AndroidEntryPoint
@Route(path = "/home/SearchActivity")
class SearchActivity :
    BaseMviActivity<CommonTitleBarFrameBinding, SearchViewModel>(),
    TextView.OnEditorActionListener,
    TextWatcher {

    private lateinit var mHistoryFragment: SearchHistoryFragment
    private lateinit var mResultFragment: SearchResultFragment
    private lateinit var inputView: ClearEditText

//    private lateinit var mMappingFragment: SearchMappingFragment
    private var mCurrentTag: String = SearchHistoryFragment::class.simpleName!!

    override fun createViewBinding(): CommonTitleBarFrameBinding {
        return CommonTitleBarFrameBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<SearchViewModel> {
        return SearchViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        if (savedInstanceState == null) {
            mHistoryFragment = Router.getInstance().build("/home/SearchHistoryFragment").navigation() as SearchHistoryFragment
            mResultFragment = Router.getInstance().build("/home/SearchResultFragment").navigation() as SearchResultFragment
//            mMappingFragment = Router.getInstance().build("/home/SearchMappingFragment").navigation() as SearchMappingFragment
            supportFragmentManager.beginTransaction()
                .add(com.at.common.R.id.fl_content, mHistoryFragment, mHistoryFragment.javaClass.simpleName)
                .add(com.at.common.R.id.fl_content, mResultFragment, mResultFragment.javaClass.simpleName)
//                .add(com.at.common.R.id.fl_content, mMappingFragment, mMappingFragment.javaClass.simpleName)
                .hide(mResultFragment)
//                .hide(mMappingFragment)
                .commit()
        } else {
            mHistoryFragment = supportFragmentManager.findFragmentByTag(SearchHistoryFragment::class.simpleName) as SearchHistoryFragment
            mResultFragment = supportFragmentManager.findFragmentByTag(SearchResultFragment::class.simpleName) as SearchResultFragment
            mCurrentTag = savedInstanceState.getString(Constants.KEY_DATA, mHistoryFragment.javaClass.simpleName)!!
        }

        requireTitleBar().apply {
            addRightTextButton("取消", com.melancholy.widget.R.id.backButton)
                .setOnClickListener(this)
            setBackgroundColor(Color.WHITE)
            val binding = HomeSearchTitleBarTopSearchBinding.inflate(
                layoutInflater,
                this,
                false
            )
            setCenterView(binding.root)
            binding.setInput.addTextChangedListener(this@SearchActivity)
            binding.setInput.setOnEditorActionListener(this@SearchActivity)
            inputView = binding.setInput
        }
    }

    override fun loadData() {
    }

    private fun showFragment(fragment: Fragment) {
        val ft: FragmentTransaction = supportFragmentManager.beginTransaction()
        if (SearchHistoryFragment::class.simpleName == mCurrentTag) {
            ft.hide(mHistoryFragment)
        } else {
            ft.hide(mResultFragment)
        }

        ft.show(fragment).commit()
        mCurrentTag = fragment.javaClass.simpleName
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(SearchUiState::keyword) {
                if (inputView.text.isNullOrEmpty() && !it.isNullOrEmpty()) {
                    inputView.text = Editable.Factory.getInstance().newEditable(it)
                }
            }
        }
    }

    override fun onEditorAction(view: TextView, action: Int, event: KeyEvent?): Boolean {
        if (view.text.isNotEmpty()) {
            showFragment(mResultFragment)
            getViewModel().sendEvent(SearchUiEvent.AddHistory(view.text.toString()))
        }
        return true
    }

    override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
    }

    override fun afterTextChanged(s: Editable) {
        getViewModel().sendEvent(SearchUiEvent.SearchDataKey(s.toString()))
        if (s.isEmpty()) {
            showFragment(mHistoryFragment)
        } else {
            showFragment(mResultFragment)
        }
    }
}
