package com.at.video.home.ui.search.history

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.PageStatus
import com.at.video.home.R
import com.at.video.home.databinding.HomeSearchHistoryHeaderBinding
import com.at.video.home.ui.search.SearchUiEvent
import com.at.video.home.ui.search.SearchUiState
import com.at.video.home.ui.search.SearchViewModel
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/28 19:23
 * @Description:
 */

@AndroidEntryPoint
@Route(path = "/home/SearchHistoryFragment")
class SearchHistoryFragment: BaseMviFragment<CommonStatusRefreshListBinding,
        SearchHistoryViewModel>(),View.OnClickListener {
    private val mAdapter: SearchHistoryAdapter = SearchHistoryAdapter()
    private val mActivityViewModel by viewModels<SearchViewModel>(ownerProducer = { requireActivity() })

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): CommonStatusRefreshListBinding {
        return CommonStatusRefreshListBinding.inflate(inflater, container, false)
    }

    override fun getViewModelClass(): Class<SearchHistoryViewModel> {
        return SearchHistoryViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().includedRefreshList.apply {
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(requireContext(), 2)
            rvList.adapter = mAdapter
            val headerBinding: HomeSearchHistoryHeaderBinding = HomeSearchHistoryHeaderBinding
                .inflate(layoutInflater, rvList, false)
            headerBinding.ivClear.setOnClickListener(this@SearchHistoryFragment)
            mAdapter.addHeaderView(headerBinding.root)
            mAdapter.setOnItemClickListener { _, view, _ -> mActivityViewModel.sendEvent(
                SearchUiEvent.SearchDataKey((view as TextView).text.toString()))}
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(SearchHistoryUiEvent.InitHistoryData)
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(SearchHistoryUiState::data){
                mAdapter.setList(it)
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(SearchHistoryUiState::pageStatus) {
                requireBinding().apply {
                    when(it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            mActivityViewModel.state.collectState(SearchUiState::history){
                if(!it.isNullOrEmpty()){
                    getViewModel().sendEvent(SearchHistoryUiEvent.AddHistoryData(it))
                }
            }
        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        if(!hidden){
            loadData()
        }
    }


    override fun onClick(v: View) {
        when(v.id){
            R.id.iv_clear -> {
                getViewModel().sendEvent(SearchHistoryUiEvent.ClearHistoryData)
                mAdapter.setList(null)
            }
        }
    }
}