package com.at.video.home.ui.search.result

import android.annotation.SuppressLint
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import com.at.video.home.R
import com.at.video.home.entity.LineBean
import com.at.video.home.entity.VideoBean
import com.at.video.home.entity.VideoUrl
import com.at.video.home.ui.search.stringToHighLight
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.decoration.DefaultDecoration
import com.melancholy.widget.shape.view.ShapeMaterialButton
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeMaterialButtonExt
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/12 13:22
 * @Description:
 */

class SearchResultAdapter : BaseMultiItemQuickAdapter<VideoBean, BaseViewHolder>() {
    private val mPool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
    private var mKeyword: String = ""

    init {
        mPool.setMaxRecycledViews(0, 30)
        addItemType(0, R.layout.home_item_movie_free_view)
        addItemType(1, R.layout.home_item_movie_pay_view)
        addItemType(2, R.layout.home_item_teleplay_free_view)
        addItemType(3, R.layout.home_item_teleplay_pay_view)
    }

    override fun onCreateDefViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        val holder = super.onCreateDefViewHolder(parent, viewType)
        if (viewType == 2 || viewType == 3) {
            val rvLevelList: RecyclerView = holder.itemView.findViewById(R.id.rv_level_list)
            rvLevelList.itemAnimator = null
            rvLevelList.setHasFixedSize(true)
            rvLevelList.setRecycledViewPool(mPool)
            val padding: Int = parent.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_8)
            rvLevelList.addItemDecoration(DefaultDecoration.create(padding))
            rvLevelList.adapter = TeleplayLevelAdapter()
        }
        return holder
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    private fun changeTagStyle(tagView: ShapeMaterialButtonExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    override fun convert(holder: BaseViewHolder, data: VideoBean) {

        val categoryName: String = if (data.categ_name.isNullOrEmpty()) "" else data.categ_name
        val sub: String = if (data.subcateg_name.isNullOrEmpty()) "" else " /${data.subcateg_name}"
        val language: String = if (data.lang.isNullOrEmpty()) "" else " /${data.lang}"

        val actors = data.actors?.split(",")?.joinToString("/")
        holder.setText(R.id.tv_title, data.name!!.stringToHighLight(mKeyword))
            .setText(R.id.tv_categoryName, "$categoryName$sub$language")
            .setText(R.id.tv_actor, actors ?: "")
        holder.getView<ImageView>(R.id.div_img).loadByGlide(
                GlideImageConfig(
                    data.cover_vertical.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
                ), WeakReference(context)
            )
        if (data.itemType == 1 || data.itemType == 3) {
            holder.setText(R.id.sb_pay_money, data.price)
            changeTagStyle(holder.getView<ShapeMaterialButtonExt>(R.id.sb_pay_money))
            holder.setGone(R.id.sb_pay_money, !StyleUtils.tagEnable)
        } else {
            changeTagStyle(holder.getView<ShapeTextViewExt>(R.id.stv_tag))
            holder.setGone(R.id.stv_tag, !StyleUtils.tagEnable)
        }
        if (data.itemType == 2 || data.itemType == 3) {
            val lineJson = data.play_url
            lineJson?.let {
                val levels = JsonUtils.getListData<LineBean>(lineJson)
                val rvLevelList: RecyclerView = holder.getView(R.id.rv_level_list)
                (rvLevelList.adapter as TeleplayLevelAdapter).setNewInstance(mutableListOf<String>().apply {
                    if (!levels.isNullOrEmpty()) {
                        val urls: MutableList<VideoUrl>? = levels[0].urls?.toMutableList()
                        if (!urls.isNullOrEmpty()) {
                            for (i in urls.indices) {
                                add("${i + 1}")
                            }
                        }

                    }
                })
            }

        }
        holder.getView<ShapeMaterialButton>(R.id.smb_browser).setOnClickListener {
            holder.itemView.performClick()
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setKeyword(keyword: String) {
        mKeyword = keyword
        notifyDataSetChanged()
    }

    fun getKeyword(): String {
        return mKeyword
    }
}