package com.at.video.home.ui.search.result

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.home.entity.SearchInfo
import com.at.video.home.entity.VideoBean
import com.at.video.home.ui.search.SearchUiEvent
import com.at.video.home.ui.search.SearchUiState
import com.at.video.home.ui.search.SearchViewModel
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.decoration.DefaultDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/28 19:23
 * @Description:
 */

@AndroidEntryPoint
@Route(path = "/home/SearchResultFragment")
class SearchResultFragment: BaseMviFragment<CommonStatusRefreshListBinding,
        SearchResultViewModel>(), OnRefreshLoadMoreListener, OnItemClickListener {
    private val mAdapter: SearchResultAdapter = SearchResultAdapter()
    private val mActivityViewModel by viewModels<SearchViewModel>(ownerProducer = { requireActivity() })


    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): CommonStatusRefreshListBinding {
        return CommonStatusRefreshListBinding.inflate(inflater, container, false)
    }

    override fun getViewModelClass(): Class<SearchResultViewModel> {
        return SearchResultViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            msvView.setViewForState(com.at.common.R.layout.common_default_search_empty, MultiStateView.ViewState.EMPTY)
            includedRefreshList.apply {
                srlRefresh.setOnRefreshLoadMoreListener(this@SearchResultFragment)
                srlRefresh.setEnableRefresh(false)
                srlRefresh.setEnableRefresh(false)
                rvList.layoutManager = LinearLayoutManager(requireContext())
                val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_12)
                val cPadding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
                rvList.addItemDecoration(DefaultDecoration.create(0, padding, 0,
                    padding, 0, cPadding))
                rvList.setHasFixedSize(true)
                rvList.itemAnimator = null
                rvList.adapter = mAdapter
                mAdapter.setOnItemClickListener(this@SearchResultFragment)
            }
        }

    }

    override fun loadData() {

    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenStarted {
            getViewModel().state.collectState(SearchResultUiState::pageStatus){
                requireBinding().apply {
                    when(it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenStarted {
            getViewModel().state.collectState(SearchResultUiState::refreshStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenStarted {
            getViewModel().state.collectState(SearchResultUiState::loadMoreStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenStarted {
            getViewModel().state.collectState(SearchResultUiState::data){
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect{
                when(it){
                    is SearchResultUiEffect.SendSearchHis->{//保存历史
                        mActivityViewModel.sendEvent(SearchUiEvent.AddHistory(it.keyword))
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenStarted {
            mActivityViewModel.state.collectState(SearchUiState::keyword){
                if(!it.isNullOrEmpty()){
                    getViewModel().sendEvent(SearchResultUiEvent.SearchData(it))
                } else {
                    getViewModel().sendEvent(SearchResultUiEvent.InitLoadData)
                }
                mAdapter.setKeyword(it ?: "")
            }
        }

    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(SearchResultUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(SearchResultUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        //点击了内容才添加历史 或者 回车键
        mActivityViewModel.sendEvent(SearchUiEvent.AddHistory(mAdapter.getKeyword()))
        val data = adapter.data[position] as? VideoBean
        if (data?.id!=null){
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.id).navigation()
        }else{
            ToastUtils.show("播放资源不存在,请稍后重试")
        }
    }
}