package com.at.video.home.ui.widget

import android.content.res.ColorStateList
import android.graphics.Color
import android.util.StateSet
import android.view.Gravity
import android.view.View
import android.widget.TextView
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.text.BaseText
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/6 上午10:21
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/6 上午10:21
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class CustomTabText:BaseText() {


    private var tabColors = arrayListOf<ColorStateList>()
    private var tabNormalSizes:MutableList<Float> = mutableListOf()
    private var tabSelectedSizes:MutableList<Float> = mutableListOf()


    fun setTabColors(normalColors: MutableList<String>, selectedColors: MutableList<String>):CustomTabText{
        for (i in normalColors.indices){
            tabColors.add(createColorStateList(normalColors[i].toArgb(),selectedColors[i].toArgb()))
        }
        return this
    }

    fun setTabNormalSizes(normalSizes: MutableList<Float>):CustomTabText{
        tabNormalSizes = normalSizes
        return this
    }

    fun setTabSelectedSizes(selectedSizes: MutableList<Float>):CustomTabText{
        tabSelectedSizes = selectedSizes
        return this
    }

    private fun createColorStateList(defaultColor: Int, selectedColor: Int): ColorStateList {
        val states = arrayOfNulls<IntArray>(2)
        val colors = IntArray(2)
        var i = 0
        states[i] = intArrayOf(android.R.attr.state_selected)
        colors[i] = selectedColor
        i++
        // Default enabled state
        states[i] =intArrayOf(android.R.attr.state_enabled)
        colors[i] = defaultColor
        i++
        return ColorStateList(states, colors)
    }

    override fun bind() {
        tabLayout?.post {
            tabLayout?.apply {
                for (i in 0 until tabCount) {
                    getTabAt(i)?.let {
                        it.customView = TextView(context).apply {
                            text = it.text
                            textSize = if (selectedTabPosition==i) tabSelectedSizes[i] else tabNormalSizes[i]
                            if (selectedTabPosition==i)
                                paint?.isFakeBoldText = selectTextBold
                            else
                                paint?.isFakeBoldText = normalTextBold
                            gravity = Gravity.CENTER
                            setTextColor(tabColors[i])
                        }
                    }
                }

                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabReselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabSelectedSizes[selectedTabPosition]
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors[selectedTabPosition])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabSelectedSizes[selectedTabPosition]
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                    override fun onTabUnselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabNormalSizes[selectedTabPosition]
                                    paint?.isFakeBoldText = normalTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors[selectedTabPosition])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabNormalSizes[selectedTabPosition]
                                paint?.isFakeBoldText = normalTextBold
                            }
                        }
                    }

                    override fun onTabSelected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabSelectedSizes[selectedTabPosition]
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors[selectedTabPosition])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabSelectedSizes[selectedTabPosition]
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                })
            }
        }
    }
}