package com.at.video.home.ui.widget

import android.content.res.ColorStateList
import android.graphics.Color
import android.util.Log
import android.util.StateSet
import android.view.Gravity
import android.view.View
import android.widget.TextView
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.text.BaseText
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/6 上午10:21
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/6 上午10:21
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class CustomTabText1:BaseText() {


    private var tabColors = ColorStateList.valueOf(Color.BLACK)
    private var tabNormalSize:Float= 12f
    private var tabSelectedSize = 16F


    fun setTabColor(normalColor: String, selectedColor: String):CustomTabText1{
        tabColors = createColorStateList(normalColor.toArgb(),selectedColor.toArgb())
        return this
    }

    fun setTabNormalSizes(normalSize: Float):CustomTabText1{
        tabNormalSize = normalSize
        return this
    }

    fun setTabSelectedSizes(selectedSize: Float):CustomTabText1{
        tabSelectedSize = selectedSize
        return this
    }

    private fun createColorStateList(defaultColor: Int, selectedColor: Int): ColorStateList {
        val states = arrayOfNulls<IntArray>(2)
        val colors = IntArray(2)
        var i = 0
        states[i] = intArrayOf(android.R.attr.state_selected)
        colors[i] = selectedColor
        i++
        // Default enabled state
        states[i] =intArrayOf(android.R.attr.state_enabled)
        colors[i] = defaultColor
        i++
        return ColorStateList(states, colors)
    }

    override fun bind() {
        tabLayout?.post {
            tabLayout?.apply {
                Log.e("tablayout",tabCount.toString())
                for (i in 0 until tabCount) {
                    getTabAt(i)?.let {
                        if(it.customView !=null){
                            (it.customView as? TextView)?.apply {
                                text = it.text
                                textSize = if (selectedTabPosition==i) tabSelectedSize else tabNormalSize
                                if (selectedTabPosition==i)
                                    paint?.isFakeBoldText = selectTextBold
                                else
                                    paint?.isFakeBoldText = normalTextBold
                                gravity = Gravity.CENTER
                                setTextColor(tabColors)
                            }
                        }else{
                            it.customView = TextView(context).apply {
                                text = it.text
                                textSize = if (selectedTabPosition==i) tabSelectedSize else tabNormalSize
                                if (selectedTabPosition==i)
                                    paint?.isFakeBoldText = selectTextBold
                                else
                                    paint?.isFakeBoldText = normalTextBold
                                gravity = Gravity.CENTER
                                setTextColor(tabColors)
                            }
                        }

                    }
                }

                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabReselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabSelectedSize
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors)
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabSelectedSize
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                    override fun onTabUnselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabNormalSize
                                    paint?.isFakeBoldText = normalTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors)
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabNormalSize
                                paint?.isFakeBoldText = normalTextBold
                            }
                        }
                    }

                    override fun onTabSelected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    text = it.text
                                    textSize = tabSelectedSize
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(tabColors)
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                textSize = tabSelectedSize
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                })
            }
        }
    }
}