package com.at.video.home.ui.widget

import android.animation.ObjectAnimator
import android.annotation.SuppressLint
import android.content.Context
import android.os.Build
import android.util.AttributeSet
import android.util.Log
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.widget.*
import androidx.annotation.RequiresApi
import androidx.core.content.ContextCompat
import com.at.video.home.R
import com.at.video.home.entity.NoticeBean
import com.video.module_widget.entity.ButtonTextConfig
import com.video.module_widget.entity.home.NoticeBarConfig
import com.video.module_widget.utils.getSp
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.ShapeLinearLayoutExt

/**
 * @Create_time: 2022/5/12 15:34
 * @Description: 顶部公告View
 */

class TopNoticeView : ShapeLinearLayoutExt, View.OnClickListener {
    private lateinit var mAdapter: NoticeAdapter

    private var style: NoticeBarConfig? = null

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        super(context, attrs, defStyleAttr) {
        init(context)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int) :
        super(context, attrs, defStyleAttr, defStyleRes) {
        init(context)
    }

    private lateinit var ivNotice: ImageView
    private lateinit var ivClose: ImageView
    private lateinit var avfNotice: AdapterViewFlipper

    fun changeStyle(noticeStyle: NoticeBarConfig? = null) {
        style = noticeStyle
        if (style != null) {
            changeIconByType(ivNotice, 1, style!!.iconStyle)
            changeIconByType(ivClose, 2, style!!.closeStyle)
            mAdapter.changeStyle(style!!.text)
            avfNotice.flipInterval = style!!.duration.toInt()
            setBgStyle()
            reBuilder()
        } else {
            setBackgroundResource(R.color.color_FFFFF6EA)
        }
    }

    private fun setBgStyle() {
        when (style!!.style) {
            2 -> {
                setBorderRadius(2)
                setBackgroundColor(style!!.background!!.toArgb())
                setBorderWidth(0)
            }
            3 -> {
                setBackgroundColor(style!!.background!!.toArgb())
                setBorderRadius(2)
                setBorderColor("rgba(255,97,63,1)".toArgb())
                setBorderWidth(1)
            }
            else -> {
                setBorderWidth(0)
                setBackgroundColor(style!!.background!!.toArgb())
            }
        }
    }

    private fun init(context: Context) {
        LayoutInflater.from(context).inflate(
            R.layout.home_home_top_notice,
            this,
            true
        )
        ivNotice = findViewById<ImageView>(R.id.iv_notice)
        ivClose = findViewById<ImageView>(R.id.iv_close)
        mAdapter = NoticeAdapter()
        gravity = Gravity.CENTER_VERTICAL
        val padding: Int = context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
        setPadding(padding, paddingTop, padding, paddingBottom)
        ivClose.setOnClickListener(this)
        avfNotice = findViewById(R.id.avf_notice)
        avfNotice.adapter = mAdapter
//        val duration: Int = IMainService.getMainService().getSysConfig().getConfigAnnouncementHoldTime() * 1000
        val duration = 3 * 1000
        avfNotice.flipInterval = duration
        avfNotice.post {
            val avfHeight: Float = avfNotice.height.toFloat()

            @SuppressLint("ObjectAnimatorBinding")
            val inAnim = ObjectAnimator.ofFloat(null, "Y", avfHeight, 0f)
            inAnim.duration = 1000
            @SuppressLint("ObjectAnimatorBinding")
            val outAnim = ObjectAnimator.ofFloat(
                null,
                "Y",
                0f,
                -avfHeight
            )
            outAnim.duration = 1000
            avfNotice.inAnimation = inAnim
            avfNotice.outAnimation = outAnim
        }
    }

    private fun changeIconByType(img: ImageView, type: Int, style: Int) {
        val params = LinearLayout.LayoutParams(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20), resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20))
        val icon = when (style) {
            1 -> {
                if (type == 1) {
                    params.width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_32)
                    params.height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_18)
                    com.video.module_widget.R.drawable.vector_drawable_icon_notice1
                } else com.video.module_widget.R.drawable.vector_drawable_icon_close1
            }
            2 -> if (type == 1) com.video.module_widget.R.drawable.vector_drawable_icon_notice2
            else {
                com.video.module_widget.R.drawable.vector_drawable_icon_close2
            }
            else -> if (type == 1) com.video.module_widget.R.drawable.vector_drawable_icon_notice3
            else {
                com.video.module_widget.R.drawable.vector_drawable_icon_close2
            }
        }
        img.layoutParams = params
        img.setImageResource(icon)
    }

    override fun onClick(view: View) {
        if (parent != null) {
            (parent as ViewGroup).removeView(this)
        }
    }

    fun setData(list: List<NoticeBean>) {
        mAdapter.setData(list)
    }

    fun setOnItemClickListener(l: OnItemClickListener) {
        mAdapter.setOnItemClickListener(l)
    }

    private class NoticeAdapter : BaseAdapter(), OnClickListener {
        var style: ButtonTextConfig? = null
        private val mData: MutableList<NoticeBean> = mutableListOf()
        private var mOnItemClickListener: OnItemClickListener? = null
        fun setData(list: List<NoticeBean>) {
            mData.clear()
            mData.addAll(list)
            notifyDataSetChanged()
        }

        fun changeStyle(itemStyle: ButtonTextConfig?) {
            style = itemStyle
            notifyDataSetChanged()
        }

        fun setOnItemClickListener(l: OnItemClickListener) {
            mOnItemClickListener = l
        }

        override fun getCount(): Int {
            return mData.size
        }

        override fun getItem(position: Int): Any {
            return mData[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val rootView: TextView = if (convertView == null) {
                val text = TextView(parent.context)
                val params: ViewGroup.LayoutParams = ViewGroup.LayoutParams(MATCH_PARENT, MATCH_PARENT)
                text.layoutParams = params
                text.gravity = Gravity.CENTER_VERTICAL
                text.setSingleLine()
                if (style != null) {
                    text.setTextSize(TypedValue.COMPLEX_UNIT_PX, text.context.getSp(style!!.size))
                    text.setTextColor(style!!.color!!.toArgb())
                    text.letterSpacing = style!!.letterSpacing.toFloat()
                } else {
                    text.setTextSize(
                        TypedValue.COMPLEX_UNIT_PX,
                        parent.resources.getDimensionPixelSize(
                            com.at.common.R.dimen.sp_14
                        ).toFloat()
                    )
                    text.setTextColor(ContextCompat.getColor(parent.context, R.color.color_FFDB801A))
                }

                text.setOnClickListener(this)
                text
            } else {
                convertView as TextView
            }
            val data: NoticeBean = mData[position]
            rootView.text = data.title
            rootView.tag = data
            return rootView
        }

        override fun onClick(view: View) {
            mOnItemClickListener?.onItemClick(view.tag as NoticeBean)
        }
    }

    interface OnItemClickListener {
        fun onItemClick(data: NoticeBean)
    }
}
