package com.at.video.main

import com.at.common.global.CacheHelper
import com.at.common.utils.JsonUtils
import com.at.video.main.entity.*

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/6 下午4:21
 */
class ConfigManager {
    @Volatile
    private var mConfigInfo: ConfigInfo? = null

    companion object {
        private const val KEY_SYS_INFO: String = "key_config_info"

        @JvmStatic
        val instance: ConfigManager by lazy(LazyThreadSafetyMode.SYNCHRONIZED) { ConfigManager() }
    }

    @Synchronized
    fun saveConfig(appInfo:AppResInfo) {
        mConfigInfo = parsing(appInfo)
        val json: String = JsonUtils.toJson(mConfigInfo!!)
        CacheHelper.getInstance().putString(KEY_SYS_INFO, json)
    }

    private fun parsing(appInfo: AppResInfo): ConfigInfo {
        return ConfigInfo(
            appInfo.userAgreement,
            appInfo.privacyPolicy,
            appInfo.applicationCode,
            appInfo.appUpdateContent,
            appInfo.appDownloadUrl,
            appInfo.advLibId,
            appInfo.videoLibId,
            appInfo.imageLibId,
            appInfo.articleLibId,
            appInfo.isConfigEmail,
            appInfo.minio_pkg_address,
            appInfo.appName,
            appInfo.upgrade,
            appInfo.versionName
            )
    }

    @Synchronized
    internal fun hasConfig(): Boolean = mConfigInfo != null

    @Synchronized
    fun getSysConfig(): ConfigInfo = mConfigInfo!!

    @Synchronized
    fun loadConfig() {
        val json: String? = CacheHelper.getInstance().getString(KEY_SYS_INFO, null)
        if(!json.isNullOrEmpty()) {
            mConfigInfo = JsonUtils.fromJson(json)
        }
    }
}