package com.at.video.main.service

import android.app.Service
import android.content.Intent
import android.os.IBinder
import android.util.Log
import com.at.common.global.CacheHelper
import com.at.common.utils.JsonUtils
import com.melancholy.network.ApiFactory
import com.melancholy.utils.FileUtils
import com.video.module_widget.api.ApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.AdvType
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.imageUrl
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.launch
import java.io.*

/**
 * @Create_time: 2022/5/30 19:35
 * @Description:
 */

class AdService : Service() {
    private var mAdJob: Job? = null
    override fun onBind(p0: Intent?): IBinder? {
        return null
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        saveAd()
        return START_NOT_STICKY
    }

    private fun saveAd() {
        mAdJob = GlobalScope.launch(Dispatchers.IO) {
            val paths = FileUtils.getExternalFilesDirs(
                this@AdService,
                "ad"
            )
            val parent: File = if (paths.isNullOrEmpty()) File(filesDir, "ad") else paths[0]
            val isExists = parent.exists()
            if (!isExists) {
                parent.mkdir()
            }
//            val launchUrl = CacheHelper.getInstance().getString("launch_addr","")
//            if(!launchUrl.isNullOrEmpty()){
//                val fileType = launchUrl.substring(launchUrl.lastIndexOf("."),launchUrl.length)
//
//                val launchPath = File(parent,"launch${fileType}")
//                saveLaunchAd(launchPath,launchUrl)
//            }
//            val themeUrl = CacheHelper.getInstance().getString("theme_addr","")
//            if(!themeUrl.isNullOrEmpty()){
//                val themePath = File(parent,"theme.json")
//                saveThemeStyle(themePath,themeUrl)
//            }
            getAd().catch {
            }.collect { it1 ->
                val json: String? = CacheHelper.getInstance().getString("ads", null)
                val localAdvs = json?.let {
                    com.video.module_widget.utils.JsonUtils.getListData<AdvertisingBean>(
                        it
                    )
                }
                // 下载新的
                if (!it1.isNullOrEmpty()) {
                    val imgList = mutableListOf<AdvertisingBean>()
                    imgList.addAll(filterDelete(localAdvs, it1))
                    it1.forEach {
                        if (!isDownload(localAdvs, it.id!!)) { // 判断是否已经下载
                            try {
                                it.image_url?.apply {
                                    var fileType = substring(
                                        lastIndexOf("."),
                                        length
                                    )
                                    if (fileType.length > 4 || fileType.length < 3) {
                                        fileType = ".png"
                                    }
                                    val path = File(parent, "${System.currentTimeMillis()}$fileType")
                                    if (!path.exists()) {
                                        path.createNewFile()
                                    }
                                    saveFile(path, this.imageUrl()!!)
                                    imgList.add(it.copy(addr = path.absolutePath))
                                }
                            } catch (e: Exception) {
                            }
                        }
                    }
                    CacheHelper.getInstance()
                        .putString("ads", JsonUtils.toJson(imgList))
                    // 删除旧的

//                    if(!localAdvs.isNullOrEmpty()){
//                        localAdvs.forEach {
//                            try {
//                                File(it.addr).delete()
//                            } catch (e: Exception) {
//                            }
//                        }
//                    }
                } else {
                    CacheHelper.getInstance().remove("ads")
                }
            }
        }
    }

    private fun isDownload(localAdv: List<AdvertisingBean>?, id: String): Boolean {
        val res = localAdv?.filter { it -> it.id == id }
        if (res != null) {
            if (res.isNotEmpty()) {
                return true
            }
        }
        return false
    }

    /**
     * 移除后台已经删除或禁用的广告
     */
    private fun filterDelete(localAdv: List<AdvertisingBean>?, serverAdv: List<AdvertisingBean>?): List<AdvertisingBean> {
        var advs = mutableListOf<AdvertisingBean>()
        localAdv?.forEach {
            if (serverAdv.isNullOrEmpty()) {
                val file = it.addr?.let { it1 -> File(it1) }
                file?.delete()
                return advs
            } else {
                val res = serverAdv.filter { it1 -> it1.id == it.id }
                if (res.isEmpty()) {
                    val file = it.addr?.let { it1 -> File(it1) }
                    file?.delete()
                } else {
                    advs.add(it)
                }
            }
        }
        return advs
    }

    private suspend fun saveLaunchAd(path: File, url: String) {
        saveFile(path, url)
        CacheHelper.getInstance()
            .putString("launch_ad", path.absolutePath)
    }

    private suspend fun saveThemeStyle(path: File, url: String) {
        saveFile(path, url)
        CacheHelper.getInstance()
            .putString("theme_style", path.absolutePath)
    }

    private suspend fun saveFile(path: File, url: String) {
        try {
            Log.e("ADV", "开始下载广告url:$url ->path:$path")
            val res = ApiFactory.getInstance().create(ApiService::class.java).download(url)
            val inputStream = res.byteStream()
            val fos = FileOutputStream(path)
            try {
                var len = 0
                var buffer = ByteArray(2048 * 2)
                while (inputStream.read(buffer).also { len = it } != -1) {
                    fos.write(buffer, 0, len)
                }
            } catch (e: Exception) {
            } finally {
                fos.close()
                inputStream.close()
            }
//            val fileSize = path.length()
//            Log.e("File","$fileSize")
//            CacheHelper.getInstance()
//                .putString("ad", JsonUtils.toJson(it.copy(addr = path.absolutePath)))
        } catch (e: Exception) {
        }
    }

    private fun getAd(): Flow<List<AdvertisingBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = (AdvType.StartAdv.ordinal + 1).toString()
            map["library_id"] = GlobalConsts.LIBRARY_ID
//            map["library_id"] = ConfigManager.instance.getSysConfig().advLibId
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getAdvertisingList(map)
            if (!bean.list.isNullOrEmpty()) {
                emit(bean.list!!)
            } else {
                emit(null)
            }
        }
    }

    override fun onDestroy() {
        mAdJob?.cancel()
        super.onDestroy()
    }
}
