package com.at.video.main.ui.main

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.export.mine.IMineService
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.ApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.AdvType
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.utils.TrackersUtil
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/27 14:08
 * @Description:
 */

class MainUiState : IUiState

sealed class MainUiEvent : IUiEvent {
    object InitData : MainUiEvent()
    object GetTrackInfo : MainUiEvent()
}

sealed class MainUiEffect : IUiEffect {
    data class BottomAd(val info: AdvertisingBean) : MainUiEffect()
}

@HiltViewModel
class MainViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<MainUiState, MainUiEvent, MainUiEffect>(application) {

    override fun providerInitialState(): MainUiState {
        return MainUiState()
    }

    override fun handleEvent(event: MainUiEvent) {
        when (event) {
            is MainUiEvent.InitData -> {
                getBottomAd()
                viewModelScope.launch {
                    getVideoTrackInfo().collect()
                }
            }
            is MainUiEvent.GetTrackInfo -> {
                viewModelScope.launch {
                    getVideoTrackInfo().collect()
                }
            }
        }
    }

    /**
     * 获取底部广告
     */
    private fun getBottomAd() {
        viewModelScope.launch {
            getAd().catch {

            }.collect {
                if (!it.isNullOrEmpty()) {
                    setEffect { MainUiEffect.BottomAd(it[0]) }
                }
            }
        }
    }


    /**
     * 获取广告
     */
    private fun getAd(): Flow<List<AdvertisingBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = (AdvType.BottomAdv.ordinal + 1).toString()
            map["library_id"] = GlobalConsts.LIBRARY_ID
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getAdvertisingList(map)
            if (!bean.list.isNullOrEmpty()) {
                emit(bean.list!!)
            } else {
                emit(null)
            }
        }
    }

    /**
     * 获取视频track信息
     */
    private fun getVideoTrackInfo(): Flow<List<String>?> {
        return flow {
            try {
                if (IMineService.getMineService().getUser().value != null) {
                    val bean =
                        ApiFactory.getInstance()
                            .create(com.at.video.main.http.ApiService::class.java)
                            .getTrackerList()
                    if (bean.success() && !bean.trackers.isNullOrEmpty()) {
                        TrackersUtil.addTracker(bean.trackers)
                        emit(bean.trackers)
                    } else {
                        emit(null)
                    }
                } else {
                    emit(null)
                }
            } catch (e: Exception) {
                emit(null)

            }
        }
    }
}