package com.at.video.main.ui.splash

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Bundle
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.ImageConfig
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.load
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.showAndHideBar
import com.at.common.utils.LoadingUtil
import com.at.video.main.R
import com.at.video.main.databinding.MainActivitySplashBinding
import com.at.video.main.entity.AdvertisingInfo
import com.at.video.main.service.AdService
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.facade.Postcard
import com.melancholy.router.api.facade.callback.NavigationCallback
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.FileUtils
import dagger.hilt.android.AndroidEntryPoint
import java.io.File

/**
 * @Create_time: 2022/4/27 10:38
 * @Description: 启动页面
 */

@AndroidEntryPoint
@Route(path = "/main/SplashActivity")
@SuppressLint("CustomSplashScreen")
class SplashActivity :
    BaseMviActivity<MainActivitySplashBinding, SplashViewModel>(),
    View.OnClickListener,
    NavigationCallback {

    private var mIsReady: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun immersiveBar() {
        showAndHideBar(false)
    }

    override fun createViewBinding(): MainActivitySplashBinding {
        return MainActivitySplashBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<SplashViewModel> {
        return SplashViewModel::class.java
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireBinding().ctvTimer.setOnClickListener(this)
        requireBinding().ivAd.setOnClickListener(this)
        // Android 12需要这层逻辑
        /*val content: View = findViewById(android.R.id.content)
        content.viewTreeObserver.addOnPreDrawListener(object : ViewTreeObserver.OnPreDrawListener {

                override fun onPreDraw(): Boolean {
                    return if(mIsReady) {
                        //准备好了移除监听器
                        content.viewTreeObserver.removeOnPreDrawListener(this)
                        true
                    } else {
                        //没有准备好，继续显示闪屏页
                        false
                    }
                }
        })*/
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is SplashUiEffect.Jump -> jump()
                    is SplashUiEffect.Loading -> LoadingUtil.showLoading(
                        this@SplashActivity,
                        it.content
                    )
                    is SplashUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is SplashUiEffect.Finish -> {
                        HintDialog.showDialog(supportFragmentManager, ::callback)
                    }
                    else -> {}
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(SplashUiState::data) {
                if (it != null) {
                    requireBinding().apply {
                        flAd.tag = it
                        ivAd.load(ImageConfig(File(it.addr), placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default, errorId = com.video.module_widget.R.drawable.common_ic_ad_default))
//                        window.setBackgroundDrawable(null)
                        flAd.visibility = View.VISIBLE
                        ctvTimer.visibility = View.VISIBLE
                        ctvTimer.setSumTime(it.showTime * 1000)
                        ctvTimer.startTimer()
                    }
                    mIsReady = true
                }
            }
        }
    }

    override fun loadData() {
        setThemeCacheFile()
//        test()
        getViewModel().sendEvent(SplashUiEvent.InitData)
//        var jsonStr:String? = null
//        val themePath = CacheHelper.getInstance().getString("theme_style","")
//        jsonStr = if(!themePath.isNullOrEmpty()){
//            getFile(this,themePath)?.let { String(it) }
//        }else{
//            getAssertsFile(applicationContext,"data.json")?.let { String(it) }
//        }
//        if(!jsonStr.isNullOrEmpty()){
//            val json = JSONObject(jsonStr)
//            parseData(json)
//        }
    }

    fun test() {
        requireBinding().apply {
            flAd.visibility = View.VISIBLE
            ctvTimer.visibility = View.VISIBLE
            ctvTimer.setSumTime(3 * 1000)
            ctvTimer.startTimer()
        }
    }

    private fun setThemeCacheFile() {
        val paths = FileUtils.getExternalFilesDirs(
            this,
            "theme"
        )
        val parent: File = if (paths.isNullOrEmpty()) File(filesDir, "theme") else paths[0]
        val isExists = parent.exists()
        if (!isExists) {
            parent.mkdir()
        }
        val themePath = File(parent, "theme.json")
        CacheHelper.getInstance()
            .putString("theme_style", themePath.absolutePath)
    }

    override fun onClick(view: View) {
        if (view.id == R.id.ctv_timer) {
            jump()
        } else if (view.id == R.id.iv_ad) {
            val ad: AdvertisingInfo? = requireBinding().flAd.tag as? AdvertisingInfo
            if (ad != null) {
                requireBinding().ctvTimer.stopTimer()
                jump(ad.jumpAddr)
            }
        }
    }

    private fun callback(isFinish: Boolean) {
        if (isFinish) {
            finish()
        } else {
            getViewModel().sendEvent(SplashUiEvent.InitData)
        }
    }

    @JvmOverloads
    private fun jump(jumpAddr: String? = null) {
        Router.getInstance().build("/main/MainActivity")
            .navigation(
                this,
                callback = object : NavigationCallback {
                    override fun onArrival(postcard: Postcard) {
                        if (jumpAddr != null) {
                            Router.getInstance().build("/webview/WebViewActivity")
                                .withString(Constants.KEY_DATA, jumpAddr)
                                .navigation(this@SplashActivity, callback = this@SplashActivity)
                        } else {
//                        window.setBackgroundDrawable(null)
                            startService(Intent(application, AdService::class.java))
                            finish()
                        }
                    }

                    override fun onFound(postcard: Postcard) {
                    }

                    override fun onInterrupt(postcard: Postcard) {
                    }

                    override fun onLost(postcard: Postcard) {
                    }
                }
            )
    }

    override fun onArrival(postcard: Postcard) {
//        window.setBackgroundDrawable(null)
        startService(Intent(application, AdService::class.java))
        finish()
    }

    override fun onFound(postcard: Postcard) {
    }

    override fun onInterrupt(postcard: Postcard) {
    }

    override fun onLost(postcard: Postcard) {
    }
}
