package com.at.video.main.widget

import android.content.Context
import android.os.Build
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.annotation.RequiresApi
import com.at.common.global.Constants
import com.at.video.main.R
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.home.Adv
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.loadByGlide
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/12 14:46
 * @Description: 底部广告View
 */

class BottomAdView : FrameLayout, View.OnClickListener {
    private lateinit var mIvAd: DiversityImageView

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        super(context, attrs, defStyleAttr) {
        init(context)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int) :
        super(context, attrs, defStyleAttr, defStyleRes) {
        init(context)
    }

    private fun init(context: Context) {
        LayoutInflater.from(context).inflate(
            R.layout.main_main_bottom_ad,
            this,
            true
        )
        mIvAd = findViewById(R.id.iv_ad)
        findViewById<View>(R.id.iv_close).setOnClickListener(this)
        setOnClickListener(this)
    }

    override fun onClick(view: View) {
        if (view.id == R.id.iv_close) {
            if (parent != null) {
                (parent as ViewGroup).removeView(this)
            }
        } else {
            val data: AdvertisingBean = mIvAd.tag as AdvertisingBean
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.jump_addr!!)
                .navigation()
        }
    }

    fun setData(data: AdvertisingBean, style: Adv?) {
//        mIvAd.load(ImageConfig(data.image_url.imageUrl(), placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
//            errorId =com.video.module_widget.R.drawable.common_ic_ad_default))
        mIvAd.loadByGlide(
            GlideImageConfig(
                data.image_url.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                errorId = com.video.module_widget.R.drawable.common_ic_ad_default
            ),
            WeakReference(context)
        )
        mIvAd.tag = data
        style?.let {
            mIvAd.setFullRadius(it.radius.toFloat())
        }
    }
}
