package com.at.video.main.widget

import android.content.Context
import android.graphics.Typeface
import android.os.Build
import android.util.AttributeSet
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.annotation.RequiresApi
import androidx.cardview.widget.CardView
import com.at.common.global.Constants
import com.at.common.utils.JsonUtils
import com.at.video.main.R
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DensityUtils
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.WordStyle
import com.video.module_widget.entity.home.Adv
import com.video.module_widget.utils.toRgb

/**
 * @Create_time: 2022/5/12 14:46
 * @Description: 底部广告View
 */

class BottomTxtAdView : FrameLayout, View.OnClickListener {
    private lateinit var mIvAd: TextView
    private lateinit var cardView: CardView

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        super(context, attrs, defStyleAttr) {
        init(context)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int) :
        super(context, attrs, defStyleAttr, defStyleRes) {
        init(context)
    }

    private fun init(context: Context) {
        val root = LayoutInflater.from(context).inflate(
            R.layout.main_main_bottom_txt_ad,
            this,
            true
        )
        mIvAd = findViewById(R.id.tv_content)
        cardView = findViewById(R.id.root_view)

        findViewById<View>(R.id.iv_close).setOnClickListener(this)
        setOnClickListener(this)
    }

    override fun onClick(view: View) {
        if (view.id == R.id.iv_close) {
            if (parent != null) {
                (parent as ViewGroup).removeView(this)
            }
        } else {
            val data: AdvertisingBean = mIvAd.tag as AdvertisingBean
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.jump_addr!!)
                .navigation()
        }
    }

    fun setData(data: AdvertisingBean, style: Adv?) {
        mIvAd.text = data.image_url
        mIvAd.tag = data
        val wordStyle = data.word_style?.let { parseStyle(it) }
        val params = cardView.layoutParams
        val textParams: FrameLayout.LayoutParams = mIvAd.layoutParams as FrameLayout.LayoutParams

        if (wordStyle != null) {
            wordStyle.background?.let { cardView.setCardBackgroundColor(it.toRgb()) }
            wordStyle.height?.let {
                params.height = DensityUtils.dp2px(context, it.replace("px", "").toFloat())
            }
            wordStyle.textAlign?.let {
                textParams.gravity = when (it) {
                    "left" -> Gravity.LEFT or Gravity.CENTER_VERTICAL
                    "right" -> Gravity.RIGHT or Gravity.CENTER_VERTICAL
                    else -> Gravity.CENTER
                }
            }
            wordStyle.fontWeight?.let {
                mIvAd.typeface = if (it != "normal") Typeface.DEFAULT_BOLD else Typeface.DEFAULT
            }
            wordStyle.fontSize?.let {
                mIvAd.textSize = it.replace("px", "").toFloat()
            }
            wordStyle.color?.let {
                mIvAd.setTextColor(it.toRgb())
            }
        }
        style?.let {
            cardView.radius = it.radius.toFloat()
        }
        cardView.layoutParams = params
        mIvAd.layoutParams = textParams
    }

    private fun parseStyle(json: String): WordStyle? {
        return JsonUtils.fromJson(json)
    }
}
