package com.at.video.main.widget

import android.R
import android.content.Context
import android.os.CountDownTimer
import android.util.AttributeSet
import android.view.View
import com.melancholy.widget.shape.view.ShapeTextView

/**
 * @Create_time: 2022/4/29 09:34
 * @Description:
 */

class CountdownTextView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null,
      defStyleAttr: Int = R.attr.textViewStyle): ShapeTextView(context, attrs, defStyleAttr),
    View.OnClickListener {

    private var mCountDownTimer: CountDownTimer? = null
    private var mSumTime: Int = 60000
    private var mIntervalTime: Int = 1000
    private var mFormatText: String = ""
    private var mOnClickListener: OnClickListener? = null

    init {
        super.setOnClickListener(this)
        mFormatText = text.toString()
    }

    override fun setOnClickListener(l: OnClickListener?) {
        mOnClickListener = l
    }

    fun setSumTime(sumTime: Int) {
        mSumTime = sumTime + 1000
    }

    fun startTimer(){
        if (mCountDownTimer == null) {
            mCountDownTimer = Timer(mSumTime.toLong(), mIntervalTime.toLong())
        }
        mCountDownTimer?.let {
            it.cancel()
            it.start()
        }
    }

    fun stopTimer(){
        mCountDownTimer?.cancel()
        mCountDownTimer = null
    }

    private inner class Timer(sumTime: Long, intervalTime: Long): CountDownTimer(sumTime,
        intervalTime) {
        override fun onFinish() {
            mOnClickListener?.onClick(this@CountdownTextView)
        }

        override fun onTick(millisUntilFinished: Long) {
            text = String.format(mFormatText, millisUntilFinished / 1000)
        }

    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        stopTimer()
    }

    override fun onClick(v: View) {
        stopTimer()
        mOnClickListener?.onClick(this@CountdownTextView)
    }
}