package com.at.video.member.ui.buy

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.R
import com.at.video.member.entity.GoldsItem
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.decoration.DefaultDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/5 16:55
 * @Description: 我的购买 开通会员
 */
@AndroidEntryPoint
@Route(path = "/member/MineBuyActivity", extras = 10086)
class MineBuyActivity : BaseMviActivity<CommonTitleBarRefreshListBinding, MineBuyViewModel>(),
    OnRefreshLoadMoreListener {
    private val mAdapter: MineBuyAdapter = MineBuyAdapter()

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<MineBuyViewModel> {
        return MineBuyViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitle("我的购买")
        }
        requireBinding().apply {
            includedStatusRefreshList.includedRefreshList.apply {
                srlRefresh.setOnRefreshLoadMoreListener(this@MineBuyActivity)
                srlRefresh.setEnableRefresh(false)
                srlRefresh.setEnableLoadMore(false)
                rvList.layoutManager = LinearLayoutManager(this@MineBuyActivity)
                val cPadding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_10)
                val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
                rvList.addItemDecoration(
                    DefaultDecoration.create(
                        padding, padding, padding, padding, cPadding, cPadding, isIncludeEdge = true
                    )
                )
                rvList.setHasFixedSize(true)
                rvList.itemAnimator = null
                rvList.adapter = mAdapter
            }
        }
        mAdapter.setDiffCallback(DiffBuyCallback())
        mAdapter.setOnItemClickListener { adapter, _, position ->
            run {
                val data = adapter.getItem(position) as GoldsItem
                when (data.use) {
                    2 -> { // 电影
                        Router.getInstance().build("/player/PlayerActivity")
                            .withInt(Constants.KEY_DATA, data.useId).navigation()
                    }
                    3 -> { // 小说
                        Router.getInstance().build("/novel/NovelDetailActivity")
                            .withInt("novelId", data.useId).withString("title", data.name)
                            .navigation()
                    }
                    4 -> { // 图集
                        Router.getInstance().build("/atlas/DetailActivity")
                            .withInt("atlasId", data.useId).navigation()
                    }
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(MineBuyUiEvent.InitLoadData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MineBuyUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                                val emptyRoot = msvView.getView(msvView.viewState) as FrameLayout
                                emptyRoot.findViewById<TextView>(com.at.common.R.id.tv_label).text =
                                    "暂无购买记录"
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MineBuyUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MineBuyUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MineBuyUiState::data) {
                mAdapter.setDiffNewData(it.toMutableList())
            }
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MineBuyUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MineBuyUiEvent.LoadDataMore)
    }
}
