package com.at.video.member.ui.buy

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.entity.GoldsItem
import com.at.video.member.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/5 16:57
 * @Description:
 */

data class MineBuyUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<GoldsItem> = listOf()
) : IUiState

sealed class MineBuyUiEvent : IUiEvent {
    object InitLoadData : MineBuyUiEvent()
    object RefreshData : MineBuyUiEvent()
    object LoadDataMore : MineBuyUiEvent()
}

sealed class MineBuyUiEffect : IUiEffect

class MineBuyViewModel(application: Application) : BaseAndroidViewModel<MineBuyUiState,
    MineBuyUiEvent, MineBuyUiEffect>(application) {

    private var mOffset: Int = 0
    private val mLimit: Int = 9
    override fun providerInitialState(): MineBuyUiState {
        return MineBuyUiState()
    }

    override fun handleEvent(event: MineBuyUiEvent) {
        when (event) {
            is MineBuyUiEvent.InitLoadData -> initLoadData()
            is MineBuyUiEvent.RefreshData -> refreshData()
            is MineBuyUiEvent.LoadDataMore -> loadDataMore()
        }
    }

    private fun initLoadData() {
        setState { copy(pageStatus = PageStatus.Empty) }
//        viewModelScope.launch {
//            getBuyList().onStart {
//                setState { copy(pageStatus = PageStatus.Loading) }
//            }.catch {
//                setState { copy(pageStatus = PageStatus.Error(it)) }
//            }.collect {
//                if(it.isNullOrEmpty()) {
//                    setState { copy(pageStatus = PageStatus.Empty) }
//                } else {
//                    val hasMore: Boolean = it.size == mLimit
//                    if(hasMore) {
//                        mOffset += mLimit
//                    }
//                    setState { copy(pageStatus = PageStatus.Success, data = it,
//                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
//                }
//            }
//        }
    }

    private fun refreshData() {
        setState { copy(pageStatus = PageStatus.Empty) }
//        viewModelScope.launch {
//            mOffset = 0
//            getBuyList().onStart {
//                setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
//                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
//            }.catch {
//                setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
//            }.collect {
//                if(it.isNullOrEmpty()) {
//                    setState { copy(pageStatus = PageStatus.Empty,
//                        refreshStatus = RefreshStatus.RefreshSuccess) }
//                } else {
//                    val hasMore: Boolean = it.size == mLimit
//                    if(hasMore) {
//                        mOffset += mLimit
//                    }
//                    setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = it,
//                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
//                }
//            }
//        }
    }

    private fun loadDataMore() {
        viewModelScope.launch {
            getBuyList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    val hasMore: Boolean = it.size == mLimit
                    if (hasMore) {
                        mOffset += mLimit
                    }
                    setState {
                        copy(
                            data = data + it,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun getBuyList(): Flow<List<GoldsItem>?> {
        return flow {
            val map = mutableMapOf<String, Any>()
            map["offset"] = mOffset
            map["limit"] = mLimit
            val result = ApiFactory.getInstance().create(ApiService::class.java).getBuyList(map)
            if (result.success()) {
                emit(result.data?.list)
            } else {
                throw Error(result.msg)
            }
        }
    }
}
