package com.at.video.member.ui.detail

import android.graphics.Color
import android.os.Bundle
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.R
import com.chad.library.adapter.base.BaseBinderAdapter
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/5 17:35
 * @Description: 消费明细
 */

// @Route(path = "/member/DetailActivity", extras = 10086)
@AndroidEntryPoint
@Route(path = "/member/DetailActivity")
class DetailActivity :
    BaseMviActivity<CommonTitleBarRefreshListBinding, DetailViewModel>(),
    OnRefreshLoadMoreListener {

    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()

    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<DetailViewModel> {
        return DetailViewModel::class.java
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireBinding().apply {
            requireTitleBar().apply {
                addBackButton(Color.BLACK)
                setTitle("消费明细")
            }
            includedStatusRefreshList.includedRefreshList.apply {
                srlRefresh.setOnRefreshLoadMoreListener(this@DetailActivity)
                srlRefresh.setEnableLoadMore(false)
                srlRefresh.setEnableRefresh(false)
                rvList.layoutManager = LinearLayoutManager(this@DetailActivity)
                rvList.itemAnimator = null
                rvList.setHasFixedSize(true)
                rvList.adapter = mAdapter
            }
        }
        mAdapter.addItemBinder(DetailHeaderBinder())
        mAdapter.addItemBinder(DetailItemBinder())
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                srlRefresh.setEnableLoadMore(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::loadMoreStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> srlRefresh.finishLoadMore(false)
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(DetailUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DetailUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DetailUiEvent.LoadMoreData)
    }
}
