package com.at.video.member.ui.gold

import android.graphics.Color
import android.os.Bundle
import android.view.View
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.at.common.immersiveStatusBar
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.databinding.MemberActivityGoldBinding
import com.at.video.member.ui.member.DiffMemberCallback
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.melancholy.router.annotation.annotation.Route
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import kotlinx.coroutines.flow.collect

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/8 下午8:20
 */
@Route(path = "/member/GoldActivity")
//@Route(path = "/member/GoldActivity", extras = 10086)
class GoldActivity : BaseMviActivity<MemberActivityGoldBinding, GoldViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener, View.OnClickListener {

    private val adapter = GoldAdapter()

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            setTitle("金币充值")
            addBackButton(Color.BLACK)
        }
        requireBinding().apply {
            srlRefreshGold.setEnableLoadMore(false)
            srlRefreshGold.setEnableRefresh(false)
            srlRefreshGold.setOnRefreshLoadMoreListener(this@GoldActivity)
            rvListGold.layoutManager = LinearLayoutManager(this@GoldActivity)
            rvListGold.adapter = adapter
//            stvNext.setOnClickListener(this@GoldActivity)
        }
        adapter.setOnItemClickListener(this)
        adapter.setDiffCallback(DiffMemberCallback())
    }

    override fun immersiveBar() {
        immersiveStatusBar()
        setLightStatusBar(true)
    }

    override fun loadData() {
        getViewModel().sendEvent(GoldUiEvent.InitLoadData)
    }

    override fun createViewBinding(): MemberActivityGoldBinding {
        return MemberActivityGoldBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<GoldViewModel> = GoldViewModel::class.java
    override fun onRefresh(refreshLayout: RefreshLayout) {
         getViewModel().sendEvent(GoldUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(GoldUiEvent.LoadDataMore)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GoldUiState::pageStatus) {
                when (it) {
                    PageStatus.Success -> {
                        requireBinding().apply {
                            srlRefreshGold.setEnableRefresh(true)
                            srlRefreshGold.setEnableLoadMore(true)
                        }
                    }
                    else -> {}
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GoldUiState::refreshStatus) {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            requireBinding().srlRefreshGold.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!requireBinding().srlRefreshGold.isRefreshing) {
                                requireBinding().srlRefreshGold.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            requireBinding().srlRefreshGold.finishRefresh(false)
                        }
                        else -> {}
                    }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GoldUiState::loadMoreStatus) {
                requireBinding().apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefreshGold.finishLoadMore()
                            } else {
                                srlRefreshGold.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefreshGold.isLoading) {
                                srlRefreshGold.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefreshGold.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GoldUiState::data) {
                adapter.setDiffNewData(it)
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when(it){
                    is GoldUiEffect.RefreshPos->{
                        adapter.notifyItemChanged(it.pos)
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GoldUiState::pos){
                val srlRefreshGold = requireBinding().srlRefreshGold
                srlRefreshGold.setEnableRefresh(it==-1)
            }
        }
    }

    override fun onClick(p0: View?) {
        getViewModel().sendEvent(GoldUiEvent.GoBuy)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        getViewModel().sendEvent(GoldUiEvent.SelPos(position))
        getViewModel().sendEvent(GoldUiEvent.GoBuy)
    }

}