package com.at.video.member.ui.gold

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.member.entity.ComboInfo
import com.at.video.member.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/8 下午8:40
 */
data class GoldUiState(
    val msg: String = "",
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: MutableList<ComboInfo> = mutableListOf(),
    val pos: Int=-1
) : IUiState

sealed class GoldUiEvent : IUiEvent {
    object InitLoadData : GoldUiEvent()
    object RefreshData : GoldUiEvent()
    object LoadDataMore : GoldUiEvent()
    data class SelPos(val pos: Int) : GoldUiEvent()
    object GoBuy:GoldUiEvent()
}

sealed class GoldUiEffect : IUiEffect{
    data class RefreshPos(val pos: Int):GoldUiEffect()
}

class GoldViewModel(application: Application) :
    BaseAndroidViewModel<GoldUiState, GoldUiEvent, GoldUiEffect>(application) {
    private var mOffset: Int = 0
    private val mLimit: Int = 10

    override fun handleEvent(event: GoldUiEvent) {
        when (event) {
            is GoldUiEvent.InitLoadData -> initData()
            is GoldUiEvent.RefreshData -> refreshData()
            is GoldUiEvent.LoadDataMore -> loadData()
            is GoldUiEvent.SelPos -> selPos(event.pos)
            is GoldUiEvent.GoBuy->{goBuy()}
        }
    }

    //购买
    private fun goBuy() {
        val pos = state.value.pos
        if (pos==-1){
            ToastUtils.show("请选择购买套餐")
        }else{
            // val comboInfo = state.value.data[pos]
            // ToastUtils.show(comboInfo.des+"充值入口")
            ToastUtils.show("功能暂未开放!")
            // TODO: 预留充值接口
        }
    }

    private fun selPos(pos: Int) {
        val data = state.value.data
        val pos1 = state.value.pos
        if (data.size < pos || data.size < pos1) {
            return
        }

        if (pos1 != -1) {
            data[pos1].isSelected = false
        }
        if (pos1 != pos) {
            data[pos].isSelected = true
        }

        setState {
            copy(data = data)
        }
        //相等
        if (pos1!=-1){
            setEffect {
                GoldUiEffect.RefreshPos(pos1)
            }
        }
        setState { copy(pos=if (pos1==pos){ -1 }else{ pos }) }
        setEffect {
            GoldUiEffect.RefreshPos(pos)
        }
    }

    private fun loadData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = "2"
            map["offset"] = mOffset
            map["limit"] = mLimit

            setState {
                            copy(
                                data = mutableListOf(
                                   ComboInfo(
                                       createdAt= 1655801251,
                                            days= 0,
                                            des= "描述1",
                                            enable= 1,
                                            exchange= 2,
                                            goldNumber= 9999,
                                            id= 17,
                                            index= 14,
                                            name= "金币套餐1",
                                            originalPrice= 19900,
                                            price= 10000,
                                            type=2,
                                            updatedAt=1656573301
                                   ) ,
                                    ComboInfo(
                                        createdAt= 1655801319,
                                        days= 0,
                                        des= "描述21",
                                        enable= 1,
                                        exchange= 2,
                                        goldNumber= 999,
                                        id= 18,
                                        index= 15,
                                        name= "金币套餐2",
                                        originalPrice= 2000,
                                        price= 10000,
                                        type=2,
                                        updatedAt=1656573304
                                    )
                                ),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                            )
                        }
//            val info = ApiFactory.getInstance().create(ApiService::class.java)
//                .getComboList(map)
//            if (info.success()) {
//                val list = info.data?.list
//                if (list.isNullOrEmpty()) {
//                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
//                } else {
//                    if (list.size < mLimit) {
//                        setState {
//                            copy(
//                                data = (data + list.toMutableList()).toMutableList(),
//                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
//                            )
//                        }
//                    } else {
//                        mOffset += mLimit
//                        setState {
//                            copy(
//                                data = (data + list.toMutableList()).toMutableList(),
//                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)
//                            )
//                        }
//                    }
//                }
//            } else {
//                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(info.msg))) }
//            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState {
                copy(
                    refreshStatus = RefreshStatus.RefreshLoading,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle,
                    pos = -1
                )
            }
            mOffset = 0
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = "2"
            map["offset"] = mOffset
            map["limit"] = mLimit
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .getComboList(map)
            if (info.success()) {
                val list = info.data?.list
                if (list.isNullOrEmpty()) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            refreshStatus = RefreshStatus.RefreshSuccess, data = mutableListOf(),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                } else {
                    if (list.size < mLimit) {
                        setState {
                            copy(
                                refreshStatus = RefreshStatus.RefreshSuccess,
                                data = list.toMutableList(),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                            )
                        }
                    } else {
                        mOffset += mLimit
                        setState {
                            copy(
                                refreshStatus = RefreshStatus.RefreshSuccess,
                                data = list.toMutableList(),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)
                            )
                        }
                    }
                }
            } else {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(info.msg))) }
            }
        }
    }

    private fun initData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(pageStatus = PageStatus.Loading) }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = "2"
            map["offset"] = mOffset
            map["limit"] = mLimit
            setState {
                copy(
                    data = mutableListOf(
                        ComboInfo(
                            createdAt= 1655801251,
                            days= 0,
                            des= "描述1",
                            enable= 1,
                            exchange= 2,
                            goldNumber= 9999,
                            id= 17,
                            index= 14,
                            name= "金币套餐1",
                            originalPrice= 19900,
                            price= 10000,
                            type=2,
                            updatedAt=1656573301
                        ) ,
                        ComboInfo(
                            createdAt= 1655801319,
                            days= 0,
                            des= "描述21",
                            enable= 1,
                            exchange= 2,
                            goldNumber= 999,
                            id= 18,
                            index= 15,
                            name= "金币套餐2",
                            originalPrice= 2000,
                            price= 10000,
                            type=2,
                            updatedAt=1656573304
                        )
                    ),
                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                )
            }
//            val info = ApiFactory.getInstance().create(ApiService::class.java)
//                .getComboList(map)
//            if (info.success()) {
//                val list = info.data?.list
//                if (list.isNullOrEmpty()) {
//                    setState { copy(pageStatus = PageStatus.Empty) }
//                } else {
//                    if (list.size < mLimit) {
//                        setState {
//                            copy(
//                                pageStatus = PageStatus.Success, data = list.toMutableList(),
//                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
//                            )
//                        }
//                    } else {
//                        mOffset += mLimit
//                        setState {
//                            copy(
//                                pageStatus = PageStatus.Success, data = list.toMutableList(),
//                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)
//                            )
//                        }
//                    }
//                    //默认选择第一项
//                    sendEvent(GoldUiEvent.SelPos(0))
//                }
//            } else {
//                setState { copy(pageStatus = PageStatus.Error(Error(info.msg))) }
//            }
        }
    }

    override fun providerInitialState(): GoldUiState {
        return GoldUiState()
    }
}