package com.at.video.member.ui.member

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.R
import com.at.video.member.databinding.MemberLayoutFragmentBinding
import com.at.video.member.entity.ComboInfo
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.decoration.DefaultDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/27 13:11
 * @Description: 会员页面
 */

@AndroidEntryPoint
@Route(path = "/member/MemberFragment")
class MemberFragment : BaseMviFragment<MemberLayoutFragmentBinding, MemberViewModel>(),
    OnItemChildClickListener, OnRefreshLoadMoreListener,View.OnClickListener {

    private val mAdapter: MemberAdapter = MemberAdapter()

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): MemberLayoutFragmentBinding {
        return MemberLayoutFragmentBinding.inflate(inflater, container, false)
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<MemberViewModel> {
        return MemberViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        val type: String? = arguments?.getString(Constants.KEY_DATA)
        getViewModel().sendEvent(MemberUiEvent.Type(type))
        requireTitleBar().apply {
            setBackgroundColor(Color.parseColor("#222222"))
            setTitleColor(Color.WHITE)
            if (type != null) {
                setTitle("开通会员")
                addBackButton(Color.BLACK)
            } else {
                setTitle("会员专区")
            }
        }
        requireBinding().apply {

                srlRefresh.setOnRefreshLoadMoreListener(this@MemberFragment)
                srlRefresh.setEnableRefresh(false)
                srlRefresh.setEnableLoadMore(false)
                rvList.layoutManager = GridLayoutManager(requireContext(),2)
                rvList.itemAnimator = null
                rvList.setHasFixedSize(true)
                val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
                rvList.addItemDecoration(
                    DefaultDecoration.create(
                        padding, padding, padding,
                        padding,padding
                    )
                )
                rvList.adapter = mAdapter
        }
        mAdapter.setDiffCallback(DiffMemberCallback())
        mAdapter.setOnItemChildClickListener(this)
//        vipStyle()
    }

    private fun vipStyle(){
        requireBinding().apply {
            tvLogin.setTextColor(Color.BLACK)
            tvVipDes.setTextColor(Color.BLACK)
            sbBuyVip.text = "立即续费"
//            llMemberInfo.background = resources.getDrawable(R.drawable.member_login_bg)
        }
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MemberUiState::pageStatus) {
                requireBinding().apply {
                    when(it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING

                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                .text = it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            srlRefresh.setEnableLoadMore(true)
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MemberUiState::refreshStatus) {
                requireBinding().apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MemberUiState::loadMoreStatus) {
                requireBinding().apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MemberUiState::data) {
                mAdapter.setDiffNewData(it)
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(MemberUiEvent.InitLoadData)
    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        Router.getInstance().build("/member/MemberPayActivity")
            .withParcelable(Constants.KEY_DATA, adapter.getItem(position) as ComboInfo).navigation()
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MemberUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MemberUiEvent.LoadDataMore)
    }

    override fun onClick(v: View) {
        when(v.id){
            R.id.tv_login->{
                Router.getInstance().build("/mine/LoginActivity").navigation()
            }
            R.id.div_header -> {
                Router.getInstance().build("/mine/UserInfoActivity").navigation()
            }
            R.id.sb_buy_vip ->{
                Router.getInstance().build("/member/RechargeActivity").navigation()
            }
        }
    }
}