package com.at.video.member.ui.member

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.entity.ComboInfo
import com.at.video.member.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/4/27 13:13
 * @Description:
 */

data class MemberUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: MutableList<ComboInfo> = mutableListOf()
): IUiState

sealed class MemberUiEvent: IUiEvent {
    data class Type(val type: String?): MemberUiEvent()
    object InitLoadData: MemberUiEvent()
    object RefreshData: MemberUiEvent()
    object LoadDataMore: MemberUiEvent()
}

sealed class MemberUiEffect: IUiEffect {

}

class MemberViewModel(application: Application):
    BaseAndroidViewModel<MemberUiState, MemberUiEvent, MemberUiEffect>(application) {
    private var mType: String? = null
    private var mOffset: Int = 0
    private val mLimit: Int = 9

    override fun providerInitialState(): MemberUiState {
        return MemberUiState()
    }

    override fun handleEvent(event: MemberUiEvent) {
        when(event) {
            is MemberUiEvent.Type -> mType = event.type
            is MemberUiEvent.InitLoadData -> initLoadData()
            is MemberUiEvent.RefreshData -> refreshData()
            is MemberUiEvent.LoadDataMore -> loadDataMore()
        }
    }

    private fun initLoadData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(pageStatus = PageStatus.Loading) }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = mType ?: "1"
            map["offset"] = mOffset
            map["limit"] = mLimit
//            val info = ApiFactory.getInstance().create(ApiService::class.java)
//                .getComboList(map)
//            if(info.success()) {
//                val list = info.data?.list
//                if(list.isNullOrEmpty()) {
//                    setState { copy(pageStatus = PageStatus.Empty) }
//                } else {
//                    if(list.size < mLimit) {
//                        setState { copy(pageStatus = PageStatus.Success, data = list.toMutableList(),
//                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
//                    } else {
//                        mOffset += mLimit
//                        setState { copy(pageStatus = PageStatus.Success, data = list.toMutableList(),
//                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
//                    }
//                }
//            } else {
//                setState { copy(pageStatus = PageStatus.Error(Error(info.msg))) }
//            }
            val list = arrayListOf<ComboInfo>(
                ComboInfo( createdAt = 1655280199,
                            days= 365,
                            des= "会员期：365天",
                            enable= 1,
                            exchange= 1,
                            goldNumber= 9999,
                            id= 1,
                            index= 1,
                            name= "年度会员",
                            originalPrice= 15000,
                            price= 10000,
                            type= 1,
                            updatedAt=1655801564),
                ComboInfo(
                    createdAt= 1655448903,
                        days= 90,
                        des= "会员期：90天",
                        enable= 1,
                        exchange= 1,
                        goldNumber= 999,
                        id=12,
                        index= 10,
                        name= "季度会员",
                        originalPrice= 9900,
                        price= 5000,
                        type= 1,
                        updatedAt= 1655801583
                )    ,
                ComboInfo(
                    createdAt= 1655448903,
                        days= 30,
                        des= "会员期：30天",
                        enable= 1,
                        exchange= 2,
                        goldNumber= 0,
                        id=12,
                        index= 10,
                        name= "月度会员",
                        originalPrice=5000,
                        price= 3000,
                        type= 1,
                        updatedAt= 1655801583
                )
            )
            setState { copy(pageStatus = PageStatus.Success, data = list.toMutableList(),
                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
        }
    }

    private fun refreshData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
                loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
            mOffset = 0
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = mType ?: "1"
            map["offset"] = mOffset
            map["limit"] = mLimit
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .getComboList(map)
            if(info.success()) {
                val list = info.data?.list
                if(list.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty,
                        refreshStatus = RefreshStatus.RefreshSuccess, data = mutableListOf(),
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    if(list.size < mLimit) {
                        setState { copy(refreshStatus = RefreshStatus.RefreshSuccess,
                            data = list.toMutableList(),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        mOffset += mLimit
                        setState { copy(refreshStatus = RefreshStatus.RefreshSuccess,
                            data = list.toMutableList(),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
                    }
                }
            } else {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(info.msg))) }
            }
        }
    }

    private fun loadDataMore() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["enable"] = 1
            map["type"] = mType ?: "1"
            map["offset"] = mOffset
            map["limit"] = mLimit
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .getComboList(map)
            if(info.success()) {
                val list = info.data?.list
                if(list.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    if(list.size < mLimit) {
                        setState { copy(data = (data + list.toMutableList()).toMutableList(),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        mOffset += mLimit
                        setState { copy(data = (data + list.toMutableList()).toMutableList(),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
                    }
                }
            } else {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(info.msg))) }
            }
        }
    }
}