package com.at.video.member.ui.pay

import android.app.Dialog
import android.graphics.Color
import android.os.Bundle
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.setLightStatusBar
import com.at.common.utils.ToastUtils
import com.at.video.member.R
import com.at.video.member.databinding.MemberActivityPayBinding
import com.at.video.member.entity.ComboInfo
import com.at.video.member.ui.MemberDialog
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import kotlinx.coroutines.flow.collect

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/10 下午4:47
 */
@Route(path = "/member/MemberPayActivity")
class MemberPayActivity : BaseMviActivity<MemberActivityPayBinding, MemberPayViewModel>(),
    View.OnClickListener {

    private lateinit var mComboInfo: ComboInfo

    override fun createViewBinding(): MemberActivityPayBinding {
        return MemberActivityPayBinding.inflate(layoutInflater)
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        mComboInfo = intent.getParcelableExtra(Constants.KEY_DATA)!!
        getViewModel().sendEvent(MemPayUiEvent.Combo(mComboInfo))
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            setTitle("支付")
            addBackButton(Color.BLACK)
        }
        requireBinding().apply {
            tvName.text = mComboInfo.name
            tvPrice.text = getFloatNoMoreThanTwoDigits(mComboInfo.price / 100f)
            tvGoldNum.visibility = if (mComboInfo.exchange!= 1) View.GONE else View.VISIBLE
            smbGoldEx.visibility = if (mComboInfo.exchange!= 1) View.GONE else View.VISIBLE
            tvGoldNum.text = "可以支付${mComboInfo.goldNumber}金币直接兑换"
            smbGoldEx.setOnClickListener(this@MemberPayActivity)
            stvBuyNow.setOnClickListener(this@MemberPayActivity)
        }
    }
    private fun getFloatNoMoreThanTwoDigits(number: Float): String {
        return String.format("%.2f",number)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is MemPayUiEffect.ByGoldState -> {
                        if (!it.state) {//失败
                            showNoDialog()
                        } else {//购买成功
                            ToastUtils.show("购买成功")
                        }
                    }
                    is MemPayUiEffect.Toast -> {
                        ToastUtils.show(it.toast)
                    }
                }
            }
        }
    }

    //展示dialog
    private var dialog: Dialog? = null
    private fun showNoDialog() {
        if (dialog == null) {
            dialog= MemberDialog.createNoGoldDialog(this, object : MemberDialog.BuyGoldListener {
                override fun buyGold() {
                    dialog?.dismiss()
                    Router.getInstance().build("/member/GoldActivity").navigation()
                }
            })
        } else {
            dialog?.show()
        }
    }

    override fun loadData() {

    }

    override fun getViewModelClass(): Class<MemberPayViewModel> = MemberPayViewModel::class.java
    override fun onClick(p0: View) {
        when (p0.id) {
            R.id.smb_goldEx -> {
                getViewModel().sendEvent(MemPayUiEvent.GoldExchange)
            }
            R.id.stv_buyNow -> {
                getViewModel().sendEvent(MemPayUiEvent.Buy)
            }
        }
    }
}