package com.at.video.member.ui.pay

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.utils.ToastUtils
import com.at.video.export.mine.IMineService
import com.at.video.member.entity.ComboInfo
import com.at.video.member.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email: 兑换
 * @date :   2022/6/9 下午5:17
 */
data class MemPayUiState(val msg: String = "") : IUiState
sealed class MemPayUiEvent : IUiEvent {
    data class Combo(val comboInfo: ComboInfo) : MemPayUiEvent()
    object GoldExchange : MemPayUiEvent()
    object Buy : MemPayUiEvent()
}

sealed class MemPayUiEffect : IUiEffect {
    data class ByGoldState(val state: Boolean = false) : MemPayUiEffect()
    data class Toast(val toast: String = "") : MemPayUiEffect()
}

class MemberPayViewModel(application: Application) :
    BaseAndroidViewModel<MemPayUiState, MemPayUiEvent, MemPayUiEffect>(application) {
    private var combo: ComboInfo? = null
    override fun handleEvent(event: MemPayUiEvent) {
        when (event) {
            is MemPayUiEvent.Combo -> combo = event.comboInfo
            is MemPayUiEvent.GoldExchange -> goldExchange()
            is MemPayUiEvent.Buy -> goBuy()
        }
    }

    //直接购买
    private fun goBuy() {
        ToastUtils.show("功能暂未开放!")
    }

    //金币兑换
    private fun goldExchange() {
        if (combo == null) {
            return
        }
        viewModelScope.launch {
            val info =
                ApiFactory.getInstance().create(ApiService::class.java).buy(combo!!.id)
            if (info.success()) {
                //更新数据
                // info.data?.apply {
                //     IMineService.getMineService().getUser().apply {
                //
                //     }
                // }
                setEffect {
                    MemPayUiEffect.ByGoldState(true)
                }
            } else {
                // setEffect {
                //     MemPayUiEffect.Toast(info.msg ?: "购买失败")
                // }
                setEffect {
                    MemPayUiEffect.ByGoldState(false)
                }
            }
        }
    }

    override fun providerInitialState(): MemPayUiState {
        return MemPayUiState()
    }
}