package com.at.video.mine.http

import com.at.common.network.BaseBean
import com.at.common.network.PageBean
import com.at.video.mine.entity.*
import com.melancholy.network.OKHttp
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.BaseResponse
import com.video.module_widget.entity.UploadData
import okhttp3.MultipartBody
import okhttp3.ResponseBody
import retrofit2.http.*

/**
 * @Create_time: 2022/5/19 09:47
 * @Description:
 */

interface ApiService {


    //获取用户信息
    @GET("users/{id}")
    suspend fun userinfo(@Path("id") id: Long): BaseBean<UserInfo>


    //重置密码
    @POST("v1/user/reset_password")
    suspend fun resetPassword(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse


    //观看历史
    @GET("users/history")
    suspend fun getHistory(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): BaseBean<HisInfo>

    //删除更多
    @HTTP(method = "DELETE", path = "users/dynamic/batch", hasBody = true)
    suspend fun deleteMore(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<Any>

    @GET("home/version/app/{id}")
    suspend fun getVersionInfo(@Path("id") appId: String): BaseBean<AppVersionInfo>

    // 获取应用信息
    @GET("v1/user/application")
    suspend fun getAppInfo(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): AppInfo

    //获取最新消息
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/message/latest_msg")
    suspend fun getLatestMessage(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): BaseBean<NewMessageInfo>

    //下载文件
    @Streaming
    @GET
    suspend fun download(@Url url: String): ResponseBody

    //访客登录
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/login")
    suspend fun serviceLogin(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<MessageInfo>

    //获取历史会话
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/message/unread")
    suspend fun getMsgCount(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): BaseBean<MessageCount>

    // 新接口

    // 获取随机账号
    @GET("v1/user/random_account")
    suspend fun getUserAccount(): RegisterData

    //注册账号
    @POST("v1/user/create")
    suspend fun register(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //校验输入数据
    @GET("v1/user/check_format")
    suspend fun checkInputData(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //登陆账号
    @POST("v1/user/login")
    suspend fun login(@Body map: @JvmSuppressWildcards Map<String, Any>): UserLoginInfo

    //登出
    @POST("v1/user/logout")
    suspend fun logout(
        @Header("Authorization") token: String,
        @Body map: @JvmSuppressWildcards Map<String, Any>
    ): BaseResponse

//    //是否存在邮箱配置
//    @POST
//    suspend fun isEmailConfig(@Url url:String = "${GlobalConsts.CMS_URL}/v1/email_config/existed",@Body map: @JvmSuppressWildcards Map<String, Any>):EmailConfig

    //发送验证码
    @POST("v1/code/send")
    suspend fun sendCode(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //校验邮箱验证码
    @POST("v1/code/verify")
    suspend fun validation(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //修改用户信息
    @POST("v1/user/update")
    suspend fun modifyUser(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //上传文件
    @Multipart
    @POST("v1/file/upload")
    suspend fun updateFile(@Part part: MultipartBody.Part): UploadData

    //获取地址区域
    @GET("v1/geo")
    suspend fun getAddressArea(): CityInfo

    //获取地址区域
    @GET("v1/country_code")
    suspend fun getCountryCode(): CountryCodeInfo

    //获取用户信息
    @GET("v1/user")
    suspend fun userinfo(): UserLoginInfo

    @GET("v1/user/collect_record/query")
    suspend fun collectList(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): CollectList

    //删除观看历史
    @POST("v1/user/watch_history/delete")
    suspend fun deleteWatchHis(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseResponse

    //获取观看历史
    @GET("v1/user/watch_history/query")
    suspend fun getWatchHisList(@QueryMap map: @JvmSuppressWildcards Map<String, Any>): HistoryInfo
}