package com.at.video.mine.http.interceptor

import android.app.Activity
import android.app.ActivityManager
import android.app.Application
import android.content.ComponentName
import android.content.Context.ACTIVITY_SERVICE
import android.content.Intent
import com.at.common.global.Constants
import com.at.common.utils.ToastUtils
import com.at.video.export.mine.IToken
import com.at.video.mine.entity.User
import com.at.video.mine.manager.UserManager
import com.at.video.mine.ui.login.LoginActivity
import com.melancholy.network.INetworkInterceptor
import com.melancholy.register.annotation.RegisterService
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.video.module_widget.constants.GlobalConsts
import okhttp3.*
import org.json.JSONObject

/**
 * @Create_time: 2022/5/26 21:14
 * @Description: 判断token是否失效，如果失效就刷新token拦截器
 */

@RegisterService(serviceClass = INetworkInterceptor::class)
class CheckTokenInvalidInterceptor: INetworkInterceptor {

    override fun priority(): Int  = 2

    override fun intercept(chain: Interceptor.Chain): Response {
        val request: Request = chain.request()
        val response: Response = chain.proceed(request)
        //判断是否是token失效了
        if((response.networkResponse?.code ?: 200) == 401) {
            if(!isExistActivity(LoginActivity::class.java)) {
                Router.getInstance().build("/mine/LoginActivity").navigation()
            }
            return response
            //双重检验token是否存在，如果不存在，不进行处理了，应该是跳转登录页了
//            val token: String? = UserManager.instance.tokenFlow.value?.getUserToken()
//            if(token == null) {
//                if(!isExistActivity(LoginActivity::class.java)) {
//                    Router.getInstance().build("/mine/LoginActivity").navigation()
//                }
//                return response
//            } else {
//                //在这里可能token刷新成功了，直接使用当前的token调用当前接口，也有可能token刷新失败了，需要重新刷新token操作
//                if(token != request.header("Authorization")) {
//                    return retryRequest(chain, request, token)
//                }
//            }
//            synchronized(CheckTokenInvalidInterceptor::class.java) {
//                //刷新token,如果token刷新还是无效了，直接跳转登录页面
//                //刷新token成功，保存token信息，重新请求上次请求的token失效的接口
//                val iToken: IToken? = UserManager.instance.tokenFlow.value
//                if(iToken == null) {
//                    if(!isExistActivity(LoginActivity::class.java)) {
//                        Router.getInstance().build("/mine/LoginActivity").navigation()
//                    }
//                    return response
//                } else {
//                    //在这里可能token刷新成功了，直接使用当前的token调用当前接口，也有可能token刷新失败了，需要重新刷新token操作
//                    if(iToken.getUserToken() != request.header("Authorization")) {
//                        return retryRequest(chain, request, iToken.getUserToken())
//                    }
//                }

//                if(UserManager.instance.userFlow.value == null){ // 登录信息不存在，直接到登录界面
//                    if(!isExistActivity(LoginActivity::class.java)) {
//                        Router.getInstance().build("/mine/LoginActivity").navigation()
//                    }
//                    return response
//                }else{
//                    val requestBuilder: Request.Builder = Request.Builder()
//                        .url("${GlobalConsts.CMS_URL}/users/login")
//
//                    val requestBody: FormBody = FormBody.Builder()
//                        .add("login_source","2")
//                        .add("belong_app_id",Constants.APP_ID)
//                        .add("login_type","1")
//                        .add("username", UserManager.instance.userFlow.value!!.getUserName())
//                        .add("password",UserManager.instance.userFlow.value!!.getPassword()).build()
//                    requestBuilder.post(requestBody)
//                    val tokenResponse: Response = chain.proceed(requestBuilder.build())
//                    when {
//                        tokenResponse.code == 401 -> {
//                            //刷新token也失效了，重新登了
//                            UserManager.instance.clearInfo()
//                            ToastUtils.show("登录已退出")
//                            Router.getInstance().build("/main/MainActivity").navigation()
//                            Router.getInstance().build("/mine/LoginActivity").navigation()
//                            return tokenResponse
//                        }
//                        tokenResponse.isSuccessful -> {
//                            //解析数据，保存token
//                            val json = JSONObject(tokenResponse.body!!.string())
//                            val data: JSONObject = json.optJSONObject("data")!!
//                            val newToken: String = data.optString("token")
////                        val tokenRefresh: String = data.optString("tokenRefresh")
//                            UserManager.instance.saveToken(newToken)
//                            //重新请求接口
//                            return retryRequest(chain, request, newToken)
//                        }
//                        else -> {
//                            //可能是网络错误或者其他错误
//                            return tokenResponse
//                        }
//                    }
//                }
//            }
        }
        return response
    }

    /**
     * 判断某个Activity是否已经启动
     */
    @Suppress("DEPRECATION")
    private fun isExistActivity(activity: Class<out Activity>): Boolean {
        val application: Application = AppInstance.getApplication()
        val intent = Intent(application, activity)
        val cmpName: ComponentName? = intent.resolveActivity(application.packageManager)
        if(cmpName != null) {
            // 说明系统中存在这个activity
            val am: ActivityManager = application.getSystemService(ACTIVITY_SERVICE)
                    as ActivityManager
            // 这里获取的是APP栈的数量，一般也就两个
            val taskInfoList: List<ActivityManager.RunningTaskInfo> = am.getRunningTasks(10)
            // 只是拿当前运行的栈
            //val runningTaskInfo: ActivityManager.RunningTaskInfo = taskInfoList[0]
            //val numActivities: Int = taskInfoList[0].numActivities
            for (taskInfo: ActivityManager.RunningTaskInfo in taskInfoList) {
                if (taskInfo.baseActivity?.equals(cmpName) == true) {
                    // 说明它已经启动了
                    return true
                }
            }
        }
        return false
    }

    /**
     * 重新请求
     * @param chain
     * @param request
     * @param token
     */
    private fun retryRequest(chain: Interceptor.Chain, request: Request, token: String): Response {
        return chain.proceed(request.newBuilder().removeHeader("Authorization")
            .addHeader("Authorization", token)
            .build())
    }
}