package com.at.video.mine.manager

import com.at.common.global.CacheHelper
import com.at.common.utils.JsonUtils
import com.at.video.export.mine.IToken
import com.at.video.export.mine.IUser
import com.at.video.mine.entity.UserLoginInfo
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/20 13:42
 * @Description: 用户信息管理
 */

class UserManager {
    @Volatile
    private var mInfo: UserLoginInfo? = null
    private val _userFlow: MutableStateFlow<IUser?> = MutableStateFlow(null)
    val userFlow: StateFlow<IUser?> = _userFlow.asStateFlow()
    private val _tokenFlow: MutableStateFlow<IToken?> = MutableStateFlow(null)
    val tokenFlow: StateFlow<IToken?> = _tokenFlow.asStateFlow()

    companion object {
        private const val KEY_LOGIN_INFO: String = "key_login_info"

        @JvmStatic
        val instance: UserManager by lazy(LazyThreadSafetyMode.SYNCHRONIZED) { UserManager() }
    }

    @Synchronized
    fun saveToken(token: String) {
        if (mInfo != null) {
            mInfo = mInfo!!.copy(session = token)
            val json: String = JsonUtils.toJson(mInfo!!)
            CacheHelper.getInstance().putString(KEY_LOGIN_INFO, json)
            _userFlow.value = mInfo
            _tokenFlow.value = mInfo
        }
    }

    /**
     * 保存用户信息
     */
    @Synchronized
    fun saveInfo(info: UserLoginInfo) {
        mInfo = info
        val json: String = JsonUtils.toJson(info)
        CacheHelper.getInstance().putString(KEY_LOGIN_INFO, json)
        _userFlow.value = info
        _tokenFlow.value = info
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUser(info: UserLoginInfo) {
        mInfo = mInfo?.copy(user_additional = info.user_additional, user = info.user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserNickName(name: String) {
        val user = mInfo?.user?.copy(nickname = name)
        mInfo = mInfo?.copy(user = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     * 更新用户信息
     */
    @Synchronized
    fun updateUserPhone(phone: String, phoneCode: String) {
        val user = mInfo?.user?.copy(phone_number = phone, mobile_country_code = phoneCode)
        mInfo = mInfo?.copy(user = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserEmail(email: String) {
        val user = mInfo?.user?.copy(email = email)
        mInfo = mInfo?.copy(user = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserGender(gender: Int) {
        val user = mInfo?.user_additional?.copy(gender = gender)
        mInfo = mInfo?.copy(user_additional = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserBirthday(birthday: String) {
        val user = mInfo?.user_additional?.copy(birthday = birthday)
        mInfo = mInfo?.copy(user_additional = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserAddr(addr: String) {
        val user = mInfo?.user_additional?.copy(location = addr)
        mInfo = mInfo?.copy(user_additional = user)
        saveInfo()
    }

    /**
     * 更新用户信息
     */
    @Synchronized
    fun updateUserAvatar(avatar: String) {
        val user = mInfo?.user?.copy(avatar = avatar)
        mInfo = mInfo?.copy(user = user)
        saveInfo()
    }

    private fun saveInfo() {
        val json: String = JsonUtils.toJson(mInfo)
        CacheHelper.getInstance().putString(KEY_LOGIN_INFO, json)
        _userFlow.value = mInfo
    }

    /**
     * 加载用户信息
     */
    @Synchronized
    fun loadInfo() {
        val json: String? = CacheHelper.getInstance().getString(KEY_LOGIN_INFO, null)
        if (!json.isNullOrEmpty()) {
            mInfo = JsonUtils.fromJson(json)
            if (mInfo != null) {
                _userFlow.value = mInfo
                _tokenFlow.value = mInfo
//                HeartbeatService.start()
            } else {
                CacheHelper.getInstance().remove(KEY_LOGIN_INFO)
            }
        }
    }

    /**
     * 清除用户信息
     */
    @Synchronized
    fun clearInfo() {
        GlobalScope.launch(Dispatchers.IO) {
            if (mInfo != null) {
//                HeartbeatService.stop()
                val token: String = mInfo!!.getUserToken()
                CacheHelper.getInstance().remove(KEY_LOGIN_INFO)
                mInfo = null
                _userFlow.value = null
                _tokenFlow.value = null
                // 调用接口退出登录
                ApiFactory.getInstance().create(ApiService::class.java).logout(token = token, map = mapOf())
            }
        }
    }

    fun clearToken() {
        _tokenFlow.value = null
    }
}
