package com.at.video.mine.ui.collection

import android.graphics.Color
import android.graphics.Typeface
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.os.bundleOf
import androidx.lifecycle.lifecycleScope
import com.at.common.adapter.FragmentAdapter
import com.at.common.adapter.PagerData
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.video.mine.databinding.MineActivityCollectBinding
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.ext.buildIndicator
import com.loper7.tab_expand.ext.buildText
import com.loper7.tab_expand.indicator.LinearIndicator
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.tabs.TabLayoutMediators
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor
import com.video.module_widget.widget.CustomTabText
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/9 上午9:52
 */
@AndroidEntryPoint
//@Route(path = "/mine/CollectActivity")
@Route(path = "/mine/CollectActivity", extras = 10086)
class CollectActivity : BaseMviActivity<MineActivityCollectBinding, CollectViewModel>(),
    TabLayoutMediators.TabConfigurationStrategy, View.OnClickListener {
    private lateinit var mEditText: TextView
    private val tabTitle: List<String> = listOf("影视", "文章", "图集")
    private var mMediator: TabLayoutMediators? = null

    override fun createViewBinding(): MineActivityCollectBinding {
        return MineActivityCollectBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<CollectViewModel> {
        return CollectViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("我的收藏")
            mEditText = addRightTextButton("编辑", android.R.id.edit)
            mEditText.textSize = DensityUtils.px2sp(
                this@CollectActivity, resources.getDimension(com.at.common.R.dimen.sp_16).toInt()
            )
            mEditText.typeface = Typeface.DEFAULT
            mEditText.setOnClickListener(this@CollectActivity)
        }
        requireBinding().apply {
            val params: ViewGroup.LayoutParams = tbTab.layoutParams
            params.height = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_40)
            tbTab.layoutParams = params
            tbTab.tabMode = TabLayout.MODE_SCROLLABLE
            tbTab.buildIndicator<LinearIndicator>()
                .setHeight(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_2))
                .setWidth(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20))
                .setColor(StyleUtils.getPrimaryColor().toArgb()).bind()
            tbTab.buildText<CustomTabText>().setTabColors(
                    mutableListOf("#666666".toColor()),
                    mutableListOf(StyleUtils.getPrimaryColor().toArgb())
                ).setTabNormalSizes(
                    mutableListOf(
                        DensityUtils.px2sp(
                            this@CollectActivity,
                            resources.getDimensionPixelSize(com.at.common.R.dimen.sp_16)
                        )
                    )
                ).setTabSelectedSizes(
                    mutableListOf(
                        DensityUtils.px2sp(
                            this@CollectActivity,
                            resources.getDimensionPixelSize(com.at.common.R.dimen.sp_16)
                        )
                    )
                ).setNormalTextBold(false).setSelectTextBold(true).bind()
            val adapter = FragmentAdapter(supportFragmentManager, lifecycle)
            vpViewPager.adapter = adapter
            mMediator = TabLayoutMediators(
                tbTab, vpViewPager, this@CollectActivity, true
            )
            mMediator!!.attach()
            adapter.add(
                PagerData(
                    "/mine/MoviesFragment", 0L, bundleOf(Constants.KEY_DATA to 1)
                )
            )
            adapter.add(
                PagerData(
                    "/mine/OtherFragment", 1L, bundleOf(Constants.KEY_DATA to 3)
                )
            )
            adapter.add(
                PagerData(
                    "/mine/OtherFragment", 2L, bundleOf(Constants.KEY_DATA to 2)
                )
            )
            vpViewPager.offscreenPageLimit = adapter.itemCount
        }
    }

    override fun onConfigureTab(tab: TabLayout.Tab, position: Int) {
        tab.text = tabTitle[position]
    }

    override fun loadData() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(CollectUiState::isEdit) {
                if (it) {
                    mEditText.text = "取消"
                    requireBinding().apply {
                        vpViewPager.isUserInputEnabled = false
                        tbTab.visibility = View.GONE
                    }
                } else {
                    mEditText.text = "编辑"
                    requireBinding().apply {
                        vpViewPager.isUserInputEnabled = true
                        tbTab.visibility = View.VISIBLE
                    }
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            android.R.id.edit -> getViewModel().sendEvent(CollectUiEvent.Edit)
        }
    }
}