package com.at.video.mine.ui.collection

import android.app.Application
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.ui.history.HistoryUiEffect
import com.at.video.mine.ui.history.HistoryUiEvent
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email: 可以编辑
 * @date :   2022/5/9 上午10:00
 */
data class CollectUiState(
    val isEdit: Boolean = false
): IUiState

sealed class CollectUiEvent: IUiEvent {
    object Edit: CollectUiEvent()
    object SelectAll: CollectUiEvent()
    object Delete: CollectUiEvent()
    object CancelEdit:CollectUiEvent()
}

sealed class CollectUiEffect: IUiEffect {
    object SelectAll: CollectUiEffect()
    object Delete: CollectUiEffect()
}

@HiltViewModel
class CollectViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<CollectUiState, CollectUiEvent, CollectUiEffect>(application) {

    override fun providerInitialState(): CollectUiState {
        return CollectUiState()
    }

    override fun handleEvent(event: CollectUiEvent) {
        when(event) {
            is CollectUiEvent.Edit -> setState { copy(isEdit = !isEdit) }
            is CollectUiEvent.SelectAll -> setEffect { CollectUiEffect.SelectAll }
            is CollectUiEvent.Delete -> setEffect { CollectUiEffect.Delete }
            is CollectUiEvent.CancelEdit ->
                if(state.value.isEdit){
                    setState { copy(isEdit = !isEdit)  }
            }
        }
    }
}