package com.at.video.mine.ui.collection.movies

import android.graphics.drawable.LayerDrawable
import android.graphics.drawable.StateListDrawable
import android.widget.ImageView
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.ImageConfig
import com.at.common.global.Constants
import com.at.common.load
import com.at.video.mine.R
import com.at.video.mine.entity.CollectBean
import com.at.video.mine.entity.CollectInfo
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.utils.toArgb
import java.lang.ref.WeakReference

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/9 上午11:12
 * @style : 0 影视 1 小说 2 图集
 */

class MoviesAdapter: BaseMultiItemQuickAdapter<CollectBean, BaseViewHolder>() {
    private var mIsEdit: Boolean = false
    init {
        addItemType(1, R.layout.mine_item_movies_free)
        addItemType(2, R.layout.mine_item_movies_pay)
        addChildClickViewIds(R.id.iv_selected)
    }

    override fun convert(holder: BaseViewHolder, item: CollectBean) {
        holder.getView<ImageView>(R.id.div_image)
            .loadByGlide(
                GlideImageConfig(item.cover.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                errorId = com.video.module_widget.R.drawable.common_ic_image_video_default,),
                WeakReference(context)
            )
        holder.setText(R.id.tv_name, item.name)
//        if(item.chargingState == 2) {
//            holder.setText(R.id.smb_glod, item.chargingCoin.toString())
//        }
        holder.setGone(R.id.iv_selected, !mIsEdit)
        changeCheck(holder,item)
        holder.itemView.setOnClickListener{
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, item.resource_id.toString()).navigation()
        }
    }

    override fun convert(holder: BaseViewHolder, item: CollectBean, payloads: List<Any>) {
        holder.setGone(R.id.iv_selected, !mIsEdit)
//        holder.setImageResource(R.id.iv_selected, if(item.isSelected)
//            R.drawable.mine_ic_checkbox_selected else R.drawable.mine_ic_checkbox_normal)
        changeCheck(holder,item)
    }

    private fun changeCheck(holder: BaseViewHolder, item: CollectBean){

        val vectorDrawable = VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_selected_bg,context.theme)
        val vectorDrawable1 = VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_selected_1,context.theme)
        vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
        val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable,vectorDrawable1))
        val normalDrawable =  VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_normal,context.theme)
//        holder.setImageResource(R.id.iv_selected, if(item.isSelected)
//            R.drawable.mine_ic_checkbox_selected else R.drawable.mine_ic_checkbox_normal)
        holder.setImageDrawable(R.id.iv_selected, if(item.is_selected)
            layerDrawAble else normalDrawable)
    }

    fun setEdit(isEdit: Boolean) {
        if(mIsEdit != isEdit) {
            mIsEdit = isEdit
            notifyItemRangeChanged(0, data.size, "notify")
        }
    }
}