package com.at.video.mine.ui.collection.other

import android.graphics.drawable.LayerDrawable
import android.widget.ImageView
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.ImageConfig
import com.at.common.load
import com.at.video.mine.R
import com.at.video.mine.entity.CollectBean
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.utils.toArgb
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/28 16:19
 * @Description:
 */

class OtherAdapter constructor(private val mType: Int): BaseQuickAdapter<CollectBean,
        BaseViewHolder>(R.layout.mine_item_other) {
    private var mIsEdit: Boolean = false

    init {
        addChildClickViewIds(R.id.iv_selected)
    }

    override fun convert(holder: BaseViewHolder, item: CollectBean) {
        if(mType == 2) {
            holder.getView<ImageView>(R.id.div_image)
                .loadByGlide(
                    GlideImageConfig(item.cover.imageUrl(),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_image_novel_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_image_novel_default),
                        WeakReference(context)
                    )
        } else {
            holder.getView<ImageView>(R.id.div_image)
                .loadByGlide(GlideImageConfig(item.cover.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default),
                    WeakReference(context)
                    )
        }
        holder.setText(R.id.tv_name, item.name)
        holder.setGone(R.id.iv_selected, !mIsEdit)
        changeCheck(holder,item)
        holder.itemView.setOnClickListener {
            if (mIsEdit){
                holder.getView<ImageView>(R.id.iv_selected).performClick()
            }else{
                when (item.type) {
                    3 -> {//小说
                        Router.getInstance().build("/novel/NovelDetailActivity")
                            .withString("novelId", item.resource_id.toString()).withString("title", item.name ?:"")
                            .navigation()
                    }
                    else -> {//图集
                        Router.getInstance().build("/atlas/DetailActivity").withString("atlasId",item.resource_id.toString()).navigation()
                    }
                }
            }

        }
    }

    override fun convert(holder: BaseViewHolder, item: CollectBean, payloads: List<Any>) {
        holder.setGone(R.id.iv_selected, !mIsEdit)
       changeCheck(holder,item)
    }
    private fun changeCheck(holder: BaseViewHolder, item: CollectBean){

        val vectorDrawable = VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_selected_bg,context.theme)
        val vectorDrawable1 = VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_selected_1,context.theme)
        vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
        val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable,vectorDrawable1))
        val normalDrawable =  VectorDrawableCompat.create(context.resources,R.drawable.mine_ic_checkbox_normal,context.theme)
//        holder.setImageResource(R.id.iv_selected, if(item.isSelected)
//            R.drawable.mine_ic_checkbox_selected else R.drawable.mine_ic_checkbox_normal)
        holder.setImageDrawable(R.id.iv_selected, if(item.is_selected)
            layerDrawAble else normalDrawable)
    }


    fun setEdit(isEdit: Boolean) {
        if(mIsEdit != isEdit) {
            mIsEdit = isEdit
            notifyItemRangeChanged(0, data.size, "notify")
        }
    }
}