package com.at.video.mine.ui.email.raw

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityRawEmailBinding
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

/**
 * @author:  stone
 * @email:
 * @date :   2022/5/6 下午1:50
 * @description: 绑定邮箱或修改邮箱
 */
@AndroidEntryPoint
@Route(path = "/mine/RawEmailActivity")
class RawEmailActivity : BaseMviActivity<MineActivityRawEmailBinding, RawEmailViewModel>(),
    View.OnClickListener {

    override fun createViewBinding(): MineActivityRawEmailBinding {
        return MineActivityRawEmailBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<RawEmailViewModel> {
        return RawEmailViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("修改邮箱")
        }
        requireBinding().apply {
            tvEmail.addTextChangedListener(EmailTextWatcher())
            tvEmail.text = IMineService.getMineService().getUser().value?.getUserEmail()
            cetCode.addTextChangedListener(CodeTextWatcher())
            ttvCode.setOnClickListener(this@RawEmailActivity)
            ttvCode.setTextColor(StyleUtils.getPrimaryColor().toArgb())
            sbNext.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            sbNext.setOnClickListener(this@RawEmailActivity)
        }
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is RawEmailUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@RawEmailActivity, it.content)
                    }
                    is RawEmailUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is RawEmailUiEffect.Toast -> ToastUtils.show(it.msg)
                    is RawEmailUiEffect.CodeSuccess -> requireBinding().ttvCode.startTimer()
                    is RawEmailUiEffect.ValidationSuccess -> {
                        Router.getInstance().build("/mine/ReplaceEmailActivity")
                            .withString(Constants.KEY_DATA, it.code)
                            .withString(Constants.KEY_DATA_1, it.oldEmail)
                            .navigation(this@RawEmailActivity, 120)
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RawEmailUiState::code) {
                requireBinding().apply {
                    if (cetCode.text.toString() != it) {
                        cetCode.setText(it)
                    }
                    sbNext.isEnabled = it.isNotEmpty()
                }
            }
        }
    }

    override fun onClick(p0: View) {
        when (p0.id) {
            //获取验证码
            R.id.ttv_code -> getViewModel().sendEvent(RawEmailUiEvent.GetCode)
            //确认
            R.id.sb_next -> getViewModel().sendEvent(RawEmailUiEvent.Confirm)
        }
    }

    private inner class EmailTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(RawEmailUiEvent.Email(p0.toString()))
        }
    }

    private inner class CodeTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(RawEmailUiEvent.Code(p0.toString()))
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 120 && resultCode == RESULT_OK && data != null) {
            if (data.getBooleanExtra("status",false)){
                onBack()
            }
        }
    }
}