package com.at.video.mine.ui.email.raw

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.constants.GlobalConsts
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/6 下午1:51
 */
data class RawEmailUiState(
    val email: String = "",
    val code: String = ""
): IUiState

sealed class RawEmailUiEvent: IUiEvent {
    data class Email(val email: String): RawEmailUiEvent()
    data class Code(val code: String): RawEmailUiEvent()
    object GetCode: RawEmailUiEvent()
    object Confirm: RawEmailUiEvent()
}

sealed class RawEmailUiEffect: IUiEffect {
    data class Loading(val content: String = ""): RawEmailUiEffect()
    object Dismiss: RawEmailUiEffect()
    data class Toast(val msg: String): RawEmailUiEffect()
    object CodeSuccess: RawEmailUiEffect()
    data class ValidationSuccess(val code: String,val oldEmail:String): RawEmailUiEffect()
}


class RawEmailViewModel(application: Application): BaseAndroidViewModel<RawEmailUiState,
        RawEmailUiEvent, RawEmailUiEffect>(application) {
    var isExit = false
    override fun providerInitialState(): RawEmailUiState {
        return RawEmailUiState()
    }

    override fun handleEvent(event: RawEmailUiEvent) {
        when(event) {
            is RawEmailUiEvent.Email -> setState { copy(email = event.email) }
            is RawEmailUiEvent.Code -> setState { copy(code = event.code) }
            is RawEmailUiEvent.GetCode -> getCode()
            is RawEmailUiEvent.Confirm -> validationChange()
        }
    }

    /**
     * 获取验证码
     */
    private fun getCode() {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["email"] = state.value.email
            map["type"] = 4
            map["way"] = 1
            map["belong_app_id"] = Constants.APP_ID

            setEffect {
                RawEmailUiEffect.Loading("获取验证码中")
            }
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .sendCode(map = map)
            setEffect { RawEmailUiEffect.Dismiss }
            if(info.success()) {
                setEffect { RawEmailUiEffect.Toast("获取验证码成功") }
                setEffect { RawEmailUiEffect.CodeSuccess }
            } else {
                setEffect { RawEmailUiEffect.Toast(info.message ?: "获取验证码失败") }
            }
        }
    }

    /**
     * 邮箱变更
     */
    private fun validationChange() {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["email"] = state.value.email
            map["code"] = state.value.code
            setEffect { RawEmailUiEffect.Loading("验证中") }
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .validation(map = map)
            setEffect { RawEmailUiEffect.Dismiss }
            if(info.success()) {
                setEffect { RawEmailUiEffect.Toast("获取验证码成功") }
                setEffect { RawEmailUiEffect.ValidationSuccess(state.value.code,state.value.email) }
            } else {
                setEffect { RawEmailUiEffect.Toast(info.message ?: "获取验证码失败") }
            }
        }
    }
}