package com.at.video.mine.ui.email.replace

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.module_export_main.IMainService
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityReplaceEmailBinding
import com.melancholy.router.annotation.annotation.Route
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

/**
 * @author:  stone
 * @email:
 * @date :   2022/5/6 下午1:50
 * @description: 绑定邮箱或修改邮箱
 */
@AndroidEntryPoint
@Route(path = "/mine/ReplaceEmailActivity")
class ReplaceEmailActivity :
    BaseMviActivity<MineActivityReplaceEmailBinding, ReplaceEmailViewModel>(),
    View.OnClickListener {
    private var mOldCode: String? = null
    private var oldEmail: String? = null

    override fun createViewBinding(): MineActivityReplaceEmailBinding {
        return MineActivityReplaceEmailBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ReplaceEmailViewModel> {
        return ReplaceEmailViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
//        mOldCode = intent.getStringExtra(Constants.KEY_DATA)
//        oldEmail = intent.getStringExtra(Constants.KEY_DATA_1)
//        oldEmail?.apply {
//            getViewModel().sendEvent(ReplaceEmailUiEvent.OldEmail(this))
//        }
        val historyEmail: String? = IMineService.getMineService().getUser().value?.getUserEmail()
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            if (historyEmail.isNullOrEmpty()) {
                setTitle("绑定邮箱")
            } else {
                setTitle("修改邮箱")
//                getViewModel().sendEvent(ReplaceEmailUiEvent.OldEmail(historyEmail))
//                getViewModel().sendEvent(ReplaceEmailUiEvent.Email(historyEmail))
            }
        }
        requireBinding().apply {
            tvEmailLabel.text = "邮箱"
//            if (!historyEmail.isNullOrEmpty()) {
//                cetEmail.setText(historyEmail)
//            }
            cetEmail.addTextChangedListener(EmailTextWatcher())
            cetCode.addTextChangedListener(CodeTextWatcher())
            ttvCode.setOnClickListener(this@ReplaceEmailActivity)
            ttvCode.setTextColor(StyleUtils.getPrimaryColor().toArgb())
            sbConfirm.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            val isExit = IMainService.getMainService().getSysConfig().getIsConfigEmail()
            llCode.visibility = if (isExit) View.VISIBLE else View.GONE
            sbConfirm.setOnClickListener(this@ReplaceEmailActivity)
        }
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is ReplaceEmailUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@ReplaceEmailActivity, it.content)
                    }
                    is ReplaceEmailUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is ReplaceEmailUiEffect.Toast -> {
                        ToastUtils.show(it.msg)
                        if (it.msg.contains("修改成功")) {
                            if (!oldEmail.isNullOrEmpty()) {
                                val intent = Intent()
                                intent.putExtra("status", true)
                                setResult(RESULT_OK, intent)
                            }
                            onBack()
                        }
                    }
                    is ReplaceEmailUiEffect.CodeSuccess -> requireBinding().ttvCode.startTimer()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ReplaceEmailUiState::email) {
                requireBinding().apply {
                    if (cetEmail.text.toString() != it && !it.isNullOrEmpty()) {
                        cetEmail.setText(it)
                    }
                    if (llCode.visibility != View.GONE) {
                        sbConfirm.isEnabled =
                            it.isNotEmpty() && cetCode.text.toString().isNotEmpty()
                    } else {
                        sbConfirm.isEnabled = it.isNotEmpty()
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ReplaceEmailUiState::code) {
                requireBinding().apply {
                    if (llCode.visibility != View.GONE) {
                        if (cetCode.text.toString() != it) {
                            cetCode.setText(it)
                        }
                        sbConfirm.isEnabled =
                            it.isNotEmpty() && cetEmail.text.toString().isNotEmpty()
                    }
                }
            }
        }
    }

    override fun onClick(p0: View) {
        when (p0.id) {
            //获取验证码
            R.id.ttv_code -> getViewModel().sendEvent(ReplaceEmailUiEvent.GetCode)
            //确认
            R.id.sb_confirm -> getViewModel().sendEvent(ReplaceEmailUiEvent.Confirm(mOldCode))
        }
    }

    private inner class EmailTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ReplaceEmailUiEvent.Email(p0.toString()))
        }
    }

    private inner class CodeTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ReplaceEmailUiEvent.Code(p0.toString()))
        }
    }
}