package com.at.video.mine.ui.history

import android.annotation.SuppressLint
import android.graphics.Color
import android.graphics.Typeface
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.os.bundleOf
import androidx.lifecycle.lifecycleScope
import com.at.common.adapter.FragmentAdapter
import com.at.common.adapter.PagerData
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityHistoryBinding
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.ext.buildIndicator
import com.loper7.tab_expand.ext.buildText
import com.loper7.tab_expand.indicator.LinearIndicator
import com.loper7.tab_expand.text.BaseText
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.TitleBar
import com.melancholy.widget.tabs.TabLayoutMediators
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor
import com.video.module_widget.widget.CustomTabText
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/7 下午3:12
 */
@AndroidEntryPoint
@Route(path = "/mine/HistoryActivity", extras = 10086)
//@Route(path = "/mine/HistoryActivity")
class HistoryActivity : BaseMviActivity<MineActivityHistoryBinding, HistoryViewModel>(),
    TabLayoutMediators.TabConfigurationStrategy, View.OnClickListener {
    private lateinit var mEditText: TextView
    private val tabTitle: List<String> = listOf("影视", "文章", "图集")
    private var mMediator: TabLayoutMediators? = null
    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    private var titlebar: TitleBar? = null

    @SuppressLint("NotifyDataSetChanged")
    override fun initialize(savedInstanceState: Bundle?) {
        titlebar = requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            setTitle("观看历史")
            addBackButton(Color.BLACK)
            mEditText = addRightTextButton("编辑", android.R.id.edit)
            mEditText.textSize = DensityUtils.px2sp(
                this@HistoryActivity, resources.getDimension(com.at.common.R.dimen.sp_16).toInt()
            )
            mEditText.typeface = Typeface.DEFAULT
            mEditText.setOnClickListener(this@HistoryActivity)
        }

        requireBinding().apply {
            val params: ViewGroup.LayoutParams = tbTab.layoutParams
            params.height = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_40)
            tbTab.layoutParams = params
            tbTab.tabMode = TabLayout.MODE_FIXED
            tbTab.buildIndicator<LinearIndicator>()
                .setHeight(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_2))
                .setWidth(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20))
                .setColor(StyleUtils.getPrimaryColor().toArgb()).bind()
//            tbTab.buildText<BaseText>()
//                .setNormalTextBold(false)
//                .setNormalTextSize(
//                    DensityUtils.px2sp(
//                        this@HistoryActivity,
//                        resources.getDimensionPixelSize(com.at.common.R.dimen.sp_14)
//                    )
//                )
//                .setSelectTextBold(true)
//                .setSelectTextSize(
//                    DensityUtils.px2sp(
//                        this@HistoryActivity,
//                        resources.getDimensionPixelSize(com.at.common.R.dimen.sp_20)
//                    )
//                )
//                .bind()
            tbTab.buildText<CustomTabText>().setTabColors(
                mutableListOf("#666666".toColor()),
                mutableListOf(StyleUtils.getPrimaryColor().toArgb())
            ).setTabNormalSizes(
                mutableListOf(
                    DensityUtils.px2sp(
                        this@HistoryActivity,
                        resources.getDimensionPixelSize(com.at.common.R.dimen.sp_16)
                    )
                )
            ).setTabSelectedSizes(
                mutableListOf(
                    DensityUtils.px2sp(
                        this@HistoryActivity,
                        resources.getDimensionPixelSize(com.at.common.R.dimen.sp_16)
                    )
                )
            ).setNormalTextBold(false).setSelectTextBold(true).bind()

            val adapter = FragmentAdapter(supportFragmentManager, lifecycle)
            vpViewPager.adapter = adapter

            mMediator = TabLayoutMediators(
                tbTab, vpViewPager, this@HistoryActivity, true
            )
            mMediator!!.attach()
            adapter.add(
                PagerData(
                    "/mine/history/MoviesFragment", 0L, bundleOf(Constants.KEY_DATA to 1)
                )
            )
            adapter.add(
                PagerData(
                    "/mine/history/OtherFragment", 1L, bundleOf(Constants.KEY_DATA to 3)
                )
            )
            adapter.add(
                PagerData(
                    "/mine/history/OtherFragment", 2L, bundleOf(Constants.KEY_DATA to 2)
                )
            )
            vpViewPager.offscreenPageLimit = adapter.itemCount
        }
    }

    override fun loadData() {

    }

    @SuppressLint("NotifyDataSetChanged")
    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(HistoryUiState::isEdit) {
                mEditText.text = if (it) "取消" else "编辑"
                titlebar?.setTitle(if (it) "选择内容" else "观看历史")
                requireBinding().apply {
//                    vTab.visibility = if (it) View.GONE else View.VISIBLE
                    vpViewPager.isUserInputEnabled = !it
                    tbTab.visibility = if (it) View.GONE else View.VISIBLE
                }
            }
        }

    }


    override fun createViewBinding(): MineActivityHistoryBinding {
        return MineActivityHistoryBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<HistoryViewModel> {
        return HistoryViewModel::class.java
    }

    override fun onClick(p0: View) {
        when (p0.id) {
            android.R.id.edit -> {// 切换编辑
                getViewModel().sendEvent(HistoryUiEvent.Edit)
            }
        }
    }

    override fun onConfigureTab(tab: TabLayout.Tab, position: Int) {
        tab.text = tabTitle[position]
    }
}