package com.at.video.mine.ui.history

import android.app.Application
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/1 下午5:58
 */

data class HistoryUiState(
    val selAllAble: Boolean = false,
    val delAble: Boolean = false,
    val msg: String = "",
    val isEdit: Boolean = false,
    val isDelete: Boolean = false,
    val isSelectAll: Boolean = false
) : IUiState

sealed class HistoryUiEvent : IUiEvent {
    object Edit : HistoryUiEvent()
    object SelectAll : HistoryUiEvent()
    object Delete : HistoryUiEvent()
    object CancelEdit : HistoryUiEvent()
}

sealed class HistoryUiEffect : IUiEffect {
    object SelectAll : HistoryUiEffect()
    object Delete : HistoryUiEffect()
}

class HistoryViewModel(application: Application) :
    BaseAndroidViewModel<HistoryUiState, HistoryUiEvent, HistoryUiEffect>(application) {
    override fun handleEvent(event: HistoryUiEvent) {
        when (event) {
            is HistoryUiEvent.Edit -> setState { copy(isEdit = !isEdit) }
            is HistoryUiEvent.SelectAll -> setEffect { HistoryUiEffect.SelectAll }
            is HistoryUiEvent.Delete -> setEffect { HistoryUiEffect.Delete }
            is HistoryUiEvent.CancelEdit ->
                if (state.value.isEdit) {
                    setState { copy(isEdit = !isEdit) }
                }
        }
    }

    override fun providerInitialState(): HistoryUiState {
        return HistoryUiState()
    }
}