package com.at.video.mine.ui.history.movies

import android.graphics.Color
import android.graphics.drawable.LayerDrawable
import android.graphics.drawable.StateListDrawable
import android.widget.ImageView
import android.widget.TextView
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.ImageConfig
import com.at.common.global.Constants
import com.at.common.load
import com.at.video.mine.R
import com.at.video.mine.TimeUtils
import com.at.video.mine.entity.ChapterInfo
import com.at.video.mine.entity.CollectBean
import com.at.video.mine.entity.HistoryBean
import com.at.video.mine.widget.VideoHistoryProgress
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.getFloatDigits
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.utils.toArgb
import java.lang.ref.WeakReference

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/9 上午11:12
 * @style : 0 影视 1 小说 2 图集
 */

class MoviesAdapter :
    BaseQuickAdapter<HistoryBean, BaseViewHolder>(R.layout.mine_history_item_movies) {
    private var mIsEdit: Boolean = false

    init {
        addChildClickViewIds(R.id.iv_selected)
    }

    override fun convert(holder: BaseViewHolder, item: HistoryBean) {
        holder.getView<ImageView>(R.id.div_image).loadByGlide(
            GlideImageConfig(
                item.cover.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
            ), WeakReference(context)
        )
        if (item.percentage.isNullOrEmpty()) {
            holder.setGone(R.id.tv_history_time, true)
        } else {
            var time: String? = try {
                TimeUtils.stringForTime(item.percentage.toLong() * 1000 ?: 0L)
            } catch (e: Exception) {
                item.percentage
            }
            holder.setGone(R.id.tv_history_time, time.isNullOrEmpty())
            holder.setText(R.id.tv_history_time, time)
        }
        holder.setText(R.id.tv_name, item.name)
//            .setText(R.id.tv_history_time, item.percentage)
        holder.setGone(R.id.iv_selected, !mIsEdit)

        changeCheck(holder, item.is_selected)

        val progressTxt = holder.getView<TextView>(R.id.tv_progress)
        val progress = holder.getView<VideoHistoryProgress>(R.id.video_progress)
        progress.setProgressStartColor(StyleUtils.primaryColor)
        progress.setProgressEndColor(StyleUtils.primaryColor)


        val proVal = if (item.progress.isNullOrEmpty()) {
            0f
        } else {
            item.progress.toFloat()
        }
        val totalTime: Long = if (item.percentage.isNullOrEmpty()) {
            0L
        } else {
            item.percentage.toLong() ?: 0L
        }
        if (totalTime > 0L) {
            val percent: Float = (proVal / totalTime).getFloatDigits(2).toFloat()
            val currentProgress = percent * 100
            progress.progressCurrent = currentProgress
            progressTxt.text =
                if (currentProgress < 1f) "观看不足1%" else if (currentProgress >= 100f) "已看完" else "观看至${currentProgress.toInt()}%"
        }

        holder.itemView.setOnClickListener {
            if (mIsEdit) {
                holder.getView<ImageView>(R.id.iv_selected).performClick()
            } else {
                Router.getInstance().build("/player/PlayerActivity")
                    .withString(Constants.KEY_DATA, item.resource_id!!).navigation()
            }
        }
    }

    private fun changeCheck(holder: BaseViewHolder, isSelect: Boolean) {

        val vectorDrawable = VectorDrawableCompat.create(
            context.resources, R.drawable.mine_ic_checkbox_selected_bg, context.theme
        )
        val vectorDrawable1 = VectorDrawableCompat.create(
            context.resources, R.drawable.mine_ic_checkbox_selected_1, context.theme
        )
        vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
        val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable, vectorDrawable1))
        val normalDrawable = VectorDrawableCompat.create(
            context.resources, R.drawable.mine_ic_checkbox_normal, context.theme
        )
        holder.setImageDrawable(
            R.id.iv_selected, if (isSelect) layerDrawAble else normalDrawable
        )
    }

    override fun convert(holder: BaseViewHolder, item: HistoryBean, payloads: List<Any>) {
        holder.setGone(R.id.iv_selected, !mIsEdit)
        changeCheck(holder, item.is_selected)
    }

    fun setEdit(isEdit: Boolean) {
        if (mIsEdit != isEdit) {
            mIsEdit = isEdit
            notifyItemRangeChanged(0, data.size, "notify")
        }
    }
}