package com.at.video.mine.ui.history.other

import android.graphics.drawable.LayerDrawable
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.video.mine.R
import com.at.video.mine.entity.HistoryBean
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.imageUrl
import com.video.module_widget.strToFloat
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.utils.toArgb
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/28 16:19
 * @Description:
 */

class OtherAdapter constructor(private val mType: Int) :
    BaseQuickAdapter<HistoryBean, BaseViewHolder>(R.layout.mine_history_item_other) {
    private var mIsEdit: Boolean = false

    init {
        addChildClickViewIds(R.id.iv_selected)
    }

    override fun convert(holder: BaseViewHolder, item: HistoryBean) {
        if (mType == 2) {
            holder.getView<ImageView>(R.id.div_image).loadByGlide(
                GlideImageConfig(
                    item.cover.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_novel_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_novel_default
                ),
                WeakReference(context)
            )
        } else {
            holder.getView<ImageView>(R.id.div_image).loadByGlide(
                GlideImageConfig(
                    item.cover.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                ),
                WeakReference(context)
            )
        }
        holder.setText(R.id.tv_name, item.name)
        holder.setGone(R.id.iv_selected, !mIsEdit)

        changeCheck(holder, item.is_selected)

        val progress = holder.getView<TextView>(R.id.tv_progress)
        try {
            val proVal: Int = ((item.percentage?.strToFloat() ?: 0f) * 100).toInt()
            progress.text =
                if (proVal < 1) "观看不足1%" else if (proVal >= 100f) "已看完" else "观看至${proVal}%"
        } catch (_: Exception) {
        }
//        when (item.type) {
//            3 -> { // 小说
// //                val proVal =
// //                    if (item.length != 0f && item.position <= item.length) ((item.position) / item.length * 100).toInt() else 0
//                try {
//                    val proVal =
//                        ((item.progress?.toInt() ?: 0) + 1) * 100 / (item.percentage?.toInt() ?: 1)
//                    progress.text =
//                        if (proVal < 1) "观看不足1%" else if (proVal == 100) "已看完" else "观看至$proVal%"
//                } catch (e: Exception) {
//                }
//            }
//            else -> {
//                try {
//                    val proVal = (item.percentage?.strToFloat() ?: 0f) * 100
//                    progress.text =
//                        if (proVal < 1f) "观看不足1%" else if (proVal >= 100f) "已看完" else "观看至${proVal.toInt()}%"
//                } catch (e: Exception) {
//                }
//            }
//        }

        val itemPosition = getItemPosition(item)
        val prePos = itemPosition - 1
        val label = holder.getView<TextView>(R.id.tv_history_time_label)
        label.visibility = View.GONE

//        label.text = item.hisTag
//        if (itemPosition == 0 || (prePos > -1 && getItem(prePos).hisTag != item.hisTag)) {
//            label.visibility = View.VISIBLE
//        } else {
//            label.visibility = View.GONE
//        }
        holder.itemView.setOnClickListener {
            if (mIsEdit) {
                holder.getView<ImageView>(R.id.iv_selected).performClick()
            } else {
                when (item.type) {
                    3 -> { // 小说
                        Router.getInstance().build("/novel/NovelDetailActivity")
                            .withString("novelId", item.resource_id!!)
                            .withString("title", item.name!!).navigation()
                    }
                    else -> { // 图集
                        Router.getInstance().build("/atlas/DetailActivity")
                            .withString("atlasId", item.resource_id!!).navigation()
                    }
                }
            }
        }
    }

    private fun changeCheck(holder: BaseViewHolder, isSelect: Boolean) {
        val vectorDrawable = VectorDrawableCompat.create(
            context.resources,
            R.drawable.mine_ic_checkbox_selected_bg,
            context.theme
        )
        val vectorDrawable1 = VectorDrawableCompat.create(
            context.resources,
            R.drawable.mine_ic_checkbox_selected_1,
            context.theme
        )
        vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
        val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable, vectorDrawable1))
        val normalDrawable = VectorDrawableCompat.create(
            context.resources,
            R.drawable.mine_ic_checkbox_normal,
            context.theme
        )
        holder.setImageDrawable(
            R.id.iv_selected,
            if (isSelect) layerDrawAble else normalDrawable
        )
    }

    override fun convert(holder: BaseViewHolder, item: HistoryBean, payloads: List<Any>) {
        holder.setGone(R.id.iv_selected, !mIsEdit)
        changeCheck(holder, item.is_selected)
    }

    fun setEdit(isEdit: Boolean) {
        if (mIsEdit != isEdit) {
            mIsEdit = isEdit
            notifyItemRangeChanged(0, data.size, "notify")
        }
    }
}
