package com.at.video.mine.ui.login

import android.content.Intent
import android.graphics.drawable.LayerDrawable
import android.graphics.drawable.StateListDrawable
import android.os.Bundle
import android.text.*
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import android.view.View
import android.widget.CompoundButton
import android.widget.Toast
import androidx.core.app.ActivityCompat
import androidx.lifecycle.lifecycleScope
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.module_export_main.IMainService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityLoginBinding
import com.at.video.mine.manager.UserManager
import com.at.video.mine.other.AgreementSpan
import com.melancholy.router.annotation.annotation.Autowired
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.facade.Postcard
import com.melancholy.router.api.facade.callback.NavigationCallback
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.dialog.MessageDialog
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlin.system.exitProcess

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/28 上午10:25
 */

@AndroidEntryPoint
@Route(path = "/mine/LoginActivity")
class LoginActivity :
    BaseMviActivity<MineActivityLoginBinding, LoginViewModel>(),
    View.OnClickListener,
    CompoundButton.OnCheckedChangeListener {

    override fun createViewBinding(): MineActivityLoginBinding {
        return MineActivityLoginBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<LoginViewModel> {
        return LoginViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    @Autowired
    @JvmField
    var account: String? = null

    @Autowired
    @JvmField
    var password: String? = null

    override fun initialize(savedInstanceState: Bundle?) {
        Router.getInstance().inject(this)
        UserManager.instance.clearInfo()
        requireTitleBar().apply {
//            setBackgroundResource(R.drawable.mine_ic_mine_top_background)
            addRightImageButton(
                R.drawable.mine_ic_close,
                com.melancholy.widget.R.id.backButton
            )
                .setOnClickListener(this)
        }
        requireBinding().apply {
            etAccount.addTextChangedListener(AccountTextChange())
            etPwd.addTextChangedListener(PasswordTextChange())
            ivPasswordVisible.setOnClickListener(this@LoginActivity)
//            tvAgreement.movementMethod = LinkMovementMethod.getInstance()
//            tvAgreement.highlightColor = Color.TRANSPARENT
//            tvAgreement.text = agreementTest()
//            pcbCheck.setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())

//            tvAgreement.setOnClickListener {
//                pcbCheck.toggle()
//            }
            pcbCheck.text = agreement()
            pcbCheck.setOnCheckedChangeListener(this@LoginActivity)
            val drawable = StateListDrawable()
            drawable.addState(
                intArrayOf(-android.R.attr.state_checked),
                VectorDrawableCompat.create(resources, R.drawable.mine_ic_checkbox_normal, theme)
            )
            val vectorDrawable = VectorDrawableCompat.create(
                resources,
                R.drawable.mine_ic_checkbox_selected_bg,
                theme
            )
            val vectorDrawable1 = VectorDrawableCompat.create(
                resources,
                R.drawable.mine_ic_checkbox_selected_1,
                theme
            )
            vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
            val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable, vectorDrawable1))
            drawable.addState(intArrayOf(android.R.attr.state_checked), layerDrawAble)
            pcbCheck.buttonDrawable = drawable
            sbLogin.apply {
                setOnClickListener(this@LoginActivity)
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            tvForgetPwd.setOnClickListener(this@LoginActivity)
            tvRegister.setOnClickListener(this@LoginActivity)
        }
        // 初始化
        fillAccountAndPwd(account, password)
    }

    private fun agreement(): SpannableStringBuilder {
        val builder = SpannableStringBuilder("已阅读并同意用户协议和隐私协议")
        val userAgreement = IMainService.getMainService().getSysConfig().getConfigUserAgreement()
        builder.setSpan(
            AgreementSpan(userAgreement, "用户协议"),
            6,
            10,
            Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        val privacyPolicy = IMainService.getMainService().getSysConfig().getConfigPrivacyPolicy()
        builder.setSpan(
            AgreementSpan(privacyPolicy, "隐私协议"),
            11,
            15,
            Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        return builder
    }

    override fun loadData() {
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is LoginUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@LoginActivity, it.content)
                    }
                    is LoginUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is LoginUiEffect.AccountError -> {
                        MessageDialog.showDialog(
                            supportFragmentManager,
                            it.msg,
                            closeLabel = "退出应用",
                            callback = object : MessageDialog.CloseCallback {
                                override fun close() {
                                    ActivityCompat.finishAffinity(this@LoginActivity)
                                    exitProcess(0)
                                }
                            }
                        )
                    }
                    is LoginUiEffect.Toast -> ToastUtils.show(it.msg)
                    is LoginUiEffect.LoginSuccess -> {
                        val path: String? = intent.getStringExtra("path")
                        if (path.isNullOrEmpty()) {
                            finish()
                        } else {
                            val extras: Bundle = intent.extras!!
                            extras.remove("path")
                            Router.getInstance().build(path)
                                .with(extras)
                                .navigation(
                                    callback = object : NavigationCallback {
                                        override fun onArrival(postcard: Postcard) {
                                            finish()
                                        }

                                        override fun onFound(postcard: Postcard) {
                                        }

                                        override fun onInterrupt(postcard: Postcard) {
                                        }

                                        override fun onLost(postcard: Postcard) {
                                        }
                                    }
                                )
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(LoginUiState::isVisible) {
                requireBinding().apply {
                    etPwd.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivPasswordVisible.setImageResource(
                        if (it) R.drawable.mine_ic_password_visible
                        else R.drawable.mine_ic_password_invisible
                    )
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(LoginUiState::userName) {
                requireBinding().apply {
                    if (etAccount.text.toString() != it) {
                        etAccount.setText(it)
                    }
                    sbLogin.isEnabled = it.isNotEmpty() && etPwd.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(LoginUiState::password) {
                requireBinding().apply {
                    if (etPwd.text.toString() != it) {
                        etPwd.setText(it)
                    }
                    sbLogin.isEnabled = it.isNotEmpty() && etAccount.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(LoginUiState::isCheck) {
                requireBinding().apply {
//                    if (pcbCheck.isChecked != it) {
//                        pcbCheck.isChecked = it
//                    }
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            R.id.iv_password_visible -> getViewModel().sendEvent(LoginUiEvent.PasswordVisible)
            // 登陆
            R.id.sb_login ->
                if (requireBinding().pcbCheck.isChecked) {
                    getViewModel().sendEvent(LoginUiEvent.Login)
                } else {
                    ToastUtils.show("请阅读并同意用户协议")
                }

            // 忘记密码
            R.id.tv_forget_pwd -> {
                val isExit = IMainService.getMainService().getSysConfig().getIsConfigEmail()
                if (isExit) {
                    Router.getInstance().build("/mine/BackPasswordActivity").navigation()
                } else {
                    MessageDialog.showDialog(supportFragmentManager, "该功能暂未正式对外提供服务，敬请期待。")
                }
            }
            // 一键注册
            R.id.tv_register -> {
                Router.getInstance().build("/mine/RegisterActivity").navigation(this, 110)
            }
        }
    }

    private inner class AccountTextChange : TextWatcher {

        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(LoginUiEvent.Account(p0.toString()))
        }
    }

    private inner class PasswordTextChange : TextWatcher {

        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(LoginUiEvent.Password(p0.toString()))
        }
    }

    override fun onCheckedChanged(buttonView: CompoundButton?, isChecked: Boolean) {
//        getViewModel().sendEvent(LoginUiEvent.Agreement)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 110 && resultCode == RESULT_OK && data != null) {
            fillAccountAndPwd(data.getStringExtra("account"), data.getStringExtra("password"))
        }
    }

    // 填充数据
    private fun fillAccountAndPwd(account: String?, password: String?) {
        // Log.i("TAG", "fillAccountAndPwd: $account  $password")
        if (account.isNullOrEmpty() || password.isNullOrEmpty()) {
            return
        }
        requireBinding().apply {
            etAccount.setText(account)
            etPwd.setText(password)
        }
    }
}
