package com.at.video.mine.ui.mine

import android.annotation.SuppressLint
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import com.at.common.ImageConfig
import com.at.common.load
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineFragmentMineBinding
import com.at.video.mine.manager.UserManager
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DateUtils
import com.video.module_widget.fileUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.loadByGlide
import dagger.hilt.android.AndroidEntryPoint
import java.lang.ref.WeakReference
import java.util.concurrent.Executors
import java.util.concurrent.ScheduledFuture
import java.util.concurrent.TimeUnit

/**
 * @Create_time: 2022/4/27 13:38
 * @Description: 我的
 */

@AndroidEntryPoint
@Route(path = "/mine/MineFragment")
class MineFragment :
    BaseMviFragment<MineFragmentMineBinding, MineViewModel>(),
    View.OnClickListener {

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): MineFragmentMineBinding {
        return MineFragmentMineBinding.inflate(inflater, container, false)
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<MineViewModel> {
        return MineViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            divHeader.setOnClickListener(this@MineFragment)
            stvLogin.setOnClickListener(this@MineFragment)
            stvRegister.setOnClickListener(this@MineFragment)
            stvRecharge.setOnClickListener(this@MineFragment)
            tvMineBuy.setOnClickListener(this@MineFragment)
            tvDetail.setOnClickListener(this@MineFragment)
//            tvInvitation.setOnClickListener(this@MineFragment)
            tvService.setOnClickListener(this@MineFragment)
            tvSettings.setOnClickListener(this@MineFragment)
//            sllBuyVip.setOnClickListener(this@MineFragment)
            tvHistory.setOnClickListener(this@MineFragment)
            tvCollect.setOnClickListener(this@MineFragment)
        }
    }

    @SuppressLint("SetTextI18n")
    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            IMineService.getMineService().getUser().collect {
                requireBinding().apply {
                    if (it == null) {
                        divHeader.load(ImageConfig(R.drawable.mine_ic_default_header))
                        gUser.visibility = View.VISIBLE
                        gMember.visibility = View.GONE
                        gNoMember.visibility = View.GONE
//                        gUser.visibility = View.GONE
//                        gNoMember.visibility = View.GONE
//                        gMember.visibility = View.VISIBLE
//                        stvVipName.text = "开通VIP会员"
//                        tvVipTime.text = "开通会员"
//                        stvHint.text = "开通VIP会员免费观看所有视频"
                    } else {
                        gUser.visibility = View.GONE
                        val nickname: String = it.getUserNickname().ifEmpty { it.getUserAccount() }
                        stvName.text = nickname
                        stvGold.visibility = View.GONE
                        stvGold.text = it.getUserGold().toString()
                        if (it.isMember()) {
                            divHeader.loadByGlide(
                                GlideImageConfig(
                                    it.getUserAvatar().fileUrl(),
                                    placeholderId = R.drawable.mine_ic_default_header,
                                    errorId = R.drawable.mine_ic_default_header
                                ),
                                WeakReference(context)
                            )
                            gNoMember.visibility = View.GONE
                            gMember.visibility = View.VISIBLE
                            stvVipName.text = "VIP会员"
                            tvVipTime.text =
                                "${
                                DateUtils.format(
                                    it.getUserExpirationTime() * 1000,
                                    "yyyy/MM/dd"
                                )
                                }到期"
//                            stvHint.text = "VIP会员免费观看所有视频"
                        } else {
                            val avatar = if (it.getUserAvatar().isEmpty()) R.drawable.mine_ic_default_header else it.getUserAvatar().fileUrl()
                            divHeader.load(
                                ImageConfig(
                                    avatar,
                                    placeholderId = R.drawable.mine_ic_default_header,
                                    errorId = R.drawable.mine_ic_default_header
                                )
                            )
                            gMember.visibility = View.GONE
                            gNoMember.visibility = View.VISIBLE
                            stvVipName.text = "开通VIP会员"
                            tvVipTime.text = "开通会员"
//                            stvHint.text = "开通VIP会员免费观看所有视频"
                        }
                    }
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            UserManager.instance.userFlow.collect {
                if (it == null) {
                    getViewModel().sendEvent(MineUiEvent.InitMsgCount)
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MineUiState::msgCount) {
                requireBinding().apply {
                    if (it > 0) {
                        badge.visibility = View.VISIBLE
                        badge.badgeCount = it
                    } else {
                        badge.visibility = View.GONE
                    }
                }
            }
        }
    }

    override fun loadData() {
    }

    override fun onResume() {
        super.onResume()
//        if (IMineService.getMineService().getUser().value?.getUserId() != null) {
//            getViewModel().sendEvent(MineUiEvent.UserInfo)
//            getViewModel().sendEvent(MineUiEvent.GetMeg)
//            startTimer()
//        } else {
//            requireBinding().badge.visibility = View.GONE
//        }
    }

    override fun onHiddenChanged(hidden: Boolean) {
        Log.e("TAG", "setUserVisibleHint:$hidden")
//        if (!hidden) {
//            if (IMineService.getMineService().getUser().value?.getUserId() != null) {
//                getViewModel().sendEvent(MineUiEvent.GetMeg)
//                getViewModel().sendEvent(MineUiEvent.UserInfo)
//                startTimer()
//            } else {
//                stopTimer()
//                requireBinding().badge.visibility = View.GONE
//            }
//        } else {
//            stopTimer()
//        }
    }

    private val mExecutor = Executors.newSingleThreadScheduledExecutor()
    private var mFuture: ScheduledFuture<*>? = null

    private fun startTimer() {
        mFuture = mExecutor.scheduleWithFixedDelay({
            if (IMineService.getMineService().getUser().value?.getUserId() != null) {
                getViewModel().sendEvent(MineUiEvent.GetMeg)
            }
        }, 1000, 1000 * 3, TimeUnit.MILLISECONDS)
    }

    private fun stopTimer() {
        mFuture?.run {
            if (!isCancelled) {
                cancel(true)
            }
        }
    }

    override fun onStop() {
//        stopTimer()
        super.onStop()
    }

    override fun onClick(v: View) {
        when (v.id) {
            // 登录注册
            R.id.stv_login -> {
                Router.getInstance().build("/mine/LoginActivity").navigation()
            }
            R.id.stv_register -> {
                Router.getInstance().build("/mine/RegisterActivity").withInt("comFrom", 0)
                    .navigation()
            }
            R.id.div_header -> {
                Router.getInstance().build("/mine/UserInfoActivity").navigation()
            }
            // 充值
            R.id.stv_recharge -> {
                Router.getInstance().build("/member/GoldActivity").navigation()
            }
            // 购买会员
//            R.id.sll_buy_vip -> {
//                Router.getInstance().build("/member/RechargeActivity").navigation()
//            }
            // 我的购买
            R.id.tv_mine_buy -> {
                Router.getInstance().build("/member/MineBuyActivity").navigation()
            }
            // 消费明细
            R.id.tv_detail -> {
                Router.getInstance().build("/member/DetailActivity").navigation()
            }
            // 邀请好友
            R.id.tv_invitation -> {
                Router.getInstance().build("/mine/InvitationActivity").navigation()
            }
            // 联系客服
            R.id.tv_service -> {
                Router.getInstance().build("/service/ServiceActivity").navigation()
            }
            // 设置页面
            R.id.tv_settings -> {
                Router.getInstance().build("/mine/SettingActivity").navigation()
            }
            // 历史
            R.id.tv_history -> {
                Router.getInstance().build("/mine/HistoryActivity").navigation()
            }
            // 收藏
            R.id.tv_collect -> {
                Router.getInstance().build("/mine/CollectActivity").navigation()
            }
        }
    }
}
