package com.at.video.mine.ui.password.back

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityBackPasswordBinding
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

/**
 * @description:
 * @author:  stone
 * @email: 找回密码
 * @date :   2022/4/28 下午4:33
 */
@AndroidEntryPoint
@Route(path = "/mine/BackPasswordActivity")
class BackPasswordActivity :
    BaseMviActivity<MineActivityBackPasswordBinding, BackPasswordViewModel>(),
    View.OnClickListener {

    override fun createViewBinding(): MineActivityBackPasswordBinding {
        return MineActivityBackPasswordBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<BackPasswordViewModel> =
        BackPasswordViewModel::class.java

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("找回密码")
        }
        requireBinding().run {
            ttvCode.setOnClickListener(this@BackPasswordActivity)
            sbNext.setOnClickListener(this@BackPasswordActivity)
            sbNext.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.5))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
//            etAccount.addTextChangedListener(AccountTextWatcher())
            etEmail.addTextChangedListener(EmailTextWatcher())
            etCode.addTextChangedListener(CodeTextWatcher())
            ttvCode.setTextColor(StyleUtils.getPrimaryColor().toArgb())
        }
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is BackPasswordUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@BackPasswordActivity, it.content)
                    }
                    is BackPasswordUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is BackPasswordUiEffect.Toast -> ToastUtils.show(it.msg)
                    is BackPasswordUiEffect.CodeSuccess -> requireBinding().ttvCode.startTimer()
                    is BackPasswordUiEffect.ValidationSuccess -> {
                        Router.getInstance().build("/mine/resetPasswordActivity")
                            .withString(Constants.KEY_DATA, it.email)
                            .withString(Constants.KEY_DATA_1, it.code).navigation()
                    }
                }
            }
        }
//        lifecycleScope.launchWhenResumed {
//            getViewModel().state.collectState(BackPasswordUiState::account) {
//                requireBinding().apply {
//                    if(etAccount.text.toString() != it) {
//                        etAccount.setText(it)
//                    }
//                    sbNext.isEnabled = it.isNotEmpty() && etEmail.text.toString().isNotEmpty() &&
//                            etCode.text.toString().isNotEmpty()
//                }
//            }
//        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(BackPasswordUiState::email) {
                requireBinding().apply {
                    if (etEmail.text.toString() != it) {
                        etEmail.setText(it)
                    }
//                    sbNext.isEnabled = it.isNotEmpty() && etAccount.text.toString().isNotEmpty() &&
//                            etCode.text.toString().isNotEmpty()
                    sbNext.isEnabled = it.isNotEmpty() && etCode.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(BackPasswordUiState::code) {
                requireBinding().apply {
                    if (etCode.text.toString() != it) {
                        etCode.setText(it)
                    }
//                    sbNext.isEnabled = it.isNotEmpty() && etAccount.text.toString().isNotEmpty() &&
//                            etEmail.text.toString().isNotEmpty()

                    sbNext.isEnabled = it.isNotEmpty() && etEmail.text.toString().isNotEmpty()
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            //获取邮箱验证码
            R.id.ttv_code -> {
                val email = requireBinding().etEmail.editableText.toString()
                if (email.isNullOrEmpty()) {
                    ToastUtils.show("请先输入邮箱")
                } else getViewModel().sendEvent(BackPasswordUiEvent.GetCode)
            }
            //确定按钮
            R.id.sb_next -> getViewModel().sendEvent(BackPasswordUiEvent.Validation)
        }
    }

    private inner class AccountTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(BackPasswordUiEvent.Account(p0.toString()))
        }
    }

    private inner class EmailTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(BackPasswordUiEvent.Email(p0.toString()))
        }
    }

    private inner class CodeTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(BackPasswordUiEvent.Code(p0.toString()))
        }
    }
}