package com.at.video.mine.ui.password.modify

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityModifyPasswordBinding
import com.at.video.mine.manager.UserManager
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import kotlinx.coroutines.flow.collect

/**
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午1:42
 * @description: 修改密码
 */
@Route(path = "/mine/ModifyPasswordActivity", extras = 10086)
class ModifyPasswordActivity: BaseMviActivity<MineActivityModifyPasswordBinding,
        ModifyPasswordViewModel>(), View.OnClickListener {

    override fun createViewBinding(): MineActivityModifyPasswordBinding {
        return MineActivityModifyPasswordBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ModifyPasswordViewModel> {
        return ModifyPasswordViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("修改密码")
        }
        requireBinding().apply {
            cetRaw.addTextChangedListener(RawTextWatcher())
            ivRawVisible.setOnClickListener(this@ModifyPasswordActivity)
            cetNew.addTextChangedListener(NewTextWatcher())
            ivNewVisible.setOnClickListener(this@ModifyPasswordActivity)
            cetConfirm.addTextChangedListener(ConfirmTextWatcher())
            ivConfirmVisible.setOnClickListener(this@ModifyPasswordActivity)
            sbConfirm.setOnClickListener(this@ModifyPasswordActivity)
            sbConfirm.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
        }
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when(it) {
                    is ModifyPasswordUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@ModifyPasswordActivity, it.content)
                    }
                    is ModifyPasswordUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is ModifyPasswordUiEffect.Toast -> ToastUtils.show(it.msg)
                    is ModifyPasswordUiEffect.Success -> {
                        UserManager.instance.clearInfo()
                        Router.getInstance().build("/mine/LoginActivity").navigation()
                        finish()
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::isRawVisible) {
                requireBinding().apply {
                    cetRaw.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivRawVisible.setImageResource(if(it) R.drawable.mine_ic_password_visible
                    else R.drawable.mine_ic_password_invisible)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::rawPassword) {
                requireBinding().apply {
                    if(cetRaw.text.toString() != it) {
                        cetRaw.setText(it)
                    }
                    sbConfirm.isEnabled = it.isNotEmpty() && cetNew.text.toString().isNotEmpty() &&
                            cetConfirm.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::isNewVisible) {
                requireBinding().apply {
                    cetNew.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivNewVisible.setImageResource(if(it) R.drawable.mine_ic_password_visible
                    else R.drawable.mine_ic_password_invisible)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::newPassword) {
                requireBinding().apply {
                    if(cetNew.text.toString() != it) {
                        cetNew.setText(it)
                    }
                    sbConfirm.isEnabled = it.isNotEmpty() && cetRaw.text.toString().isNotEmpty() &&
                            cetConfirm.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::isConfirmVisible) {
                requireBinding().apply {
                    cetConfirm.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivConfirmVisible.setImageResource(if(it) R.drawable.mine_ic_password_visible
                    else R.drawable.mine_ic_password_invisible)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ModifyPasswordUiState::confirmPassword) {
                requireBinding().apply {
                    if(cetConfirm.text.toString() != it) {
                        cetConfirm.setText(it)
                    }
                    sbConfirm.isEnabled = it.isNotEmpty() && cetRaw.text.toString().isNotEmpty() &&
                            cetNew.text.toString().isNotEmpty()
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            //显示或隐藏原密码
            R.id.iv_raw_visible -> getViewModel().sendEvent(ModifyPasswordUiEvent.RawVisible)
            //显示或隐藏原密码
            R.id.iv_new_visible -> getViewModel().sendEvent(ModifyPasswordUiEvent.NewVisible)
            //显示或隐藏原密码
            R.id.iv_confirm_visible -> getViewModel().sendEvent(ModifyPasswordUiEvent.ConfirmVisible)
            //确认
            R.id.sb_confirm -> getViewModel().sendEvent(ModifyPasswordUiEvent.Confirm)
        }
    }

    private inner class RawTextWatcher: TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            
        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            
        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ModifyPasswordUiEvent.RawPassword(p0.toString()))
        }
    }

    private inner class NewTextWatcher: TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ModifyPasswordUiEvent.NewPassword(p0.toString()))
        }
    }

    private inner class ConfirmTextWatcher: TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ModifyPasswordUiEvent.ConfirmPassword(p0.toString()))
        }
    }
}