package com.at.video.mine.ui.password.modify

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.export.mine.IMineService
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午1:43
 */
data class ModifyPasswordUiState(
    val isRawVisible: Boolean = false,
    val rawPassword: String = "",
    val isNewVisible: Boolean = false,
    val newPassword: String = "",
    val isConfirmVisible: Boolean = false,
    val confirmPassword: String = ""
): IUiState

sealed class ModifyPasswordUiEvent: IUiEvent {
    object RawVisible: ModifyPasswordUiEvent()
    data class RawPassword(val rawPassword: String): ModifyPasswordUiEvent()
    object NewVisible: ModifyPasswordUiEvent()
    data class NewPassword(val newPassword: String): ModifyPasswordUiEvent()
    object ConfirmVisible: ModifyPasswordUiEvent()
    data class ConfirmPassword(val confirmPassword: String): ModifyPasswordUiEvent()
    object Confirm: ModifyPasswordUiEvent()
}

sealed class ModifyPasswordUiEffect: IUiEffect {
    data class Loading(val content: String = ""): ModifyPasswordUiEffect()
    object Dismiss: ModifyPasswordUiEffect()
    data class Toast(val msg: String): ModifyPasswordUiEffect()
    object Success: ModifyPasswordUiEffect()
}


class ModifyPasswordViewModel(application: Application): BaseAndroidViewModel<ModifyPasswordUiState,
        ModifyPasswordUiEvent, ModifyPasswordUiEffect>(application) {

    override fun providerInitialState(): ModifyPasswordUiState {
        return ModifyPasswordUiState()
    }

    override fun handleEvent(event: ModifyPasswordUiEvent) {
        when(event) {
            is ModifyPasswordUiEvent.RawVisible -> setState { copy(isRawVisible = !isRawVisible) }
            is ModifyPasswordUiEvent.RawPassword -> setState { copy(rawPassword = event.rawPassword) }
            is ModifyPasswordUiEvent.NewVisible -> setState { copy(isNewVisible = !isNewVisible) }
            is ModifyPasswordUiEvent.NewPassword -> setState { copy(newPassword = event.newPassword) }
            is ModifyPasswordUiEvent.ConfirmVisible -> setState { copy(isConfirmVisible = !isConfirmVisible) }
            is ModifyPasswordUiEvent.ConfirmPassword -> setState { copy(confirmPassword = event.confirmPassword) }
            is ModifyPasswordUiEvent.Confirm -> modifyPassword()
        }
    }

    private fun modifyPassword() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { ModifyPasswordUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val map: MutableMap<String, Any> = mutableMapOf()
            map["old_password"] = state.value.rawPassword
            map["password"] = state.value.newPassword
            map["confirm_password"] = state.value.confirmPassword
            val data = ApiFactory.getInstance().create(ApiService::class.java).modifyUser( map = map)
            if(data.success()) {
//                UserManager.instance.updateUser(data.data!!)
                setEffect { ModifyPasswordUiEffect.Toast("修改密码成功,请重新登录！") }
                setEffect { ModifyPasswordUiEffect.Success }
            } else {
                setEffect { ModifyPasswordUiEffect.Toast(data.message!!) }
            }
            setEffect { ModifyPasswordUiEffect.Dismiss }
        }
    }
}