package com.at.video.mine.ui.password.reset

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.export.mine.IMineService
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.at.video.mine.ui.register.RegisterUiEffect
import com.melancholy.network.ApiFactory
import com.video.module_widget.entity.BaseResponse
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午1:43
 */
data class ResetPasswordUiState(
    val isNewOneVisible: Boolean = false,
    val newOnePassword: String = "",
    val isNewTwoVisible: Boolean = false,
    val newTwoPassword: String = "",
    val isCheckPasswordVaild: Boolean = false
) : IUiState

sealed class ResetPasswordUiEvent : IUiEvent {
    object NewOneVisible : ResetPasswordUiEvent()
    data class NewOnePassword(val newOnePassword: String) : ResetPasswordUiEvent()
    object NewTwoVisible : ResetPasswordUiEvent()
    data class NewTwoPassword(val newTwoPassword: String) : ResetPasswordUiEvent()
    data class Confirm(val email: String, val code: String) : ResetPasswordUiEvent()
    data class CheckPasswordVaild(val passowrd: String) : ResetPasswordUiEvent()
}

sealed class ResetPasswordUiEffect : IUiEffect {
    data class Loading(val content: String = "") : ResetPasswordUiEffect()
    object Dismiss : ResetPasswordUiEffect()
    data class Toast(val msg: String) : ResetPasswordUiEffect()
    object Success : ResetPasswordUiEffect()
    data class CheckPwdSuccess(val password: String) : ResetPasswordUiEffect()
    data class CheckPwdFail(val msg: BaseResponse) : ResetPasswordUiEffect()
}


class ResetPasswordViewModel(application: Application) :
    BaseAndroidViewModel<ResetPasswordUiState, ResetPasswordUiEvent, ResetPasswordUiEffect>(
        application
    ) {

    override fun providerInitialState(): ResetPasswordUiState {
        return ResetPasswordUiState()
    }

    override fun handleEvent(event: ResetPasswordUiEvent) {
        when (event) {
            is ResetPasswordUiEvent.NewOneVisible -> setState { copy(isNewOneVisible = !isNewOneVisible) }
            is ResetPasswordUiEvent.NewOnePassword -> setState { copy(newOnePassword = event.newOnePassword) }
            is ResetPasswordUiEvent.NewTwoVisible -> setState { copy(isNewTwoVisible = !isNewTwoVisible) }
            is ResetPasswordUiEvent.NewTwoPassword -> setState { copy(newTwoPassword = event.newTwoPassword) }
            is ResetPasswordUiEvent.Confirm -> resetPassword(event.email, event.code)
            is ResetPasswordUiEvent.CheckPasswordVaild -> checkPassword(event.passowrd)
            else -> {

            }
        }
    }

    private fun resetPassword(email: String, code: String) {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { ResetPasswordUiEffect.Loading("重置中") }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["password"] = state.value.newOnePassword
            map["confirm_password"] = state.value.newTwoPassword
            map["code"] = code
            map["belong_app_id"] = Constants.APP_ID
            map["email"] = email
            map["way"] = 1
            val data = ApiFactory.getInstance().create(ApiService::class.java).resetPassword(map)
            if (data.success()) {
                setEffect { ResetPasswordUiEffect.Toast("修改密码成功") }
                setEffect { ResetPasswordUiEffect.Success }
            } else {
                setEffect { ResetPasswordUiEffect.Toast(data.message ?: "服务器异常，请稍后再试") }
            }
            setEffect { ResetPasswordUiEffect.Dismiss }
        }
    }

    /**
     * @desc 检查密码合法性
     *
     */
    private fun checkPassword(pwd: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["password"] = pwd
            val res =
                ApiFactory.getInstance().create(ApiService::class.java).checkInputData(map = map)
            if (res.success()) {
                setEffect { ResetPasswordUiEffect.CheckPwdSuccess(pwd) }
            } else {
                setEffect { ResetPasswordUiEffect.CheckPwdFail(msg = res) }
            }
        }
    }
}