package com.at.video.mine.ui.phone

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import androidx.lifecycle.lifecycleScope
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityPhoneBinding
import com.at.video.mine.entity.CountryCode
import com.melancholy.router.annotation.annotation.Autowired
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/17 下午4:04
 */
@AndroidEntryPoint
@Route(path = "/mine/ModifyPhoneActivity")
class ModifyPhoneActivity : BaseMviActivity<MineActivityPhoneBinding, ModifyPhoneViewModel>(),
    TextWatcher, View.OnClickListener {

    @Autowired
    @JvmField
    var phone: String? = null
    override fun initialize(savedInstanceState: Bundle?) {
        Router.getInstance().inject(this)
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("修改手机号")
        }
        requireBinding().apply {
            cetPhone.addTextChangedListener(this@ModifyPhoneActivity)
            sbConfirm.setOnClickListener(this@ModifyPhoneActivity)
            if (!phone.isNullOrEmpty()) {
                cetPhone.setText(phone)
            }
            sbConfirm.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
        }
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is PhoneUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@ModifyPhoneActivity, it.content)
                    }
                    is PhoneUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is PhoneUiEffect.Toast -> {
                        ToastUtils.show(it.msg)
                        if (it.msg.contains("成功")) {
                            onBack()
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PhoneUiState::countryCodes) {
                val adapter =
                    ArrayAdapter<CountryCode>(baseContext, android.R.layout.simple_spinner_item, it)
                requireBinding().apply {
                    spCountryCode.adapter = adapter
                    for (i in it.indices) {
                        if (it[i].phone_code == "86") {
                            spCountryCode.setSelection(i, true)
                        }
                    }
                    spCountryCode.onItemSelectedListener =
                        object : AdapterView.OnItemSelectedListener {
                            override fun onItemSelected(
                                parent: AdapterView<*>?, view: View?, position: Int, id: Long
                            ) {
//                            tvPhoneCode.text = "+${it[position].phone_code}"
                                getViewModel().sendEvent(PhoneUiEvent.PhoneCode(it[position].phone_code!!))
                            }

                            override fun onNothingSelected(parent: AdapterView<*>?) {
                            }
                        }
                }
            }
        }
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun loadData() {
        getViewModel().sendEvent(PhoneUiEvent.LoadCountCode)
    }

    override fun createViewBinding(): MineActivityPhoneBinding =
        MineActivityPhoneBinding.inflate(layoutInflater)

    override fun getViewModelClass(): Class<ModifyPhoneViewModel> = ModifyPhoneViewModel::class.java
    override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
    }

    override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
    }

    override fun afterTextChanged(p0: Editable?) {
        getViewModel().sendEvent(PhoneUiEvent.Phone(p0.toString()))
    }

    override fun onClick(p0: View) {
        if (p0.id == R.id.sb_confirm) {
            getViewModel().sendEvent(PhoneUiEvent.Confirm)
        }
    }
}
