package com.at.video.mine.ui.register

import android.Manifest
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.drawable.LayerDrawable
import android.graphics.drawable.StateListDrawable
import android.os.Bundle
import android.text.*
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import android.util.Log
import android.view.View
import android.widget.CompoundButton
import androidx.core.view.drawToBitmap
import androidx.lifecycle.lifecycleScope
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.module_export_main.IMainService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityRegisterBinding
import com.at.video.mine.other.AgreementSpan
import com.hjq.permissions.OnPermissionCallback
import com.hjq.permissions.XXPermissions
import com.lxj.xpopup.util.KeyboardUtils
import com.melancholy.router.annotation.annotation.Autowired
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.dialog.HintDialog
import com.video.module_widget.dialog.MessageDialog
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/28 下午5:53
 */
@AndroidEntryPoint
@Route(path = "/mine/RegisterActivity")
class RegisterActivity : BaseMviActivity<MineActivityRegisterBinding, RegisterViewModel>(),
    View.OnClickListener, CompoundButton.OnCheckedChangeListener {
    private lateinit var mPermissions: XXPermissions

    @Autowired
    @JvmField
    var comFrom: Int = 1 // 0 =>从首页来 1 从登录来

    override fun createViewBinding(): MineActivityRegisterBinding {
        return MineActivityRegisterBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<RegisterViewModel> {
        return RegisterViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        Router.getInstance().inject(this)
        mPermissions = XXPermissions.with(this)
        requireTitleBar().apply {
            addRightImageButton(
                R.drawable.mine_ic_close, com.melancholy.widget.R.id.backButton
            ).setOnClickListener(this)
        }
        requireBinding().apply {
            etAccount.addTextChangedListener(AccountTextChange())
            etPwd.addTextChangedListener(PasswordTextChange())
            ivSwitchAccount.setOnClickListener(this@RegisterActivity)
            ivPasswordVisible.setOnClickListener(this@RegisterActivity)
            pcbCheck.text = agreement()
            pcbCheck.setOnCheckedChangeListener(this@RegisterActivity)

            val drawable = StateListDrawable()
            drawable.addState(
                intArrayOf(-android.R.attr.state_checked),
                VectorDrawableCompat.create(resources, R.drawable.mine_ic_checkbox_normal, theme)
            )
            val vectorDrawable = VectorDrawableCompat.create(
                resources, R.drawable.mine_ic_checkbox_selected_bg, theme
            )
            val vectorDrawable1 = VectorDrawableCompat.create(
                resources, R.drawable.mine_ic_checkbox_selected_1, theme
            )
            vectorDrawable?.setTint(StyleUtils.getPrimaryColor().toArgb())
            val layerDrawAble = LayerDrawable(arrayOf(vectorDrawable, vectorDrawable1))
            drawable.addState(intArrayOf(android.R.attr.state_checked), layerDrawAble)
            pcbCheck.buttonDrawable = drawable
//            pcbCheck.buttonTintList = StyleUtils.createColorStateList(R.color.color_FFFFFFFF,StyleUtils.getPrimaryColor().toArgb())
//            tvAgreement.movementMethod = LinkMovementMethod.getInstance()
//            tvAgreement.highlightColor = Color.TRANSPARENT
//            tvAgreement.text = agreementTest()
//            pcbCheck.setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
            sbRegister.apply {
                setOnClickListener(this@RegisterActivity)
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.6))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
            tvForgetPwd.setOnClickListener(this@RegisterActivity)
            tvAccounts.setOnClickListener(this@RegisterActivity)
        }
    }

    private fun agreement(): SpannableStringBuilder {
        val builder = SpannableStringBuilder("已阅读并同意用户协议和隐私协议")
        val userAgreement = IMainService.getMainService().getSysConfig().getConfigUserAgreement()
        builder.setSpan(
            AgreementSpan(userAgreement, "用户协议"), 6, 10, Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        val privacyPolicy = IMainService.getMainService().getSysConfig().getConfigPrivacyPolicy()
        builder.setSpan(
            AgreementSpan(privacyPolicy, "隐私协议"), 11, 15, Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        return builder
    }

    private fun agreementTest(): SpannableStringBuilder {
        val builder = SpannableStringBuilder("已阅读并同意用户协议和隐私协议")
        val userAgreement = "测试用户协议"
        builder.setSpan(
            AgreementSpan(userAgreement, "用户协议"), 6, 10, Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        val privacyPolicy = "测试隐私协议"
        builder.setSpan(
            AgreementSpan(privacyPolicy, "隐私协议"), 11, 15, Spanned.SPAN_INCLUSIVE_EXCLUSIVE
        )
        return builder
    }

    override fun loadData() {
    }

    private val SAVE_PIC = "save_pic"

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is RegisterUiEffect.Loading -> LoadingUtil.showLoading(
                        this@RegisterActivity, it.content
                    )
                    is RegisterUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is RegisterUiEffect.Register -> {
                        HintDialog.showDialog(
                            supportFragmentManager,
                            title = "需要加入您的“照片”",
                            leftLabel = "不允许",
                            rightLabel = "好",
                            content = "是否允许此app访问您的相册？把注册的账号和密码保存到相册中，当您忘记账号和密码时，可在相册中找回。"
                        ) { res ->
                            if (res) {
                                ToastUtils.show("注册成功！务必保管好账号密码")
                                skipLogin()
                            } else {
                                savePicture()
                            }
                        }
                        // }
                    }
                    is RegisterUiEffect.Toast -> {
                        ToastUtils.show(it.msg)
                        if (it.msg == "注册成功！账号密码保存至相册失败" || it.msg == "注册成功！账号密码已保存相册") {
                            skipLogin()
                        }
                    }
                    is RegisterUiEffect.CheckSuccess -> {
                        requireBinding().apply {
                            if (it.isAccount) {
                                tvAccountTip.visibility = View.GONE
                            } else {
                                tvPasswordTip.visibility = View.GONE
                            }
//                            Log.e("USERNAME", "observer: ${tvAccountTip.visibility == View.GONE && tvPasswordTip.visibility == View.GONE}")
                            sbRegister.isEnabled =
                                tvAccountTip.visibility == View.GONE && tvPasswordTip.visibility == View.GONE
                                checkAccountAndPwdIsEmpty()
                        }
                    }
                    is RegisterUiEffect.CheckFail -> {
                        requireBinding().apply {
                            if (it.msg.reason == "username") {
                                tvAccountTip.visibility = View.VISIBLE
                                tvAccountTip.text = it.msg.message
                                sbRegister.isEnabled = false
                            } else if (it.msg.reason == "password") {
                                tvPasswordTip.visibility = View.VISIBLE
                                tvPasswordTip.text = it.msg.message
                                sbRegister.isEnabled = false
                            }
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RegisterUiState::isVisible) {
                requireBinding().apply {
                    etPwd.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivPasswordVisible.setImageResource(
                        if (it) R.drawable.mine_ic_password_visible
                        else R.drawable.mine_ic_password_invisible
                    )
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RegisterUiState::userName) {
                requireBinding().apply {
                    if (etAccount.text.toString() != it) {
                        etAccount.setText(it)
                    }
//                    Log.e("USERNAME", "observer: ${tvAccountTip.visibility == View.GONE && tvPasswordTip.visibility == View.GONE}")
                    sbRegister.isEnabled = it.isNotEmpty() && etPwd.text.toString()
                        .isNotEmpty() && tvAccountTip.visibility == View.GONE && tvPasswordTip.visibility == View.GONE
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RegisterUiState::isCheck) {
                requireBinding().apply {
//                    if (pcbCheck.isChecked != it) {
//                        pcbCheck.isChecked = it
//                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(RegisterUiState::password) {
                requireBinding().apply {
                    if (etPwd.text.toString() != it) {
                        etPwd.setText(it)
                    }
                    sbRegister.isEnabled = it.isNotEmpty() && etAccount.text.toString()
                        .isNotEmpty() && tvAccountTip.visibility == View.GONE && tvPasswordTip.visibility == View.GONE
                }
            }
        }
    }

    private fun skipLogin() {
        requireBinding().apply {
            val account = etAccount.text.toString().trim()
            val password = etPwd.text.toString().trim()
            if (comFrom == 0) { // 跳转到登录页
                Router.getInstance().build("/mine/LoginActivity").withString("account", account)
                    .withString("password", password).navigation()
            } else { // 回调到登录页
                val intent = Intent()
                intent.putExtra("account", account)
                intent.putExtra("password", password)
                setResult(RESULT_OK, intent)
            }
            onBack()
        }
    }

    override fun onClick(view: View) {
        KeyboardUtils.hideSoftInput(window)
        when (view.id) {
            // 换一换
            R.id.iv_switch_account -> getViewModel().sendEvent(RegisterUiEvent.RandomAccount)
            // 显示和隐藏密码
            R.id.iv_password_visible -> getViewModel().sendEvent(RegisterUiEvent.PasswordVisible)
            // 注册
            R.id.sb_register -> {
                if (requireBinding().pcbCheck.isChecked) {
                    getViewModel().sendEvent(RegisterUiEvent.Register)
                } else {
                    ToastUtils.show("请阅读并同意协议")
                }
            }
            // 忘记密码
            R.id.tv_forget_pwd -> {
//                Router.getInstance().build("/mine/BackPasswordActivity").navigation()

                val isExit = IMainService.getMainService().getSysConfig().getIsConfigEmail()
                if (isExit) {
                    Router.getInstance().build("/mine/BackPasswordActivity").navigation()
                } else {
                    MessageDialog.showDialog(supportFragmentManager, "该功能暂未正式对外提供服务，敬请期待。")
                }
            }
            // 有账号
            R.id.tv_accounts -> {
                if (comFrom == 0) {
                    Router.getInstance().build("/mine/LoginActivity").navigation()
                }
                onBack()
            }
        }
    }

    private fun savePicture() {
        mPermissions.permission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
            .request(object : OnPermissionCallback {
                override fun onGranted(permissions: MutableList<String>?, all: Boolean) {
                    if (all) {
                        val isVisible: Boolean = getViewModel().state.value.isVisible
                        if (!isVisible) {
                            requireBinding().etPwd.transformationMethod =
                                HideReturnsTransformationMethod.getInstance()
                        }
                        val bitmap: Bitmap = Bitmap.createBitmap(window.decorView.drawToBitmap())
                        if (!isVisible) {
                            requireBinding().etPwd.transformationMethod =
                                PasswordTransformationMethod.getInstance()
                        }
                        getViewModel().sendEvent(RegisterUiEvent.Picture(bitmap))
                    } else {
                        ToastUtils.show("注册成功,未获取权限,保存失败")
                        skipLogin()
                    }
                }

                override fun onDenied(permissions: MutableList<String>?, never: Boolean) {
                    super.onDenied(permissions, never)
                    ToastUtils.show("注册成功,权限未被允许,请手动开启权限,保存失败")
                    skipLogin()
                }
            })
    }

    private inner class AccountTextChange : TextWatcher {

        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            getViewModel().sendEvent(RegisterUiEvent.Account(requireBinding().etAccount.editableText.toString()))
            getViewModel().sendEvent(RegisterUiEvent.CheckAccount)
        }

        override fun afterTextChanged(p0: Editable) {

        }
    }

    private inner class PasswordTextChange : TextWatcher {

        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            getViewModel().sendEvent(RegisterUiEvent.Password(requireBinding().etPwd.editableText.toString()))
            getViewModel().sendEvent(RegisterUiEvent.CheckPwd)
        }

        override fun afterTextChanged(p0: Editable) {

        }
    }

    override fun onCheckedChanged(buttonView: CompoundButton?, isChecked: Boolean) {
        getViewModel().sendEvent(RegisterUiEvent.Agreement)
    }

    private fun checkAccountAndPwdIsEmpty() {
        requireBinding().apply {
            val pwd = etPwd.editableText.toString()
            val acount = etAccount.editableText.toString()
            if (pwd.isNullOrEmpty() ||acount.isNullOrEmpty()) {
                requireBinding().sbRegister.isEnabled = false
            }
        }

    }
}
