package com.at.video.mine.ui.settings.userinfo.dialog

import android.os.Bundle
import android.view.*
import android.widget.RadioGroup
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineDialogBottomGenderBinding
import com.melancholy.utils.ResourceUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/5 下午2:11
 */
class BottomGenderDialog: BaseBindingDialogFragment<MineDialogBottomGenderBinding>(),
    View.OnClickListener, RadioGroup.OnCheckedChangeListener {
    private val mGender: Int? = IMineService.getMineService().getUser().value?.getUserGender()
    private var mSelectGender: Int = mGender ?: 2
    private var mCallback: ((Int) -> Unit)? = null

    companion object {
        @JvmStatic
        @JvmOverloads
        fun showDialog(ft: FragmentManager, callback: (Int) -> Unit, tag: String? = null) {
            BottomGenderDialog()
                .setCallBack(callback)
                .show(ft, tag)
        }
    }

    override fun getCustomTheme(): Int = R.style.DialogTheme

    override fun getWidth(): Int = WindowManager.LayoutParams.MATCH_PARENT

    override fun getGravity(): Int = Gravity.BOTTOM

    override fun getCancelable(): Boolean = false

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): MineDialogBottomGenderBinding {
        return MineDialogBottomGenderBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvCancel.setOnClickListener(this@BottomGenderDialog)
            tvConfirm.setOnClickListener(this@BottomGenderDialog)
            tvConfirm.setTextColor(StyleUtils.getPrimaryColor().toArgb())
            rbWoman.setTextColor(StyleUtils.createColorStateList(R.color.color_FF000000,StyleUtils.getPrimaryColor().toArgb()))
            rbMan.setTextColor(StyleUtils.createColorStateList(R.color.color_FF000000,StyleUtils.getPrimaryColor().toArgb()))
            rgSex.check(if(mSelectGender == 2) R.id.rb_woman else R.id.rb_man)
            rgSex.setOnCheckedChangeListener(this@BottomGenderDialog)
        }
    }

    override fun loadData() {

    }

    fun setCallBack(callback: (Int) -> Unit): BottomGenderDialog {
        mCallback = callback
        return this
    }

    override fun onClick(view: View) {
        when (view.id) {
            R.id.tv_confirm -> {
                mCallback?.invoke(mSelectGender)
            }
        }
        dismiss()
    }

    override fun onCheckedChanged(group: RadioGroup, checkedId: Int) {
        when(checkedId) {
            R.id.rb_woman -> mSelectGender = 2
            R.id.rb_man -> mSelectGender = 1
        }
    }
}