package com.at.video.mine.ui.settings.userinfo.dialog

import android.annotation.SuppressLint
import android.os.Bundle
import android.text.Editable
import android.text.InputFilter
import android.text.Spanned
import android.text.TextWatcher
import android.util.Log
import android.view.*
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineDialogBottomNicknameBinding
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/5 上午11:18
 */
class BottomNicknameDialog: BaseBindingDialogFragment<MineDialogBottomNicknameBinding>(),
    View.OnClickListener, TextWatcher {
    private val MAX_LEN = 20
    private var mCallback: ((String) -> Unit)? = null
    private val mNickname: String? = IMineService.getMineService().getUser().value?.getUserNickname()

    companion object {
        @JvmStatic
        @JvmOverloads
        fun showDialog(ft: FragmentManager, callback: (String) -> Unit, tag: String? = null) {
            val dialog = BottomNicknameDialog()
            dialog.setCallback(callback)
            dialog.show(ft, tag)
        }
    }

    override fun getCustomTheme(): Int = R.style.DialogTheme

    override fun getWidth(): Int = WindowManager.LayoutParams.MATCH_PARENT

    override fun getGravity(): Int = Gravity.BOTTOM

    override fun getCancelable(): Boolean = false

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): MineDialogBottomNicknameBinding {
        return MineDialogBottomNicknameBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvCancel.setOnClickListener(this@BottomNicknameDialog)
            tvSave.setOnClickListener(this@BottomNicknameDialog)
            tvSave.setTextColor(StyleUtils.getPrimaryColor().toArgb())
            etNickname.addTextChangedListener(this@BottomNicknameDialog)
            etNickname.setText(mNickname)
//            etNickname.filters = arrayOf(CustomInputFiler())
        }
    }

    override fun loadData() {

    }

    fun setCallback(callback: (String) -> Unit): BottomNicknameDialog {
        mCallback = callback
        return this
    }

    override fun onClick(view: View) {
        when (view.id) {
            R.id.tv_save -> {
                requireBinding().apply {
                    val nickname = etNickname.text.toString().trim().replace("/t","").replace("/n","")
                    mCallback?.invoke(nickname)
                }
            }
        }
        dismiss()
    }

    override fun beforeTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
//        Log.e("INPUT", "beforeTextChanged: s:$s,start:$start,before:$before,count:$count" )


    }

    override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
//        Log.e("INPUT", "onTextChanged: s:$s,start:$start,before:$before,count:$count" )
//       val len =  Character.codePointCount(s,0,s.length)
//       Log.e("INPUT", "onTextChanged: s:$s,start:$start,before:$before,count:$count,len:$len" )

    }

    @SuppressLint("SetTextI18n")
    override fun afterTextChanged(p0: Editable) {
        requireBinding().apply {
            val len =  Character.codePointCount(p0.toString(),0,p0.toString().length)
            tvLimit.text = "${p0.toString().length}/20"
            tvSave.isEnabled = p0.isNotEmpty() && p0.toString() != mNickname
        }
    }

    class CustomInputFiler :InputFilter{
        override fun filter(
            source: CharSequence,
            start: Int,
            end: Int,
            dest: Spanned?,
            dstart: Int,
            dend: Int
        ): CharSequence? {
            val len = Character.codePointCount(dest!!.toString(), 0, dest.length)
//            Log.e("INPUT", "filter: source:$source,start:$start,end:$end,dest:$dest,dstart:$dstart,dend:$dend,len:$len" )

            var keep: Int = 20 - (len - (dend - dstart))
            return if (keep <= 0) {
                ""
            } else if (keep >= 1) {
//                Log.e("INPUT", "filter: $keep" )
                null // keep original
            } else {
                keep += start
                if (Character.isHighSurrogate(source[keep - 1])) {
                    --keep
                    if (keep == start) {
                        return ""
                    }
                }
                source.subSequence(start, keep)
            }

//            if (Character.codePointCount(charSequence.toString(), 0, charSequence.toString().length) <= 20) {
//                return null
//            } else {
//                return charSequence
//            }
        }
    }

}

