package com.at.video.mine.ui.userinfo

import android.app.Activity
import android.content.ContentValues
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Color
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.os.Environment.DIRECTORY_PICTURES
import android.provider.MediaStore
import android.view.View
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.lifecycle.lifecycleScope
import com.at.common.ImageConfig
import com.at.common.load
import com.at.common.mvi.BaseMviActivity
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityUserinfoBinding
import com.at.video.mine.entity.City
import com.at.video.mine.ui.settings.userinfo.dialog.BottomGenderDialog
import com.at.video.mine.ui.settings.userinfo.dialog.BottomHeadDialog
import com.at.video.mine.ui.settings.userinfo.dialog.BottomNicknameDialog
import com.bigkoo.pickerview.builder.OptionsPickerBuilder
import com.bigkoo.pickerview.builder.TimePickerBuilder
import com.bigkoo.pickerview.view.OptionsPickerView
import com.bigkoo.pickerview.view.TimePickerView
import com.contrarywind.view.WheelView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.DateUtils
import com.melancholy.utils.DensityUtils
import com.melancholy.utils.FileUtils
import com.melancholy.utils.MediaUtils
import com.video.module_widget.fileUrl
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect
import java.io.File
import java.util.*

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午3:15
 */
@AndroidEntryPoint
// @Route(path = "/mine/UserInfoActivity")
@Route(path = "/mine/UserInfoActivity", extras = 10086)
class UserInfoActivity :
    BaseMviActivity<MineActivityUserinfoBinding, UserInfoViewModel>(),
    View.OnClickListener {
    private var mFile: File? = null
    private var mCropFile: File? = null
    private var mTimePicker: TimePickerView? = null
    private var mProvincePicker: OptionsPickerView<City>? = null

    private var mPhone: String? = null

    override fun createViewBinding(): MineActivityUserinfoBinding {
        return MineActivityUserinfoBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<UserInfoViewModel> {
        return UserInfoViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("个人资料")
        }
        requireBinding().apply {
            divHeadImg.setOnClickListener(this@UserInfoActivity)
            llNickname.setOnClickListener(this@UserInfoActivity)
            llEmail.setOnClickListener(this@UserInfoActivity)
            llGender.setOnClickListener(this@UserInfoActivity)
            llBirthday.setOnClickListener(this@UserInfoActivity)
            llBirthplace.setOnClickListener(this@UserInfoActivity)
            llPhone.setOnClickListener(this@UserInfoActivity)
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(UserInfoUiEvent.Province)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is UserInfoUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@UserInfoActivity, it.content)
                    }
                    is UserInfoUiEffect.Dismiss -> {
                        LoadingUtil.dismiss()
                    }
                    is UserInfoUiEffect.Toast -> ToastUtils.show(it.msg)
                    is UserInfoUiEffect.Location -> location(it.provinces, it.cities)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            IMineService.getMineService().getUser().collect {
                requireBinding().apply {
                    val url: String? = it?.getUserAvatar()?.fileUrl()
                    val defaultId: Int = R.drawable.mine_ic_upload_header
                    divHeadImg.load(
                        ImageConfig(
                            url,
                            placeholderId = defaultId,
                            errorId = defaultId
                        )
                    )
                    if (it != null) {
                        tvNickname.text = it.getUserNickname()
                        tvBirthday.tag = it.getUserBirthday().toLong()
                        tvBirthplace.text = getAddress(it.getUserAddress())
                        if (it.getUserBirthday() != "0") {
                            tvBirthday.text =
                                DateUtils.format(it.getUserBirthday().toLong() * 1000, "yyyy-MM-dd")
                        }
                    }
                    tvEmail.text = it?.getUserEmail()
                    val gender: Int? = it?.getUserGender()
                    if (gender != null) {
                        val sex: String? = if (gender == 1) "男" else if (gender == 2) "女" else null
                        tvGender.text = sex
                    }

                    if (!it?.getPhoneCountryCode().isNullOrEmpty() && !it?.getUserPhoneNumber()
                        .isNullOrEmpty()
                    ) {

                        mPhone = it?.getUserPhoneNumber()
                        tvPhone.text = "+${it?.getPhoneCountryCode()} ${it?.getUserPhoneNumber()}"
                    } else {
                        tvPhone.text = ""
                    }
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            // 选择头像
            R.id.div_head_img -> {
                BottomHeadDialog.showDialog(supportFragmentManager, {
                    if (it == 0) { // 从相册选取
                        MediaUtils.selectFile(this, 101)
                    } else { // 拍照
                        takingPictures()
                    }
                })
            }
            // 昵称
            R.id.ll_nickname -> {
                BottomNicknameDialog.showDialog(supportFragmentManager, {
                    getViewModel().sendEvent(UserInfoUiEvent.Nickname(it))
                })
            }
            // 邮箱
            R.id.ll_email -> {
//                val path: String = if(IMineService.getMineService().getUser()
//                        .value?.getUserEmail().isNullOrEmpty()) {
//                    "/mine/ReplaceEmailActivity"
//                } else {
//                    "/mine/RawEmailActivity"
//                }
                Router.getInstance().build("/mine/ReplaceEmailActivity").navigation()
            }
            // 手机号
            R.id.ll_phone -> {
                requireBinding().apply {
                    Router.getInstance().build("/mine/ModifyPhoneActivity")
                        .withString("phone", mPhone ?: "").navigation()
                }
            }
            // 性别
            R.id.ll_gender -> {
                BottomGenderDialog.showDialog(supportFragmentManager, {
                    getViewModel().sendEvent(UserInfoUiEvent.Gender(it))
                })
            }
            // 生日
            R.id.ll_birthday -> timePicker()
            // 所在地
            R.id.ll_birthplace -> getViewModel().sendEvent(UserInfoUiEvent.Location)
        }
    }

    private var option1Id = ""
    private var option2Id = ""

    private fun getAddress(address: String): String {
        val adds = address.split("-")
        if (adds.isEmpty()) return ""
        if (adds.size == 1) return adds[0]
        if (adds.size == 2) {
            val options = adds[1].split(",")
            if (options.size > 1) {
                option1Id = options[0]
                option2Id = options[1]
            }
            return adds[0]
        }
        return ""
    }

    /**
     * 时间选择器
     */
    private fun timePicker() {
        if (mTimePicker == null) {
            val time: Long? = requireBinding().tvBirthday.tag as? Long
            val curData = Calendar.getInstance()
            if (time != null && time != 0L) {
                curData.timeInMillis = time * 1000
            }
            val startDate = Calendar.getInstance()
            startDate.set(1900, 0, 1)
            val endDate = Calendar.getInstance()
            endDate.timeInMillis = System.currentTimeMillis()
            mTimePicker = TimePickerBuilder(this) { date, _ ->
                getViewModel().sendEvent(UserInfoUiEvent.Birthday(date.time / 1000))
            }
                .setLayoutRes(R.layout.mine_pickerview_custom_time) {
                    it.findViewById<TextView>(R.id.tv_save).apply {
                        setOnClickListener {
                            mTimePicker?.returnData()
                            mTimePicker?.dismiss()
                        }
                        setTextColor(StyleUtils.getPrimaryColor().toArgb())
                    }

                    it.findViewById<View>(R.id.tv_cancel).setOnClickListener {
                        mTimePicker?.dismiss()
                    }
                }
                .setContentTextSize(
                    DensityUtils.px2sp(
                        this,
                        resources.getDimensionPixelSize(
                            com.at.common.R.dimen.sp_14
                        )
                    ).toInt()
                )
                .setRangDate(startDate, endDate)
                .setDate(curData)
                .setType(booleanArrayOf(true, true, true, false, false, false))
                .setLabel("", "", "", "", "", "")
                .setLineSpacingMultiplier(3.5f)
                .isCenterLabel(false)
                .setDividerType(WheelView.DividerType.FILL)
                .setDividerColor(ContextCompat.getColor(this, R.color.color_FFF1F3F7))
                .setItemVisibleCount(3)
                .setTextColorCenter(StyleUtils.primaryColor)
                .setOutSideCancelable(false)
                .build()
        }
        mTimePicker?.show()
    }

    /**
     * 所在地
     */
    private fun location(provinces: List<City>, cities: List<List<City>>) {
        if (mProvincePicker == null) {
            mProvincePicker = OptionsPickerBuilder(this) { options1, options2, _, _ ->
                getViewModel().sendEvent(UserInfoUiEvent.Address("${provinces[options1].name},${cities[options1][options2].name}-${provinces[options1].id},${cities[options1][options2].id}"))
            }
                .setLayoutRes(R.layout.mine_pickerview_custom_options) {
                    it.findViewById<TextView>(R.id.tv_save).apply {
                        setOnClickListener {
                            mProvincePicker?.returnData()
                            mProvincePicker?.dismiss()
                        }
                        setTextColor(StyleUtils.primaryColor)
                    }
                    it.findViewById<View>(R.id.tv_cancel).setOnClickListener {
                        mProvincePicker?.dismiss()
                    }
                }
                .setDecorView(findViewById(android.R.id.content))
                .setLineSpacingMultiplier(3f)
                .setItemVisibleCount(5)
                .setDividerColor(Color.TRANSPARENT)
                .setTextColorCenter(StyleUtils.getPrimaryColor().toArgb())
                .setOutSideCancelable(false)
                .build()
        }
        mProvincePicker?.setPicker(provinces, cities, null)
        mProvincePicker?.apply {
            var option1 = 0
            for (i in provinces.indices) {
                if (provinces[i].id == option1Id) option1 = i
            }
            var option2 = 0
            for (i in cities[option1].indices) {
                if (cities[option1][i].id == option1Id) option2 = i
            }

            setSelectOptions(option1, option2)
        }
        mProvincePicker?.show()
    }

    /**
     * 拍照
     */
    private fun takingPictures() {
        try {
            if (mFile != null) {
                mFile!!.delete()
                mFile = null
            }
        } catch (e: Exception) {
        }
        mFile = File(
            FileUtils.getExternalCacheDirs(this)?.get(0) ?: cacheDir,
            "${System.currentTimeMillis()}.jpg"
        )
        MediaUtils.takingPictures(
            this,
            100,
            FileUtils.fileToUri(this, mFile!!)
        )
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            when (requestCode) {
                100 -> clip(FileUtils.fileToUri(this, mFile!!))
                101 -> clip(data!!.data!!)
                102 -> getViewModel().sendEvent(UserInfoUiEvent.Avatar(mCropFile!!))
            }
        }
    }

    /**
     * 裁剪
     */
    private fun clip(desUri: Uri) {
        try {
            if (mCropFile != null) {
                mCropFile!!.delete()
                mCropFile = null
            }
        } catch (e: Exception) {
        }
        val fileName = "${System.currentTimeMillis()}.jpg"
        val outUri: Uri = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q &&
            !Environment.isExternalStorageLegacy()
        ) {
            mCropFile = File(
                Environment.getExternalStoragePublicDirectory(DIRECTORY_PICTURES),
                fileName
            )
            val values = ContentValues()
            values.put(MediaStore.Images.Media.DATA, mCropFile!!.absolutePath)
            values.put(MediaStore.Images.Media.DISPLAY_NAME, fileName)
            values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg")
            contentResolver.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values)!!
        } else {
            mCropFile = File(
                FileUtils.getExternalCacheDirs(this)?.get(0) ?: cacheDir,
                fileName
            )
            if (mCropFile != null && !mCropFile!!.exists()) {
                mCropFile!!.createNewFile()
            }
            Uri.fromFile(mCropFile!!)
        }
        MediaUtils.openPhotoClip(
            this, desUri, outUri, 1, 1, 200,
            200, true, Bitmap.CompressFormat.JPEG, false, 102
        )
    }
}
