package com.at.video.mine.widget

import android.content.Context
import android.text.Spannable
import android.text.method.MovementMethod
import android.util.AttributeSet
import android.view.MotionEvent
import com.google.android.material.R.attr.checkboxStyle
import com.google.android.material.checkbox.MaterialCheckBox
import com.melancholy.widget.fixtouch.ISpanTouchFix
import com.melancholy.widget.fixtouch.LinkTouchMovementMethod

/**
 * @Create_time: 2020/8/26 16:06
 * @Author: wr
 * @Description: 支持协议Check View
 */
class ProtocolCheckBox: MaterialCheckBox, ISpanTouchFix {
    //记录当前 Touch 事件对应的点是不是点在了 span 上面
    private var mTouchSpanHit: Boolean = false
    //记录每次真正传入的press，每次更改mTouchSpanHint，需要再调用一次setPressed，确保press状态正确
    private var mIsPressedRecord = false
    //当前View是否应该消耗事件
    private var mNeedForceEventToParent = false

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = checkboxStyle):
            super(context, attrs, defStyleAttr) {
        setMovementMethodDefault()
    }

    private fun setNeedForceEventToParent(needForceEventToParent: Boolean) {
        mNeedForceEventToParent = needForceEventToParent
        isFocusable = !needForceEventToParent
        isClickable = !needForceEventToParent
        isLongClickable = !needForceEventToParent
    }

    /**
     * 使用者主动调用
     */
    private fun setMovementMethodDefault() {
        setMovementMethodCompat(LinkTouchMovementMethod.getInstance())
    }

    private fun setMovementMethodCompat(movement: MovementMethod?) {
        movementMethod = movement
        if (mNeedForceEventToParent) {
            setNeedForceEventToParent(true)
        }
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        if (text !is Spannable) {
            return super.onTouchEvent(event)
        }
        mTouchSpanHit = true
        // 调用super.onTouchEvent,会走到LinkTouchMovementMethod
        // 会走到LinkTouchMovementMethod#onTouchEvent会修改mTouchSpanHint
        val ret: Boolean = super.onTouchEvent(event)
        return if (mNeedForceEventToParent) {
            mTouchSpanHit
        } else ret
    }

    override fun setTouchSpanHit(hit: Boolean) {
        if (mTouchSpanHit != hit) {
            mTouchSpanHit = hit
            isPressed = mIsPressedRecord
        }
    }

    override fun performClick(): Boolean {
        return if (!mTouchSpanHit && !mNeedForceEventToParent) {
            super.performClick()
        } else false
    }

    override fun performLongClick(): Boolean {
        return if (!mTouchSpanHit && !mNeedForceEventToParent) {
            super.performLongClick()
        } else false
    }

    override fun setPressed(pressed: Boolean) {
        mIsPressedRecord = pressed
        if (!mTouchSpanHit) {
            onSetPressed(pressed)
        }
    }

    private fun onSetPressed(pressed: Boolean) {
        super.setPressed(pressed)
    }
}