package com.at.video.mine.widget

import android.animation.ObjectAnimator
import android.annotation.SuppressLint
import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.util.Log
import android.view.View
import com.at.video.mine.R
import kotlin.math.acos
import kotlin.math.sin

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/6/27 下午1:17
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/27 下午1:17
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class VideoHistoryProgress : View {



    constructor(context: Context, attrs: AttributeSet):super(context,attrs){
        obtainStyledAttrs(attrs)
        initPaint()
    }
    @SuppressLint("ObjectAnimatorBinding")
    var progressCurrent = 0f // 当前进度
        set(value){
            field = value
            invalidate()
        }
    private  var progressMax = 100 // 最大进度
    private var progressColor:Int = R.color.color_FF21B5FF
    private var progressStartColor:Int =  R.color.color_FF21B5FF
    private var progressEndColor:Int =  R.color.color_FF2EBDFF
    private var progressBackground:Int = Color.argb(16,255,255,255)
    private var progressHeight:Float = 10f
    private var progressBorder:Int = 0
    @SuppressLint("ResourceAsColor")
    private fun obtainStyledAttrs(attrs: AttributeSet){
        val ta = context.obtainStyledAttributes(attrs,R.styleable.VideoHistoryProgress)
        progressMax = ta.getInt(R.styleable.VideoHistoryProgress_progress_max,progressMax)
        progressCurrent = ta.getFloat(R.styleable.VideoHistoryProgress_progress_current,progressCurrent)
        progressColor = ta.getColor(R.styleable.VideoHistoryProgress_progress_color,progressColor)
        progressBackground = ta.getColor(R.styleable.VideoHistoryProgress_progress_background,progressBackground)
        progressStartColor = ta.getColor(R.styleable.VideoHistoryProgress_progress_start_color,progressColor)
        progressEndColor = ta.getColor(R.styleable.VideoHistoryProgress_progress_end_color,progressColor)
        progressHeight = ta.getDimension(R.styleable.VideoHistoryProgress_progress_height,progressHeight)
        progressBorder = ta.getInt(R.styleable.VideoHistoryProgress_progress_border,progressBorder)
        ta.recycle()
    }


    fun setProgressStartColor(color:Int){
        progressStartColor = color
        invalidate()
    }

    fun setProgressEndColor(color:Int){
        progressEndColor = color
        invalidate()
    }

    private lateinit var borerPaint: Paint
    private lateinit var paint: Paint
    private lateinit var bgPaint: Paint
    private lateinit  var path: Path


    @SuppressLint("ResourceAsColor")
    private fun initPaint(){
        paint = Paint()
        paint.style = Paint.Style.FILL
        paint.isAntiAlias = true

        borerPaint = Paint()
        borerPaint.style = Paint.Style.STROKE
        borerPaint.color = progressBackground
        borerPaint.isAntiAlias = true

        bgPaint = Paint()
        bgPaint.style = Paint.Style.FILL
        bgPaint.color = progressBackground
        bgPaint.isAntiAlias = true

        path = Path()
        mPath= Path()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val width = MeasureSpec.getSize(widthMeasureSpec)
        setMeasuredDimension(width,b.toInt())
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        a = 3 * progressHeight
        b = progressHeight
    }

    var mRectF: RectF? = null
    var mPath: Path? = null
    private val mPaintFlagsDrawFilter =  PaintFlagsDrawFilter(0, Paint.ANTI_ALIAS_FLAG or Paint.FILTER_BITMAP_FLAG)
    override fun draw(canvas: Canvas) {
        if(mRectF == null){
            mRectF = RectF(0f,0f, measuredWidth.toFloat(), measuredHeight.toFloat())
        }
        mPath?.addRoundRect(mRectF!!, floatArrayOf(0f,0f,0f,0f,a,b,a,b), Path.Direction.CW)
        canvas.drawFilter = mPaintFlagsDrawFilter
        canvas.clipPath(mPath!!)
        super.draw(canvas)
//        canvas.translate(0f,-b)
//        if(progressBorder!=0){
//            drawBorder(canvas)
//        }
        canvas.drawColor(progressBackground)
        drawProgress1(canvas)
    }

    private var curWidth = 0f
    private var mLinearGradient: LinearGradient? =null
    private var a = 2 * progressHeight
    private var b = progressHeight

    private fun drawProgress1(canvas: Canvas) {
        if(progressMax == 0)return
        if(progressCurrent < 0) progressCurrent = 0f
        curWidth = (width * progressCurrent / progressMax).toFloat()
        mLinearGradient =LinearGradient(0f, b, curWidth,
            2 * b,
            progressStartColor,
            progressEndColor,
            Shader.TileMode.CLAMP)
        paint.shader = mLinearGradient

        canvas.drawRect(RectF(0f ,0f,curWidth  ,measuredHeight.toFloat()),paint)

    }

    @SuppressLint("ResourceAsColor")
    private fun drawProgress(canvas: Canvas){
        if(progressMax == 0)return
        if(progressCurrent < 0f) progressCurrent = 0f
        curWidth = (width * progressCurrent / progressMax).toFloat()
        mLinearGradient = LinearGradient(0f, b, curWidth,
            2 * b,
            progressStartColor,
            progressEndColor,
            Shader.TileMode.CLAMP)
        paint.shader = mLinearGradient
        if(curWidth < a){
            val x =  a - curWidth
            val y =  b * sin(acos(x/a))
            canvas.drawArc(RectF(0F,b-y,2*(a-x),b+y ),90f,90f,true,paint)
        }else{
            canvas.drawArc(RectF(0F,0f,2*a,2*b ),90f,90f,true,paint)
            if(curWidth <= width - a){
                canvas.drawRect(RectF(a - 1,b,curWidth + 1,2*b),paint)
            }else{
                val x1 = a - (width - curWidth)
                val y1 = b * sin(acos(x1/a))
                canvas.drawRect(RectF(a,b,width - a,2*b),paint)
                canvas.drawRect(RectF(width-a,b,width-a+x1,b+y1),paint)
                canvas.drawArc(RectF(width - a -x1,2*y1, width-a+x1,2f*b),0f,90f,true,paint)
            }
        }
    }

    @SuppressLint("ResourceAsColor")
    private fun drawBorder(canvas: Canvas){
        path.addArc(RectF(0F,0f,2 * a,2*b),90f,90f)
        path.moveTo(a,2*b)
        path.lineTo(width- a,2 * b)
        path.moveTo(width- a,2 * b)
        path.addArc(RectF(width - 2 * a,0f, width.toFloat(),2 * b),0f,90f)
        path.moveTo(width.toFloat(),b)
        path.lineTo(0f,b)
        canvas.drawPath(path,borerPaint)
    }

    @Synchronized
    fun startAnimal(progress:Float = 0f,duration:Long = 1000){
        val animator = ObjectAnimator.ofFloat(this, "progressCurrent", 0f, progress)
        animator.duration = duration
        animator.start()
    }
}