package com.at.video.player.custom

import android.content.Context
import android.view.View
import androidx.recyclerview.widget.RecyclerView
import com.at.video.player.R
import com.at.video.player.entity.VideoUrl
import com.at.video.player.ui.player.binder.LevelBinder
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.lxj.xpopup.core.PositionPopupView
import com.melancholy.widget.decoration.DefaultDecoration

/**
 * @Create_time: 2022/4/24 11:19
 * @Description: 选集
 */

class LevelPopup constructor(context: Context): PositionPopupView(context), OnItemClickListener {
    private lateinit var mRecyclerView: RecyclerView
    private val mAdapter: LevelAdapter = LevelAdapter()
    private var mCallback: ((Int) -> Unit)? = null

    override fun getImplLayoutId(): Int {
        return R.layout.player_custom_level
    }

    override fun onCreate() {
        super.onCreate()
        mRecyclerView = findViewById(R.id.recycler_view)
        mRecyclerView.setHasFixedSize(true)
        mRecyclerView.adapter = mAdapter
        mAdapter.setOnItemClickListener(this)
        val padding: Int = context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20)
        mRecyclerView.addItemDecoration(DefaultDecoration.create(leftPadding = padding,
            topPadding = padding, rightPadding = padding, bottomPadding = padding,
            leftAndRightSideOfPadding = padding, topAndBottomSideOfPadding = padding,
            isIncludeEdge = true))
    }

    fun setData(levelData: LevelBinder.LevelData) {
        mAdapter.setNewInstance(levelData.list)
        mAdapter.setIndex(levelData.index)
    }

    /**
     * 选集适配器
     */
    class LevelAdapter: BaseQuickAdapter<VideoUrl, BaseViewHolder>(R.layout.player_custom_item_level) {
        private var mSelectIndex: Int = 0

        override fun convert(holder: BaseViewHolder, item: VideoUrl) {
            holder.setText(R.id.tv_level, "${item.name}")
                .setEnabled(R.id.sfl_level_background,
                    holder.bindingAdapterPosition != mSelectIndex)
        }

        override fun convert(holder: BaseViewHolder, item: VideoUrl, payloads: List<Any>) {
            holder.setEnabled(R.id.sfl_level_background,
                    holder.bindingAdapterPosition != mSelectIndex)
        }

        fun setIndex(index: Int) {
            val prevIndex: Int = mSelectIndex
            mSelectIndex = index
            notifyItemChanged(mSelectIndex, "notify")
            notifyItemChanged(prevIndex, "notify")
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        mCallback?.invoke(position)
        dismiss()
    }

    fun setCallback(callback: (Int) -> Unit) {
        mCallback = callback
    }
}