package com.at.video.player.custom

import cn.jzvd.IReadData
import com.at.video.player.libtorrent.Torrent
import com.at.video.player.libtorrent.TorrentDownload
import com.at.video.player.libtorrent.bean.AppendToFile
import com.at.video.player.libtorrent.bean.Status
import com.at.video.player.ui.player.PlayerActivity
import com.liulishuo.filedownloader.BaseDownloadTask
import com.liulishuo.filedownloader.FileDownloader
import com.melancholy.utils.AppInstance
import java.io.File
import java.io.IOException
import java.io.RandomAccessFile

/**
 *@author ly
 *@name xx
 *@desc xx
 *@since 2023/2/23 16:32
 *@version 1.0.0
 **/
class MagnetReadData : IReadData {

    private var mFile: RandomAccessFile? = null

    /**
     * ijk读取buffer回调
     */
    var currentPositionTime: Long = 0
    var readPosition: Long = 0
    var readPositionTime: Long = 0

    private var videoReadPosition: Long = 0

    private var requestPos: Long = 0

    private lateinit var mTorrentDownload: TorrentDownload

    private var ismplayvideos = false

    var onMagnetReadListener: OnMagnetReadListener? = null

    /**
     * 需要块下载
     *
     * @param position
     */
    private var dealDownloading = false

    constructor(url: String, download: TorrentDownload) {
        mTorrentDownload = download
        val moviePath: File = File(url)
        if (moviePath.exists() && moviePath.isFile && moviePath.canRead()) {
            mFile = RandomAccessFile(moviePath, "r")
        }
    }

    @Throws(IOException::class)
    override fun readData(position: Long, buffer: ByteArray, offset: Int, size: Int): Int {
        //在1s内不能连续读取数据
        videoReadPosition = position
        if (readPosition == position && System.currentTimeMillis() - readPositionTime > 999) {
            try {
                Thread.sleep(1000)
            } catch (e: InterruptedException) {
                e.printStackTrace()
            }
        }
        readPosition = position
        readPositionTime = System.currentTimeMillis()
        if (mFile == null) {
            return 0
        }
        try {
            if (mFile!!.filePointer != position) {
                mFile!!.seek(position)
            }
        } catch (e: java.lang.Exception) {
        }
        // 未播放前读哪就优先下哪
        if (mTorrentDownload != null && mTorrentDownload!!.entity.state != Status.STATE_FINISHED) {
            try {
                val torrent = mTorrentDownload!!.currentTorrent
                if (existAvailableTorrent(torrent)) {
                    val pieceLength = torrent.torrentHandle.torrentFile().pieceLength()
                    //未播放的情况下优先下载需要读取请求的块
                    var i = 0
                    while (i < size) {

                        //判断当前buffer与下个buffer
                        if (torrent.torrentHandle == null) {
                            break
                        }
                        val downed = !torrent.hasPlayBytes(position + i)
                        if (downed) {
                            //多线程下载制定的需要块
                            val requestPiece = (position / pieceLength).toInt()
                            var currentPiece = 0
                            if (mTorrentDownload!!.headNum > 0) {
                                currentPiece = mTorrentDownload!!.headNum - 1
                            }
                            if (requestPiece >= currentPiece && System.currentTimeMillis() - currentPositionTime >= 2000) {
                                currentPositionTime = System.currentTimeMillis()
                                blockDownloadRequired(position)
                            }
                            torrent.setInterestedBytes(position + i, ismplayvideos)
                            return 0
                        }
                        i += pieceLength
                    }
                }
            } catch (e: java.lang.Exception) {
            }
        }
        return try {
            mFile!!.read(buffer, offset, size)
        } catch (e: java.lang.Exception) {
            0
        }
    }

    override fun getSize(): Long {
        var size: Long = 0
        try {
            size = mFile?.length() ?: 0
        } catch (e: java.lang.Exception) {
        }
        return size
    }


    @Throws(IOException::class)
    override fun close() {
        mFile?.close()
    }

    var resetPlayTime: Long = 0

    @Throws(IOException::class)
    override fun setData() {
        // mVideoView.stopPlayback();
        //设置当前下载的优先级
        if (mTorrentDownload == null || mTorrentDownload!!.currentTorrent == null) {
            return
        }
        val HeadNum = mTorrentDownload.headNum
        mTorrentDownload!!.currentTorrent.setInterested()
        if (HeadNum == 0 && System.currentTimeMillis() - resetPlayTime > 999) {
            resetPlayTime = System.currentTimeMillis()
            onMagnetReadListener?.onLoadVideo(5)
        }
    }

    /**
     * 按块下载
     */
    private fun blockDownloadRequired(position: Long) {
        try {
            if (dealDownloading) {
                return
            }
            if (mTorrentDownload == null || !existAvailableTorrent(mTorrentDownload!!.currentTorrent)) {
                return
            }
            if (mTorrentDownload!!.piece_size == 0L) {
                return
            }
            val torrent = mTorrentDownload!!.currentTorrent
            val pieceIndex = (position / torrent.torrentHandle.torrentFile().pieceLength()).toInt()
            if (requestPos == position) { //没有第一次播放启动时
                if (!torrent.hasPieces[pieceIndex]) {
                    requestPos = 0
                }
                return
            }
            dealDownloading = true
            requestPos = position
            val url = mTorrentDownload!!.seedUrl //下载3块
            val MaxNum = 3
            val header = "bytes=" + position + "-" + (position + MaxNum * 512 * 1024)
            val tempPath: String =
                AppInstance.getApplication().cacheDir.absolutePath + "player/" + "temp.mp4"
            val file = File(tempPath)
            if (!file.exists()) {
                try {
                    file.createNewFile()
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
            //下载指定位置块
            FileDownloader.getImpl().create(url).setPath(tempPath).addHeader("Range", header)
                .setAutoRetryTimes(3).setListener(
                    DownloadSpecifiedLocationBlockListener(object :
                        DownloadSpecifiedLocationBlockListener.DownloadListener {
                        override fun completed(task: BaseDownloadTask) {
                            var isSuccess = false
                            if (existAvailableTorrent(torrent)) {
                                isSuccess = AppendToFile.addTorrentData(
                                    File(tempPath),
                                    torrent.torrentHandle,
                                    mTorrentDownload!!.piece_size.toInt(),
                                    pieceIndex,
                                    MaxNum
                                )
                            }
                            if (mTorrentDownload != null) {


                            }
                            if (file.exists()) {
                                file.delete()
                            }
                            dealDownloading = false
                            if (isSuccess) {
                                onMagnetReadListener?.onLoadVideo(2)
                            }
                        }

                        override fun error(task: BaseDownloadTask?, e: Throwable?) {
                        }
                    })
                ).setCallbackProgressMinInterval(1000).start()
        } catch (e: java.lang.Exception) {

        }

    }

    /**
     * 判断是否可用的torrent
     *
     * @param torrent
     * @return
     */
    private fun existAvailableTorrent(torrent: Torrent?): Boolean {
        return torrent != null && torrent.torrentHandle != null && torrent.torrentHandle.isValid
    }

    interface OnMagnetReadListener {
        fun onLoadVideo(type: Int)
    }
}
