package com.at.video.player.custom

import android.annotation.SuppressLint
import android.content.Context
import android.content.pm.PackageManager
import android.media.AudioManager
import android.provider.Settings
import android.provider.Settings.SettingNotFoundException
import android.util.AttributeSet
import android.util.Log
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.widget.*
import cn.jzvd.IReadData
import cn.jzvd.JZDataSource
import cn.jzvd.JZUtils
import cn.jzvd.JzvdStd
import com.at.common.global.Constants
import com.at.video.player.R
import com.at.video.player.entity.VideoBean
import com.at.video.player.ui.player.binder.LevelBinder
import com.google.android.material.button.MaterialButton
import com.lxj.xpopup.XPopup
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.DiversityImageView
import com.melancholy.widget.shape.layout.ShapeLinearLayout
import com.melancholy.widget.shape.view.ShapeTextView
import com.video.module_widget.entity.Adv
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.getPx
import com.video.module_widget.utils.loadByGlide
import tv.danmaku.ijk.media.player.misc.IMediaDataSource
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/4/18 17:19
 * @Description:
 */

class PlayerView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null) :
    JzvdStd(context, attrs), CountdownTextView.OnSkipListener {
    //竖屏状态下单独的返回按钮
    private lateinit var mIvBack: ImageView

    //竖屏状态下顶部操作栏
    private lateinit var mRlTopControl: RelativeLayout

    //横屏状态下顶部操作栏
    private lateinit var mRlLandscapeTopControl: RelativeLayout

    //横屏状态下右边操作栏
    private lateinit var mLlRightControl: LinearLayout

    //竖屏状态下底部操作栏
    private lateinit var mLlBottomControl: LinearLayout
    private lateinit var mIvPlayerStatus: ImageView
    private lateinit var mTvTime: TextView

    //横屏状态下底部操作栏
    private lateinit var mLlLandscapeBottomControl: LinearLayout
    private lateinit var mIvLandscapePlayerStatus: ImageView
    private lateinit var mSkLandscapeProgress: SeekBar

    //进度拖动
    private lateinit var mStvDragProgress: ShapeTextView

    //亮度
    private lateinit var mSllDragBrightnessProgress: ShapeLinearLayout
    private lateinit var mPbDragBrightnessProgress: ProgressBar

    //亮度
    private lateinit var mSllDragVolumeProgress: ShapeLinearLayout
    private lateinit var mPbDragVolumeProgress: ProgressBar

    //loading记载框
    private lateinit var mSllLoading: ShapeLinearLayout

    //调节倍速
    private var mSpeedPopup: SpeedPopup? = null

    private var mOnViewClickListener: OnViewClickListener? = null

    //判断是否处于画中画
    private var mIsInPictureInPictureMode: Boolean = false

    //高度
    private var mViewHeight: Int = 0

    //是否在加载中
    private var mIsLoading: Boolean = false
    private lateinit var mInitLoading: ShapeLinearLayout
    private lateinit var mInitRetry: LinearLayout
    private lateinit var mIvAd: ImageView
    private lateinit var mCtvTimer: CountdownTextView
    private var mOnSkipListener: CountdownTextView.OnSkipListener? = null
    private lateinit var mFlPauseAd: FrameLayout
    private lateinit var mDivPauseAd: DiversityImageView
    private lateinit var mTvLevel: TextView
    private var mLevelPopup: LevelPopup? = null
    private var mLevelData: LevelBinder.LevelData? = null

    //下一集
    private lateinit var mIvNext: ImageView

    //
    private lateinit var mLlMember: LinearLayout
    private lateinit var mMbBuyGold: MaterialButton

    private lateinit var replayText: View

    /**
     * 返回自定义控制布局
     */
    override fun getLayoutId(): Int {
        return R.layout.player_custom_control
    }

    override fun init(context: Context) {
        super.init(context)
        SAVE_PROGRESS = false
        mIvBack = findViewById(R.id.iv_back)
        mIvBack.setOnClickListener(this)
        findViewById<View>(R.id.iv_landscape_back).setOnClickListener(this)
        mRlTopControl = findViewById(R.id.rl_top_control)
        mRlLandscapeTopControl = findViewById(R.id.rl_landscape_top_control)
        mLlRightControl = findViewById(R.id.ll_right_control)
        mLlBottomControl = findViewById(R.id.ll_bottom_control)
        mIvPlayerStatus = findViewById(R.id.iv_player_status)
        mTvTime = findViewById(R.id.tv_time)
        mLlLandscapeBottomControl = findViewById(R.id.ll_landscape_bottom_control)
        mIvLandscapePlayerStatus = findViewById(R.id.iv_landscape_player_status)
        mSkLandscapeProgress = findViewById(R.id.sk_landscape_progress)
        mSkLandscapeProgress.setOnSeekBarChangeListener(SeekBarChangeListener())
        mStvDragProgress = findViewById(R.id.stv_drag_progress)
        mSllDragBrightnessProgress = findViewById(R.id.sll_drag_brightness_progress)
        mPbDragBrightnessProgress = findViewById(R.id.pb_drag_brightness_progress)
        mSllDragVolumeProgress = findViewById(R.id.sll_drag_volume_progress)
        mPbDragVolumeProgress = findViewById(R.id.pb_drag_volume_progress)
        mSllLoading = findViewById(R.id.sll_loading)
        mInitLoading = findViewById(R.id.sll_init_loading)
        mInitRetry = findViewById(R.id.ll_init_retry)
        mIvAd = findViewById(R.id.iv_ad)
        mIvAd.setOnClickListener(this)
        mCtvTimer = findViewById(R.id.ctv_timer)
        mCtvTimer.setOnSkipListener(this)
        mFlPauseAd = findViewById(R.id.fl_pause_ad)
        mFlPauseAd.setOnClickListener(this)
        mDivPauseAd = findViewById(R.id.div_pause_ad)
        findViewById<View>(R.id.iv_pause_close).setOnClickListener(this)
        //播放按钮
        mIvPlayerStatus.setOnClickListener(this)
        mIvLandscapePlayerStatus.setOnClickListener(this)
        //全屏按钮
        findViewById<View>(R.id.iv_fullscreen).setOnClickListener(this)
        //倍速
        findViewById<View>(R.id.tv_speed).setOnClickListener(this)
        //切换画中画
        val isPInP: Boolean =
            context.packageManager.hasSystemFeature(PackageManager.FEATURE_PICTURE_IN_PICTURE)
        val ivSuspension: ImageView = findViewById(R.id.iv_suspension)
        ivSuspension.visibility = if (isPInP) VISIBLE else INVISIBLE
        ivSuspension.setOnClickListener(this)
        val ivBottomSuspension: ImageView = findViewById(R.id.iv_bottom_suspension)
        ivBottomSuspension.visibility = if (isPInP) VISIBLE else INVISIBLE
        ivBottomSuspension.setOnClickListener(this)
        //选集
        mTvLevel = findViewById(R.id.tv_level)
        mTvLevel.setOnClickListener(this)
        mIvNext = findViewById(R.id.iv_next)
        mIvNext.setOnClickListener(this)
        mLlMember = findViewById(R.id.ll_member)
        mMbBuyGold = findViewById(R.id.mb_buy_gold)
        mMbBuyGold.setOnClickListener(this)
        replayText = findViewById(R.id.replay_text)
        replayText.setOnClickListener(this)
        findViewById<View>(R.id.stv_buy_member).setOnClickListener(this)
    }

    /**
     * 设置loading
     */
    fun loading() {
        releaseAllVideos()
        mRlTopControl.visibility = INVISIBLE
        mIsLoading = true
        mInitLoading.visibility = VISIBLE
        mInitRetry.visibility = INVISIBLE
    }

//    override fun setUp(jzDataSource: JZDataSource?, screen: Int, mediaInterfaceClass: Class<*>?) {
//        super.setUp(jzDataSource, screen, Jz::class.java)
//    }

    /**
     * 播放失败
     */
    fun error() {
        mInitLoading.visibility = VISIBLE
        mInitRetry.visibility = INVISIBLE
    }

    fun setOnSkipListener(l: CountdownTextView.OnSkipListener?) {
        mOnSkipListener = l
    }

    fun setAd(info: AdvertisingBean) {
        mIvAd.tag = info
        mIvAd.loadByGlide(
            GlideImageConfig(
                info.image_url.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                errorId = com.video.module_widget.R.drawable.common_ic_ad_default
            ), WeakReference(context)
        )
        mCtvTimer.setSumTime(info.show_time * 1000)
        mCtvTimer.startTimer()
        mIvAd.visibility = VISIBLE
        mCtvTimer.visibility = VISIBLE
        mInitLoading.visibility = View.INVISIBLE
        mInitRetry.visibility = View.INVISIBLE
        mLlMember.visibility = INVISIBLE
    }

    fun setPauseAd(info: AdvertisingBean, style: Adv? = null) {
        mFlPauseAd.tag = info
        mDivPauseAd.loadByGlide(
            GlideImageConfig(
                info.image_url.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                errorId = com.video.module_widget.R.drawable.common_ic_ad_default
            ), WeakReference(context)
        )
        style?.let {
            mDivPauseAd.setFullRadius(context.getPx(it.radius).toFloat())
        }

        mFlPauseAd.visibility = VISIBLE
        val params: ViewGroup.LayoutParams = mFlPauseAd.layoutParams
        if (screen == SCREEN_FULLSCREEN) {
            params.width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_350)
            params.height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_200)
        } else {
            params.width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_198)
            params.height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_113)
        }
        mFlPauseAd.layoutParams = params
    }

    fun hidePauseAd() {
        mFlPauseAd.visibility = View.GONE
    }

    fun hideAdUi() {
        mIvAd.visibility = INVISIBLE
        mCtvTimer.visibility = INVISIBLE
    }

    fun setLevelData(levelData: LevelBinder.LevelData) {
        mLevelData = levelData
        mIvNext.visibility = if (levelData.index + 1 < levelData.list.size) VISIBLE else INVISIBLE
    }

    fun setOnViewClickLister(listener: OnViewClickListener?) {
        mOnViewClickListener = listener
    }

    fun setRead(iReadData: IReadData?) {
        this.iReadData = iReadData
    }

    override fun setUp(url: String?, title: String?) {
        super.setUp(url, title)
    }

    override fun setUp(jzDataSource: JZDataSource?, screen: Int, mediaInterfaceClass: Class<*>?) {
        super.setUp(jzDataSource, screen, JZCustomReaderMediaIjk::class.java)
    }

    fun setPay(info: VideoBean) {
        mLlMember.visibility = VISIBLE
        mMbBuyGold.text = "${info.price}金币"
        mIvAd.visibility = INVISIBLE
        mCtvTimer.visibility = INVISIBLE
        mInitLoading.visibility = INVISIBLE
        mInitRetry.visibility = INVISIBLE
    }

    override fun startVideo() {
        mIsLoading = false
        mLlMember.visibility = INVISIBLE
        mIvAd.visibility = INVISIBLE
        mCtvTimer.visibility = INVISIBLE
        mInitLoading.visibility = INVISIBLE
        mInitRetry.visibility = INVISIBLE
        super.startVideo()
    }

    fun setIsInPictureInPictureMode(isInPictureInPictureMode: Boolean) {
        mIsInPictureInPictureMode = isInPictureInPictureMode
        val params = layoutParams
        if (isInPictureInPictureMode) {
            mViewHeight = if (screen == SCREEN_FULLSCREEN) {
                measuredWidth
            } else {
                measuredHeight
            }
            params.height = ViewGroup.LayoutParams.MATCH_PARENT
        } else {
            params.height = mViewHeight
        }
        layoutParams = params
        setAllControlsVisiblity(
            INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
        )
    }

    /**
     * 点击事件
     */
    override fun onClick(v: View) {
        when (v.id) {
            R.id.iv_back, R.id.iv_landscape_back -> {
                startDismissControlViewTimer()
                setVideoImageDisplayType(VIDEO_IMAGE_DISPLAY_TYPE_ADAPTER)
                mOnViewClickListener?.onBack()
            }
            //切换全屏
            R.id.iv_fullscreen -> {
                startDismissControlViewTimer()
                gotoFullscreen()
                setVideoImageDisplayType(VIDEO_IMAGE_DISPLAY_TYPE_FILL_SCROP)
            }
            //倍速
            R.id.tv_speed -> {
                cancelDismissControlViewTimer()
                if (mSpeedPopup == null) {
                    mSpeedPopup = SpeedPopup(context)
                    mSpeedPopup!!.setCallback(::callbackSpeed)
                }
                val popupWidth: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_210)
                val popupHeight: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_290)
                val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_50)

                XPopup.Builder(context).isViewMode(true).offsetX(width - popupWidth - padding)
                    .offsetY((height - popupHeight) / 2).asCustom(mSpeedPopup).show()
                changeUiToControlsClear()
                Log.d("TAG123456", "tv_speed")
            }
            //选集
            R.id.tv_level -> {
                cancelDismissControlViewTimer()
                val popupWidth: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_299)
                val popupHeight: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_290)
                val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_50)
                if (mLevelPopup == null) {
                    mLevelPopup = LevelPopup(context)
                    mLevelPopup!!.setCallback(::callbackLevel)
                }
                mLevelPopup!!.setData(mLevelData!!)
                XPopup.Builder(context).isViewMode(true).offsetX(width - popupWidth - padding)
                    .offsetY((height - popupHeight) / 2).asCustom(mLevelPopup!!).show()
                changeUiToControlsClear()
            }
            //播放按钮
            R.id.iv_player_status, R.id.replay_text, R.id.iv_landscape_player_status -> {
                startDismissControlViewTimer()
                clickStart()
            }
            //画中画
            R.id.iv_suspension, R.id.iv_bottom_suspension -> {
                cancelDismissControlViewTimer()
                mOnViewClickListener?.onSuspension()
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
                )
            }
            R.id.iv_pause_close -> {
                mFlPauseAd.visibility = INVISIBLE
            }
            R.id.iv_next -> {
                mOnViewClickListener?.onLevel(mLevelData!!, mLevelData!!.index + 1)
            }
            //买金币
            R.id.mb_buy_gold -> {
//                Router.getInstance().build("/member/GoldActivity").navigation()
                mOnViewClickListener?.onPay()
            }
            //买会员
            R.id.stv_buy_member -> {
                Router.getInstance().build("/member/RechargeActivity").navigation()
            }
            R.id.iv_ad, R.id.fl_pause_ad -> {
                val data: AdvertisingBean = v.tag as AdvertisingBean
                if (!data.jump_addr.isNullOrEmpty()) {
                    Router.getInstance().build("/webview/WebViewActivity")
                        .withString(Constants.KEY_DATA, data.jump_addr!!).navigation()
                }
            }
            else -> {
                if (mIsLoading) {
                    return
                }
                super.onClick(v)
            }
        }
    }

    override fun clickStart() {
        super.clickStart()
        if (state == STATE_PLAYING) {
            mFlPauseAd.visibility = INVISIBLE
        }
        mOnViewClickListener?.onPauseOrPlay()
    }

    override fun changeUiToError() {
        when (screen) {
            SCREEN_NORMAL -> {
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, VISIBLE
                )
                updateStartImage()
            }
            SCREEN_FULLSCREEN -> {
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, VISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, VISIBLE
                )
                updateStartImage()
            }
            SCREEN_TINY -> {}
        }
    }

    fun clickStartOrPause() {
        clickStart()
    }

    private fun callbackSpeed(speed: Float) {
        mediaInterface.setSpeed(speed)
    }

    private fun callbackLevel(position: Int) {
        mOnViewClickListener?.onLevel(mLevelData!!, position)
    }

    fun seekTo(time: Long) {
        if (mediaInterface != null) {
            mediaInterface.seekTo(time)
        }
    }

    /**
     * 隐藏布局
     */
    override fun dissmissControlView() {
        /**
         * 当状态不等于NORMAL 不等于错误 不等于完成
         */
        if (state != STATE_NORMAL && state != STATE_ERROR && state != STATE_AUTO_COMPLETE) {
            Log.d("TAG123456", "dissmissControlView")
            post {
                cancelDismissControlViewTimer()
                changeUiToControlsClear()
            }
        }
    }

    /**
     * 加载中
     */
    override fun changeUiToPreparing() {
        when (screen) {
            SCREEN_NORMAL, SCREEN_FULLSCREEN -> {
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, INVISIBLE, VISIBLE, VISIBLE, INVISIBLE, INVISIBLE
                )
                updateStartImage()
            }
            SCREEN_TINY -> {}
        }
        Log.d("TAG123456", "changeUiToPreparing")
    }

    /**
     * 更新播放状态图标
     */
    override fun updateStartImage() {
        super.updateStartImage()
        when (state) {
            STATE_PLAYING -> {
                mIvPlayerStatus.setImageResource(R.drawable.player_ic_pause)
                mIvLandscapePlayerStatus.setImageResource(R.drawable.player_ic_pause)
            }
            STATE_ERROR -> {
                mIvPlayerStatus.setImageResource(R.drawable.player_ic_player)
                mIvLandscapePlayerStatus.setImageResource(R.drawable.player_ic_player)
            }
            STATE_AUTO_COMPLETE -> {
                mIvPlayerStatus.setImageResource(cn.jzvd.R.drawable.jz_click_replay_selector)
                mIvLandscapePlayerStatus.setImageResource(cn.jzvd.R.drawable.jz_click_replay_selector)
                replayTextView.visibility = VISIBLE
            }
            else -> {
                mIvPlayerStatus.setImageResource(R.drawable.player_ic_player)
                mIvLandscapePlayerStatus.setImageResource(R.drawable.player_ic_player)
            }
        }
    }

    override fun setScreenNormal() {
        super.setScreenNormal()
        mIvBack.visibility = VISIBLE
        mRlTopControl.visibility = VISIBLE
        mRlLandscapeTopControl.visibility = INVISIBLE
        //mLlRightControl.visibility = INVISIBLE
        mLlBottomControl.visibility = VISIBLE
        mLlLandscapeBottomControl.visibility = INVISIBLE
    }

    override fun setScreenFullscreen() {
        super.setScreenFullscreen()
        mIvBack.visibility = INVISIBLE
        mRlTopControl.visibility = INVISIBLE
        mRlLandscapeTopControl.visibility = VISIBLE
        //mLlRightControl.visibility = VISIBLE
        mLlBottomControl.visibility = INVISIBLE
        mLlLandscapeBottomControl.visibility = VISIBLE
        Log.d("TAG123456", "setScreenFullscreen")
    }

    override fun onStatePlaying() {
        super.onStatePlaying()
        Log.d("TAG123456", "onStatePlaying $state")
    }

    override fun changeUrl(jzDataSource: JZDataSource?, seekToInAdvance: Long) {
        super.changeUrl(jzDataSource, seekToInAdvance)
        Log.d("TAG12.3456", "changeUrl")
    }

    /**
     * 准备好了
     */
    override fun onPrepared() {
        super.onPrepared()
        onStatePlaying()
        mOnViewClickListener?.onPrepared()
    }

    override fun onCompletion() {
        if (screen == SCREEN_FULLSCREEN) {
            onStateAutoComplete()
        } else {
            super.onCompletion()
        }
        if (mLevelData!!.index + 1 < mLevelData!!.list.size) {
            mOnViewClickListener?.onLevel(mLevelData!!, mLevelData!!.index + 1)
        }
    }

    @SuppressLint("SetTextI18n")
    override fun onStateAutoComplete() {
        super.onStateAutoComplete()
        mSkLandscapeProgress.progress = 100
        val total: String = totalTimeTextView.text.toString()
        mTvTime.text = "$total/$total"
        Log.d("TAG123456", "onStateAutoComplete")
    }

    /**
     * 播放显示
     */
    override fun changeUiToPlayingShow() {
        super.changeUiToPlayingShow()
        Log.d("TAG123456", "changeUiToPlayingShow")
    }

    /**
     * 暂停显示
     */
    override fun changeUiToPauseShow() {
        super.changeUiToPauseShow()
        Log.d("TAG123456", "changeUiToPauseShow")
    }

    /**
     * 暂停清除
     */
    override fun changeUiToPauseClear() {
        super.changeUiToPauseClear()
        Log.d("TAG123456", "changeUiToPauseClear $state")
    }

    /**
     * 播放完成
     */
    override fun changeUiToComplete() {
        Log.d("TAG123456", "changeUiToComplete")
        when (screen) {
            SCREEN_NORMAL, SCREEN_FULLSCREEN -> {
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
                )
                updateStartImage()
            }
            SCREEN_TINY -> {}
        }
    }

    /**
     * 最底层的点击事件
     */
    override fun onClickUiToggle() {
        if (mIsLoading) {
            return
        }
        //这是事件
        if (bottomContainer.visibility != VISIBLE) {
            clarity.text = jzDataSource.currentKey.toString()
        }
        if (state == STATE_PREPARING) {
            changeUiToPreparing()
        } else if (state == STATE_PLAYING) {
            if (bottomContainer.visibility == VISIBLE) {
                changeUiToPlayingClear()
            } else {
                changeUiToPlayingShow()
            }
        } else if (state == STATE_PAUSE) {
            if (bottomContainer.visibility == VISIBLE) {
                changeUiToPauseClear()
            } else {
                changeUiToPauseShow()
            }
        } else if (state == STATE_PREPARING_PLAYING) {
            startDismissControlViewTimer()
            if (bottomContainer.visibility == VISIBLE) {
                changeUiToPreparingClear()
            } else {
                changeUiToPreparingShow()
            }
        }
    }

    private fun changeUiToPreparingShow() {
        setAllControlsVisiblity(
            VISIBLE, VISIBLE, INVISIBLE, VISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
        )
    }

    private fun changeUiToPreparingClear() {
        setAllControlsVisiblity(
            INVISIBLE, INVISIBLE, INVISIBLE, VISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
        )
    }

    override fun changeStartButtonSize(size: Int) {
        var lp = startButton.layoutParams
        lp.height = size
        lp.width = size
        startButton.layoutParams = lp
        lp = loadingProgressBar.layoutParams
        val loadingSize: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20)
        lp.height = loadingSize
        lp.width = loadingSize
        loadingProgressBar.layoutParams = lp
        val params: ViewGroup.LayoutParams = mFlPauseAd.layoutParams
        if (screen == SCREEN_FULLSCREEN) {
            params.width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_350)
            params.height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_200)
        } else {
            params.width = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_198)
            params.height = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_113)
        }
        mFlPauseAd.layoutParams = params
        Log.d("TAG123456", "changeStartButtonSize")
    }

    override fun setAllControlsVisiblity(
        topCon: Int,
        bottomCon: Int,
        startBtn: Int,
        loadingPro: Int,
        posterImg: Int,
        bottomPro: Int,
        retryLayout: Int
    ) {
        super.setAllControlsVisiblity(
            topCon, bottomCon, startBtn, loadingPro, posterImg, bottomPro, retryLayout
        )
        startButton.visibility = if (mIsInPictureInPictureMode) INVISIBLE else startBtn
        mSllLoading.visibility = if (mIsInPictureInPictureMode) INVISIBLE else loadingPro
        when {
            mIsInPictureInPictureMode -> {
                mIvBack.visibility = INVISIBLE
                mRlLandscapeTopControl.visibility = INVISIBLE
                //mLlRightControl.visibility = INVISIBLE
                mLlLandscapeBottomControl.visibility = INVISIBLE
                mRlTopControl.visibility = INVISIBLE
                mLlBottomControl.visibility = INVISIBLE
            }
            screen == SCREEN_FULLSCREEN -> {
                mIvBack.visibility =
                    if (retryLayout == VISIBLE || state == STATE_AUTO_COMPLETE) VISIBLE else INVISIBLE
                mRlLandscapeTopControl.visibility = topCon
                //mLlRightControl.visibility = topCon
                mLlLandscapeBottomControl.visibility = bottomCon
                mRlTopControl.visibility = INVISIBLE
                mLlBottomControl.visibility = INVISIBLE
            }
            else -> {
                mIvBack.visibility = VISIBLE
                mRlLandscapeTopControl.visibility = INVISIBLE
                //mLlRightControl.visibility = INVISIBLE
                mLlLandscapeBottomControl.visibility = INVISIBLE
                mRlTopControl.visibility = topCon
                mLlBottomControl.visibility = bottomCon
            }
        }
        Log.d("TAG123456", "setAllControlsVisiblity")
    }

    override fun onStatePreparingPlaying() {
        super.onStatePreparingPlaying()
        Log.d("TAG123456", "onStatePreparingPlaying")
    }

    override fun changeUIToPreparingPlaying() {
        Log.d("TAG123456", "changeUIToPreparingPlaying $screen")
        when (screen) {
            SCREEN_NORMAL, SCREEN_FULLSCREEN -> {
                setAllControlsVisiblity(
                    INVISIBLE, INVISIBLE, INVISIBLE, VISIBLE, INVISIBLE, INVISIBLE, INVISIBLE
                )
                updateStartImage()
            }
            SCREEN_TINY -> {}
        }
    }

    @SuppressLint("SetTextI18n")
    override fun onProgress(progress: Int, position: Long, duration: Long) {
        super.onProgress(progress, position, duration)
        mSkLandscapeProgress.progress = progress
        mTvTime.text = "${currentTimeTextView.text}/${totalTimeTextView.text}"
    }

    override fun setBufferProgress(bufferProgress: Int) {
        super.setBufferProgress(bufferProgress)
        mSkLandscapeProgress.secondaryProgress = bufferProgress
    }

    @SuppressLint("SetTextI18n")
    override fun resetProgressAndTime() {
        super.resetProgressAndTime()
        mSkLandscapeProgress.progress = 0
        mSkLandscapeProgress.secondaryProgress = 0
        mTvTime.text = "${currentTimeTextView.text}/${totalTimeTextView.text}"
    }

    @SuppressLint("SetTextI18n")
    override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
        super.onProgressChanged(seekBar, progress, fromUser)
        if (fromUser) {
            mTvTime.text = "${currentTimeTextView.text}/${totalTimeTextView.text}"
        }
    }


    internal inner class SeekBarChangeListener : SeekBar.OnSeekBarChangeListener {
        @SuppressLint("SetTextI18n")
        override fun onProgressChanged(seekBar: SeekBar, progress: Int, fromUser: Boolean) {
            if (fromUser) {
                currentTimeTextView.text = JZUtils.stringForTime(progress * duration / 100)
                mTvTime.text = "${currentTimeTextView.text}/${totalTimeTextView.text}"
            }
        }

        override fun onStartTrackingTouch(seekBar: SeekBar) {
            cancelProgressTimer()
            var vpdown = parent
            while (vpdown != null) {
                vpdown.requestDisallowInterceptTouchEvent(true)
                vpdown = vpdown.parent
            }
            cancelDismissControlViewTimer()
        }

        override fun onStopTrackingTouch(seekBar: SeekBar) {
            startProgressTimer()
            var vpup = parent
            while (vpup != null) {
                vpup.requestDisallowInterceptTouchEvent(false)
                vpup = vpup.parent
            }
            if (state != STATE_PLAYING && state != STATE_PAUSE) return
            val time = seekBar.progress * duration / 100
            val total = duration
            if (time <= total && total > 0) {
                seekToManulPosition = seekBar.progress
                mediaInterface.seekTo(time)
                startDismissControlViewTimer()
            }
        }

    }

    override fun touchActionMove(x: Float, y: Float) {
        val deltaX = x - mDownX
        var deltaY = y - mDownY
        val absDeltaX = Math.abs(deltaX)
        val absDeltaY = Math.abs(deltaY)
        if (screen == SCREEN_FULLSCREEN || screen == SCREEN_NORMAL) {
            //拖动的是NavigationBar和状态栏
            if (mDownX > JZUtils.getScreenWidth(context) || mDownY < JZUtils.getStatusBarHeight(
                    context
                )
            ) {
                return
            }
            if (!mChangePosition && !mChangeVolume && !mChangeBrightness) {
                if (absDeltaX > THRESHOLD || absDeltaY > THRESHOLD) {
                    cancelProgressTimer()
                    if (absDeltaX >= THRESHOLD) {
                        // 全屏模式下的CURRENT_STATE_ERROR状态下,不响应进度拖动事件.
                        // 否则会因为mediaplayer的状态非法导致App Crash
                        if (state != STATE_ERROR) {
                            mChangePosition = true
                            mGestureDownPosition = currentPositionWhenPlaying
                        }
                    } else {
                        //如果y轴滑动距离超过设置的处理范围，那么进行滑动事件处理
                        if (mDownX < measuredWidth * 0.5f) { //左侧改变亮度
                            mChangeBrightness = true
                            val lp = JZUtils.getWindow(context).attributes
                            if (lp.screenBrightness < 0) {
                                try {
                                    mGestureDownBrightness = Settings.System.getInt(
                                        context.contentResolver, Settings.System.SCREEN_BRIGHTNESS
                                    ).toFloat()
                                    Log.i(
                                        TAG, "current system brightness: $mGestureDownBrightness"
                                    )
                                } catch (e: SettingNotFoundException) {
                                    e.printStackTrace()
                                }
                            } else {
                                mGestureDownBrightness = lp.screenBrightness * 255
                                Log.i(
                                    TAG, "current activity brightness: $mGestureDownBrightness"
                                )
                            }
                        } else { //右侧改变声音
                            mChangeVolume = true
                            mAudioManager?.apply {
                                mGestureDownVolume = getStreamVolume(AudioManager.STREAM_MUSIC)
                            }
                        }
                    }
                }
            }
        }
        if (mChangePosition) {
            val totalTimeDuration = duration
            if (PROGRESS_DRAG_RATE <= 0) {
                Log.d(TAG, "error PROGRESS_DRAG_RATE value")
                PROGRESS_DRAG_RATE = 1f
            }
            mSeekTimePosition =
                (mGestureDownPosition + deltaX * totalTimeDuration / (measuredWidth * PROGRESS_DRAG_RATE)).toLong()
            if (mSeekTimePosition > totalTimeDuration) mSeekTimePosition = totalTimeDuration
            val seekTime = JZUtils.stringForTime(mSeekTimePosition)
            val totalTime = JZUtils.stringForTime(totalTimeDuration)
            showProgressDialog(deltaX, seekTime, mSeekTimePosition, totalTime, totalTimeDuration)
        }
        if (mChangeVolume) {
            deltaY = -deltaY
            val max = mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC)
            val deltaV = (max * deltaY * 3 / measuredHeight).toInt()
            mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC, mGestureDownVolume + deltaV, 0)
            //dialog中显示百分比
            val volumePercent =
                (mGestureDownVolume * 100 / max + deltaY * 3 * 100 / measuredHeight).toInt()
            showVolumeDialog(-deltaY, volumePercent)
        }

        if (mChangeBrightness) {
            deltaY = -deltaY
            val deltaV = (255 * deltaY * 3 / measuredHeight).toInt()
            val params = JZUtils.getWindow(context).attributes
            if ((mGestureDownBrightness + deltaV) / 255 >= 1) { //这和声音有区别，必须自己过滤一下负值
                params.screenBrightness = 1f
            } else if ((mGestureDownBrightness + deltaV) / 255 <= 0) {
                params.screenBrightness = 0.01f
            } else {
                params.screenBrightness = (mGestureDownBrightness + deltaV) / 255
            }
            JZUtils.getWindow(context).attributes = params
            //dialog中显示百分比
            val brightnessPercent =
                (mGestureDownBrightness * 100 / 255 + deltaY * 3 * 100 / measuredHeight).toInt()
            showBrightnessDialog(brightnessPercent)
//                        mDownY = y;
        }
    }

    /**
     * 显示亮度
     */
    override fun showBrightnessDialog(brightnessPercent: Int) {
        if (mSllDragBrightnessProgress.visibility != View.VISIBLE) {
            mSllDragBrightnessProgress.visibility = View.VISIBLE
        }
        mPbDragBrightnessProgress.progress = when {
            brightnessPercent > 100 -> 100
            brightnessPercent < 0 -> 0
            else -> brightnessPercent
        }
        changeUiToControlsClear()
    }

    /**
     * 隐藏亮度
     */
    override fun dismissBrightnessDialog() {
        mSllDragBrightnessProgress.visibility = View.INVISIBLE
    }

    @SuppressLint("SetTextI18n")
    override fun showProgressDialog(
        deltaX: Float,
        seekTime: String?,
        seekTimePosition: Long,
        totalTime: String?,
        totalTimeDuration: Long
    ) {
        if (mStvDragProgress.visibility != View.GONE) {
            mStvDragProgress.visibility = View.VISIBLE
            mStvDragProgress.setTextSize(
                TypedValue.COMPLEX_UNIT_PX, if (screen == SCREEN_FULLSCREEN) {
                    resources.getDimensionPixelSize(com.at.common.R.dimen.sp_16)
                } else {
                    resources.getDimensionPixelSize(com.at.common.R.dimen.sp_12)
                }.toFloat()
            )
        }
        mStvDragProgress.text = "$seekTime/$totalTime"
        changeUiToControlsClear()
    }

    override fun dismissProgressDialog() {
        mStvDragProgress.visibility = View.INVISIBLE
    }

    override fun showVolumeDialog(deltaY: Float, volumePercent: Int) {
        if (mSllDragVolumeProgress.visibility != View.VISIBLE) {
            mSllDragVolumeProgress.visibility = View.VISIBLE
        }
        mPbDragVolumeProgress.progress = when {
            volumePercent > 100 -> 100
            volumePercent < 0 -> 0
            else -> volumePercent
        }
        changeUiToControlsClear()
    }

    private fun changeUiToControlsClear() {
        val isShowLoading: Int = if (state == STATE_PREPARING_PLAYING) VISIBLE else INVISIBLE
        setAllControlsVisiblity(
            INVISIBLE, INVISIBLE, INVISIBLE, isShowLoading, INVISIBLE, INVISIBLE, INVISIBLE
        )
    }

    override fun dismissVolumeDialog() {
        mSllDragVolumeProgress.visibility = View.INVISIBLE
    }

    interface OnViewClickListener {
        //画中画
        fun onSuspension()

        //返回键
        fun onBack()

        //暂停
        fun onPauseOrPlay()

        //选集
        fun onLevel(data: LevelBinder.LevelData, position: Int)

        //准备好了
        fun onPrepared()

        //金币购买
        fun onPay()

        //播放异常
        fun playError()
    }

    override fun onSkip() {
        mOnSkipListener?.onSkip()
    }

}