package com.at.video.player.custom

import android.content.Context
import android.view.View
import androidx.recyclerview.widget.RecyclerView
import com.at.video.player.R
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.lxj.xpopup.core.PositionPopupView

/**
 * @Create_time: 2022/4/22 10:08
 * @Description:
 */

class SpeedPopup(context: Context): PositionPopupView(context), OnItemClickListener {
    private lateinit var mRecyclerView: RecyclerView
    private val mAdapter: SpeedAdapter = SpeedAdapter(getSpeedList())
    private var mCallback: ((Float) -> Unit)? = null

    private fun getSpeedList(): MutableList<Float> {
        return mutableListOf(2.0f, 1.75f, 1.5f, 1.25f, 1.0f, 0.75f, 0.5f)
    }

    override fun getImplLayoutId(): Int {
        return R.layout.player_custom_speed
    }

    override fun onCreate() {
        super.onCreate()
        mRecyclerView = findViewById(R.id.recycler_view)
        mRecyclerView.setHasFixedSize(true)
        mRecyclerView.adapter = mAdapter
        mAdapter.setOnItemClickListener(this)
    }

    class SpeedAdapter(data: MutableList<Float>): BaseQuickAdapter<Float,
            BaseViewHolder>(R.layout.player_custom_item_speed, data) {
        private var mSpeed: Float = 1.0f

        override fun convert(holder: BaseViewHolder, item: Float) {
            holder.setText(R.id.tv_speed, item.toString())
            if(mSpeed == item) {
                holder.setVisible(R.id.iv_checked, true)
            } else {
                holder.setVisible(R.id.iv_checked, false)
            }
        }

        override fun convert(holder: BaseViewHolder, item: Float, payloads: List<Any>) {
            if(mSpeed == item) {
                holder.setVisible(R.id.iv_checked, true)
            } else {
                holder.setVisible(R.id.iv_checked, false)
            }
        }

        fun setSpeed(speed: Float) {
            if(speed != mSpeed) {
                var prevIndex: Int = data.indexOf(mSpeed)
                if(prevIndex >= 0) {
                    notifyItemChanged(prevIndex, "notify")
                }
                mSpeed = speed
                prevIndex = data.indexOf(speed)
                if(prevIndex >= 0) {
                    notifyItemChanged(prevIndex, "notify")
                }
            }
        }

    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val speed: Float = adapter.getItem(position) as Float
        mAdapter.setSpeed(speed)
        mCallback?.invoke(speed)
    }

    fun setCallback(callback: (Float) -> Unit) {
        mCallback = callback
    }
}