/*
 * Copyright (C) 2015-2016 Sébastiaan (github.com/se-bastiaan)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.at.video.player.libtorrent;

import static com.frostwire.jlibtorrent.swig.peer_info.utp_socket;

import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.HandlerThread;
import android.util.Log;

import com.at.common.global.CacheHelper;
import com.at.video.player.libtorrent.bean.AppendToFile;
import com.at.video.player.libtorrent.bean.CacheEntity;
import com.at.video.player.libtorrent.bean.MovieHeadBean;
import com.at.video.player.libtorrent.bean.Status;
import com.at.video.player.libtorrent.execption.DirectoryModifyException;
import com.at.video.player.libtorrent.execption.TorrentInfoException;
import com.at.video.player.libtorrent.listener.DownloadHeadListener;
import com.at.video.player.libtorrent.listener.DownloadListener;
import com.at.video.player.libtorrent.listener.MovePeerConnectedAlertListener;
import com.at.video.player.libtorrent.listener.MyFileDownloadListener;
import com.at.video.player.libtorrent.listener.TorrentAddedAlertListener;
import com.at.video.player.libtorrent.listener.TorrentListener;
import com.at.video.player.utils.FileUtils;
import com.at.video.player.utils.ThreadUtils;
import com.melancholy.utils.StringUtils;
import com.video.module_widget.utils.TrackersUtil;
import com.frostwire.jlibtorrent.PeerInfo;
import com.frostwire.jlibtorrent.Priority;
import com.frostwire.jlibtorrent.SessionManager;
import com.frostwire.jlibtorrent.Sha1Hash;
import com.frostwire.jlibtorrent.TcpEndpoint;
import com.frostwire.jlibtorrent.TorrentFlags;
import com.frostwire.jlibtorrent.TorrentHandle;
import com.frostwire.jlibtorrent.TorrentInfo;
import com.frostwire.jlibtorrent.alerts.AddTorrentAlert;
import com.frostwire.jlibtorrent.alerts.PeerConnectAlert;
import com.frostwire.jlibtorrent.swig.settings_pack;
import com.frostwire.jlibtorrent.swig.torrent_flags_t;
import com.google.gson.Gson;
import com.liulishuo.filedownloader.FileDownloader;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.Interceptor;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.Buffer;
import okio.BufferedSource;
import okio.ForwardingSource;
import okio.Okio;

public final class TorrentDownload {

    private static final String TAG = TorrentDownload.class.getSimpleName();
    private Boolean isStreaming = false, isCanceled = false;

    private volatile Torrent currentTorrent;

    private Map<Object, DownloadListener> listeners = new HashMap<>();
    private CacheEntity entity;

    private final TorrentAddedAlertListener torrentAddedAlertListener;
    private HandlerThread streamingThread;
    private Handler streamingHandler;
    protected long prepareSize = 1 * 1024L * 1024L;//以前15M
    private int flagsNum;

    private String seedUrl;
    private String SHA1HASH;
    private long piece_size;

    TorrentInfo torrentInfo = null;

    private Thread headLoadThread = null;

    public long getPiece_size() {
        return piece_size;
    }

    public String getSHA1HASH() {
        return SHA1HASH;
    }

    public String getSeedUrl() {
        return seedUrl;
    }

    //播放界面每个阶段的状态
    private int playerLoadingStatus = 0;


    public volatile boolean headerAddOver = false;

    private loadingStatusListener loadingStatusListener;

    public int getPlayerLoadingStatus() {
        return playerLoadingStatus;
    }

    public TorrentDownload.loadingStatusListener getLoadingStatusListener() {
        return loadingStatusListener;
    }

    public interface loadingStatusListener {
        void statusListener(int playerLoadingStatus);

        void downloadProgress(int progress);
    }

    public TorrentDownload(final CacheEntity entity) {
        //重连标记
        reConnect = false;
        this.entity = entity;
        torrentAddedAlertListener = new TorrentAddedAlertListener(entity.getVideoTag()) {
            @Override
            public void torrentAdded(AddTorrentAlert alert) {
                // Log.i(TAG, "torrentAdded: " + alert.type());
                SessionManager sessionManager = TorrentEngine.getInstance().getTorrentSession();
                TorrentHandle th = sessionManager.find(alert.handle().infoHash());
                flagsNum = alert.params().dhtNodes().size();
                InternalTorrentListener listener = new InternalTorrentListener(entity.getVideoTag());
                currentTorrent = new Torrent(th, listener, prepareSize);
                currentTorrent.setSequentialDownload(true);
                Log.i("progress=", "added");
                sessionManager.removeListener(this);
            }
        };
    }

    /**
     * 设置下载模式 true 顺序下在 false 无序下载
     *
     * @param sequential
     */
    public void setDownloadModule(boolean sequential) {
        if (currentTorrent == null) {
            return;
        }
        TorrentHandle th = currentTorrent.getTorrentHandle();
        if (th == null || !th.isValid()) {
            return;
        }
        if (sequential) {
            th.setFlags(TorrentFlags.SEQUENTIAL_DOWNLOAD);
        } else {
            th.unsetFlags(TorrentFlags.SEQUENTIAL_DOWNLOAD);
        }
        th.saveResumeData();
    }

    public CacheEntity getEntity() {
        return entity;
    }

    public void setLoadingStatusListener(TorrentDownload.loadingStatusListener loadingStatusListener) {
        this.loadingStatusListener = loadingStatusListener;
    }

    /**
     * 获取当前的下载模式 true 有序 false 无序
     *
     * @return
     */
    public boolean getSequentialDownload() {
        if (currentTorrent == null) {
            return false;
        }
        TorrentHandle th = currentTorrent.getTorrentHandle();
        if (th == null) {
            return false;
        }

        if (!th.isValid()) {
            return false;
        }

        torrent_flags_t flags = th.status().flags();
        return flags.and_(TorrentFlags.SEQUENTIAL_DOWNLOAD).nonZero();
    }

    /**
     * Get torrent metadata, either by downloading the .torrent or fetching the magnet
     *
     * @param torrentUrl {@link String} URL to .torrent or magnet link
     * @return {@link TorrentInfo}
     */
    private TorrentInfo getTorrentInfo(String torrentUrl) throws TorrentInfoException {//Can't decode data: expected value (list, dict, int or string) in bencoded string
        // if (VideoApplication.vIdeobean.getMagnet() != null && VideoApplication.vIdeobean.getMagnet().startsWith("http")) {
        if (torrentUrl != null && torrentUrl.startsWith("http")) {
            try {
                Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行中：" + System.currentTimeMillis());
                // URL urlh = new URL(VideoApplication.vIdeobean.getMagnet());
                URL urlh = new URL(torrentUrl);
                HttpURLConnection connectionh = (HttpURLConnection) urlh.openConnection();
                connectionh.setRequestMethod("GET");
                connectionh.setConnectTimeout(500);
                connectionh.setInstanceFollowRedirects(true);
                connectionh.connect();
                InputStream inputStream = connectionh.getInputStream();
                byte[] responseByteArray = new byte[0];
                if (connectionh.getResponseCode() == 200) {
                    responseByteArray = getBytesFromInputStream(inputStream);
                }
                inputStream.close();
                connectionh.disconnect();
                if (responseByteArray.length > 0) {
                    entity.setBdcode(responseByteArray);
                    Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行完成啦：" + System.currentTimeMillis());
                    return TorrentInfo.bdecode(responseByteArray);
                }
            } catch (IOException | IllegalArgumentException e) {
                if (torrentUrl.startsWith("magnet")) {
                    torrentUrl = TrackersUtil.spliceTracker(torrentUrl);
                    //添加xpe
                    Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "异常执行执行中：" + System.currentTimeMillis());
                    if (entity.getXpe() != null && !entity.getXpe().isEmpty()) {
                        String[] xpes = entity.getXpe().split(", ");
                        for (int i = 0; i < xpes.length; i++) {
                            torrentUrl += "&x.pe=" + Uri.parse(xpes[i]).getHost() + ":" + Uri.parse(xpes[i]).getPort();
                        }
                    }
                    // 获取TorrentEngine信息
                    byte[] data = TorrentEngine.getInstance().getTorrentSession().fetchMagnet(torrentUrl, 60);
                    if (data != null) try {
                        entity.setBdcode(data);
                        Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "异常执行完毕：" + System.currentTimeMillis());
                        return TorrentInfo.bdecode(data);
                    } catch (IllegalArgumentException e1) {
                        Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "异常执行抛出异常完毕：" + System.currentTimeMillis());
                        throw new TorrentInfoException(e1);
                    }
                }
            }

        } else if (torrentUrl.startsWith("magnet")) {

            Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行中：" + System.currentTimeMillis() + " url=" + torrentUrl);
            //添加xpe
            if (entity.getXpe() != null) {
                String[] xpes = entity.getXpe().split(", ");
                for (int i = 0; i < xpes.length; i++) {
                    torrentUrl += "&x.pe=" + Uri.parse(xpes[i]).getHost() + ":" + Uri.parse(xpes[i]).getPort();
                }
            }
            byte[] data = TorrentEngine.getInstance().getTorrentSession().fetchMagnet(torrentUrl, 30);
            if (data != null) {
                try {
                    entity.setBdcode(data);
                    Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行完毕：" + System.currentTimeMillis());
                    return TorrentInfo.bdecode(data);
                } catch (IllegalArgumentException e) {
                    Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行抛出异常完毕：" + System.currentTimeMillis());
                    throw new TorrentInfoException(e);
                }
            } else {
                Log.i(TAG, "getTorrentInfo:data=null=> " + torrentUrl);
                Log.i(TAG, "getTorrentInfo: " + Thread.currentThread().getName() + "执行data[]空：" + System.currentTimeMillis());
            }

        } else if (torrentUrl.startsWith("http") || torrentUrl.startsWith("https")) {
            try {
                URL url = new URL(torrentUrl);
                HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                connection.setRequestMethod("GET");
                connection.setInstanceFollowRedirects(true);
                connection.connect();
                InputStream inputStream = connection.getInputStream();

                byte[] responseByteArray = new byte[0];
                if (connection.getResponseCode() == 200) {
                    responseByteArray = getBytesFromInputStream(inputStream);
                }

                inputStream.close();
                connection.disconnect();

                if (responseByteArray.length > 0) {
                    entity.setBdcode(responseByteArray);
                    return TorrentInfo.bdecode(responseByteArray);
                }
            } catch (IOException | IllegalArgumentException e) {
                throw new TorrentInfoException(e);
            }
        } else if (torrentUrl.startsWith("file")) {
            Uri path = Uri.parse(torrentUrl);
            File file = new File(path.getPath());

            try {
                FileInputStream fileInputStream = new FileInputStream(file);
                byte[] responseByteArray = getBytesFromInputStream(fileInputStream);
                fileInputStream.close();

                if (responseByteArray.length > 0) {
                    entity.setBdcode(responseByteArray);
                    return TorrentInfo.bdecode(responseByteArray);
                }
            } catch (IOException | IllegalArgumentException e) {
                throw new TorrentInfoException(e);
            }
        }

        return null;
    }

    private byte[] getBytesFromInputStream(InputStream inputStream) throws IOException {
        ByteArrayOutputStream byteBuffer = new ByteArrayOutputStream();

        int bufferSize = 1024;
        byte[] buffer = new byte[bufferSize];

        int len = 0;
        while ((len = inputStream.read(buffer)) != -1) {
            byteBuffer.write(buffer, 0, len);
        }

        return byteBuffer.toByteArray();
    }

    /**
     * Start stream download for specified torrent  pointer Index out of range
     */
    private File saveDirectory;
    private String currentTorrentUrl;
    private MovieHeadBean headBean;
    private int HeadNum;

    public int getHeadNum() {
        return HeadNum;
    }

    public MovieHeadBean getHeadBean() {
        return headBean;
    }

    public void startStream(final String torrentUrl, final String cachUrl) {
        String videoTag = entity.getVideoTag();
        if (isStreaming || videoTag == null) {
            if (currentTorrent != null && currentTorrent.getTorrentHandle() != null && currentTorrent.getTorrentHandle().isValid()) {
                return;
            }
        }
        playerLoadingStatus = 0;//初始化
        if (loadingStatusListener != null) {
            loadingStatusListener.statusListener(playerLoadingStatus);
        }
        isCanceled = false;
        TorrentEngine.getInstance().resumeSession();//检查session启动
        streamingThread = new HandlerThread(entity.getVideoTag());
        streamingThread.start();
        streamingHandler = new Handler(streamingThread.getLooper());
        isStreaming = true;
        streamingHandler.post(new Runnable() {
            @Override
            public void run() {
                currentTorrentUrl = torrentUrl;
                saveDirectory = new File(cachUrl);
                if (!saveDirectory.isDirectory() && !saveDirectory.mkdirs()) {
                    entity.setState(Status.STATE_ERROR);
                    for (final com.at.video.player.libtorrent.listener.DownloadListener listener : listeners.values()) {
                        ThreadUtils.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                listener.onStreamError(null, new DirectoryModifyException());
                            }
                        });
                    }
                    isStreaming = false;
                    return;
                }

                //初始化
                TorrentEngine.getInstance().getTorrentSession().removeListener(torrentAddedAlertListener);

                //创建线程池
                try {
                    //进入磁力链解析中
                    if (!(entity.getState() == Status.STATE_FINISHED)) {
                        entity.setState(Status.STATE_PARSING);
                    }
                    playerLoadingStatus = 1;//下载解析磁力链接中
                    if (loadingStatusListener != null) {
                        loadingStatusListener.statusListener(playerLoadingStatus);
                    }

                    for (final TorrentListener listener : listeners.values()) {
                        ThreadUtils.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                listener.onURLParsing();
                            }
                        });
                    }
                    //判断原来磁力链是否解析成功
                    if (entity.getBdcode() == null || entity.getBdcode().length <= 0) {
                        //明天我们在这里加一个参数就OK了.
                        long start = System.currentTimeMillis();
                        //添加阻塞队列
                        ArrayBlockingQueue<TorrentInfo> queue = new ArrayBlockingQueue(1);
                        List<Future<TorrentInfo>> futures = new ArrayList<>();
                        ExecutorService es = Executors.newFixedThreadPool(3);
                        TorrentCallable callable = new TorrentCallable(TorrentDownload.this, torrentUrl, " count=>" + 2, queue);
                        futures.add(es.submit(callable));
                        try {
                            torrentInfo = queue.poll(60, TimeUnit.SECONDS);
                            if (torrentInfo != null) {
                                Log.i(TAG, "==>inittime:拿到info耗时： " + (System.currentTimeMillis() - start) + "ms");
                                for (Future future : futures) {
                                    future.cancel(true);
                                }
                            }
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                            throw new TorrentInfoException(e);
                        } finally {
                            if (es != null) {
                                es.shutdownNow();
                            }
                        }
                    } else {
                        try {
                            torrentInfo = TorrentInfo.bdecode(entity.getBdcode());//select timeout: wait for receiving msg  received crash request for pid silence
                        } catch (Exception e) {
                            Log.e(TAG, "==>run:" + e);
                        }
                    }

                } catch (Exception e) {
                    Log.e(TAG, "TorrentInfoException run: " + e);
                    if (!isCanceled) {
                        entity.setBdcode(null);
                        entity.setState(Status.STATE_ERROR_ANALYSIS);
                        for (final TorrentListener listener : listeners.values()) {
                            ThreadUtils.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    listener.onStreamError(null, e);
                                }
                            });
                        }
                    }
                    isStreaming = false;
                    return;
                }

                if (torrentInfo == null || torrentInfo.swig() == null) {
                    if (!isCanceled) {
                        entity.setBdcode(null);
                        entity.setState(Status.STATE_ERROR_ANALYSIS);
                        for (final TorrentListener listener : listeners.values()) {
                            ThreadUtils.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    listener.onStreamError(null, new TorrentInfoException());
                                }
                            });
                        }
                    }
                    isStreaming = false;
                    return;
                }

                Log.i(TAG, "==>inittime: 拿到info了");
                // if (isCanceled) {//中途取消
                //     isStreaming = false;
                //     return;
                // }

//                if (!currentTorrentUrl.equals(torrentUrl) || isCanceled) {
//                    isStreaming = false;
//                    return;  //java.lang.NullPointerException: Attempt to read from field 'android.os.MessageQueue android.os.Looper.mQueue' on a null object reference
//                }

                //添加动态获取xpe并webseed加速
                TorrentEngine.getInstance().getTorrentSession().addListener(peerConnectedAlert);
                //磁力链解析成功，进行下载
                TorrentEngine.getInstance().getTorrentSession().addListener(torrentAddedAlertListener);

                int num = 0;
                try {
                    num = torrentInfo.numFiles();
                } catch (Exception e) {
                    Log.e(TAG, "run: " + e);
                }
                Priority[] priorities = new Priority[num];
                Arrays.fill(priorities, Priority.IGNORE);
                // WebSeed加速;刹不住车
                String headUrl = null;
                SHA1HASH = torrentInfo.infoHash().toHex();
                // 取head 下载地址
                String dmm = "dmm=";
                String spirt = "&";
                String dmmUrl = null;
                if (torrentUrl.contains(dmm)) {
                    int startHeaderIndex = torrentUrl.indexOf(dmm);
                    dmmUrl = torrentUrl.substring(startHeaderIndex + 4);
                    if (dmmUrl.contains("&")) {
                        int spirtIndex = dmmUrl.indexOf(spirt);
                        dmmUrl = dmmUrl.substring(0, spirtIndex);
                    }
                }
                headUrl = dmmUrl;

                piece_size = torrentInfo.pieceSize(0);
                //添加url的xpe节点
                ArrayList<TcpEndpoint> peers = new ArrayList<TcpEndpoint>();
                if (entity.getXpe() != null && !entity.getXpe().isEmpty()) {
                    String[] xpes = entity.getXpe().split(", ");
                    for (String xpe : xpes) {
                        if (xpe == null || xpe.isEmpty()) {
                            continue;
                        }
                        Uri uri = Uri.parse(xpe);
                        peers.add(new TcpEndpoint(uri.getHost(), uri.getPort()));
                    }
                }
                seedUrl = headUrl;
                Log.i(TAG, "==>run:headUrl " + "\n" + headUrl + "\t path:");

                try {
                    int limit = TorrentEngine.getInstance().getSettingsPack().getInteger(settings_pack.int_types.max_web_seed_connections.swigValue());
                    String finalHeadUrl = headUrl;
                    headLoadThread = new Thread(new Runnable() {
                        @Override
                        public void run() {
                            //测试下载头块信息
                            if (finalHeadUrl != null && !finalHeadUrl.isEmpty()) {
                                playerLoadingStatus = 2;//探测头部信息
                                if (loadingStatusListener != null) {
                                    loadingStatusListener.statusListener(playerLoadingStatus);
                                }
                                String name = null;
                                try {
                                    name = URLDecoder.decode(finalHeadUrl.substring(finalHeadUrl.lastIndexOf("/") + 1), "utf-8");
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
//                    }
                                Log.i(TAG, "==>run: " + name + "\thttp:" + finalHeadUrl);
                                String finalName = name;
                                String filePath = cachUrl + "head/" + name.replace(".mp4", ".zip");
                                String realPath = cachUrl + "head/" + name.replace(".mp4", "/");
                                File fileInput = new File(realPath);
                                if (fileInput.exists() && fileInput.isDirectory() && fileInput.listFiles().length > 0) {//解压过了的
                                    headBean = new MovieHeadBean();
                                    putDataToFile(fileInput, saveDirectory, 0);
                                } else {//未解压过
                                    String path = filePath;
                                    FileDownloader.getImpl().create(finalHeadUrl).setPath(path).setAutoRetryTimes(3).setListener(new MyFileDownloadListener(path, realPath.replace(".zip", ".head"), SHA1HASH, entity, loadingStatusListener, currentTorrent, copySuccessListener, new MyFileDownloadListener.onSuccess() {
                                        @Override
                                        public void onSuccess() {
                                            headBean = new MovieHeadBean();
                                            try {
                                                File file = new File(path);
                                                Log.i(TAG, "==>onSuccess size: " + file.length());
                                                file.delete();
                                            } catch (Exception e) {
                                                Log.e(TAG, "onSuccess: " + e);
                                            }
                                            putDataToFile(fileInput, saveDirectory, 0);
                                        }
                                    })).setCallbackProgressMinInterval(1000).start();
                                }
                            } else {//不存在的情况
                                headBean = new MovieHeadBean();
                                headerAddOver = true;
                            }

                        }
                    });
                    headLoadThread.start();

                    while (headBean == null) Thread.sleep(200);
                    Log.i("progress=", "==>inittime: 开始download");
                    playerLoadingStatus = 5;//开始链接下载数据
                    if (loadingStatusListener != null) {
                        loadingStatusListener.statusListener(playerLoadingStatus);
                    }
                    TorrentEngine.getInstance().getTorrentSession().download(torrentInfo, saveDirectory, null, null, null);
                } catch (Exception e) {
                    e.printStackTrace();
                    Log.i("progress=", "error download" + e.getMessage());
                }
            }
        });
    }

    /**
     * 添加数据到文件中
     *
     * @param fileInput
     * @param piece_size
     */
    public void putDataToFile(File fileInput, File outFile, int piece_size) {
        if (fileInput != null) {
            // Log.i(TAG, "==>onSucceed1: " + System.currentTimeMillis());
            while (currentTorrent == null || currentTorrent.getTorrentHandle() == null || !currentTorrent.getTorrentHandle().havePiece(0))
                ;
            playerLoadingStatus = 3;//开始添加数据
            if (loadingStatusListener != null) {
                loadingStatusListener.statusListener(playerLoadingStatus);
            }
            try {
                File[] temp = fileInput.listFiles();
                List<File> errorAddPiece = new ArrayList<>();
                // 添加头文件信息到mp4文件
                if (temp.length > 0) {
                    for (int i = 0; i < temp.length; i++) {
                        File file = temp[i];
                        int piece = Integer.parseInt(file.getName());
                        Boolean isOrNotSuccess = AppendToFile.fillInPieceBlock(file, currentTorrent.getTorrentHandle(), torrentInfo.pieceSize(piece), piece);
                        if (!isOrNotSuccess) {
                            errorAddPiece.add(file);
                        }
                    }
                }
                Boolean isAllSuccess = errorAddPiece.isEmpty();
                // 添加失败，再试一次
                if (!errorAddPiece.isEmpty()) {
                    for (int i = 0; i < errorAddPiece.size(); i++) {
                        File file = temp[i];
                        int piece = Integer.parseInt(file.getName());
                        boolean isAddSuccess = AppendToFile.fillInPieceBlock(file, currentTorrent.getTorrentHandle(), torrentInfo.pieceSize(piece), piece);
                        Log.i("progress=", "==>addHeader : " + isAddSuccess);
                    }
                }
                Log.i("progress=", "==>addHeader complete: " + isAllSuccess);
            } catch (Exception e) {

            }

            // 添加完成
            headerAddOver = true;
            if (copySuccessListener != null) {
                copySuccessListener.copySuccess();
            }

        }
    }

    //获取文件
    private File getFile(String extraUrl) {
        if (extraUrl == null || extraUrl.isEmpty()) {
            return null;
        }
        extraUrl = extraUrl.replace(".mp4", ".zip");
        File file = new File(FileUtils.getCacheHeadDir() + extraUrl);
        return file;
    }

    //添加了extraSpeed;
    private void addExtraSeed(TorrentInfo torrentInfo, String extra) {
        if (extra.contains(":16565")) {//端口
            String[] ports = {":16560", ":16561", ":16562", ":16563", ":16564"};
            for (String str : ports) {
                String wsUrl = extra.replace(":16565", str);
                Log.i(TAG, "run:extra_wsurl " + wsUrl);
                torrentInfo.addUrlSeed(wsUrl);
            }
        }
    }

    //动态设置监听并添加webseed加速
    private MovePeerConnectedAlertListener peerConnectedAlert = new MovePeerConnectedAlertListener("app") {
        @Override
        public void peerConnect(PeerConnectAlert alert) {
            SessionManager sessionManager = TorrentEngine.getInstance().getTorrentSession();
            final TorrentHandle th = sessionManager.find(alert.handle().infoHash());
            if (th == null) {
                return;
            }

            String urlWebSeed = String.format("http://%s/seed/%s", alert.endpoint().toString(), th.infoHash().toHex());
            if (urlWebSeed.contains(":6565")) {
                String url = urlWebSeed.replace(":6565", ":16565");
                th.addHttpSeed(url);
            }
        }
    };

    private boolean reConnect;

    /**
     * 播放页面重连机制，不影响播放条件
     */
    public void stopAndRestartStream(final String torrentUrl) {
        if (torrentUrl == null) {
            reConnect = true;
        }
        stopStream();
//        startStream(torrentUrl);
    }

    public boolean isReConnect() {
        return reConnect;
    }

    public void setReConnect(boolean reConnect) {
        this.reConnect = reConnect;
    }

    /**
     * Stop current torrent stream
     */
    public void stopStream() {//是否删除了资源或下载完毕了的删除session
        Log.i(TAG, "stopStream: ");
        if (entity.getState() == Status.STATE_FINISHED) {
            //去除th,避免下载完成的磁力链，删除后不能再次下载
            byte[] bdCode = entity.getBdcode();//
            if (bdCode == null || bdCode.length <= 0) {
                return;
            }
            TorrentInfo torrentInfo = TorrentInfo.bdecode(bdCode);
            SessionManager sessionManager = TorrentEngine.getInstance().getTorrentSession();
            Sha1Hash hash = torrentInfo.infoHash();

            TorrentHandle torrentHandle = sessionManager.find(hash);
            if (torrentHandle != null && torrentHandle.isValid()) {
                sessionManager.remove(torrentHandle);
            }
            return;
        }
        try {
            headLoadThread = null;
        } catch (Exception e) {

        }
        //remove all callbacks from handler
        if (streamingHandler != null) {
            streamingHandler.removeCallbacksAndMessages(null);
        }
        isCanceled = true;
        isStreaming = false;

        if (currentTorrent != null) {
            if (entity.getState() == Status.STATE_PAUSE) {
                return;
            }
            currentTorrent.pause();
            SessionManager sessionManager = TorrentEngine.getInstance().getTorrentSession();
            if (currentTorrent != null) {
                TorrentHandle torrentHandle = currentTorrent.getTorrentHandle();
                if (torrentHandle != null) {
                    sessionManager.remove(torrentHandle);
                }
            }
            currentTorrent = null;
        }

        if (streamingThread != null) {
            streamingThread.interrupt();
            //回收streamingThread 线程
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                streamingThread.getLooper().quitSafely();
            } else {
                streamingThread.getLooper().quit();
            }
            streamingThread = null;
        }

        entity.setState(Status.STATE_PAUSE);
        for (final TorrentListener listener : listeners.values()) {
            ThreadUtils.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    listener.onStreamStopped();
                }
            });
        }
    }

    public boolean isStreaming() {
        return isStreaming;
    }

    public Torrent getCurrentTorrent() {
        return currentTorrent;
    }

    public void addListener(DownloadListener listener) {
        listeners.put(listener.tag, listener);
    }

    public void removeListener(DownloadListener listener) {
        listeners.remove(listener.tag);
    }

    public void runThread(TorrentDownload download) {

    }


    private CopySuccessListener copySuccessListener;

    public CopySuccessListener getCopySuccessListener() {
        return copySuccessListener;
    }

    public void setCopySuccessListener(CopySuccessListener copySuccessListener) {
        this.copySuccessListener = copySuccessListener;
    }

    public interface CopySuccessListener {
        void copySuccess();

        void copyFail();
    }

    protected class InternalTorrentListener extends DownloadListener {

        public InternalTorrentListener(Object tag) {
            super(tag);
        }

        /**
         * 开始磁力链接
         *
         * @param torrent
         */
        public void onStreamStarted(final Torrent torrent) {
            if (tag != entity.getVideoTag()) return;
            if (entity.getState() != Status.STATE_FINISHED) {
                entity.setState(Status.STATE_WAITING);
            }

            for (final TorrentListener listener : listeners.values()) {
                ThreadUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        listener.onStreamStarted(torrent);
                    }
                });
            }
        }

        public void onStreamError(final Torrent torrent, final Exception e) {
            Log.i(TAG, "onStreamError: ");
        }

        public void onStreamReady(final Torrent torrent) {
            if (tag != entity.getVideoTag()) {
                return;
            }
            if (!(entity.getState() == Status.STATE_FINISHED)) {
                entity.setReady(true);
            }
            for (final TorrentListener listener : listeners.values()) {
                ThreadUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        listener.onStreamReady(torrent);
                    }
                });
            }

        }

        public void onStreamProgress(final Torrent torrent, final StreamStatus status) {
            long proSize = getProSize(torrent);

            if (entity.getState() != Status.STATE_FINISHED) {
                entity.setState(Status.STATE_DOWNING);
                entity.setVideoPath(torrent.getVideoFile().getAbsolutePath());
                if (entity.getVideoName() == null || entity.getVideoName().isEmpty()) {
                    entity.setVideoName(torrent.getVideoFile().getName());
                }
                entity.setTotalSize(torrent.getSize());
                entity.setProSize(proSize);
                entity.setDownSpeed((long) status.downloadSpeed);
            }
            for (final TorrentListener listener : listeners.values()) {
                ThreadUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        listener.onStreamProgress(torrent, status);
                    }
                });
            }

            int progress = (int) (((float) entity.getProSize() / entity.getTotalSize()) * 100);
            Log.i("progress=", "" + progress);
            //部分资源未下载完出现status.progress=100的情况
            if (progress >= 99 || status.progress >= 99) {
                if (!torrent.resetSwitch()) {//判断是否有未下载的块,有重置
                    entity.setState(Status.STATE_FINISHED);
                    if (entity.getCacheName() != null && !entity.getCacheName().isEmpty()) {
                        // 缓存播放地址
                        CacheHelper.getInstance().putString(entity.getCacheName(), entity.getVideoPath());
                    }
                    // entity.setVideoName(torrent.getVideoFile().getName());
                    // if (!entity.isReady()) {
                    entity.setReady(true);
                    // }
                    if (entity.getState() == Status.STATE_FINISHED) {
                        for (final TorrentListener listener : listeners.values()) {
                            ThreadUtils.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    listener.onStreamFinished(torrent);
                                }
                            });
                        }
                    }
                }
            }
        }

        @Override
        public void onStreamStopped() {
            // Not used
            Log.i(TAG, "onStreamStopped: ");
        }

        @Override
        public void onURLParsing() {
            Log.i(TAG, "onURLParsing: ");
        }

        @Override
        public void onStreamFinished(final Torrent torrent) {
            Log.i(TAG, "onStreamFinished: ");
            if (entity.isReady()) {
                entity.setReady(true);
            }
        }

        @Override
        public void onStreamPrepared(final Torrent torrent) {
            if (tag != entity.getVideoTag()) return;
            if (HeadNum != 0) {
                torrent.setNum(HeadNum);
            }
            torrent.startDownload();
            if (!(entity.getState() == Status.STATE_FINISHED)) {
                entity.setState(Status.STATE_WAITING);
            }

            for (final TorrentListener listener : listeners.values()) {
                ThreadUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        listener.onStreamPrepared(torrent);
                    }
                });
            }
        }
    }


    //重新计算下载块
    private long getProSize(Torrent torrent) {
        long proSize;
        int count = 0;
        for (int i = 0; i < torrent.hasPieces.length; i++) {
            if (torrent.hasPieces[i]) {
                count++;
            }
        }
        proSize = (long) ((((double) count) / torrent.hasPieces.length) * torrent.getSize());
        return proSize;
    }

    @Override
    public String toString() {
        return "TorrentDownload{" + "isStreaming=" + isStreaming + ", isCanceled=" + isCanceled + ", currentTorrent=" + currentTorrent + ", listeners=" + listeners + ", entity=" + entity + ", torrentAddedAlertListener=" + torrentAddedAlertListener + ", streamingThread=" + streamingThread + ", streamingHandler=" + streamingHandler + ", prepareSize=" + prepareSize + ", flagsNum=" + flagsNum + ", saveDirectory=" + saveDirectory + ", peerConnectedAlert=" + peerConnectedAlert + ", reConnect=" + reConnect + '}';
    }


    public String getConnectType(PeerInfo.ConnectionType connectionType, int flags) {
        if ((flags & utp_socket.to_int()) != 0) return "UTP";

        String connection;
        switch (connectionType) {
            case HTTP_SEED:
            case WEB_SEED:
                connection = "Web";
                break;
            default:
                connection = "BT";
        }
        return connection;
    }

    static class TorrentCallable implements Callable<TorrentInfo> {//广告

        public TorrentCallable(TorrentDownload download, String torrentUrl, String name, ArrayBlockingQueue<TorrentInfo> queue) {
            this.download = download;
            this.torrentUrl = torrentUrl;
            this.name = name;
            this.queue = queue;
        }

        public TorrentDownload download;
        private String torrentUrl;
        private String name;
        private ArrayBlockingQueue<TorrentInfo> queue;

        @Override
        public TorrentInfo call() throws TorrentInfoException {
            Log.i(TAG, "call: " + this.name + "=>torrentUrl:" + this.torrentUrl);
            TorrentInfo info = null;
//            if (info != null) {
//                Log.i(TAG, "call: name=>" + name + "=>完成解析了=>" + torrentUrl);
//            } else {
            // 解析 dt信息
            String dt = "dt=";
            String spirt = "&";
            String dtUrl = null;
            if (torrentUrl.contains(dt)) {
                int startHeaderIndex = torrentUrl.indexOf(dt);
                dtUrl = torrentUrl.substring(startHeaderIndex + 3);
                if (dtUrl.contains("&")) {
                    int spirtIndex = dtUrl.indexOf(spirt);
                    dtUrl = dtUrl.substring(0, spirtIndex);
                }
            }
            if (dtUrl != null && !dtUrl.isEmpty()) {
                info = download.getTorrentInfo(dtUrl);
            } else {
                info = download.getTorrentInfo(torrentUrl);
            }
//            }
            if (info != null) {
                Log.i(TAG, "call: name=>" + name + "=>完成解析了=>" + torrentUrl);
            }
            try {
                queue.put(info);
            } catch (InterruptedException e) {
                Log.e(TAG, "call: " + e);
            }
            return info;
        }
    }


    //测试
    public Call download(String url, final DownloadHeadListener downloadListener, final long startsPoint, Callback callback) {
        Request request = new Request.Builder().url(url).header("RANGE", "bytes=" + startsPoint + "-")//断点续传
                .build();

        // 重写ResponseBody监听请求
        Interceptor interceptor = new Interceptor() {
            @Override
            public Response intercept(Chain chain) throws IOException {
                Response originalResponse = chain.proceed(chain.request());
                return originalResponse.newBuilder().body(new DownloadResponseBody(originalResponse, startsPoint, downloadListener)).build();
            }
        };

        OkHttpClient.Builder dlOkhttp = new OkHttpClient.Builder().addNetworkInterceptor(interceptor);
        // 绕开证书
        try {
            setSSL(dlOkhttp);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // 发起请求
        Call call = dlOkhttp.build().newCall(request);
        call.enqueue(callback);
        return call;
    }

    //设置oKHttpClient
    private void setSSL(OkHttpClient.Builder dlOkhttp) {
        dlOkhttp.sslSocketFactory(SSLSocketClient.getSSLSocketFactory()).hostnameVerifier(SSLSocketClient.getHostnameVerifier())
                // .proxy(proxy)
                .build();
    }


    public class DownloadResponseBody extends ResponseBody {

        private Response originalResponse;
        private DownloadHeadListener downloadListener;
        private long oldPoint = 0;

        public DownloadResponseBody(Response originalResponse, long startsPoint, DownloadHeadListener downloadListener) {
            this.originalResponse = originalResponse;
            this.downloadListener = downloadListener;
            this.oldPoint = startsPoint;
        }

        @Override
        public MediaType contentType() {
            return originalResponse.body().contentType();
        }

        @Override
        public long contentLength() {
            return originalResponse.body().contentLength();
        }

        @Override
        public BufferedSource source() {
            return Okio.buffer(new ForwardingSource(originalResponse.body().source()) {
                private long bytesReaded = 0;

                @Override
                public long read(Buffer sink, long byteCount) throws IOException {
                    long bytesRead = super.read(sink, byteCount);
                    bytesReaded += bytesRead == -1 ? 0 : bytesRead;
                    if (downloadListener != null) {
                        downloadListener.loading((int) ((bytesReaded + oldPoint) / (1024)));
                    }
                    return bytesRead;
                }
            });
        }

    }


}