package com.at.video.player.libtorrent.listener;

import android.util.Log;

import com.at.video.player.libtorrent.Torrent;
import com.at.video.player.libtorrent.TorrentDownload;
import com.at.video.player.libtorrent.bean.CacheEntity;
import com.at.video.player.utils.FileUtils;
import com.liulishuo.filedownloader.BaseDownloadTask;
import com.liulishuo.filedownloader.FileDownloadListener;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MyFileDownloadListener extends FileDownloadListener {
    private static final String TAG = "MyFileDownloadListener";
    private TorrentDownload.loadingStatusListener loadingStatusListener;
    private volatile Torrent currentTorrent;
    private String SHA1HASH;
    private String path;
    private CacheEntity entity;
    private int playerLoadingStatus;
    private TorrentDownload.CopySuccessListener copySuccessListener;
    private String zipPath;
    private onSuccess onSuccess;

    public interface onSuccess {
        void onSuccess();
    }

    public MyFileDownloadListener(String path, String zipPath, String SHA1HASH, CacheEntity entity, TorrentDownload.loadingStatusListener loadingStatusListener, Torrent torrent, TorrentDownload.CopySuccessListener copySuccessListener, onSuccess onSuccess) {
        this.loadingStatusListener = loadingStatusListener;
        this.currentTorrent = torrent;
        this.path = path;
        this.SHA1HASH = SHA1HASH;
        this.entity = entity;
        this.copySuccessListener = copySuccessListener;
        this.onSuccess = onSuccess;
        this.zipPath = zipPath;
    }

    long start = System.currentTimeMillis();

    @Override
    protected void pending(BaseDownloadTask task, int soFarBytes, int totalBytes) {

    }

    @Override
    protected void progress(BaseDownloadTask task, int soFarBytes, int totalBytes) {
        Log.i("MyFileDownloadListener", "progress: " + soFarBytes + "/" + totalBytes);
        if (loadingStatusListener != null)
            loadingStatusListener.downloadProgress(soFarBytes / 1024);
    }

    @Override
    protected void completed(BaseDownloadTask task) {
        long time = (System.currentTimeMillis() - start);
        Log.i("MyFileDownloadListener", "==>completed:用时= " + time + " ms");
        File file = new File(path);
        Log.i(TAG, "==>completed: size:" + file.length() + " 平均速度" + (file.length() / 1024.0) / (time / 1000.0) + "kb/s");
        ExecutorService mExecutor = Executors.newSingleThreadExecutor();
        String outPath = path.replace(".7z", "");
        File fileInput = new File(outPath.replace(".zip", "/"));
        if (fileInput.exists() && fileInput.isDirectory() && fileInput.listFiles().length > 0) {//已经解压了
            Log.i(TAG, "==>complete:已经解压 ");
            if (onSuccess != null) onSuccess.onSuccess();
        } else {
            playerLoadingStatus = 4;//开始解压头文件
            if (loadingStatusListener != null) {
                loadingStatusListener.statusListener(playerLoadingStatus);
            }
            mExecutor.submit(new Runnable() {
                @Override
                public void run() {
                    try {
                        FileUtils.unzipFile(path, path.replace(".zip", "") + "/");
                        if (onSuccess != null) onSuccess.onSuccess();
                    } catch (Exception e) {
                        if (copySuccessListener != null) {
                            copySuccessListener.copyFail();
                        }

                    }
                }
            });
        }

    }

    @Override
    protected void paused(BaseDownloadTask task, int soFarBytes, int totalBytes) {

    }

    @Override
    protected void error(BaseDownloadTask task, Throwable e) {
        Log.e("error", "file downlod error" + e.getMessage());
    }

    @Override
    protected void warn(BaseDownloadTask task) {

    }
}
