package com.at.video.player.ui.detail

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.databinding.CommonTitleBarRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.player.R
import com.at.video.player.entity.*
import com.at.video.player.ui.detail.binder.DetailHeaderBinderView
import com.at.video.player.ui.detail.binder.TelevisionWorksBinderView
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb

/**
 * @Create_time: 2022/5/13 17:03
 * @Description: 演员详情
 */

@Route(path = "/player/DetailActivity")
class DetailActivity : BaseMviActivity<CommonTitleBarRefreshListBinding, DetailViewModel>(),
    OnItemChildClickListener, OnRefreshListener, OnItemClickListener {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ACTOR_DETAILS)
    private var mId: String = ""
    override fun createViewBinding(): CommonTitleBarRefreshListBinding {
        return CommonTitleBarRefreshListBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<DetailViewModel> = DetailViewModel::class.java

    override fun initialize(savedInstanceState: Bundle?) {
        intent.getStringExtra(Constants.KEY_DATA)
            ?.let {
                mId = it
                DetailUiEvent.HeaderData(it, homeStyle)
            }?.let { getViewModel().sendEvent(it) }

        requireTitleBar().apply {
            setBackgroundResource(R.color.color_FF181B22)
            intent.getStringExtra(Constants.KEY_DATA_1)?.let {
                if (it.isNotEmpty()) {
                    setTitle("$it")
                    setTitleColor(Color.WHITE)
                }
            }
            addBackButton(Color.WHITE)
        }
        requireBinding().includedStatusRefreshList.includedRefreshList.apply {
            srlRefresh.setOnRefreshListener(this@DetailActivity)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            srlRefresh.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.ACTOR_DETAILS).toArgb()
            )
            rvList.layoutManager = GridLayoutManager(this@DetailActivity, 1)
            rvList.itemAnimator = null
            rvList.setHasFixedSize(true)
            rvList.adapter = mAdapter
        }
        mAdapter.addItemBinder(
            DetailHeaderBinderView(
                JsonUtils.getHomeContentStyle(
                    homeStyle,
                    StyleConsts.SYNOPSIS
                )
            )
        )
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle,
                    StyleConsts.SWIPER_ADV
                )
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginVertical = true,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV),
                showMarginHorizontal = false
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(NoDataBinder(label = "暂无作品"))
        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        mAdapter.addItemBinder(TelevisionWorksBinderView(pool))
        mAdapter.addChildClickViewIds(R.id.tv_more, R.id.mb_extension, R.id.iv_extension)
        mAdapter.setOnItemChildClickListener(this)
        mAdapter.setOnItemClickListener(this)
    }

    override fun loadData() {
        getViewModel().sendEvent(DetailUiEvent.InitLoadData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::pageStatus) {
                requireBinding().includedStatusRefreshList.apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                    .text = it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
//                                srlRefresh.setEnableRefresh(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::refreshStatus) {
                requireBinding().includedStatusRefreshList.includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
                if (!it.isNullOrEmpty() && it[0] !is NoDataBinder.Data) {
                    requireTitleBar().setTitle("")
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(DetailUiState::isExtend) {
                mAdapter.notifyItemChanged(0, it)
            }
        }
    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        if (view.id == R.id.tv_more) {
            val data = adapter.getItem(position) as StarWorkCategoryInfo
            Router.getInstance().build("/player/WorkActivity")
                .withString(Constants.KEY_DATA, mId)
                .withParcelableArrayList(Constants.KEY_DATA_1, getViewModel().tabs)
                .withString(Constants.KEY_DATA_2, data.category.id!!)
                .navigation()
        } else if (view.id == R.id.mb_extension) {
            getViewModel().sendEvent(DetailUiEvent.ExtensionOrShrink)
        } else if (view.id == R.id.iv_extension) {
            getViewModel().sendEvent(DetailUiEvent.ExtensionOrShrink)
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(DetailUiEvent.RefreshData)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
//        when(val data = adapter.getItem(position)){
//            is AdImageBinder.Data ->  Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//            is AdTextBinder.Data ->   Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//        }
    }
}