package com.at.video.player.ui.detail.binder

import android.animation.ObjectAnimator
import android.view.LayoutInflater
import android.view.ViewGroup
import android.view.animation.AccelerateInterpolator
import android.view.animation.Animation
import android.view.animation.RotateAnimation
import android.widget.ImageView
import android.widget.TextView
import com.at.video.player.R
import com.at.video.player.entity.StarDetailBean
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.google.android.material.button.MaterialButton
import com.video.module_widget.entity.Content
import com.video.module_widget.imageUrl
import com.video.module_widget.strToFloat
import com.video.module_widget.strToInt
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.loadByGlide
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/18 10:33
 * @Description: 详情头部View
 */

class DetailHeaderBinderView(val style: Content? = null) :
    BaseItemBinder<StarDetailBean, BaseViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context).inflate(
                R.layout.player_detail_header_view, parent, false
            )
        )
    }


    override fun convert(holder: BaseViewHolder, data: StarDetailBean) {
        holder.getView<ImageView>(R.id.div_header)
            .loadByGlide(
                GlideImageConfig(
                    data.cover_vertical.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                ),
                WeakReference(context)
            )
        holder.setText(R.id.tv_name, data.name)
            .setText(R.id.tv_sub_name, data.profession)
            .setText(R.id.tv_info, data.introduction)
            .setText(R.id.tv_alias, data.name_eng)
            .setText(R.id.tv_sex, if (data.gender?.strToInt() == 1) "男" else "女")
            .setText(
                R.id.tv_height,
                if (data.height?.strToFloat()!! > 0) "${data.height}cm" else ""
            )
            .setText(
                R.id.tv_weight,
                if (data.weight?.strToFloat()!! > 0) "${data.weight}kg" else ""
            )
            .setText(R.id.tv_birthday, data.birthday)
            .setText(R.id.tv_school, data.school)
            .setText(R.id.tv_blood, data.blood_type)
            .setText(R.id.tv_region, data.area_name)
            .setText(R.id.tv_constellation, data.constellation)
//            .setText(R.id.tv_hobby, data.hobby)
            .setText(R.id.tv_position, data.profession)
        changeExtend(holder, false)
    }

    override fun convert(holder: BaseViewHolder, data: StarDetailBean, payloads: List<Any>) {
        val isExtend = payloads[0] as Boolean
        changeExtend(holder, isExtend)
    }

    private fun changeExtend(holder: BaseViewHolder, isExtend: Boolean) {
        style?.config?.let {
            if (it.style == 2) {
                holder.setGone(R.id.iv_extension, true)
                holder.setGone(R.id.mb_extension, false)
                if (isExtend) {
                    holder.setText(R.id.mb_extension, "收起")
                    holder.getView<MaterialButton>(R.id.mb_extension)
                        .setIconResource(R.drawable.player_arrow_up)
                    holder.setGone(R.id.ll_info, false)
                    holder.getView<TextView>(R.id.tv_info).maxLines = Int.MAX_VALUE
                } else {
                    holder.setText(R.id.mb_extension, "展开")
                    holder.getView<MaterialButton>(R.id.mb_extension)
                        .setIconResource(R.drawable.player_arrow_down)
                    holder.setGone(R.id.ll_info, true)
                    holder.getView<TextView>(R.id.tv_info).maxLines = it.maxLine
                }
            } else {
                holder.setGone(R.id.mb_extension, true)
                if (isExtend) {
                    val anim = RotateAnimation(
                        0F,
                        180F,
                        Animation.RELATIVE_TO_SELF,
                        0.5F,
                        Animation.RELATIVE_TO_SELF,
                        0.5f
                    )
                    anim.fillAfter = true
                    anim.duration = 500
                    anim.interpolator = AccelerateInterpolator()
                    holder.getView<ImageView>(R.id.iv_extension).startAnimation(anim)
                    holder.setGone(R.id.ll_info, false)
                    holder.getView<TextView>(R.id.tv_info).maxLines = Int.MAX_VALUE
                } else {
                    val anim = RotateAnimation(
                        0F,
                        360F,
                        Animation.RELATIVE_TO_SELF,
                        0.5F,
                        Animation.RELATIVE_TO_SELF,
                        0.5f
                    )
                    anim.fillAfter = true
                    anim.duration = 500
                    anim.interpolator = AccelerateInterpolator()
                    holder.getView<ImageView>(R.id.iv_extension).startAnimation(anim)
                    holder.setGone(R.id.ll_info, true)
                    holder.getView<TextView>(R.id.tv_info).maxLines = it.maxLine
                }
            }
        }
    }
}