package com.at.video.player.ui.detail.binder

import android.widget.FrameLayout
import android.widget.ImageView
import com.at.common.ImageConfig
import com.at.common.load
import com.at.video.player.R
import com.at.video.player.entity.StarVideoInfo
import com.chad.library.adapter.base.BaseMultiItemQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeMaterialButtonExt
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/18 14:28
 * @Description:
 */

class TelevisionWorksAdapter : BaseMultiItemQuickAdapter<StarVideoInfo, BaseViewHolder>() {

    init {
        addItemType(1, R.layout.player_item_television_works_free)
        addItemType(2, R.layout.player_item_television_works_pay)
    }

    override fun convert(holder: BaseViewHolder, item: StarVideoInfo) {
        val realImageUrl: String? = item.cover_vertical ?: item.cover_horizontal
        holder.getView<ImageView>(R.id.div_icon)
            .loadByGlide(
                GlideImageConfig(
                    realImageUrl.imageUrl(),
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
                ), WeakReference(context)
            )
//        val numStr = if(item.episode?.strToInt() == 0) "已完结" else "全${item.episode}集"
        holder.setText(R.id.tv_name, item.name)
            .setText(R.id.tv_numEpisodes, item.note)
        if (item.price_type?.toInt() == 1) {
            changeTagStyle(holder.getView<ShapeTextViewExt>(R.id.stv_free))
        } else {
            holder.setText(R.id.smb_glod, item.price)
            changeTagStyle(holder.getView<ShapeMaterialButtonExt>(R.id.smb_glod))
        }
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

    private fun changeTagStyle(tagView: ShapeMaterialButtonExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setShapeRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }

}