package com.at.video.player.ui.player

import android.util.Log
import android.view.Gravity
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.at.video.player.R
import com.at.video.player.entity.VideoUrl
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.video.module_widget.entity.Content
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor
import com.video.module_widget.widget.ShapeFrameLayoutExt

/**
 * @Create_time: 2022/5/13 13:47
 * @Description:
 */

class LevelAdapter(val isMore: Boolean = false) :
    BaseQuickAdapter<VideoUrl, BaseViewHolder>(R.layout.player_item_episode) {

    private var mSelectIndex: Int = 0
    private var type: Int = 1
    private var style: Content? = null
    override fun createBaseViewHolder(view: View): BaseViewHolder {
        if (!isMore) {
            val params = view.layoutParams
            params.width = view.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_53)
            params.height = view.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_53)
            view.layoutParams = params
        }
        return super.createBaseViewHolder(view)
    }

    override fun convert(holder: BaseViewHolder, item: VideoUrl) {
        changeButtonStyle(holder, item)
    }

    fun setType(mType: Int) {
        type = mType
    }

    fun setStyle(mStyle: Content?) {
        style = mStyle
    }

    private fun changeTxtStyle(text: TextView, isSelected: Boolean) {
        text.apply {
            val textColor =
                if (isSelected) StyleUtils.primaryColor else context.resources.getColor(R.color.color_FF48494D)
            setTextColor(textColor)
//            val borderWidth = if(isSelected) 2 else 1
//            setBorderWidth(borderWidth)
//            val borderColor = if(isSelected) StyleUtils.primaryColor else context.resources.getColor(R.color.color_FFAEB3B9)
//            setBorderColor(borderColor)
//            reBuilder()
        }
        style?.apply {
            config?.let {
                text.apply {
                    val textColor =
                        if (isSelected) it.selectTextColor?.toArgb() else context.resources.getColor(
                            R.color.color_FF48494D
                        )
                    if (textColor != null) {
                        setTextColor(textColor)
                    }
//            val borderWidth = if(isSelected) 2 else 1
//            setBorderWidth(borderWidth)
//            val borderColor = if(isSelected) StyleUtils.primaryColor else context.resources.getColor(R.color.color_FFAEB3B9)
//            setBorderColor(borderColor)
//            reBuilder()
                }
            }
        }
    }


    private fun changeContainerStyle(container: ShapeFrameLayoutExt, isSelected: Boolean) {
//        container.apply {
//            setBorderRadius(4)
//            val borderWidth = when(type){
//                1-> if(isSelected) 2 else 0
//                else-> 0
//            }
////            val bgColor = if(isSelected) "rgba(255,97,63,0.18)".toArgb() else "#F6F7F9".toColor()
////            setBackgroundColor(bgColor)
//            setBorderWidth(borderWidth)
//            val borderColor = if(isSelected) StyleUtils.primaryColor else context.resources.getColor(R.color.color_FFAEB3B9)
//            setBorderColor(borderColor)
//            reBuilder()
//        }

        style?.apply {

            config?.let {

                val borderColor = if (isSelected) it.selectedBorderColor else "rgba(0,0,0,0)"
                val borderWidth = if (isSelected) it.width else 0

                val bgColor =
                    if (isSelected) it.selectedBackground?.toArgb() else "#F6F7F9".toColor()
                container.apply {
                    setBorderRadius(it.radius)
                    if (bgColor != null) {
                        setBackgroundColor(bgColor)
                    }
                    setBorderWidth(borderWidth)
                    if (borderColor != null) {
                        setBorderColor(borderColor.toArgb())
                    }
                    reBuilder()
                }
            }
        }
    }

    private fun changeButtonStyle(holder: BaseViewHolder, item: VideoUrl) {
        // TODO 目前所有VIP相关都暂时取消
        holder.setGone(R.id.stv_flag, true)
        changeContainerStyle(
            holder.getView(R.id.scl_root),
            mSelectIndex == holder.bindingAdapterPosition
        )
        val root = holder.getView<ShapeFrameLayoutExt>(R.id.scl_root)
        val params = root.layoutParams as RecyclerView.LayoutParams
        if (item.name != null) {
            holder.setText(R.id.tv_text, "${item.name}")
            changeTxtStyle(
                holder.getView(R.id.tv_text),
                mSelectIndex == holder.bindingAdapterPosition
            )
            holder.setGone(R.id.tv_des, true)
            params.width = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_53)
            params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_53)
            if (style != null) {
                style?.apply {
                    config?.let { it ->
                        when (it.style) {
                            1 -> {
                                holder.setGone(
                                    R.id.iv_playing,
                                    mSelectIndex != holder.bindingAdapterPosition
                                )
                                holder.setGone(
                                    R.id.tv_text,
                                    mSelectIndex == holder.bindingAdapterPosition
                                )
                                val iconParams =
                                    holder.getView<ImageView>(R.id.iv_playing).layoutParams as FrameLayout.LayoutParams
                                iconParams.width =
                                    context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20)
                                iconParams.height =
                                    context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20)
                            }
                            2 -> {
                                holder.setGone(
                                    R.id.tv_text,
                                    false
                                )
                                holder.setGone(R.id.iv_playing, true)
                            }
                            else -> {
                                 holder.setGone(
                                        R.id.iv_playing,
                                        mSelectIndex != holder.bindingAdapterPosition
                                    )
                                 holder.setGone(
                                            R.id.tv_text,
                                    false
                                )
                                val iconParams =
                                    holder.getView<ImageView>(R.id.iv_playing).layoutParams as FrameLayout.LayoutParams
                                val icon = holder.getView<ImageView>(R.id.iv_playing)
                                it.selectTextColor?.toArgb()
                                    ?.let { it1 -> icon.setColorFilter(it1) }
                                iconParams.gravity = Gravity.BOTTOM or Gravity.LEFT
                                iconParams.width =
                                    context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_10)
                                iconParams.height =
                                    context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_10)
                            }
                        }
                    }
                }
            } else {
                holder.setGone(
                    R.id.iv_playing,
                    mSelectIndex != holder.bindingAdapterPosition
                )
            }
        } else {
            if (!isMore) {
                val root = holder.getView<ShapeFrameLayoutExt>(R.id.scl_root)
                val params = root.layoutParams as RecyclerView.LayoutParams
                params.width = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_130)
            }
            val tvTitle = holder.getView<TextView>(R.id.tv_text)
            val titleParams = tvTitle.layoutParams as FrameLayout.LayoutParams
            titleParams.gravity = Gravity.LEFT
            titleParams.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
            val tvDes = holder.getView<TextView>(R.id.tv_des)
            tvDes.maxLines = if (isMore) 2 else 1
            val desParams = tvDes.layoutParams as FrameLayout.LayoutParams
            desParams.leftMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_6)
            desParams.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_36)

            if (mSelectIndex == holder.bindingAdapterPosition) {
                holder.setGone(R.id.iv_playing, false)
                val iconParams =
                    holder.getView<ImageView>(R.id.iv_playing).layoutParams as FrameLayout.LayoutParams
                iconParams.gravity = Gravity.LEFT
                iconParams.leftMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_6)
                iconParams.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_10)
                titleParams.leftMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_26)

            } else {
                holder.setGone(R.id.iv_playing, true)
                titleParams.leftMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_6)

            }

            holder.setText(R.id.tv_text, "${holder.bindingAdapterPosition + 1}")
            holder.setGone(R.id.tv_des, false)
//            holder.setText(R.id.tv_des,"测试两三年了福建省菲利克斯进度款浪费时间的李开复是劳动法吉林省")
//            if(item.des!=null){
//                holder.setText(R.id.tv_des,item.des)
//            }else{
//                holder.setGone(R.id.tv_des,true)
//            }
        }
    }


    override fun convert(holder: BaseViewHolder, item: VideoUrl, payloads: List<Any>) {
//        Log.e("Level", "convert: notify->${holder.bindingAdapterPosition}" )

        changeButtonStyle(holder, item)
    }

    fun setIndex(index: Int) {
        val prevIndex: Int = mSelectIndex
        mSelectIndex = index
//        Log.e("Level", "setIndex: prevIndex:${prevIndex}" )

        notifyItemChanged(mSelectIndex, "notify")
        notifyItemChanged(prevIndex, "notify")
    }
}