package com.at.video.player.ui.player

import android.app.AppOpsManager
import android.app.PendingIntent
import android.app.PictureInPictureParams
import android.app.RemoteAction
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.res.Configuration
import android.graphics.drawable.Icon
import android.net.Uri
import android.os.*
import android.util.Base64
import android.util.Log
import android.util.Rational
import android.view.View
import android.widget.TextView
import androidx.annotation.DrawableRes
import androidx.annotation.RequiresApi
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import cn.jzvd.IReadData
import cn.jzvd.JZDataSource
import cn.jzvd.Jzvd
import cn.jzvd.JzvdStd
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.player.R
import com.at.video.player.custom.*
import com.at.video.player.databinding.PlayerActivityPlayerBinding
import com.at.video.player.entity.WatchHistorys
import com.at.video.player.libtorrent.StreamStatus
import com.at.video.player.libtorrent.Torrent
import com.at.video.player.libtorrent.TorrentDownload
import com.at.video.player.libtorrent.TorrentEngine
import com.at.video.player.libtorrent.bean.AppendToFile
import com.at.video.player.libtorrent.bean.CacheEntity
import com.at.video.player.libtorrent.bean.Status
import com.at.video.player.libtorrent.listener.DownloadListener
import com.at.video.player.ui.player.binder.*
import com.at.video.player.ui.player.synopsis.EpisodePopup
import com.at.video.player.ui.player.synopsis.SynopsisPopup
import com.at.video.player.utils.CacheUtils
import com.at.video.player.utils.ScreenRotateUtils
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.liulishuo.filedownloader.BaseDownloadTask
import com.liulishuo.filedownloader.FileDownloader
import com.lxj.xpopup.XPopup
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.StringUtils
import com.scwang.smart.refresh.layout.SmartRefreshLayout
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.dialog.ShareDialog
import com.video.module_widget.entity.FooterBar
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.TrackersUtil
import com.video.module_widget.utils.toArgb
import com.youth.banner.util.LogUtils
import dagger.hilt.android.AndroidEntryPoint
import tv.danmaku.ijk.media.player.IjkMediaPlayer
import java.io.File
import java.io.FileNotFoundException
import java.io.IOException
import java.io.RandomAccessFile
import java.net.URLDecoder

/**
 * @Create_time: 2022/4/18 17:21
 * @Description:
 */

@AndroidEntryPoint
// @Route(path = "/player/PlayerActivity")
@Route(path = "/player/PlayerActivity", extras = 10086)
class PlayerActivity : BaseMviActivity<PlayerActivityPlayerBinding, PlayerViewModel>(),
    PlayerView.OnViewClickListener, ScreenRotateUtils.OrientationChangeListener,
    OnItemChildClickListener, VideoListBinder.OnItemChildLsViewClickListener, View.OnClickListener,
    OnRefreshListener, GridSpanSizeLookup, CountdownTextView.OnSkipListener,
    RouteBinder.OnRouterListener, LevelBinder.OnLevelListener, OnItemClickListener {
    private var mPictureInPictureParamsBuilder: PictureInPictureParams.Builder? = null
    private var mIsPlaying: Boolean = false
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private var mSynopsisPopup: SynopsisPopup? = null
    private var episodePopup: EpisodePopup? = null

    private var homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.VIDEO_PLAYER)

    private var currentPlayTime: Long = 0

    private var footerBar: FooterBar? =
        JsonUtils.getObjectByJson(StyleConsts.VIDEO_PLAYER, StyleConsts.FOOTER_BAR)

    // 先下载再播放(下载器)
    private var mTorrentDownload: TorrentDownload? = null

    // 下载监听事件
    private var mBitDownloadListener: DownloadListener? = null

    private var mFile: RandomAccessFile? = null

    // 播放缓存地址
    private var mVideoPayerUrl: String? = null

    @Volatile
    private var isPlayerIng: Boolean = false

    private var mMagnetReadData: MagnetReadData? = null

    companion object {
        private const val ACTION_MEDIA_CONTROL: String = "action_media_control"
        private const val EXTRA_CONTROL_TYPE: String = "extra_control_type"

        private const val REQUEST_PLAY: String = "play"
        private const val REQUEST_PAUSE: String = "pause"
        private const val REQUEST_RETREAT_QUICKLY: String = "retreat_quickly"
        private const val REQUEST_FAST_FORWARD: String = "fast_forward"
        const val TIME_SKIP = 1000L
    }

    override fun createViewBinding(): PlayerActivityPlayerBinding {
        return PlayerActivityPlayerBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<PlayerViewModel> = PlayerViewModel::class.java

    override fun initialize(savedInstanceState: Bundle?) {
        FileDownloader.setup(this@PlayerActivity)
        getViewModel().sendEvent(
            PlayerUiEvent.HeaderData(
                intent.getStringExtra(Constants.KEY_DATA)!!, homeStyle
            )
        )
        changeBottomBtnStyle()
        requireBinding().apply {
            pvPlayer.setOnViewClickLister(this@PlayerActivity)
            pvPlayer.setOnSkipListener(this@PlayerActivity)
            mbCollect.setOnClickListener(this@PlayerActivity)
            mbPraise.setOnClickListener(this@PlayerActivity)
            mbShare.setOnClickListener(this@PlayerActivity)
            pvPlayer.loading()
            ScreenRotateUtils.getInstance(applicationContext)
                .setOrientationChangeListener(this@PlayerActivity)
            val srlRefresh: SmartRefreshLayout = findViewById(com.at.common.R.id.srl_refresh)
            srlRefresh.setOnRefreshListener(this@PlayerActivity)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            srlRefresh.setBackgroundColor(
                StyleUtils.getPageBackground(StyleConsts.VIDEO_PLAYER).toArgb()
            )
            val rvList: RecyclerView = findViewById(com.at.common.R.id.rv_list)
            rvList.layoutManager = GridLayoutManager(this@PlayerActivity, 1)
            val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
            pool.setMaxRecycledViews(0, 30)
            rvList.setRecycledViewPool(pool)
            rvList.adapter = mAdapter
            rvList.setHasFixedSize(true)
//            rvList.addItemDecoration(PlayerDecoration())
        }
        mAdapter.setOnItemChildClickListener(this)
        mAdapter.addItemBinder(SynopsisBinder())
        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 10)
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginVertical = true,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV),
                showMarginHorizontal = false
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginVertical = true))
        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.SWIPER_ADV
                ), showMarginVertical = true
            )
        )
        mAdapter.addItemBinder(RouteBinder(pool, this))
        mAdapter.addItemBinder(LevelBinder(this))
        mAdapter.addItemBinder(
            VideoListBinder(
                this, JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.VIDEO_GRID)
            )
        )
        mAdapter.addChildClickViewIds(R.id.tv_synopsis, R.id.tv_more, R.id.mb_switch)
        mAdapter.setOnItemClickListener(this)
    }

    override fun loadData() {
//        getViewModel().sendEvent(PlayerUiEvent.InitData)
    }

    private fun changeBottomBtnStyle() {
        var praiseDrawable = com.video.module_widget.R.drawable.vector_drawable_raise_normal1
        var collectDrawable = com.video.module_widget.R.drawable.vector_drawable_collect_normal1
        var shareDrawable = com.video.module_widget.R.drawable.vector_drawable_share1
        footerBar?.let {
            when (it.style) {
                2 -> {
                    praiseDrawable = com.video.module_widget.R.drawable.vector_drawable_dianzan
                    collectDrawable = com.video.module_widget.R.drawable.vector_drawable_shoucang
                    shareDrawable = com.video.module_widget.R.drawable.vector_drawable_fenxiang
                }
                3 -> {
                    praiseDrawable =
                        com.video.module_widget.R.drawable.vector_drawable_raise_normal3
                    collectDrawable =
                        com.video.module_widget.R.drawable.vector_drawable_collect_normal3
                    shareDrawable = com.video.module_widget.R.drawable.vector_drawable_share3
                }
            }

            requireBinding().apply {
                mbPraise.setIconResource(praiseDrawable)
                mbCollect.setIconResource(collectDrawable)
                mbShare.setIconResource(shareDrawable)
            }
        }
    }

    private fun getPraiseResId(isSelected: Boolean): Int {
        var praiseDrawable =
            if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected1 else com.video.module_widget.R.drawable.vector_drawable_raise_normal1
        footerBar?.let {
            when (it.style) {
                2 -> {
                    praiseDrawable =
                        if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected2 else com.video.module_widget.R.drawable.vector_drawable_raise_normal2
                }
                3 -> {
                    praiseDrawable =
                        if (isSelected) com.video.module_widget.R.drawable.vector_drawable_raise_selected3 else com.video.module_widget.R.drawable.vector_drawable_raise_normal3
                }
            }
        }
        return praiseDrawable
    }

    private fun getCollectResId(isSelected: Boolean): Int {
        var collectDrawable =
            if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected1 else com.video.module_widget.R.drawable.vector_drawable_collect_normal1
        footerBar?.let {
            when (it.style) {
                2 -> {
                    collectDrawable =
                        if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected2 else com.video.module_widget.R.drawable.vector_drawable_collect_normal2
                }
                3 -> {
                    collectDrawable =
                        if (isSelected) com.video.module_widget.R.drawable.vector_drawable_collect_selected3 else com.video.module_widget.R.drawable.vector_drawable_collect_normal3
                }
            }
        }
        return collectDrawable
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        setIntent(intent)
        isPlayerIng = false
        mTorrentDownload?.stopStream()
        mTorrentDownload = null
        if (mSynopsisPopup != null) {
            mSynopsisPopup!!.dismiss()
            mSynopsisPopup = null
        }
        if (episodePopup != null) {
            episodePopup!!.dismiss()
            episodePopup = null
        }
        getViewModel().sendEvent(
            PlayerUiEvent.HeaderData(
                intent!!.getStringExtra(Constants.KEY_DATA)!!, homeStyle
            )
        )
        requireBinding().pvPlayer.hidePauseAd()
        val rvList: RecyclerView = findViewById(com.at.common.R.id.rv_list)
        rvList.scrollToPosition(0)
        getViewModel().sendEvent(PlayerUiEvent.InitData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::pageStatus) {
                val srlRefresh: SmartRefreshLayout = findViewById(com.at.common.R.id.srl_refresh)
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING
                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                            Log.d("TAG", "observerssserror: ${it.error.message}")
                            it.error.message?.apply {
                                if (contains("不存在")) {
                                    ToastUtils.show("资源不存在")
//                                    onBack()
                                }
                            }
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            if (srlRefresh.isRefreshing) {
                                srlRefresh.finishRefresh()
                            }
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::refreshStatus) {
                val srlRefresh: SmartRefreshLayout = findViewById(com.at.common.R.id.srl_refresh)
                when (it) {
                    is RefreshStatus.RefreshSuccess -> {
                        srlRefresh.finishRefresh()
                    }
                    is RefreshStatus.RefreshLoading -> {
                        if (!srlRefresh.isRefreshing) {
                            srlRefresh.autoRefreshAnimationOnly()
                        }
                    }
                    is RefreshStatus.RefreshFail -> {
                        srlRefresh.finishRefresh(false)
                    }
                    else -> {}
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::info) {
                if (it != null) {
                    requireBinding().apply {
                        mbPraise.text = if (it.is_star) "已点赞" else "点赞"
                        mbCollect.text = if (it.is_collect == 1) "已收藏" else "收藏"
                        mbPraise.setIconResource(getPraiseResId(it.is_star))
                        mbCollect.setIconResource(getCollectResId(it.is_collect == 1))
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::level) {
                if (it != null) {
                    requireBinding().pvPlayer.setLevelData(it)
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(PlayerUiState::likeLsCollectStatus) {
                if (it.parentPos < 0) {
                    return@collectState
                }
                val parent = mAdapter.data[it.parentPos]
                if (parent is VideoListBinder.Data) {
                    val child = parent.data[it.childPos]
                    if (child is ItemFreeBinder.Free) {
                        child.info.apply {
                            is_collect = it.isCollect
                        }
                    } else if (child is ItemPayBinder.Pay) {
                        child.info.apply {
                            is_collect = it.isCollect
                        }
                    }
                    mAdapter.notifyItemChanged(
                        it.parentPos, VideoListBinder.REFRESH_COLLECT_STATUS
                    )
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is PlayerUiEffect.Loading -> LoadingUtil.showLoading(
                        this@PlayerActivity, it.content
                    )
                    is PlayerUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is PlayerUiEffect.Toast -> ToastUtils.show(it.msg)
                    is PlayerUiEffect.ChangeDataEffect -> mAdapter.notifyItemChanged(
                        it.pos, it.data
                    )
                    is PlayerUiEffect.PlayBeforeAd -> {
                        homeStyle?.let { it1 ->
//                            if (it1.playerBeforeAdv?.show == true) {
                            requireBinding().pvPlayer.setAd(it.info)
//                            }
                        }
                    }
                    is PlayerUiEffect.PayForPlay -> requireBinding().pvPlayer.setPay(it.info)
                    is PlayerUiEffect.PlayLoading -> requireBinding().pvPlayer.loading()
                    is PlayerUiEffect.Play -> {
                        requireBinding().apply {
                            if (it.router.list.isNullOrEmpty()) {
                                ToastUtils.show("播放线路异常!")
                                return@apply
                            }
                            if (it.level.list.isNullOrEmpty()) {
                                ToastUtils.show("播放异常")
                                return@apply
                            }
                            var level = it.level.list[it.level.index].url
                            if (level.isNullOrEmpty()) {
                                ToastUtils.show("播放资源异常")
                                return@apply
                            }
                            // 磁力链接处理
                            if (!level.isNullOrEmpty() && level.startsWith("magnet:?")) {
                                // 先判断是否缓存完的视频
                                val level = URLDecoder.decode(level, "utf-8")
                                val cacheKey = CacheUtils.hashKeyForDisk(level)
                                Log.i("cache key", "=$cacheKey")
                                if (CacheHelper.getInstance().contains(cacheKey)) {
                                    val url: String? =
                                        CacheHelper.getInstance().getString(cacheKey, null)
                                    if (url.isNullOrEmpty() || !File(url).exists()) {
                                        loadVideo(
                                            level, it, this
                                        )
                                    } else {
                                        normalPlayUrl(
                                            CacheHelper.getInstance().getString(cacheKey, ""), ""
                                        )
                                    }
                                } else {
                                    loadVideo(
                                        level, it, this
                                    )
                                }
                            } else {
                                mTorrentDownload?.stopStream()
                                normalPlayUrl(level, "")
                            }
                        }
                    }
                    is PlayerUiEffect.PlayPauseAd -> {
                        requireBinding().apply {
                            if (pvPlayer.state == Jzvd.STATE_PAUSE) {
                                // 播放广告
                                homeStyle?.let { it1 ->
                                    if (it1.playerPauseAdv?.show == true) {
                                        pvPlayer.setPauseAd(it.info, homeStyle?.playerPauseAdv)
                                    }
                                }
                            }
                        }
                    }
                    is PlayerUiEffect.VideoInfoFail -> {
                        ToastUtils.show("资源不存在")
                    }
                    else -> {}
                }
            }
        }
    }

    private fun normalPlayUrl(url: String?, title: String?) {
        isPlayerIng = true
        requireBinding().pvPlayer.setRead(null)
        Log.i("play url:", "=" + url)
        requireBinding().pvPlayer.setUp(
            url, title
        )
        requireBinding().pvPlayer.startVideo()
//    }
    }

    /**
     * 磁力链接播放
     */
    private fun loadVideo(
        level: String?,
        it: PlayerUiEffect.Play,
        playerActivityPlayerBinding: PlayerActivityPlayerBinding
    ) {
        if (mTorrentDownload != null) {
            mTorrentDownload!!.stopStream()
            if (mBitDownloadListener != null) {
                mTorrentDownload!!.removeListener(mBitDownloadListener)
            }
        }

        mTorrentDownload = TorrentDownload(CacheEntity().apply {
            videoURL = level
            videoTag = it.level.list[it.level.index].name
            cacheName = CacheUtils.hashKeyForDisk(level)
        })
        // 有序播放
        mTorrentDownload!!.setDownloadModule(true)

        mBitDownloadListener = object : DownloadListener(mTorrentDownload) {
            override fun onStreamPrepared(torrent: Torrent?) {
                torrent?.isSequentialDownload = true
            }

            override fun onStreamStarted(torrent: Torrent?) {
                torrent?.isSequentialDownload = true

            }

            override fun onStreamError(
                torrent: Torrent?, e: java.lang.Exception?
            ) {
                // 取 ws 地址
                try {
                    val ws = "&ws="
                    if (level != null && level.contains(ws)) {
                        mTorrentDownload?.stopStream()
                        val startPos = level.indexOf(ws)
                        var newUrl = level.substring(startPos + 4, level.length)
                        val index = newUrl.indexOfFirst { it == '&' }
                        if (index > 0) {
                            newUrl = newUrl.substring(0, index)
                        }
                        isPlayerIng = true
                        normalPlayUrl(newUrl, "")
                    } else {
                        requireBinding().pvPlayer.error()
                    }
                } catch (e: Exception) {

                }
            }

            override fun onStreamReady(torrent: Torrent?) {
            }

            override fun onStreamProgress(
                torrent: Torrent?, status: StreamStatus?
            ) {
                Log.i("progress=", "" + torrent?.isSequentialDownload)
                mVideoPayerUrl = torrent?.videoFile?.absolutePath
                val moviePath: File = File(mVideoPayerUrl)
                if (moviePath.exists() && moviePath.isFile && moviePath.canRead()) {
                    mFile = RandomAccessFile(moviePath, "r")
                }
//                Log.i(
//                    "progress=",
//                    "torrent=${torrent != null}  " + "file=${mFile != null} " + "isPLay=${!isPlayerIng}" + " header=${mTorrentDownload!!.headerAddOver}"
//                )
                if (torrent != null && mFile != null && !isPlayerIng && mTorrentDownload!!.headerAddOver) {
                    currentPlayTime = System.currentTimeMillis()
                    if (mFile!!.length() > 0) {
                        isPlayerIng = true
                        LogUtils.i("player_start_play")

                        playerActivityPlayerBinding.pvPlayer.setUp(
                            JZDataSource(
                                Uri.parse(mVideoPayerUrl)
                            ), Jzvd.SCREEN_NORMAL, JZCustomReaderMediaIjk::class.java
                        )
                        mMagnetReadData = MagnetReadData(mVideoPayerUrl!!, mTorrentDownload!!)
                        mMagnetReadData!!.onMagnetReadListener =
                            object : MagnetReadData.OnMagnetReadListener {
                                override fun onLoadVideo(type: Int) {
                                    loadVideo(type)
                                }

                            }
                        playerActivityPlayerBinding.pvPlayer.setRead(mMagnetReadData!!)
                        playerActivityPlayerBinding.pvPlayer.startVideo()

                    }
                }
            }

            override fun onStreamStopped() {
                Log.i("progress=", " stop")
            }

            override fun onURLParsing() {
            }

            override fun onStreamFinished(torrent: Torrent?) {
                Log.i("progress=", " completed")

            }

        }
        mTorrentDownload!!.addListener(mBitDownloadListener)
        requireBinding().pvPlayer.hideAdUi()
        requireBinding().pvPlayer.loading()
        mTorrentDownload!!.setDownloadModule(true)
        isPlayerIng = false
        mTorrentDownload!!.startStream(
            TrackersUtil.spliceTracker(level), cacheDir.absolutePath + "/player/"
        )

    }

    override fun onResume() {
        super.onResume()
        ScreenRotateUtils.getInstance(this).start(this)
        if (mIsPlaying) {
            Jzvd.goOnPlayOnResume()
        } else {
            getViewModel().sendEvent(
                PlayerUiEvent.HeaderData(
                    intent!!.getStringExtra(Constants.KEY_DATA)!!, homeStyle
                )
            )
            requireBinding().pvPlayer.loading()
            getViewModel().sendEvent(PlayerUiEvent.InitData)
        }
    }

    /**
     * 判断头部是否下载成功
     *
     * @param torrent
     * @return
     */
    private fun downHeadSuccess(torrent: Torrent): Boolean {
        if (existAvailableTorrent(torrent)) {
            for (i in 0 until torrent.num) {
                if (existAvailableTorrent(torrent) && !torrent.torrentHandle.havePiece(i)) {
                    return false
                }
            }
        }
        return true
    }

    /**
     * 判断是否可用的torrent
     *
     * @param torrent
     * @return
     */
    private fun existAvailableTorrent(torrent: Torrent?): Boolean {
        return torrent != null && torrent.torrentHandle != null && torrent.torrentHandle.isValid
    }

    override fun onBackPressedSupport(): Boolean {
        return Jzvd.backPress()
    }

    override fun onPause() {
        super.onPause()
        ScreenRotateUtils.getInstance(this).stop()
        mIsPlaying = requireBinding().pvPlayer.state == Jzvd.STATE_PLAYING
        if (!(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && isInPictureInPictureMode) && mIsPlaying) {
            Jzvd.goOnPlayOnPause()
        }
    }

    override fun onDestroy() {
        mHandler.removeCallbacksAndMessages(null)
        super.onDestroy()
        try {
            ScreenRotateUtils.getInstance(this.applicationContext)
                .setOrientationChangeListener(null)
            Jzvd.releaseAllVideos()
            mTorrentDownload?.stopStream()
            mTorrentDownload = null
            if (!mVideoPayerUrl.isNullOrEmpty()) com.at.video.player.utils.FileUtils.deleteCache(
                mVideoPayerUrl
            )
            mMagnetReadData = null
        } catch (e: Exception) {

        }

    }

    override fun onSuspension() {
        val appOpsManager: AppOpsManager =
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
                getSystemService(Context.APP_OPS_SERVICE) as AppOpsManager
            } else {
                TODO("VERSION.SDK_INT < KITKAT")
            }

        val result = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            appOpsManager.unsafeCheckOpNoThrow(
                AppOpsManager.OPSTR_PICTURE_IN_PICTURE, application.applicationInfo.uid, packageName
            )
        } else {
            appOpsManager.checkOpNoThrow(
                AppOpsManager.OPSTR_PICTURE_IN_PICTURE, application.applicationInfo.uid, packageName
            )
        }

        if (AppOpsManager.MODE_ALLOWED != result) {
            Log.d("TAG123456", "没有开启画中画")
            val intent = Intent("android.settings.PICTURE_IN_PICTURE_SETTINGS")
            intent.data = Uri.parse("package:$packageName")
            startActivity(intent)
            return
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            mPictureInPictureParamsBuilder = PictureInPictureParams.Builder()
            // 设置宽高比例值，第一个参数表示分子，第二个参数表示分母
            // 下面的10/5=2，表示画中画窗口的宽度是高度的两倍
            val aspectRatio = Rational(16, 9)
            // 设置画中画窗口的宽高比例
            mPictureInPictureParamsBuilder!!.setAspectRatio(aspectRatio)
            /*updatePictureInPictureActions(R.drawable.player_ic_suspension_pause, "暂停播放", "暂停播放",
                REQUEST_PAUSE, 102)*/
            updatePictureInPictureActions(
                R.drawable.player_ic_suspension_play, "开始播放", "开始播放", REQUEST_PLAY, 101
            )

            // 进入画中画模式，注意enterPictureInPictureMode是Android8.0之后新增的方法
            enterPictureInPictureMode(mPictureInPictureParamsBuilder!!.build())
        }
    }

    override fun onPauseOrPlay() {
        if (requireBinding().pvPlayer.state == JzvdStd.STATE_PLAYING) {
            mHandler.sendEmptyMessage(0)
        } else if (requireBinding().pvPlayer.state == Jzvd.STATE_PAUSE) {
            getViewModel().sendEvent(PlayerUiEvent.PauseAd)
            mHandler.removeCallbacksAndMessages(null)
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    private fun updatePictureInPictureActions(
        @DrawableRes iconId: Int,
        title: String,
        description: String,
        controlType: String,
        requestCode: Int
    ) {
        val actions: MutableList<RemoteAction> = mutableListOf()
        actions.add(
            RemoteAction(
                Icon.createWithResource(this, R.drawable.player_ic_suspension_retreat_quickly),
                "快退",
                "快退",
                PendingIntent.getBroadcast(
                    this, 100, Intent(ACTION_MEDIA_CONTROL).putExtra(
                        EXTRA_CONTROL_TYPE, REQUEST_RETREAT_QUICKLY
                    ), PendingIntent.FLAG_IMMUTABLE
                )
            )
        )

        actions.add(
            RemoteAction(
                Icon.createWithResource(this, iconId),
                title,
                description,
                PendingIntent.getBroadcast(
                    this, requestCode, Intent(ACTION_MEDIA_CONTROL).putExtra(
                        EXTRA_CONTROL_TYPE, controlType
                    ), PendingIntent.FLAG_IMMUTABLE
                )
            )
        )

        actions.add(
            RemoteAction(
                Icon.createWithResource(this, R.drawable.player_ic_suspension_fast_forward),
                "快进",
                "快进",
                PendingIntent.getBroadcast(
                    this, 103, Intent(ACTION_MEDIA_CONTROL).putExtra(
                        EXTRA_CONTROL_TYPE, REQUEST_FAST_FORWARD
                    ), PendingIntent.FLAG_IMMUTABLE
                )
            )
        )

        mPictureInPictureParamsBuilder!!.setActions(actions)

        setPictureInPictureParams(mPictureInPictureParamsBuilder!!.build())
    }

    override fun onPictureInPictureModeChanged(
        isInPictureInPictureMode: Boolean, newConfig: Configuration?
    ) {
        super.onPictureInPictureModeChanged(isInPictureInPictureMode, newConfig)
        if (isInPictureInPictureMode) {
            registerReceiver(mReceiver, IntentFilter(ACTION_MEDIA_CONTROL))
        } else {
            unregisterReceiver(mReceiver)
        }
        requireBinding().pvPlayer.setIsInPictureInPictureMode(isInPictureInPictureMode)
    }

    private val mReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val extra: String? = intent.getStringExtra(EXTRA_CONTROL_TYPE)
            if (extra == REQUEST_PLAY) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    updatePictureInPictureActions(
                        R.drawable.player_ic_suspension_pause, "暂停播放", "暂停播放", REQUEST_PAUSE, 102
                    )
                }
                requireBinding().pvPlayer.clickStartOrPause()
                mIsPlaying = true
            } else if (extra == REQUEST_PAUSE) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    updatePictureInPictureActions(
                        R.drawable.player_ic_suspension_play, "开始播放", "开始播放", REQUEST_PLAY, 101
                    )
                }
                mIsPlaying = false
                requireBinding().pvPlayer.clickStartOrPause()
            } else if (extra == REQUEST_RETREAT_QUICKLY) {
                requireBinding().pvPlayer.seekTo((requireBinding().pvPlayer.seekToManulPosition - 15000).toLong())
            } else if (extra == REQUEST_FAST_FORWARD) {
                requireBinding().pvPlayer.seekTo((requireBinding().pvPlayer.seekToManulPosition + 15000).toLong())
            }
        }
    }

    override fun orientationChange(orientation: Int) {
        if (Jzvd.CURRENT_JZVD != null && (requireBinding().pvPlayer.state == Jzvd.STATE_PLAYING || requireBinding().pvPlayer.state == Jzvd.STATE_PAUSE) && requireBinding().pvPlayer.screen != Jzvd.SCREEN_TINY) {
            if (orientation in 45..315 && requireBinding().pvPlayer.screen == Jzvd.SCREEN_NORMAL) {
                changeScreenFullLandscape(ScreenRotateUtils.orientationDirection)
            } else if ((orientation in 0..44 || orientation > 315) && requireBinding().pvPlayer.screen == Jzvd.SCREEN_FULLSCREEN) {
                changeScreenNormal()
            }
        }
    }

    /**
     * 竖屏并退出全屏
     */
    private fun changeScreenNormal() {
        if (requireBinding().pvPlayer.screen == Jzvd.SCREEN_FULLSCREEN) {
            requireBinding().pvPlayer.autoQuitFullscreen()
        }
    }

    /**
     * 横屏
     */
    private fun changeScreenFullLandscape(x: Float) {
        // 从竖屏状态进入横屏
        if (requireBinding().pvPlayer.screen != Jzvd.SCREEN_FULLSCREEN) {
            if (System.currentTimeMillis() - Jzvd.lastAutoFullscreenTime > 2000) {
                requireBinding().pvPlayer.autoFullscreen(x)
                Jzvd.lastAutoFullscreenTime = System.currentTimeMillis()
            }
        }
    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        when (view.id) {
            R.id.tv_synopsis -> {
                val bean: SynopsisBinder.SynopsisData =
                    adapter.getItem(position) as SynopsisBinder.SynopsisData
                val content: View = requireBinding().llContent
                if (mSynopsisPopup == null) {
                    mSynopsisPopup = SynopsisPopup(this, bean.data, bean.actorList, bean.extra)
                }
                XPopup.Builder(this).popupWidth(content.width).popupHeight(content.measuredHeight)
                    .enableDrag(false).hasShadowBg(false).asCustom(mSynopsisPopup).show()
            }
            R.id.tv_more -> {
                val bean: LevelBinder.LevelData = adapter.getItem(position) as LevelBinder.LevelData
                val content: View = requireBinding().llContent
                if (episodePopup == null) {
                    episodePopup = EpisodePopup(this, bean, 1, this)
                } else {
                    episodePopup?.setIndex(bean.index)
                }
                episodePopup?.setStyle(
                    StyleUtils.getHomeContentStyle(
                        homeStyle!!, StyleConsts.PARTS
                    )
                )
                XPopup.Builder(this).popupWidth(content.width).popupHeight(content.measuredHeight)
                    .enableDrag(false).hasShadowBg(false).asCustom(episodePopup).show()
            }
            R.id.mb_switch -> {
                getViewModel().sendEvent(
                    PlayerUiEvent.Switch(
                        position, adapter.getItem(position) as VideoListBinder.Data
                    )
                )
            }
        }
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.mb_praise -> getViewModel().sendEvent(PlayerUiEvent.CollectOrLike(-1, 1))
            R.id.mb_collect -> getViewModel().sendEvent(PlayerUiEvent.CollectOrLike(-1, 2))
            R.id.mb_share -> {
                if (GlobalConsts.SHARE_URL.isNullOrEmpty()) {
                    ToastUtils.show("功能暂未开放")
                } else {
                    ShareDialog().show(supportFragmentManager, "share_address")
                }
            }
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(PlayerUiEvent.RefreshData)
    }

    override fun getSpanSize(
        gridLayoutManager: GridLayoutManager, viewType: Int, position: Int
    ): Int {
        return if (viewType == 7 || viewType == 8) 1 else gridLayoutManager.spanCount
    }

    override fun onSkip() {
        getViewModel().sendEvent(PlayerUiEvent.Play)
    }

    override fun onRouter(data: RouteBinder.RouteData, position: Int) {
        requireBinding().pvPlayer.hidePauseAd()
        // 重新加载播放前广告
        getViewModel().sendEvent(PlayerUiEvent.SwitchRouter(data, position))
    }

    override fun onLevel(data: LevelBinder.LevelData, position: Int) {
        requireBinding().pvPlayer.hidePauseAd()
        // 重新加载播放前广告
        getViewModel().sendEvent(PlayerUiEvent.SwitchLevel(data, position))
    }

    override fun onPrepared() {
        val info: WatchHistorys? = getViewModel().state.value.watchHistory
        info?.let {
            val progress: Double = it.progress?.toDouble() ?: 0.0
            try {
                val total: Long = it.percentage?.toLong() ?: 0L
                val curPositon = progress * total / 100
                Log.e("PROGRESS", "onPrepared: $curPositon")
                requireBinding().apply {
                    pvPlayer.seekTo((progress * total / 100).toLong())
                }
            } catch (e: Exception) {
            }
        }
        mHandler.sendEmptyMessage(0)
    }

    override fun onPay() {
        getViewModel().sendEvent(PlayerUiEvent.PayForPlay)
    }

    override fun playError() {
    }

    private val mHandler: Handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            requireBinding().apply {
                val total = pvPlayer.duration / 1000
                val playIndex: Int = getViewModel().state.value.level?.index ?: 0

                if (pvPlayer.state == Jzvd.STATE_PLAYING) {
                    val position: Float = pvPlayer.currentPositionWhenPlaying / 1000f
                    val totalTime = pvPlayer.totalTimeTextView.text.toString()
                    val progress = pvPlayer.progressBar.progress
                    Log.e("PROGRESS", "handleMessage: $progress")
                    if (total > 0 && progress > 0) {
                        getViewModel().sendEvent(
                            PlayerUiEvent.UpdatePlayTime(
                                playIndex, progress.toFloat(), total.toString()
                            )
                        )
                    }
                } else if (pvPlayer.state == Jzvd.STATE_AUTO_COMPLETE) {
                    Log.e("PROGRESS", "播放完成")
                    if (total > 0) {
                        getViewModel().sendEvent(
                            PlayerUiEvent.UpdatePlayTime(
                                playIndex, 100f, total.toString()
                            )
                        )
                    }
                }
            }
            sendEmptyMessageDelayed(0, 10_000)
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: Any = adapter.getItem(position)!!
        if (data is ItemFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        } else if (data is ItemPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        }
//        else if(data is AdImageBinder.Data) {
//            Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//        } else if(data is AdTextBinder.Data) {
//            Router.getInstance().build("/webview/WebViewActivity")
//                .withString(Constants.KEY_DATA, data.data.jumpAddr)
//                .navigation()
//        }
    }

    override fun onlistViewClick(
        parentPos: Int, adapter: BaseQuickAdapter<*, *>, view: View, position: Int
    ) {
        if (view.id == R.id.iv_collect) {
            getViewModel().sendEvent(PlayerUiEvent.CollectOrLike(parentPos, 2, position))
        }
    }

    var loadVideoTime: Long = 0

    private fun loadVideo(i: Int) {
        //3s内不能反复调用
        if (System.currentTimeMillis() - loadVideoTime < 3000) {
            return
        }
        if (mMagnetReadData == null) {
            return
        }
        loadVideoTime = System.currentTimeMillis()
        try {
            val mUrl = mTorrentDownload!!.entity.videoPath
            val moviePath: File = File(mUrl)
            if (moviePath.exists() && moviePath.isFile && moviePath.canRead()) {
                mFile = RandomAccessFile(moviePath, "r")
            }
            if (mFile != null && !isPlayerIng) {
                isPlayerIng = true
                requireBinding().pvPlayer.setUp(
                    JZDataSource(
                        Uri.parse(mUrl)
                    ), Jzvd.SCREEN_NORMAL, JZCustomReaderMediaIjk::class.java
                )
                requireBinding().pvPlayer.setRead(mMagnetReadData!!)
                requireBinding().pvPlayer.startVideo()
            }
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        }
    }
}
