package com.at.video.player.ui.player.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import com.at.video.player.R
import com.at.video.player.entity.VideoBean
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.loadByGlide
import com.video.module_widget.widget.ShapeTextViewExt
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/5/13 15:04
 * @Description:
 */

class ItemFreeBinder(val type: Int = 1) : BaseItemBinder<ItemFreeBinder.Free, BaseViewHolder>() {

    private fun getRootParamsByStyleType(): FrameLayout.LayoutParams {
        val params = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_137)
        )

        when (type) {
            1 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_266)
            }
            2 -> {
                params.height = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_190)
            }
        }
        return params
    }


    private fun getImageParamsByType() = when (type) {
        1 -> {
            com.at.common.R.dimen.dp_221
        }
        2 -> {
            com.at.common.R.dimen.dp_150
        }
        else -> {
            com.at.common.R.dimen.dp_221
        }
    }

    private fun changeTagStyle(tagView: ShapeTextViewExt) {
        val params = tagView.layoutParams as FrameLayout.LayoutParams
        when (StyleUtils.tagStyle) {
            2 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10)
                params.rightMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_4)
                tagView.reBuilder()
            }
            3 -> {
                tagView.setBorderRadius(com.at.common.R.dimen.dp_10, com.at.common.R.dimen.dp_10)
                params.topMargin = StyleUtils.getDpByDimen(com.at.common.R.dimen.dp_8)
                tagView.reBuilder()
            }
        }
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        val view: View = LayoutInflater.from(parent.context).inflate(
            R.layout.player_item_free_view, parent, false
        )
        return BaseViewHolder(view)
    }

    override fun convert(holder: BaseViewHolder, item: Free) {
        val root = holder.getView<FrameLayout>(R.id.fl_root)
        root.layoutParams = getRootParamsByStyleType()
        val image = holder.getView<DiversityImageView>(R.id.div_image)
        val params = image.layoutParams
        params.height = StyleUtils.getDpByDimen(getImageParamsByType())

        holder.getView<ImageView>(R.id.div_image).loadByGlide(
            GlideImageConfig(
                item.info.cover_vertical.imageUrl(),
                placeholderId = com.video.module_widget.R.drawable.common_ic_image_video_default,
                errorId = com.video.module_widget.R.drawable.common_ic_image_video_default
            ), WeakReference(context)
        )
//        val numStr =
//            if (item.info.episode_total?.strToInt() == 0) "已完结" else "全${item.info.episode_total}集"
        // 是否收藏
        val collect =
            if (item.info.is_collect == 1) R.drawable.player_ic_collect_checked else R.drawable.player_ic_collect_default
        holder.setImageResource(R.id.iv_collect, collect)
        holder.setGone(R.id.iv_collect, false)

        holder.setText(R.id.stv_name, item.info.highlight).setText(R.id.tv_update, item.info.note)
            .setText(R.id.tv_title, item.info.name).setText(R.id.tv_score, "${item.info.score}")
//        if (StyleUtils.tagEnable) {
//            changeTagStyle(holder.getView(R.id.stv_free))
//            holder.setGone(R.id.stv_free, false)
//        } else {
//            holder.setGone(R.id.stv_free, true)
//        }

    }

    data class Free(val info: VideoBean)
}