package com.at.video.player.ui.player.binder

import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.at.video.player.databinding.PlayerTitleListView1Binding
import com.at.video.player.entity.VideoUrl
import com.at.video.player.ui.player.LevelAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.QuickViewBindingItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.video.module_widget.entity.Content
import com.video.module_widget.widget.RightIntervalDecoration

/**
 * @Create_time: 2022/5/13 10:20
 * @Description: 播放线路
 */

class LevelBinder constructor(private val mOnLevelListener: OnLevelListener):
    QuickViewBindingItemBinder<LevelBinder.LevelData, PlayerTitleListView1Binding>(),
    OnItemClickListener {

    private var mPosition:Int = 0

    override fun onCreateViewBinding(layoutInflater: LayoutInflater, parent: ViewGroup,
                                     viewType: Int): PlayerTitleListView1Binding {
        val binding: PlayerTitleListView1Binding = PlayerTitleListView1Binding.inflate(layoutInflater,
            parent, false)
        binding.tvTitle.text = "选集"
        binding.rvList.itemAnimator = null
        binding.rvList.setHasFixedSize(true)
//        val params = binding.rvList.layoutParams
//        params.height = parent.context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_53)
//        binding.rvList.layoutParams = params
        val pool: RecyclerView.RecycledViewPool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 10)
        binding.rvList.setRecycledViewPool(pool)
        val padding: Int = parent.context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
        binding.rvList.addItemDecoration(RightIntervalDecoration())
        val adapter = LevelAdapter()
        adapter.setOnItemClickListener(this)
        binding.rvList.adapter = adapter


        return binding
    }

    override fun convert(holder: BinderVBHolder<PlayerTitleListView1Binding>, data: LevelData) {
        holder.viewBinding.rvList.tag = data

        val adapter = (holder.viewBinding.rvList.adapter as LevelAdapter)
        adapter.setNewInstance(data.list)
        adapter.setStyle(data.style)
        if(data.list.size == 0){
            holder.viewBinding.tvTitle.visibility = View.GONE
        }else{
            holder.viewBinding.tvTitle.visibility = View.VISIBLE
        }

        adapter.setIndex(data.index)
        holder.viewBinding.rvList.scrollToPosition(data.index)
//        Log.e("Level", "选中: ${data.index} style: ${data.style}" )
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val level: LevelData = adapter.recyclerView.tag as LevelData
        mOnLevelListener.onLevel(level, position)
    }

    data class LevelData (var list: MutableList<VideoUrl>, var index: Int, val style:Content? = null)

    interface OnLevelListener {
        fun onLevel(data: LevelData, position: Int)
    }
}
