package com.at.video.player.ui.player.binder

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.at.video.player.databinding.PlayerTitleListViewBinding
import com.at.video.player.entity.Line
import com.at.video.player.entity.LineBean
import com.at.video.player.ui.player.TagAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.QuickViewBindingItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.melancholy.widget.decoration.DefaultDecoration

/**
 * @Create_time: 2022/5/13 10:20
 * @Description: 播放线路
 */

class RouteBinder constructor(private val pool: RecyclerView.RecycledViewPool,
                              private val mOnRouterListener: OnRouterListener):
    QuickViewBindingItemBinder<RouteBinder.RouteData, PlayerTitleListViewBinding>(),
    OnItemClickListener {

    override fun onCreateViewBinding(layoutInflater: LayoutInflater, parent: ViewGroup,
                                     viewType: Int): PlayerTitleListViewBinding {
        val binding: PlayerTitleListViewBinding = PlayerTitleListViewBinding.inflate(layoutInflater,
            parent, false)
        binding.tvTitle.text = "播放线路"
        binding.rvList.itemAnimator = null
        binding.rvList.setHasFixedSize(true)
        binding.rvList.setRecycledViewPool(pool)
        val padding: Int = parent.context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
        val cPadding: Int = parent.context.resources.getDimensionPixelSize(com.at.common.R.dimen.dp_12)
        binding.rvList.addItemDecoration(DefaultDecoration.create(0, 0, padding,
            0, cPadding))
        val adapter = TagAdapter()
        adapter.setOnItemClickListener(this)
        binding.rvList.adapter = adapter
        return binding
    }

    override fun convert(holder: BinderVBHolder<PlayerTitleListViewBinding>, data: RouteData) {
        holder.viewBinding.rvList.tag = data
        val adapter: TagAdapter = (holder.viewBinding.rvList.adapter as TagAdapter)
        adapter.setNewInstance(data.list)
        adapter.setIndex(data.index)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: RouteData = adapter.recyclerView.tag as RouteData
        mOnRouterListener.onRouter(data, position)
    }

    data class RouteData (val list: MutableList<LineBean>, var index: Int = 0)

    interface OnRouterListener {
        fun onRouter(data: RouteData, position: Int)
    }
}
