package com.at.video.player.ui.player.binder

import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.video.player.R
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.google.android.material.button.MaterialButton
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.entity.Content
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.getSp
import com.video.module_widget.utils.toArgb
import com.video.module_widget.utils.toColor
import com.video.module_widget.widget.IntervalDecoration

class VideoListBinder(
    private val onChild: OnItemChildLsViewClickListener,
    private val style: Content? = null
) : BaseItemBinder<VideoListBinder.Data, BaseViewHolder>(), OnItemClickListener {

    data class Data(
        val data: MutableList<Any>,
        val type: Int = 1,
        val datas: List<List<Any>>? = null,
        val curPos: Int = 0
    )

    companion object {
        const val REFRESH_COLLECT_STATUS = "refresh_collect_status"
    }

    private val intervalDecoration = IntervalDecoration()
    override fun convert(holder: BaseViewHolder, data: Data) {
        val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
        mAdapter.addItemBinder(ItemFreeBinder(data.type))
        mAdapter.addItemBinder(ItemPayBinder(data.type))

        holder.getView<RecyclerView>(R.id.rv_data).apply {
            adapter = mAdapter
            when (data.type) {
                1 -> {
                    layoutManager = GridLayoutManager(context, 2)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
                2 -> {
                    layoutManager = GridLayoutManager(context, 3)
                    removeItemDecoration(intervalDecoration)
                    addItemDecoration(intervalDecoration)
                }
            }
        }
        style?.let {
            it.config?.title?.let { it1 ->
                holder.setTextColor(R.id.tv_title, (it1.color ?: "rgba(0,0,0,1)").toArgb())
                val title = holder.getView<TextView>(R.id.tv_title)
                title.setTextSize(TypedValue.COMPLEX_UNIT_PX, context.getSp(it1.size))
            }
        }
        val change = StyleUtils.change
        if (change != null) {
            val switch = holder.getView<MaterialButton>(R.id.mb_switch)
            holder.setGone(R.id.mb_switch, !change.enable)
            when (change.style) {
                1 -> {
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_START
                    switch.backgroundTintList = StyleUtils.createColorStateList(
                        context.toColor(R.color.color_FFF8F8F8),
                        context.toColor(R.color.color_FFF8F8F8)
                    )
                }
                2 -> {
                    switch.iconTint = StyleUtils.createColorStateList(
                        context.toColor(R.color.color_FF4B5363),
                        context.toColor(R.color.color_FF4B5363)
                    )
                    switch.backgroundTintList = StyleUtils.createColorStateList(
                        context.toColor(R.color.color_FFF8F8F8),
                        context.toColor(R.color.color_FFF8F8F8)
                    )
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_END
                }
                else -> {
                    switch.iconGravity = MaterialButton.ICON_GRAVITY_TEXT_END
                    switch.iconTint = StyleUtils.createColorStateList(
                        context.toColor(R.color.color_FFFF613F),
                        context.toColor(R.color.color_FFFF613F)
                    )
                    switch.backgroundTintList = StyleUtils.createColorStateList(
                        context.toColor(R.color.color_FFFFF6F4),
                        context.toColor(R.color.color_FFFFF6F4)
                    )
                }
            }
        }
        mAdapter.setNewInstance(data.data)
        mAdapter.setOnItemClickListener(this)
        mAdapter.addChildClickViewIds(R.id.iv_collect)
        mAdapter.setOnItemChildClickListener { adapter, view, position ->
            onChild.onlistViewClick(
                holder.layoutPosition,
                adapter,
                view,
                position
            )
        }
    }

    override fun convert(holder: BaseViewHolder, data: Data, payloads: List<Any>) {
        if (payloads.isNotEmpty()) {
            payloads.forEach {
                when (it) {
                    REFRESH_COLLECT_STATUS -> {
                        val adapter1 =
                            holder.getView<RecyclerView>(R.id.rv_data).adapter as BaseBinderAdapter
                        adapter1.setNewInstance(data.data)
                        adapter1.notifyDataSetChanged()
                    }
                    is Data -> {
                        val adapter1 =
                            holder.getView<RecyclerView>(R.id.rv_data).adapter as BaseBinderAdapter
                        adapter1.setNewInstance(it.data)
                        adapter1.notifyDataSetChanged()
                    }
                }
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder =
        BaseViewHolder(
            LayoutInflater.from(parent.context).inflate(R.layout.player_video_list, parent, false)
        )

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data: Any = adapter.getItem(position)!!
        if (data is ItemFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        } else if (data is ItemPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        }
    }

    /**
     * 子RecylerView点击事件传递接口
     */
    interface OnItemChildLsViewClickListener {
        fun onlistViewClick(
            parentPos: Int,
            adapter: BaseQuickAdapter<*, *>,
            view: View,
            position: Int
        )
    }
}
