package com.at.video.player.ui.player.synopsis

import android.annotation.SuppressLint
import android.content.Context
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.at.common.global.Constants
import com.at.video.player.R
import com.at.video.player.entity.Extra
import com.at.video.player.entity.RoleBean
import com.at.video.player.entity.VideoBean
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.lxj.xpopup.core.BottomPopupView
import com.melancholy.router.api.launcher.Router
import com.melancholy.widget.decoration.DefaultDecoration
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.strToInt
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.toArgb
import com.willy.ratingbar.ScaleRatingBar
import java.lang.StringBuilder

/**
 * @Create_time: 2022/5/13 15:54
 * @Description:
 */

class SynopsisPopup(context: Context, val data: VideoBean, private val person: List<RoleBean>?,val extra: Extra?):
    BottomPopupView(context), View.OnClickListener, OnItemClickListener {

    override fun getImplLayoutId(): Int = R.layout.player_popup_sysnopsis
    private var homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.VIDEO_SYNOPSIS)

    @SuppressLint("SetTextI18n")
    override fun onCreate() {
        super.onCreate()
        findViewById<View>(R.id.iv_close).setOnClickListener(this)
        val recyclerView: RecyclerView = findViewById(R.id.recycler_view)
        val tvTitle =findViewById<TextView>(R.id.tv_title)
        val tvHeat =findViewById<TextView>(R.id.tv_heat)
        val tvArea =findViewById<TextView>(R.id.tv_area)
        val tvInfo =findViewById<TextView>(R.id.tv_info)
        val tvInfoTitle =findViewById<TextView>(R.id.tv_intro_title)
        val tvScore =findViewById<TextView>(R.id.tv_score)
        val rbScore = findViewById<ScaleRatingBar>(R.id.srb_score)
        val llRoot = findViewById<LinearLayout>(R.id.ll_bg)
        tvTitle.text = data.name
        tvHeat.text = "${extra?.pop_view?.strToInt()?.plus(1)}次播放"
        val str = StringBuilder()
        if(data.area?.isNotEmpty() == true) str.append("${data.area} ")
        if(!data.year.isNullOrEmpty()) str.append("${data.year} ")
        if(!data.subcateg_name.isNullOrEmpty()) str.append("${data.subcateg_name}")
        tvArea.text = str.toString()
        tvInfo.text = data.introduction
        tvInfoTitle.visibility = if(data.introduction.isNullOrEmpty()) View.GONE else View.VISIBLE
        rbScore.rating = ((data.score?.toFloat()?.div(2)) ?: 0f)
        tvScore.text = data.score.toString()

        var fillDrawable = VectorDrawableCompat.create(resources, com.video.module_widget.R.drawable.vector_drawable_icon_rating1,null)
//        fillDrawable?.setTint()
        homeStyle?.let { it ->
            it.background?.let { it1 -> llRoot.setBackgroundColor(it1.toArgb()) }
            val score = JsonUtils.getHomeContentStyle(homeStyle,"score")
            val viewCount =  JsonUtils.getHomeContentStyle(homeStyle,"viewCounts")
            score?.config?.let { it2 ->
                it2.color?.let { it1 -> tvScore.setTextColor(it1.toArgb()) }
                when(it2.style){
                    1->{
//                        rbScore.setFilledDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_rating1)
                        if (fillDrawable != null) {
                            it2.fill?.let { it1 -> fillDrawable?.setTint(it1.toArgb()) }
                            rbScore.setFilledDrawable(fillDrawable!!)
                        }
                        rbScore.setEmptyDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_normal_rating1)
                    }
                    2->{
                        fillDrawable = VectorDrawableCompat.create(resources, com.video.module_widget.R.drawable.vector_drawable_icon_rating2,null)
                        if (fillDrawable != null) {
                            it2.fill?.let { it1 -> fillDrawable?.setTint(it1.toArgb()) }
                            rbScore.setFilledDrawable(fillDrawable!!)
                        }
//                        rbScore.setFilledDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_rating2)
                        rbScore.setEmptyDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_normal_rating2)

                    }
                    else ->{
//                        rbScore.setFilledDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_rating3)
                        fillDrawable = VectorDrawableCompat.create(resources, com.video.module_widget.R.drawable.vector_drawable_icon_rating3,null)
                        if (fillDrawable != null) {
                            it2.fill?.let { it1 -> fillDrawable?.setTint(it1.toArgb()) }
                            rbScore.setFilledDrawable(fillDrawable!!)
                        }
                        rbScore.setEmptyDrawableRes(com.video.module_widget.R.drawable.vector_drawable_icon_normal_rating3)
                    }
                }
            }
            viewCount?.config?.let { it1 ->
                        tvHeat.visibility = if(it1.enable) View.VISIBLE else View.GONE
            }
        }
       if(!person.isNullOrEmpty()){
           val adapter = StarAdapter()
           adapter.setOnItemClickListener(this)
           recyclerView.adapter = adapter
           recyclerView.setHasFixedSize(true)
           recyclerView.itemAnimator = null
           val padding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_16)
           val cPadding: Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_8)
           recyclerView.addItemDecoration(DefaultDecoration.create(padding, 0,
               0, 0, cPadding))
           adapter.addData(person)
       }else{
           recyclerView.visibility = GONE
           findViewById<View>(R.id.v_line).visibility = GONE
       }

    }

    override fun onClick(p0: View?) {
        dismiss()
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position) as RoleBean
        Router.getInstance().build("/player/DetailActivity")
            .withString(Constants.KEY_DATA_1,data.person_name!!)
            .withString(Constants.KEY_DATA, data.id!!).navigation()

    }
}