package com.at.video.player.ui.work

import android.graphics.drawable.GradientDrawable
import android.os.Bundle
import android.view.ViewGroup
import com.at.common.adapter.FragmentAdapter
import com.at.common.adapter.PagerData
import com.at.common.binding.BaseBindingActivity
import com.at.common.global.Constants
import com.at.common.setLightStatusBar
import com.at.video.player.R
import com.at.video.player.databinding.PlayerTabViewPagerBinding
import com.at.video.player.entity.TabInfo
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.ext.buildIndicator
import com.loper7.tab_expand.ext.buildText
import com.loper7.tab_expand.ext.toPx
import com.loper7.tab_expand.indicator.BaseIndicator
import com.loper7.tab_expand.indicator.LinearIndicator
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.tabs.TabLayoutMediators
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.*
import com.video.module_widget.widget.CustomTabText
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/13 17:04
 * @Description: 演员详情作品
 */

@AndroidEntryPoint
@Route(path = "/player/WorkActivity")
class WorkActivity :
    BaseBindingActivity<PlayerTabViewPagerBinding>(),
    TabLayoutMediators.TabConfigurationStrategy {
    private var mMediator: TabLayoutMediators? = null
    private lateinit var adapter: FragmentAdapter
    private lateinit var mTabs: ArrayList<TabInfo>
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ACTOR_DETAIL_MORE)

    override fun createViewBinding(): PlayerTabViewPagerBinding {
        return PlayerTabViewPagerBinding.inflate(layoutInflater)
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        val id: String = intent.getStringExtra(Constants.KEY_DATA)!!
        val cateId = intent.getStringExtra(Constants.KEY_DATA_2)
        mTabs = intent.getParcelableArrayListExtra(Constants.KEY_DATA_1)!!
        requireTitleBar().apply {
            addBackButton()
            setTitle("更多")
        }
        requireBinding().apply {
            tbTab.tabMode = TabLayout.MODE_SCROLLABLE
            changTabStyle(tbTab)
            adapter = FragmentAdapter(supportFragmentManager, lifecycle)
            var currentIndex = 0
            for (i: Int in 0 until mTabs.size) {
                val it = mTabs[i]
                val bundle = Bundle()
                bundle.putString(Constants.KEY_DATA, id)
                bundle.putString(Constants.KEY_DATA_1, it.id)
                val itemId: Long = it.id.toLong()
                adapter.add(PagerData("/player/TelevisionFragment", itemId, bundle))
                if (cateId == it.id) {
                    currentIndex = i
                }
            }
            val params: ViewGroup.LayoutParams = tbTab.layoutParams
            params.height = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_32)
            params.width = ViewGroup.LayoutParams.WRAP_CONTENT

            tbTab.layoutParams = params
            vpViewPager.offscreenPageLimit = adapter.itemCount
            vpViewPager.adapter = adapter
            vpViewPager.currentItem = currentIndex
            mMediator = TabLayoutMediators(
                tbTab,
                vpViewPager,
                this@WorkActivity,
                true
            )
            mMediator!!.attach()
        }
    }
    private fun changTabStyle(tab: TabLayout) {
        val tabStyle: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.TABS)
        tabStyle?.config?.let {
            val text = it.text

            when (it.style) {
                1 -> {
                    val indicator = it.indicator
                    val height = indicator?.height ?: 2
                    val color = indicator?.color?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    val normalColor = text?.color?.toArgb() ?: "#666666".toColor()
                    val selectedColor = text?.selectedColor?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    tab.buildIndicator<LinearIndicator>()
                        .setHeight(getDp(height).toInt())
                        .setWidth(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20))
                        .setColor(color)
                        .bind()
                    tab.buildText<CustomTabText>()
                        .setTabColors(mutableListOf(normalColor), mutableListOf(selectedColor))
                        .setTabNormalSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.size!!).toInt()
                                )
                            )
                        )
                        .setTabSelectedSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.selectedSize!!).toInt()
                                )
                            )
                        )
                        .setSelectTextBold(true)
                        .setNormalTextBold(false)
                        .bind()
                }
                2 -> {
                    val bg = it.fill?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    tab.buildIndicator<LinearIndicator>()
                        .setAngle(2.toPx())
                        .setColor(bg)
                        .setHeight(BaseIndicator.MATCH)
                        .setWidth(BaseIndicator.MATCH)
                        .bind()
                    val gradientDrawable1 = GradientDrawable()
                    gradientDrawable1.setStroke(2, bg)
                    gradientDrawable1.cornerRadius = 2.toPx().toFloat()
                    tab.background = gradientDrawable1
                    val selectedColor = text?.selectedColor?.toArgb() ?: "#FFFFFF".toColor()
                    val normalColor = text?.color?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    tab.buildText<CustomTabText>()
                        .setTabColors(mutableListOf(normalColor), mutableListOf(selectedColor))
                        .setTabNormalSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.size!!).toInt()
                                )
                            )
                        )
                        .setTabSelectedSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.selectedSize!!).toInt()
                                )
                            )
                        )
                        .setSelectTextBold(true)
                        .setNormalTextBold(false)
                        .bind()
                }
                3 -> {
                    val bg = it.fill?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    tab.buildIndicator<LinearIndicator>()
                        .setColor(bg)
                        .setHeight(BaseIndicator.MATCH)
                        .setWidth(BaseIndicator.MATCH)
                        .bind()
                    val gradientDrawable1 = GradientDrawable()
                    gradientDrawable1.setStroke(2, bg)
                    gradientDrawable1.cornerRadius = 14.toPx().toFloat()
                    tab.background = gradientDrawable1
                    val selectedColor = text?.selectedColor?.toArgb() ?: "#FFFFFF".toColor()
                    val normalColor = text?.color?.toArgb() ?: StyleUtils.getPrimaryColor().toArgb()
                    tab.buildText<CustomTabText>()
                        .setTabColors(mutableListOf(normalColor), mutableListOf(selectedColor))
                        .setTabNormalSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.size!!).toInt()
                                )
                            )
                        )
                        .setTabSelectedSizes(
                            mutableListOf(
                                DensityUtils.px2sp(
                                    this@WorkActivity,
                                    getSp(text?.selectedSize!!).toInt()
                                )
                            )
                        )
                        .setSelectTextBold(true)
                        .setNormalTextBold(false)
                        .bind()
                }
            }
        }
    }

    override fun loadData() {
    }

    override fun onConfigureTab(tab: TabLayout.Tab, position: Int) {
        tab.text = mTabs[position].name
    }

    override fun onDestroy() {
        mMediator?.detach()
        mMediator = null
        super.onDestroy()
    }
}
