package com.at.video.player.ui.work.television

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.databinding.CommonStatusRefreshListBinding
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.player.entity.StarVideoInfo
import com.at.video.player.entity.VideoInfo
import com.at.video.player.ui.work.television.binder.ItemFreeBinder
import com.at.video.player.ui.work.television.binder.ItemPayBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.GridSpanSizeLookup
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.GapLineBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import com.video.module_widget.widget.LodeMoreView
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/5/18 15:10
 * @Description:
 */

@AndroidEntryPoint
@Route(path = "/player/TelevisionFragment")
class TelevisionFragment : BaseMviFragment<CommonStatusRefreshListBinding, TelevisionViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()
    private val homeStyle: HomeStyle? = JsonUtils.getStyleBean(StyleConsts.ACTOR_DETAIL_MORE)

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): CommonStatusRefreshListBinding {
        return CommonStatusRefreshListBinding.inflate(inflater, container, false)
    }

    override fun getViewModelClass(): Class<TelevisionViewModel> {
        return TelevisionViewModel::class.java
    }

    override fun getViewLifecycleOwner(): LifecycleOwner {
        return requireActivity()
    }

    override fun getViewModelKey(): String? {
        return requireArguments().getString(Constants.KEY_DATA_1)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        getViewModel().sendEvent(
            TelevisionUiEvent.HeaderData(
                requireArguments().getString(Constants.KEY_DATA)!!,
                requireArguments().getString(Constants.KEY_DATA_1)!!,
                homeStyle
            )
        )
        val dataStyle: Content? = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.WORKS)
        val spanCount = dataStyle?.config?.layout?.style ?: 2
        requireBinding().includedRefreshList.apply {
            srlRefresh.setOnRefreshListener(this@TelevisionFragment)
            srlRefresh.setOnRefreshLoadMoreListener(this@TelevisionFragment)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            val footer = LodeMoreView(requireContext())
            StyleUtils.changeLoadMore(footer)
            srlRefresh.setRefreshFooter(footer)
            rvList.itemAnimator = null
            rvList.setHasFixedSize(true)

            rvList.layoutManager = GridLayoutManager(requireContext(), spanCount + 1)
            rvList.addItemDecoration(WorkDecoration())
            rvList.adapter = mAdapter
            mAdapter.setOnItemClickListener(this@TelevisionFragment)
            srlRefresh.setBackgroundColor(Color.WHITE)
        }

        mAdapter.addItemBinder(
            BannerBinder(
                JsonUtils.getHomeContentStyle(
                    homeStyle, StyleConsts.SWIPER_ADV
                )
            )
        )
        mAdapter.addItemBinder(
            GapLineBinder(
                StyleUtils.getPageBackground(StyleConsts.ACTOR_DETAIL_MORE).toArgb()
            )
        )
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginHorizontal = false,
                style = JsonUtils.getHomeContentStyle(homeStyle, StyleConsts.BANNER_ADV),
                bg = StyleUtils.getPageBackground(StyleConsts.ACTOR_DETAIL_MORE)
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginHorizontal = false))
        mAdapter.addItemBinder(ItemFreeBinder(spanCount))
        mAdapter.addItemBinder(ItemPayBinder(spanCount))
        mAdapter.setOnItemClickListener(this)
        mAdapter.setGridSpanSizeLookup(object : GridSpanSizeLookup {
            override fun getSpanSize(
                gridLayoutManager: GridLayoutManager, viewType: Int, position: Int
            ): Int {
                if (viewType == 1 || viewType == 2 || viewType == 3 || viewType == 4) {
                    return gridLayoutManager.spanCount
                }
                return 1
            }
        })
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(TelevisionUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.viewState = MultiStateView.ViewState.LOADING
                            }
                        }
                        is PageStatus.Error -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.getView(MultiStateView.ViewState.ERROR)!!
                                    .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                    it.error.message
                                msvView.viewState = MultiStateView.ViewState.ERROR
                            }
                        }
                        is PageStatus.Empty -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(false)
                                msvView.viewState = MultiStateView.ViewState.EMPTY
                            }
                        }
                        is PageStatus.Success -> {
                            includedRefreshList.apply {
                                srlRefresh.setEnableRefresh(true)
                                srlRefresh.setEnableLoadMore(true)
                                msvView.viewState = MultiStateView.ViewState.CONTENT
                            }
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(TelevisionUiState::refreshStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(TelevisionUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(TelevisionUiState::loadMoreStatus) {
                requireBinding().includedRefreshList.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.resetNoMoreData()
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(TelevisionUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(TelevisionUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(TelevisionUiEvent.LoadMoreData)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position)
        if (data is ItemFreeBinder.Free) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        } else if (data is ItemPayBinder.Pay) {
            Router.getInstance().build("/player/PlayerActivity")
                .withString(Constants.KEY_DATA, data.info.id!!).navigation()
        }
    }
}
