
package com.at.video.player.utils;

import android.content.Context;
import android.os.Environment;
import android.os.StatFs;
import android.text.TextUtils;
import android.util.Log;

import com.at.video.player.libtorrent.bean.CacheEntity;
import com.frostwire.jlibtorrent.TorrentBuilder;
import com.frostwire.jlibtorrent.TorrentInfo;
import com.frostwire.jlibtorrent.swig.add_torrent_params;
import com.frostwire.jlibtorrent.swig.error_code;
import com.frostwire.jlibtorrent.swig.sha1_hash;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;


public class FileUtils {
    /**
     * 获取内置SD卡路径
     *
     * @return
     */
    public static String getExternalSDPath() {
        boolean sdCardExist = Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED); //判断sd卡是否存在
        String sd_default = "";
        if (sdCardExist) {
            sd_default =
                    Environment.getExternalStorageDirectory().getAbsolutePath();
        }
        return sd_default;
    }


    public static String getSDPath() {
        String dir = getExternalSDPath();
        return mkdirs(dir);
    }


    public static String getCacheDir() {
        String dir = getExternalSDPath() + "/videoPlayer/Cache/";
        return mkdirs(dir);
    }

    public static String getCacheHeadDir() {
        String dir = getExternalSDPath() + "/videoPlayer/Head/";
        return mkdirs(dir);
    }

    public static String getDownloadDir() {
        String dir = getExternalSDPath() + "/videoPlayer/Download/";
        return mkdirs(dir);
    }

    private static String mkdirs(String dir) {
        File file = new File(dir);
        if (!file.exists()) {
            file.mkdirs();
        }
        return dir;
    }

    /**
     * 根据输入时间删除超过时间的文件
     *
     * @param path 文件夹路径（文件集根目录）
     * @param time 相差时间（如一周一个月等）
     */
    public static void aboutTimeRemove(String path, long time) {
        File file = new File(path);
        if (file.exists() && file.isDirectory()) {
            for (File child : file.listFiles()) {
                if (System.currentTimeMillis() - child.lastModified() > time) {
                    removeFiles(child.getAbsolutePath());
                }
            }
        }
    }

    /**
     * 下载头获取存放路径
     *
     * @param extraUrl
     * @return
     */
    public static File getFile(String extraUrl) {
        if (extraUrl == null || extraUrl.isEmpty()) {
            return null;
        }
        extraUrl = extraUrl.replace(".mp4", ".7z");
        File file = new File(FileUtils.getCacheHeadDir() + extraUrl);
        // Log.i("zip", "getFile: " + saveDirectory + "\n" + file);
        return file;
    }

    /**
     * 删除当前文件夹所有内容
     *
     * @param path
     */
    public static void removeFiles(String path) {
        File file = new File(path);
        if (!file.exists()) {
            return;
        }
        if (file.isDirectory()) {
            for (File child : file.listFiles()) {
                removeFiles(child.getAbsolutePath());
            }
        }
        file.delete();
    }

    /**
     * 把这个字节数转化为KB、MB、GB的最终呈现给用户 是否要小数点
     *
     * @param size
     * @return
     */
    private static DecimalFormat df = new DecimalFormat("#.0");

    public static String getPrintSize(long size) {
        String fileSizeString = "";
        String wrongSize = "0B";
        if (size == 0) {
            return wrongSize;
        }
        if (size < 1024) {
            fileSizeString = df.format((double) size) + "B";
        } else if (size < 1048576) {
            fileSizeString = df.format((double) size / 1024) + "KB";
        } else if (size < 1073741824) {
            fileSizeString = df.format((double) size / 1048576) + "MB";
        } else {
            fileSizeString = df.format((double) size / 1073741824) + "GB";
        }
        return fileSizeString;
    }

    /**
     * SD卡剩余空MB
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public static float getUsSizeMB(String path) {
        StatFs fs = new StatFs(path);
        if (fs != null) {
            return (fs.getAvailableBlocks() * (fs.getBlockSize() / (1024f * 1024f)));
        }
        return 0;
    }

    /**
     * 得到SD卡总大小GB
     *
     * @return
     */
    public static float getCountSizeGB(String path) {
        StatFs fs = new StatFs(path);
        if (fs != null) {
            return (fs.getBlockCount() * (fs.getBlockSize() / (1024f * 1024f * 1024)));
        }
        return 0;
    }

    /**
     * Delete every item below the File location
     *
     * @param file Location
     * @return {@code true} when successful delete
     */
    public static boolean recursiveDelete(File file) {
        if (file == null || !file.exists()) {
            return true;
        }

        if (file.isDirectory()) {
            String[] children = file.list();
            if (children == null) return false;
            for (String child : children) {
                recursiveDelete(new File(file, child));
            }
        }

        return file.delete();
    }

    /**
     * 获取指定目录内所有文件路径
     *
     * @param dirPath 需要查询的文件目录
     */
    private static void getAllFiles(String dirPath, List<File> mFiles) {
        File f = new File(dirPath);
        if (!f.exists()) {//判断路径是否存在
            return;
        }

        File[] files = f.listFiles();
        if (files == null) {//判断权限
            return;
        }

        for (File _file : files) {//遍历目录
            if (_file.isFile()) {
                mFiles.add(_file);
            } else if (_file.isDirectory()) {//查询子目录
                getAllFiles(_file.getAbsolutePath(), mFiles);
            }
        }
    }

    /**
     * 主动删除缓存文件
     */

    public static void deleteCache(String path) {
        List<File> mFiles = new ArrayList<>();
        getAllFiles(FileUtils.getCacheDir(), mFiles);
        //添加head目录
        if (path == null) {
            removeFiles(getCacheHeadDir());
        }
    }


    /**
     * 根据SD卡剩余空间删除文件
     */
    public static void deleteFileCache() {
        String path = getExternalSDPath();
        if (TextUtils.isEmpty(path)) return;
        float UsMB = getUsSizeMB(path);
        if (UsMB < 2 * 1024) {
            deleteCache(null);
        }
    }


    /**
     * //判断是否存在完整本地包
     *
     * @param file   检查的包文件
     * @param urlapk 磁力链接 magnet
     * @return
     */
    public static boolean isApkExistence(File file, String urlapk) {
        Log.i("zip", "isApkExistence: " + file.getPath() + "\t" + urlapk);
        try {
            if (file.length() <= 0) {
                return false;
            }

            TorrentBuilder builder = new TorrentBuilder().pieceSize(512 * 1024);
            TorrentBuilder.Result result = builder.path(file).generate();

            error_code ec = new error_code();
            add_torrent_params p = add_torrent_params.parse_magnet_uri(urlapk, ec);
            sha1_hash info_hash = p.getInfo_hash();

            TorrentInfo info = TorrentInfo.bdecode(result.entry().bencode());

            return info_hash.to_hex().equals(info.swig().info_hash().to_hex());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 将数据存到文件中
     *
     * @param context  context
     * @param data     需要保存的数据
     * @param fileName 文件名
     */
    public static void saveDataToFile(Context context, String data, String fileName) {
        FileOutputStream fileOutputStream = null;
        BufferedWriter bufferedWriter = null;
        try {
            /**
             * "data"为文件名,MODE_PRIVATE表示如果存在同名文件则覆盖，
             * 还有一个MODE_APPEND表示如果存在同名文件则会往里面追加内容
             */
            fileOutputStream = context.openFileOutput(fileName,
                    Context.MODE_PRIVATE);
            bufferedWriter = new BufferedWriter(
                    new OutputStreamWriter(fileOutputStream));
            bufferedWriter.write(data);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (bufferedWriter != null) {
                    bufferedWriter.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }


    /**
     * 从文件中读取数据
     *
     * @param context  context
     * @param fileName 文件名
     * @return 从文件中读取的数据
     */
    public static String loadDataFromFile(Context context, String fileName) {
        FileInputStream fileInputStream = null;
        BufferedReader bufferedReader = null;
        StringBuilder stringBuilder = new StringBuilder();
        try {
            /**
             * 注意这里的fileName不要用绝对路径，只需要文件名就可以了，系统会自动到data目录下去加载这个文件
             */
            fileInputStream = context.openFileInput(fileName);
            bufferedReader = new BufferedReader(
                    new InputStreamReader(fileInputStream));
            String result = "";
            while ((result = bufferedReader.readLine()) != null) {
                stringBuilder.append(result);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return stringBuilder.toString();
    }

    /**
     * 解压assets的zip压缩文件到指定目录
     *
     * @param zipPtath
     * @param outputDirectory
     * @throws IOException
     */
    public  static void unzipFile(String zipPtath, String outputDirectory) throws IOException {

        Log.i("zip", "开始解压的文件： " + zipPtath + "\n" + "解压的目标路径：" + outputDirectory);
        // 创建解压目标目录
        File file = new File(outputDirectory);
        // 如果目标目录不存在，则创建
        if (!file.exists()) {
            file.mkdirs();
        }
        // 打开压缩文件
        InputStream inputStream = new FileInputStream(zipPtath);
        ;
        ZipInputStream zipInputStream = new ZipInputStream(inputStream);

        // 读取一个进入点
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        // 使用1Mbuffer
        byte[] buffer = new byte[1024 * 1024];
        // 解压时字节计数
        int count = 0;
        // 如果进入点为空说明已经遍历完所有压缩包中文件和目录
        while (zipEntry != null) {
            Log.i("zip", "解压文件 入口 1： " + zipEntry);
            if (!zipEntry.isDirectory()) {  //如果是一个文件
                // 如果是文件
                String fileName = zipEntry.getName();
                Log.i("zip", "解压文件 原来 文件的位置： " + fileName);
                fileName = fileName.substring(fileName.lastIndexOf("/") + 1);  //截取文件的名字 去掉原文件夹名字
                Log.i("zip", "解压文件 的名字： " + fileName);
                file = new File(outputDirectory + File.separator + fileName);  //放到新的解压的文件路径

                file.createNewFile();
                FileOutputStream fileOutputStream = new FileOutputStream(file);
                while ((count = zipInputStream.read(buffer)) > 0) {
                    fileOutputStream.write(buffer, 0, count);
                }
                fileOutputStream.close();

            }

            // 定位到下一个文件入口
            zipEntry = zipInputStream.getNextEntry();
            Log.i("zip", "解压文件 入口 2： " + zipEntry);
        }
        zipInputStream.close();
        Log.i("zip", "解压完成");
    }
}
