package com.at.video.player.utils;

import static android.media.MediaCodecList.REGULAR_CODECS;

import android.media.MediaCodecInfo;
import android.media.MediaCodecList;
import android.util.Log;

/**
 * Date:2020/5/14
 * <p>
 * Author:jocker
 * <p>
 * Description: 是否支持硬件解码工具
 */
public class MediaCodecUtil {
    /**
     * 是否支持 hevc 硬解
     *
     * @return
     */
    public static boolean isH265HWDecoderSupport() {
        MediaCodecList codecList = null;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
            codecList = new MediaCodecList(REGULAR_CODECS);
        }
        // 获取设备支持的所有 codec 信息
        MediaCodecInfo[] codecInfos = new MediaCodecInfo[0];
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
            codecInfos = codecList.getCodecInfos();
        }
        for (MediaCodecInfo codecInfo : codecInfos) {
            // 解码codec & 解码器名称包含'hevc' & 不是软件codec
            if (!codecInfo.isEncoder() && (codecInfo.getName().contains("hevc")
                    && !isSWCodec(codecInfo.getName()))) {
                return true;
            }
        }

        return false;
    }

    /**
     * 是否为软件 codec
     *
     * @param codecName
     * @return
     */
    public static boolean isSWCodec(String codecName) {
        if (codecName.startsWith("OMX.google.")) {
            return true;
        }

        return !codecName.startsWith("OMX.");
    }

    /**
     * 判断4.1版本以上是否有硬解功能
     *
     * @return
     */

    public static boolean isMediaCodecList() {
        boolean is_exist = false;
        try {
            // 判断4.1版本以上是否有硬解功能
            Class<?> aClass = Class.forName("android.media.MediaCodecList");
            is_exist = true;
            Log.i("MediaCodecUtil", "isMediaCodecList: " + aClass);
        } catch (ClassNotFoundException e) {
            Log.i("MediaCodecUtil", "isMediaCodecList: " + e);
        }
        return is_exist;
    }
}
