package com.at.video.service.http

import com.at.common.network.BaseBean
import com.at.common.network.PageBean
import com.at.video.service.entity.*
import com.melancholy.network.OKHttp
import okhttp3.MultipartBody
import okhttp3.ResponseBody
import retrofit2.http.*

interface ApiService {

    //访客登录
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/login")
    suspend fun login(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<MessageInfo>

    //发送消息
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/message")
    suspend fun sendMessage(@Body map: @JvmSuppressWildcards Map<String, Any>):BaseBean<SendMessageInfo>

    //获取最新消息
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/message/latest_msg")
    suspend fun getLatestMessage(@QueryMap map:@JvmSuppressWildcards Map<String, Any>):BaseBean<NewMessageInfo>

    //获取最新消息
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user")
    suspend fun getVisitInfo(@Query("visitor_id") visitId:Int):BaseBean<VisitInfo>

    //会话转工单
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/work_order/session")
    suspend fun sessionToWorkOrder(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<Any>

    //创建工单
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/work_order/normal")
    suspend fun createWorkOrder(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<Any>

    //查询工单
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/work_order")
    suspend fun workOrderList(@QueryMap map:@JvmSuppressWildcards Map<String, Any>):BaseBean<PageBean<OrderData>>


    //查询工单沟通记录
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/work_order/chat_history")
    suspend fun chatHistory(@QueryMap map:@JvmSuppressWildcards Map<String, Any>):BaseBean<PageBean<OrderDetailMsgInfo>>

    //会话结束
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/messages/over")
    suspend fun sessionOver(@QueryMap map:@JvmSuppressWildcards Map<String, Any>):BaseBean<Any>

    //获取历史会话
    @Headers("${OKHttp.SERVICE_URL}:true")
    @GET("v1/user/message/history_all")
    suspend fun historySession(@QueryMap map:@JvmSuppressWildcards Map<String, Any>):BaseBean<PageBean<Message>>

    //上传文件
    @Headers("${OKHttp.SERVICE_URL}:true")
    @Multipart
    @POST("v1/upload")
    suspend fun updateFile(@PartMap map:Map<String,String>,@Part part: MultipartBody.Part): BaseBean<FileInfo>

    //撤销
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/work_order/revoke")
    suspend fun revoke(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<Any>

    //结单
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/work_order/over")
    suspend fun over(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<Any>

    //工单聊天
    @Headers("${OKHttp.SERVICE_URL}:true")
    @POST("v1/user/work_order/chat")
    suspend fun chat(@Body map: @JvmSuppressWildcards Map<String, Any>): BaseBean<OrderDetailMsgInfo>

    //下载文件
    @Streaming
    @GET
    suspend fun download(@Url url:String):ResponseBody


}