package com.at.video.service.http.interceptor

import android.util.Log
import com.at.common.global.Constants
import com.melancholy.network.INetworkInterceptor
import com.melancholy.network.OKHttp
import com.melancholy.register.annotation.RegisterService
import okhttp3.Headers
import okhttp3.Interceptor
import okhttp3.Response

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/5 下午5:08
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/5 下午5:08
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

@RegisterService(serviceClass = INetworkInterceptor::class)
class ResetBaseUrlInterceptor : INetworkInterceptor {

    override fun priority(): Int = 3

    override fun intercept(chain: Interceptor.Chain): Response {
        var request = chain.request()
        val originalHeaders = request.headers
        val baseUrl = originalHeaders[OKHttp.BASE_URL]
        if (!baseUrl.isNullOrEmpty()) {
            //2.删除BaseUrl这个头部
            val newHeaders: Headers = createNewHeader(originalHeaders, OKHttp.BASE_URL)
            //3.替换BaseUrl
            val url: String = request.url.toString()
            val newBaseUrl: String = url.replace(Constants.BASE_URL, baseUrl)
            //4.重新构建Request
            request = request.newBuilder().url(newBaseUrl).headers(newHeaders).build()
        }
        val serviceUrl = originalHeaders[OKHttp.SERVICE_URL]
        if (!serviceUrl.isNullOrEmpty() && Constants.SERVICE_URL.isNotEmpty()) {
            //2.删除serviceUrl这个头部
            val newHeaders: Headers = createNewHeader(originalHeaders, OKHttp.SERVICE_URL)
            val url = request.url.toString()
            val newBaseUrl = url.replace(
                Constants.BASE_URL + Constants.URL_PATH_SUFFIX,
                Constants.SERVICE_URL + "/"
            )
            Log.d("TAG", "intercept: $newBaseUrl $serviceUrl $url")
            request = request.newBuilder().url(newBaseUrl).headers(newHeaders).build()
        }
        return chain.proceed(request)
    }

    //删除BaseUrl这个头部 构建新的header
    private fun createNewHeader(originalHeaders: Headers, key: String): Headers {
        val newHeaderBuilder = Headers.Builder()
        for (name: String in originalHeaders.names()) {
            if (name != key) {
                val value: String = originalHeaders[name] ?: ""
                newHeaderBuilder.add(name, value)
            }
        }
        return newHeaderBuilder.build()
    }

}