package com.at.video.service.ui.dialog

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.at.video.service.databinding.ServiceDialogNoticeBinding

/**
 * @Create_time: 2022/5/12 16:45
 * @Description: 详情通知
 */

class NoticeDialog(val dismissCallback: DialogDismissCallback): BaseBindingDialogFragment<ServiceDialogNoticeBinding>(), View.OnClickListener {

    interface DialogDismissCallback{
        fun dismiss()
    }

    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, msg: String,callback: DialogDismissCallback) {
            val dialog = NoticeDialog(callback)
            dialog.arguments = bundleOf(Constants.KEY_DATA to msg)
            dialog.show(manager, null)
        }
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): ServiceDialogNoticeBinding {
        return ServiceDialogNoticeBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvContent.text = requireArguments().getString(Constants.KEY_DATA)!!
            tvQuery.setOnClickListener(this@NoticeDialog)
        }
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_200)
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {

    }

    override fun onClick(view: View) {
        dismissCallback.dismiss()
        dismissAllowingStateLoss()
    }
}