package com.at.video.service.ui.dialog

import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.at.common.utils.ToastUtils
import com.at.video.service.R
import com.at.video.service.databinding.ServiceDialogOrderTransformBinding
import com.at.video.service.ui.service.conversation.ConversationUiEvent
import com.at.video.service.ui.service.conversation.ConversationViewModel
import com.at.video.service.ui.service.utils.SoftKeyboardUtils
import com.melancholy.widget.shape.view.ShapeEditText

/**
 * @Create_time: 2022/5/12 16:45
 * @Description: 详情通知
 */

class WorkOrderDialog(private val viewModel: ConversationViewModel): BaseBindingDialogFragment<ServiceDialogOrderTransformBinding>(), View.OnClickListener {


    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, msg: String,viewModel: ConversationViewModel) {
            val dialog = WorkOrderDialog(viewModel)
            dialog.arguments = bundleOf(Constants.KEY_DATA to msg)
            dialog.show(manager, null)
        }
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): ServiceDialogOrderTransformBinding {
        return ServiceDialogOrderTransformBinding.inflate(inflater, container, false)
    }

   private  lateinit var inputEt:ShapeEditText

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            val txt = requireArguments().getString(Constants.KEY_DATA)!!
            stInput.setText(txt)
            inputEt = stInput
            tvTxtNum.text ="${txt.length}/50"
            tvQuery.setOnClickListener(this@WorkOrderDialog)
            tvCancel.setOnClickListener(this@WorkOrderDialog)
            stInput.addTextChangedListener(object :TextWatcher{
                override fun beforeTextChanged(
                    s: CharSequence?,
                    start: Int,
                    count: Int,
                    after: Int
                ) {
                }

                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                }

                override fun afterTextChanged(s: Editable?) {
                    tvTxtNum.text ="${s.toString().length}/50"
                }

            })
        }
    }

    override fun dismiss() {
        SoftKeyboardUtils.closeKeyboard(inputEt)
        super.dismiss()
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_300)
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {

    }

    override fun onClick(view: View) {
        when(view.id){
            R.id.tv_cancel -> dismissAllowingStateLoss()
            R.id.tv_query ->{
                if(inputEt.text.toString().isNotEmpty()){
                    viewModel.sendEvent(ConversationUiEvent.SessionToWorkOrder(inputEt.text.toString()))
                    dismissAllowingStateLoss()
                }else{
                    ToastUtils.show("请输入标题!")
                }
            }
        }

    }
}