package com.at.video.service.ui.dialog

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.ImageView
import androidx.core.content.ContentProviderCompat.requireContext
import androidx.fragment.app.FragmentManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.utils.ToastUtils
import com.at.video.service.R
import com.at.video.service.databinding.ServiceDialogOrderResponseBinding
import com.at.video.service.entity.FileData
import com.at.video.service.ui.service.order.add.OrderUploadFileAdapter
import com.at.video.service.ui.service.order.detail.WorkOrderDetailActivity
import com.at.video.service.ui.service.order.detail.WorkOrderDetailUiEvent
import com.at.video.service.ui.service.order.detail.WorkOrderDetailViewModel
import com.at.video.service.ui.service.utils.SoftKeyboardUtils
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.lxj.xpopup.core.BottomPopupView
import com.melancholy.utils.FileUtils
import com.melancholy.utils.KeyBoardUtils
import com.melancholy.utils.MediaUtils
import com.melancholy.widget.shape.view.ShapeButton
import java.io.File

/**
 * @Create_time: 2022/5/12 16:45
 * @Description: 详情通知
 */

class WorkOrderResponsePopup(context: Context,val viewModel: WorkOrderDetailViewModel,val kefuId:String): BottomPopupView(context), View.OnClickListener {

    private val mAdapter = OrderDetailUploadFileAdapter()

    override fun onClick(v: View) {
        when(v.id){
            R.id.sb_submit ->{
                val inputTxt = mEditText.text.toString()
                if(inputTxt.isNotEmpty()){
                    viewModel.sendEvent(WorkOrderDetailUiEvent.SendMessage(inputTxt,kefuId))
                    if(mAdapter.itemCount > 0){
                        for (i in 0 until mAdapter.itemCount){
                                mAdapter.removeAt(i)
                        }
                    }
                    mEditText.setText("")
                    dismiss()
                }else{
                    ToastUtils.show("请输入回复内容!")
                }

            }
            R.id.iv_pic ->{
              if(mAdapter.itemCount <5){
                  BottomHeadDialog.showDialog((context as WorkOrderDetailActivity).supportFragmentManager, {
                      if (it == 0) {//从相册选取
                          MediaUtils.selectFile(context  as WorkOrderDetailActivity, 101)
                      } else {//拍照
                            viewModel.sendEvent(WorkOrderDetailUiEvent.TakePhoto)
                          }
                  })
              }else{
                  ToastUtils.show("最多只能上传5个文件")
              }
            }
            R.id.iv_folder ->{
               if(mAdapter.itemCount < 5){
                   val intent = Intent(Intent.ACTION_GET_CONTENT)
                   intent.type = "*/*"
                   intent.addCategory(Intent.CATEGORY_OPENABLE)
                   (context as WorkOrderDetailActivity).startActivityForResult(intent,102)
               }else{
                   ToastUtils.show("最多只能上传5个文件")
               }
            }
        }
    }

    private var mFile: File? = null


    /**
     * 拍照
     */
    private fun takingPictures() {
        try {
            if (mFile != null) {
                mFile!!.delete()
                mFile = null
            }
        } catch (e: Exception) {
        }
        mFile = File(
            FileUtils.getExternalCacheDirs(context)?.get(0) ?: context.cacheDir,
            "${System.currentTimeMillis()}.jpg")
        val uri =  FileUtils.fileToUri(context as WorkOrderDetailActivity, mFile!!)
        MediaUtils.takingPictures(context as WorkOrderDetailActivity, 100,
            uri)
    }

    override fun getImplLayoutId(): Int = R.layout.service_dialog_order_response

    private lateinit var mEditText:EditText

    override fun onCreate() {
        super.onCreate()
        val sbSubmit = findViewById<ShapeButton>(R.id.sb_submit)
        val ivPic = findViewById<ImageView>(R.id.iv_pic)
        ivPic.setOnClickListener(this)
        val ivFolder = findViewById<ImageView>(R.id.iv_folder)
        ivFolder.setOnClickListener(this)
        val fileRv =findViewById<RecyclerView>(R.id.rv_file_list)
            fileRv.adapter = mAdapter
        mEditText = findViewById(R.id.st_input)
        mAdapter.addChildClickViewIds(R.id.iv_delete)
        mAdapter.setOnItemChildClickListener{_,_,position->mAdapter.removeAt(position)}
        sbSubmit.setOnClickListener(this)
    }

    override fun getPopupHeight(): Int {
        return super.getPopupHeight()
    }

    fun addData(file:FileData){
        mAdapter.addData(file)
    }


    override fun dismiss() {
        SoftKeyboardUtils.closeKeyboard(mEditText)
        super.dismiss()
    }

    override fun getMaxHeight(): Int {
        return resources.getDimensionPixelSize(com.at.common.R.dimen.dp_200)
    }

    class OrderDetailUploadFileAdapter: BaseQuickAdapter<FileData,
            BaseViewHolder>(R.layout.service_work_order_upload_file_item) {

        override fun convert(holder: BaseViewHolder, item: FileData) {
            holder.setText(R.id.tv_name,item.name)
        }


    }
}