package com.at.video.service.ui.service

import android.content.Intent
import android.content.res.ColorStateList
import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.core.os.bundleOf
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.at.common.adapter.FragmentAdapter
import com.at.common.adapter.PagerData
import com.at.common.binding.BaseBindingActivity
import com.at.common.global.Constants
import com.at.common.setLightStatusBar
import com.at.video.service.R
import com.at.video.service.databinding.ServiceActivityServiceBinding
import com.at.video.service.ui.dialog.NoticeDialog
import com.at.video.service.ui.service.emoji.EmojiWidget
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.ext.buildIndicator
import com.loper7.tab_expand.ext.buildText
import com.loper7.tab_expand.indicator.LinearIndicator
import com.loper7.tab_expand.text.BaseText
import com.melancholy.network.ApiFactory
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.DensityUtils
import com.melancholy.widget.tabs.TabLayoutMediators
import dagger.hilt.android.AndroidEntryPoint
import net.danlew.android.joda.JodaTimeAndroid

/**
 * @Create_time: 2022/5/5 20:01
 * @Description: 客服
 */

@AndroidEntryPoint
@Route(path = "/service/ServiceActivity", extras = 10086)
class ServiceActivity: BaseBindingActivity<ServiceActivityServiceBinding>() ,  TabLayoutMediators.TabConfigurationStrategy{

    private val tabTitle: List<String> = listOf("当前会话", "我的工单")
    private var mMediator: TabLayoutMediators? = null
    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }
    override fun createViewBinding(): ServiceActivityServiceBinding {
        return ServiceActivityServiceBinding.inflate(layoutInflater)
    }

    fun getColorById(id:Int) = ContextCompat.getColor(this@ServiceActivity,id)

    override fun initialize(savedInstanceState: Bundle?) {
        JodaTimeAndroid.init(this)
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitleColor(Color.BLACK)
            setTitle("客服会话")
        }
        requireBinding().apply {
            val tbTab: TabLayout = findViewById(R.id.tb_tab)
            val params: ViewGroup.LayoutParams = tbTab.layoutParams
            params.height = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_36)
            tbTab.layoutParams = params
//            val states = mutableListOf<IntArray>()
//            states.add(intArrayOf(android.R.attr.state_checked))
//            states.add(intArrayOf(-android.R.attr.state_checked))
//            tbTab.tabTextColors = ColorStateList(states.toTypedArray(), intArrayOf(getColorById(R.color.color_FF0083FF),getColorById(R.color.color_FF181818)))
            tbTab.buildIndicator<LinearIndicator>()
                .setHeight(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_3))
                .setWidth(resources.getDimensionPixelSize(com.at.common.R.dimen.dp_20))
                .setColor(ContextCompat.getColor(this@ServiceActivity,R.color.color_FF0083FF))
                .bind()
            tbTab.buildText<BaseText>()
                .setNormalTextBold(false)
                .setNormalTextSize(
                    DensityUtils.px2sp(
                        this@ServiceActivity,
                        resources.getDimension(com.at.common.R.dimen.sp_16).toInt()
                    )
                )
                .setSelectTextBold(true)
                .setSelectTextSize(
                    DensityUtils.px2sp(
                        this@ServiceActivity,
                        resources.getDimension(com.at.common.R.dimen.sp_16).toInt()
                    )
                )

                .bind()
            val adapter = FragmentAdapter(supportFragmentManager, lifecycle)
            vpViewPager.adapter = adapter

            mMediator = TabLayoutMediators(
                tbTab, vpViewPager,
                this@ServiceActivity, true
            )
            mMediator!!.attach()
            adapter.add(
                PagerData(
                    "/service/conversation/ConversationFragment", 0L,
                    bundleOf(Constants.KEY_DATA to 1)
                )
            )
            adapter.add(
                PagerData(
                    "/service/order/OrderFragment", 1L,
                    bundleOf(Constants.KEY_DATA to 2)
                )
            )
            vpViewPager.offscreenPageLimit = adapter.itemCount
            vpViewPager.isUserInputEnabled = false
            vpViewPager.registerOnPageChangeCallback(ViewPagerCallback())
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        supportFragmentManager.fragments[0].onActivityResult(requestCode,resultCode,data)
    }

    private inner class ViewPagerCallback: ViewPager2.OnPageChangeCallback() {

        override fun onPageSelected(position: Int) {
            requireTitleBar().setTitle(if(position == 0)"客服会话" else tabTitle[position])

        }

    }

    fun changePage(page:Int = 0){
        requireBinding().apply {
            vpViewPager.currentItem = page
        }
    }

    override fun loadData() {
//        NoticeDialog.showDialog(supportFragmentManager,"当前无法使用客服功能",object :NoticeDialog.DialogDismissCallback{
//            override fun dismiss() {
//                finish()
//            }
//        })
    }

    override fun onConfigureTab(tab: TabLayout.Tab, position: Int) {
        tab.text = tabTitle[position]
    }
}