package com.at.video.service.ui.service.conversation

import android.app.Activity
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Rect
import android.os.Bundle
import android.util.Log
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import android.widget.EditText
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.utils.ToastUtils
import com.at.video.service.R
import com.at.video.service.databinding.ServiceFragmentConversationBinding
import com.at.video.service.ui.dialog.BottomHeadDialog
import com.at.video.service.ui.dialog.NoticeDialog
import com.at.video.service.ui.dialog.WorkOrderDialog
import com.at.video.service.ui.service.ServiceActivity
import com.at.video.service.ui.service.emoji.EmojiWidget
import com.at.video.service.ui.service.utils.SoftKeyboardUtils
import com.at.video.service.ui.service.utils.UriUtils
import com.at.video.service.widget.ImageViewer
import com.at.video.service.widget.loader.CoilLoader
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.FileUtils
import com.melancholy.utils.MediaUtils
import com.melancholy.widget.decoration.DefaultDecoration
import com.scwang.smart.refresh.layout.SmartRefreshLayout
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint
import java.io.File
import java.util.concurrent.Executors
import java.util.concurrent.ScheduledFuture
import java.util.concurrent.TimeUnit

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/6/16 上午9:18
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 上午9:18
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@AndroidEntryPoint
@Route(path = "/service/conversation/ConversationFragment")
class ConversationFragment:BaseMviFragment<ServiceFragmentConversationBinding,ConversationViewModel>(),OnRefreshLoadMoreListener,View.OnClickListener{
    private val mAdapter: ServiceAdapter = ServiceAdapter()
    private lateinit var mEditText:EditText
    private lateinit var rvList: RecyclerView
    private var message:String = ""
    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            EmojiWidget(requireActivity(), vpEmoji, rivIndicator, llEmoji, setInput,
                ivEmoji)
            mEditText = setInput
            setInput.setOnEditorActionListener(object :TextView.OnEditorActionListener{
                override fun onEditorAction(
                    v: TextView?,
                    actionId: Int,
                    event: KeyEvent?
                ): Boolean {
                    if(actionId == EditorInfo.IME_ACTION_SEND){
                        getViewModel().sendEvent(ConversationUiEvent.SendMessage(setInput.text.toString()))
                        if(message.isEmpty()){
                            message = setInput.text.toString()
                        }
                        setInput.setText("")
                        return true
                    }
                    return false
                }

            })
            ivPic.setOnClickListener(this@ConversationFragment)
            stvAdd.setOnClickListener(this@ConversationFragment)
            stvTransform.setOnClickListener(this@ConversationFragment)
        }
        observerSoftInput()
        val srlRefresh: SmartRefreshLayout = requireViewById(com.at.common.R.id.srl_refresh)
        srlRefresh.setOnRefreshLoadMoreListener(this)
        srlRefresh.setEnableRefresh(false)
        srlRefresh.setEnableLoadMore(false)
        rvList = requireViewById(com.at.common.R.id.rv_list)
        rvList.apply {
            layoutManager = LinearLayoutManager(requireActivity())
            val padding: Int = resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_12)
            addItemDecoration(DefaultDecoration.create(padding, padding, 0, padding))
            itemAnimator = null
            setHasFixedSize(true)
            adapter = mAdapter
        }
        mAdapter.addChildClickViewIds(R.id.div_image)
        mAdapter.setOnItemChildClickListener { adapter, view, position ->
            val data = adapter.getItem(position) as ChartData
            ImageViewer.load(data.message.content_type!!) //要加载的图片数据，单张或多张
                .selection(position) //当前选中位置
                .indicator(true) //是否显示指示器，默认不显示
                .imageLoader(CoilLoader()) //加载器，*必须配置，目前内置的有GlideImageLoader或PicassoImageLoader，也可以自己实现
                .theme(R.style.ImageViewerTheme) //设置主题风格，默认：R.style.ImageViewerTheme
                .orientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) //设置屏幕方向,默认：ActivityInfo.SCREEN_ORIENTATION_BEHIND
                .start(this@ConversationFragment, view)
        }
    }
    private var mFile: File? = null

    private var mContentHeight: Int = 0
    private var mIsInputMethodOpen: Boolean = false

    private fun observerSoftInput() {
        val content: View = requireActivity().findViewById(android.R.id.content)
        content.post {
            mContentHeight = content.height
        }
        content.viewTreeObserver.addOnGlobalLayoutListener {
            if(mContentHeight == 0) {
                mContentHeight = content.height
            }
            //获取当前根视图在屏幕上显示的大小
            val r = Rect()
            content.getWindowVisibleDisplayFrame(r)
            val visibleHeight: Int = r.bottom
            val params: ViewGroup.LayoutParams = content.layoutParams
            if(mContentHeight - visibleHeight >= 200 && !mIsInputMethodOpen) {
                params.height = visibleHeight
                content.layoutParams = params
                mIsInputMethodOpen = true
                rvList.scrollToPosition(mAdapter.itemCount - 1)
            } else if (mIsInputMethodOpen && mContentHeight - visibleHeight < 200) {

                params.height = ViewGroup.LayoutParams.MATCH_PARENT
                content.layoutParams = params
                mIsInputMethodOpen = false
            }
        }
    }

    /**
     * 拍照
     */
    private fun takingPictures() {
        try {
            if (mFile != null) {
                mFile!!.delete()
                mFile = null
            }
        } catch (e: Exception) {
        }
        mFile = File(
            FileUtils.getExternalCacheDirs(requireContext())?.get(0) ?: requireContext().cacheDir,
            "${System.currentTimeMillis()}.jpg")
        MediaUtils.takingPictures(requireActivity(), 100,
            FileUtils.fileToUri(requireActivity(), mFile!!))
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            when (requestCode) {
                100 -> {
                    getViewModel().sendEvent(ConversationUiEvent.UploadFile(mFile!!))
                }//拍照
                101 -> {
                  val file = UriUtils.uriToFile(data!!.data!!,requireContext())
                   getViewModel().sendEvent(ConversationUiEvent.UploadFile(file!!))
                }//选择照片 data!!.data!!
            }
        }
    }


    override fun onPause() {
        SoftKeyboardUtils.closeKeyboard(mEditText)
        super.onPause()
    }

    override fun onResume() {
        super.onResume()
      if(mFuture==null){
          startTimer()
      }
    }
    private val mExecutor = Executors.newSingleThreadScheduledExecutor()
    private var mFuture:ScheduledFuture<*>? = null
    override fun loadData() {
        getViewModel().sendEvent(ConversationUiEvent.Login)
    }

    private fun startTimer(){
        mFuture = mExecutor.scheduleWithFixedDelay({
            getViewModel().sendEvent(ConversationUiEvent.LatestMessage)
        },3*1000,1000 *3,TimeUnit.MILLISECONDS)
    }

    private fun stopTimer(){
        mFuture?.run {
            if(!isCancelled){
                cancel(true)
            }
        }
    }

    override fun onStop() {
        stopTimer()
        super.onStop()
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ConversationUiState::loginData){
                    if(it!= null&& it.content!!.isNotEmpty()){
                        mAdapter.addData(ChartData(it,0))
                        Log.d("Data","loginData:${mAdapter.itemCount}")
                        rvList.scrollToPosition(mAdapter.itemCount -1)
                    }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ConversationUiState::sendMessage){
                if(it!= null){
                    mAdapter.addData(ChartData(it.message!!))
                    rvList.scrollToPosition(mAdapter.itemCount -1)
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ConversationUiState::newMsg){
                if(it!=null){
                    mAdapter.addData(it)
                    Log.d("Data","newMsg:${mAdapter.itemCount}")
                    rvList.scrollToPosition(mAdapter.itemCount -1)
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ConversationUiState::messageData){
                if(!it.isNullOrEmpty()){
                    mAdapter.setNewInstance(it as MutableList<ChartData>?)
                    Log.d("Data","getHistoryData:${mAdapter.itemCount}")
                    rvList.scrollToPosition(mAdapter.itemCount -1)
                    rvList.postDelayed(scrollToBottom,100)
                }
                startTimer()
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when(it){
                    is ConversationUiEffect.LoginFail -> {
                                NoticeDialog.showDialog(requireActivity().supportFragmentManager,"当前无法使用客服功能",object :
                                    NoticeDialog.DialogDismissCallback{
                                        override fun dismiss() {
                                            requireActivity().finish()
                                        }
                                    })
                    }
                    is ConversationUiEffect.Toast -> ToastUtils.show(it.msg)
                    is ConversationUiEffect.WorkOrderTurnSuccess -> {
                        ToastUtils.show("工单转换成功!")
                        (activity as ServiceActivity).changePage(1)
                    }
                    is ConversationUiEffect.TransformOrder ->{
                         WorkOrderDialog.showDialog(requireActivity().supportFragmentManager,message,getViewModel())
                    }
                }

            }
        }
    }

    val scrollToBottom = Runnable {
        if(mAdapter.itemCount > 0){
            rvList.scrollToPosition(mAdapter.itemCount -1)
        }
    }

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): ServiceFragmentConversationBinding = ServiceFragmentConversationBinding.inflate(inflater)

    override fun getViewModelClass(): Class<ConversationViewModel> = ConversationViewModel::class.java
    override fun onRefresh(refreshLayout: RefreshLayout) {
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
    }

    override fun onClick(v: View) {
        when(v.id){
            R.id.iv_pic -> {
                BottomHeadDialog.showDialog(requireActivity().supportFragmentManager, {
                    if (it == 0) {//从相册选取
                        MediaUtils.selectFile(requireActivity(), 101)
                    } else {//拍照
                        takingPictures()
                    }
                })
            }
            R.id.stv_transform ->{
                getViewModel().sendEvent(ConversationUiEvent.GetVisitInfo)
            }
            R.id.stv_add->{
                Router.getInstance().build("/service/WorkOrderAddActivity").navigation()
            }
        }
    }
}