package com.at.video.service.ui.service.emoji

import android.app.Activity
import android.content.Context
import android.graphics.Rect
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import com.at.video.service.R
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.melancholy.utils.KeyBoardUtils
import com.melancholy.widget.RectIndicatorView

/**
 * @Create_time: 2022/5/6 17:56
 * @Description:
 */

class EmojiWidget constructor(private val mActivity: Activity, mVpEmoji: ViewPager2,
    mIndicator: RectIndicatorView, private val mEmoji: View, private val mEdContent: EditText,
    private val mIvEmoji: ImageView): OnItemClickListener, View.OnClickListener {
    private var mContentHeight: Int = 0
    private var mIsShowEmoji: Boolean = true
    private var mIsInputMethodOpen: Boolean = false
    /** 表情集合  */
    private var emojis: List<List<EmojiEntry>>? = null

    init {
        mIvEmoji.setOnClickListener(this)
        emojis = EmojiConversionUtils.INSTANCE.emojiLists
        if (this.emojis == null || emojis!!.isEmpty()) {
            EmojiConversionUtils.INSTANCE.init(mActivity)
            this.emojis = EmojiConversionUtils.INSTANCE.emojiLists
        }
        val pool = RecyclerView.RecycledViewPool()
        pool.setMaxRecycledViews(0, 42)
        val adapter = EmojiAdapter(pool)
        adapter.setNewInstance(emojis!!.toMutableList())
        adapter.setItemClickListener(this)
        mVpEmoji.adapter = adapter
        mIndicator.setupWithViewPager(mVpEmoji)
        observerSoftInput()
    }

    private fun observerSoftInput() {
        val content: View = mActivity.findViewById(android.R.id.content)
        content.post {
            mContentHeight = content.height
        }
        content.viewTreeObserver.addOnGlobalLayoutListener {
            if(mContentHeight == 0) {
                mContentHeight = content.height
            }
            //获取当前根视图在屏幕上显示的大小
            val r = Rect()
            content.getWindowVisibleDisplayFrame(r)
            val visibleHeight: Int = r.bottom
            val params: ViewGroup.LayoutParams = content.layoutParams
            if(mContentHeight - visibleHeight >= 200 && !mIsInputMethodOpen) {
                if(mIsShowEmoji) {
                    mEmoji.visibility = View.GONE
                    mIsShowEmoji = false
                }
                params.height = visibleHeight
                content.layoutParams = params
                mIsInputMethodOpen = true
            } else if (mIsInputMethodOpen && mContentHeight - visibleHeight < 200) {
                if(mIsShowEmoji) {
                    mEmoji.visibility = View.VISIBLE
                }
                params.height = ViewGroup.LayoutParams.MATCH_PARENT
                content.layoutParams = params
                mIsInputMethodOpen = false
            }
        }
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val emoji: EmojiEntry = adapter.getItem(position) as EmojiEntry
        if(emoji.id == R.drawable.service_ic_emoji_item_delete) {
            val selection: Int = mEdContent.selectionStart
            val text: String = mEdContent.text.toString()
            if (selection > 0) {
                val text2 = text.substring(selection - 1)
                if ("]" == text2) {
                    val start = text.lastIndexOf("[")
                    mEdContent.text.delete(start, selection)
                    return
                }
                mEdContent.text.delete(selection - 1, selection)
            }
        }
        if (!emoji.character.isNullOrEmpty()) {
            val spannableString = EmojiConversionUtils.INSTANCE
                .addFace(mActivity, emoji.id, emoji.character)
            mEdContent.append(spannableString)
        }

    }

    override fun onClick(p0: View) {
        if(mIsShowEmoji) {
            KeyBoardUtils.openKeyboard(mEdContent)
        } else {
            mIsShowEmoji = true
            //判断输入法是否打开
            if(mIsInputMethodOpen) {
                //隐藏输入法
                //KeyBoardUtils.closeKeyboard(mEdContent)
                closeKeyboard(mEdContent)
            }
        }
    }

    private fun closeKeyboard(view: View){
        /**
         * 京东在baseActivity隐藏键盘写的
         * ((InputMethodManager) getSystemService("input_method")).hideSoftInputFromWindow(getWindow().getDecorView().getWindowToken(), 0);
         * android11，有些设备有适配问题，所以直接使用InputMethodManager
         * view.windowInsetsController?.hide(WindowInsets.Type.ime())
         */
        val manager: InputMethodManager = view.context.getSystemService(Context.INPUT_METHOD_SERVICE)
                as InputMethodManager? ?: return
        //view.isFocusable = false
        //view.isFocusableInTouchMode = false
        view.clearFocus()
        manager.hideSoftInputFromWindow(view.windowToken, 0)
    }
}