package com.at.video.service.ui.service.order

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.service.databinding.ServiceFragmentOrderBinding
import com.at.video.service.entity.OrderData
import com.at.video.service.ui.service.order.detail.WorkOrderDetailActivity
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.KeyBoardUtils
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/6/16 上午9:11
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 上午9:11
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@AndroidEntryPoint
@Route(path = "/service/order/OrderFragment")
class OrderFragment : BaseMviFragment<ServiceFragmentOrderBinding, OrderViewModel>(),
    OnRefreshLoadMoreListener {

    val tabs = listOf<StatusData>(
        StatusData("全部",0),
        StatusData("未处理",6),
        StatusData("处理中",1),
        StatusData("待反馈",2),
        StatusData("已结束",3),
        StatusData("已失效",5),
        StatusData("已撤销",4))

    var status = 0

   private lateinit var mAdapter:OrderItemAdapter
    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            val statusAdapter = OrderStatusAdapter()
            rvTab.layoutManager = LinearLayoutManager(requireContext(),RecyclerView.HORIZONTAL,false)
            rvTab.adapter = statusAdapter
            statusAdapter.setNewInstance(tabs.toMutableList())
            statusAdapter.setOnItemClickListener { adapter, _, position ->
                run {
                    status = (adapter.getItem(position) as StatusData).values
                    statusAdapter.setSelectIndex(
                        position
                    )
                    getViewModel().sendEvent(OrderUiEvent.InitWorkOrder(status))
                }
            }
            mAdapter = OrderItemAdapter()
            rvList.apply {
                layoutManager = LinearLayoutManager(requireContext())
                adapter = mAdapter
                addItemDecoration(OrderDecoration())
            }
            stvAdd.setOnClickListener {
                Router.getInstance().build("/service/WorkOrderAddActivity").navigation()
            }
            srlRefresh.setOnRefreshLoadMoreListener(this@OrderFragment)
            mAdapter.setOnItemClickListener { adapter, _, position ->
                Router.getInstance().build("/service/WorkOrderDetailActivity").withParcelable(WorkOrderDetailActivity.WORK_ORDER_DATA,adapter.getItem(position) as OrderData).navigation()
            }
        }
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(OrderUiState::workOrderData){
                if(!it.isNullOrEmpty()){
                    mAdapter.setNewInstance(it as MutableList<OrderData>?)
                }
            }
        }

        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(OrderUiState::refreshStatus) {
                requireBinding().srlRefresh.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                           finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!isRefreshing) {
                                autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                           finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(OrderUiState::pageStatus){
                requireBinding().apply {
                    when(it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING
                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                .text = it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            srlRefresh.setEnableLoadMore(true)
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }


        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(OrderUiState::loadMoreStatus) {
                requireBinding().srlRefresh.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                finishLoadMore()
                            } else {
                                finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!isLoading) {
                                autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
    }

    override fun loadData() {
    }

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): ServiceFragmentOrderBinding = ServiceFragmentOrderBinding.inflate(layoutInflater)

    override fun getViewModelClass(): Class<OrderViewModel> = OrderViewModel::class.java
    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(OrderUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(OrderUiEvent.LoadDataMore)
    }

    override fun onResume() {
        super.onResume()
        KeyBoardUtils.closeKeyboard(requireView())
        getViewModel().sendEvent(OrderUiEvent.InitWorkOrder(status))
    }
}