package com.at.video.service.ui.service.order

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.export.mine.IMineService
import com.at.video.service.entity.OrderData
import com.at.video.service.http.ApiService
import com.melancholy.network.ApiFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 *
 * @Description: OrderViewModelViewModel
 * @CreateDate: 2022/6/16 上午9:04
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 上午9:04
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class OrderUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
    val workOrderData:List<OrderData>? = null,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
) : IUiState

sealed class OrderUiEvent : IUiEvent {
    data class InitWorkOrder(val state:Int = -1):OrderUiEvent()
    object RefreshData: OrderUiEvent()
    object LoadDataMore: OrderUiEvent()
}
sealed class OrderUiEffect : IUiEffect {}

@HiltViewModel
class OrderViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<OrderUiState, OrderUiEvent, OrderUiEffect>(
        application
    ) {
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var status:Int = 0

    override fun providerInitialState(): OrderUiState {
        return OrderUiState()
    }

    override fun handleEvent(event: OrderUiEvent) {
        when(event){
            is OrderUiEvent.InitWorkOrder -> {
                status = event.state
                mOffset = 0
                initWorkOrder(0)
            }
            is OrderUiEvent.RefreshData ->{
                mOffset = 0
                initWorkOrder(1)
            }
            is OrderUiEvent.LoadDataMore ->initWorkOrder(2)
        }
    }

    private fun initWorkOrder(type:Int){
        viewModelScope.launch{
            val map: MutableMap<String, Any> = mutableMapOf()
                map["visitor_state"] = status
                map["visitor_id"] = IMineService.getMineService().getUser().value?.getUserId().toString()
                map["offset"] = mOffset
                map["limit"] = mLimit
            when(type){
                0->  setState { copy(pageStatus = PageStatus.Loading) }
                1->  setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                2-> setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }
            val res = ApiFactory.getInstance().create(ApiService::class.java).workOrderList(map)
            if(res.code == 6200 &&res.data !=null){
                if(res.data!!.list.isNullOrEmpty()&&type!=2){
                    setState { copy(pageStatus = PageStatus.Empty) }
                }else{
                    val hasMore: Boolean = res.data!!.list!!.size == mLimit
                    if(hasMore) {
                        mOffset += mLimit
                    }
                    val data = if(type == 2) state.value.workOrderData!! + res.data!!.list!! else res.data!!.list
                    setState { copy(pageStatus = PageStatus.Success, workOrderData = data,refreshStatus = RefreshStatus.RefreshSuccess,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
             }
          }else{
              setState { copy(pageStatus = PageStatus.Error(Throwable(res.msg))) }
          }
        }
    }
}