package com.at.video.service.ui.service.order.add

import android.app.Activity
import android.content.ContentResolver
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.os.Bundle
import android.provider.OpenableColumns
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.webkit.MimeTypeMap
import android.widget.EditText
import androidx.core.net.toFile
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.mvi.BaseMviActivity
import com.at.common.setLightStatusBar
import com.at.common.utils.ToastUtils
import com.at.video.service.R
import com.at.video.service.databinding.ServiceActivityWorkOrderAddBinding
import com.at.video.service.entity.FileData
import com.at.video.service.ui.dialog.NoticeDialog
import com.at.video.service.ui.service.conversation.ConversationUiEffect
import com.at.video.service.ui.service.utils.SoftKeyboardUtils
import com.at.video.service.ui.service.utils.UriUtils
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.widget.TitleBar
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect
import java.io.File
import java.math.RoundingMode
import java.text.DecimalFormat
import java.util.*


/**
 *
 * @Description: 创建工单
 * @CreateDate: 2022/6/16 下午5:06
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 下午5:06
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@AndroidEntryPoint
@Route(path = "/service/WorkOrderAddActivity")
class WorkOrderAddActivity:BaseMviActivity<ServiceActivityWorkOrderAddBinding,WorkOrderAddViewModel>(),View.OnClickListener {

    private lateinit var mAdapter: OrderUploadFileAdapter
    private lateinit var rvLists:RecyclerView
    private lateinit var mEditText: EditText
    override fun initialize(savedInstanceState: Bundle?) {
       requireBinding().apply {
            val head = llTitle as TitleBar
           head.apply {
               setBackgroundColor(Color.WHITE)
               addBackButton(Color.BLACK)
               setTitleColor(Color.BLACK)
               setTitle("创建工单")
               setOnBackListener(object :TitleBar.OnBackListener{
                   override fun onBack() {
                       finish()
                   }
               })
           }
           rvLists = rvList
           mEditText= stInputTitle

           stInputTitle.addTextChangedListener(object : TextWatcher {
               override fun beforeTextChanged(
                   s: CharSequence?,
                   start: Int,
                   count: Int,
                   after: Int
               ) {
               }

               override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
               }

               override fun afterTextChanged(s: Editable?) {
                   tvTxtNum.text ="${s.toString().length}/50"
               }

           })

           stInputContent.addTextChangedListener(object :TextWatcher{
               override fun beforeTextChanged(
                   s: CharSequence?,
                   start: Int,
                   count: Int,
                   after: Int
               ) {
               }

               override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
               }

               override fun afterTextChanged(s: Editable?) {
                   tvContentNum.text ="${s.toString().length}/255"
               }

           })

           rvList.apply {
               mAdapter = OrderUploadFileAdapter()
               adapter = mAdapter
               layoutManager = LinearLayoutManager(this@WorkOrderAddActivity)
               val divider = DividerLineDecoration()
               divider.setDrawable(ColorDrawable(resources.getColor(R.color.color_FFF3F5F8)))
               rvList.addItemDecoration(divider)
           }
           tvUnloadBtn.setOnClickListener(this@WorkOrderAddActivity)
           stvAdd.setOnClickListener(this@WorkOrderAddActivity)
           mAdapter.addChildClickViewIds(R.id.iv_delete)
           mAdapter.setOnItemChildClickListener { _, _, position -> mAdapter.removeAt(position)}
       }
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when(it){
                    is WorkOrderAddUiEffect.Toast -> ToastUtils.show(it.msg)
                    is WorkOrderAddUiEffect.AddSuccess ->{
                        ToastUtils.show("添加成功!")
                        finish()
                    }
                    is WorkOrderAddUiEffect.UploadFileProgress ->{
//                        mAdapter.updateProgress(it.progress,it.position)
                         mAdapter.getItem(it.position).progress = it.progress
                        mAdapter.notifyItemChanged(it.position)
                    }
                    else -> {}
//                    is WorkOrderAddUiEffect.UploadFileSuccess -> mAdapter.addData(it.fileInfo)
                }
            }
        }
    }

    override fun createViewBinding(): ServiceActivityWorkOrderAddBinding = ServiceActivityWorkOrderAddBinding.inflate(layoutInflater)

    override fun getViewModelClass(): Class<WorkOrderAddViewModel> = WorkOrderAddViewModel::class.java
    override fun onClick(v: View) {
        when(v.id){
            R.id.tv_unload_btn ->{
                if(mAdapter.itemCount < 5){
                    val intent = Intent(Intent.ACTION_GET_CONTENT)
                    intent.type = "*/*"
                    intent.addCategory(Intent.CATEGORY_OPENABLE)
                    startActivityForResult(intent,100)
                }else{
                 ToastUtils.show("最多只能上传5个文件")
                }
            }
            R.id.stv_add ->{
                val title = requireBinding().stInputTitle.text.toString()
                val content = requireBinding().stInputContent.text.toString()
                SoftKeyboardUtils.closeKeyboard(mEditText)
                if(title.isEmpty()){
                    ToastUtils.show("请输入工单标题!")
                    return
                }
                if(content.isEmpty()){
                    ToastUtils.show("请输入工单内容!")
                    return
                }
                getViewModel().sendEvent(WorkOrderAddUiEvent.AddWorkOrder(title,content))
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(resultCode != Activity.RESULT_OK || requestCode != 100) return
        val uri = data?.data ?: return
        val mimeType = UriUtils.getMimeType(this,uri)?:""
        val type  = if(mimeType.indexOf("image") != -1) 1 else 2
        val fileData = UriUtils.uriToFileName(uri,this,type)
        val file = UriUtils.uriToFile(uri,this,fileData.name!!)!!
        if(file.length()> 8 * 1024 * 1024){
            ToastUtils.show("单个文件不能超过8M!")
            return
        }
        mAdapter.addData(fileData)
        getViewModel().sendEvent(WorkOrderAddUiEvent.UploadFile(file,fileData,mAdapter.itemCount - 1))
    }
}