package com.at.video.service.ui.service.order.detail

import android.app.Activity
import android.content.Intent
import android.content.pm.ActivityInfo
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.provider.Settings
import android.view.View
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.ImageConfig
import com.at.common.global.Constants
import com.at.common.load
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.service.R
import com.at.video.service.databinding.ServiceWorkOrderDetailBinding
import com.at.video.service.entity.FileInfo
import com.at.video.service.entity.OrderData
import com.at.video.service.entity.OrderDetailMsgInfo
import com.at.video.service.ui.dialog.WorkOrderResponsePopup
import com.at.video.service.ui.service.utils.TimeUtils
import com.at.video.service.ui.service.utils.UriUtils
import com.at.video.service.widget.ImageViewer
import com.at.video.service.widget.loader.CoilLoader
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.hjq.permissions.XXPermissions
import com.kennyc.view.MultiStateView
import com.lxj.xpopup.XPopup
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.AppUtils
import com.melancholy.utils.FileUtils
import com.melancholy.utils.MediaUtils
import com.melancholy.widget.TitleBar
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import dagger.hilt.android.AndroidEntryPoint
import java.io.File

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/6/17 上午10:04
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/17 上午10:04
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@AndroidEntryPoint
@Route(path = "/service/WorkOrderDetailActivity", extras = 10086)
class WorkOrderDetailActivity:BaseMviActivity<ServiceWorkOrderDetailBinding,WorkOrderDetailViewModel>(),View.OnClickListener ,
    OnRefreshLoadMoreListener,OnItemClickListener,OnItemChildClickListener {

    companion object{
        const val WORK_ORDER_DATA ="work_order_data"
    }

    private lateinit var mWork: TextView
    private val mAdapter: WorkOrderDetailAdapter = WorkOrderDetailAdapter()
    private lateinit var mEditText: EditText
    private lateinit var llInput:LinearLayout
    private lateinit var rvList: RecyclerView
    private lateinit var workResponsePopup:WorkOrderResponsePopup
    private lateinit var workOrderData: OrderData
    override fun initialize(savedInstanceState: Bundle?) {
        titlebar = requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            setTitle("工单详情")
            addBackButton(Color.BLACK)
            mWork = addRightTextButton("撤销", android.R.id.edit)
            mWork.setTextColor(resources.getColor(R.color.color_FF0083FF))
            mWork.visibility = View.GONE
            mWork.setOnClickListener(this@WorkOrderDetailActivity)
        }
        workOrderData = intent.extras?.getParcelable(WORK_ORDER_DATA)!!
        requireBinding().apply {
            rvList.apply {
                layoutManager = LinearLayoutManager(this@WorkOrderDetailActivity)
                itemAnimator = null
                adapter = mAdapter
            }
            mEditText = setInput
            llInput = llInputResponse
            workResponsePopup = WorkOrderResponsePopup(this@WorkOrderDetailActivity, getViewModel(),workOrderData.kefu_id!!)
            setInput.setOnClickListener {
                XPopup.Builder(this@WorkOrderDetailActivity)
                    .popupWidth(llInputResponse.width)
                    .enableDrag(false)
                    .hasShadowBg(true)
                    .asCustom(workResponsePopup)
                    .show()
            }
            srlRefresh.setOnRefreshLoadMoreListener(this@WorkOrderDetailActivity)
            rvList.scrollToPosition(mAdapter.itemCount -1)
        }
        mAdapter.setOnItemClickListenerTemp(this)
        mAdapter.setOnChildItemClickListener(this)
        mAdapter.setOnItemClickListener { adapter, view, position ->
                val data = adapter.getItem(position) as OrderDetailMsgInfo
                if (data.content!!.indexOf("img") != -1) {
                    val imgs = data.content.split("src")
                    val img = imgs[1].split("\"")
                    if(imgs.size >1 && img.size>1){
                        ImageViewer.load(img[1]) //要加载的图片数据，单张或多张
                            .selection(1) //当前选中位置
                            .indicator(true) //是否显示指示器，默认不显示
                            .imageLoader(CoilLoader()) //加载器，*必须配置，目前内置的有GlideImageLoader或PicassoImageLoader，也可以自己实现
                            .theme(R.style.ImageViewerTheme) //设置主题风格，默认：R.style.ImageViewerTheme
                            .orientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) //设置屏幕方向,默认：ActivityInfo.SCREEN_ORIENTATION_BEHIND
                            .start(this@WorkOrderDetailActivity, null)
                    }
            }
        }
        mPermissions = XXPermissions.with(this)
    }

    private var mFile: File? = null

    /**
     * 拍照
     */
    private fun takingPictures() {
        try {
            if (mFile != null) {
                mFile!!.delete()
                mFile = null
            }
        } catch (e: Exception) {
        }
        mFile = File(
            FileUtils.getExternalCacheDirs(this)?.get(0) ?:cacheDir,
            "${System.currentTimeMillis()}.jpg")
        val uri =  FileUtils.fileToUri(this, mFile!!)
        MediaUtils.takingPictures(this, 100,
            uri)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(resultCode == Activity.RESULT_OK){
          when(requestCode){
              100->{
                  getViewModel().sendEvent(WorkOrderDetailUiEvent.UploadFile(mFile!!,UriUtils.uriToFileName(FileUtils.fileToUri(this,mFile!!),this,1)))
              }
              101,102->{
                  val uri = data?.data ?: return
                  val mimeType = UriUtils.getMimeType(this,uri)?:""
                  val type  = if(mimeType.indexOf("image") != -1) 1 else 2
                  val fileName = UriUtils.uriToFileName(uri,this,type)
                  getViewModel().sendEvent(WorkOrderDetailUiEvent.UploadFile(UriUtils.uriToFile(uri,this,fileName.name!!)!!,fileName))

              }
          }
        }
    }


    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }
    private var titlebar: TitleBar? = null

    override fun loadData() {
        requireBinding().apply {
            tvTitle.text = workOrderData.title
//            tvContent.text = "工单内容"
           if(workOrderData != null){
               divHeader.load(ImageConfig(Constants.SERVICE_URL+workOrderData.visitor_avatar, placeholderId = R.drawable.service_ic_user_default_header, errorId = R.drawable.service_ic_user_default_header))
               tvContent.text = workOrderData.content
               tvDate.text = TimeUtils.getTimeStr(workOrderData.created_at)
               if(workOrderData.appendix?.isNotEmpty() == true){
                   val adapter =  OrderDetailFileAdapter()
                   rvServiceFile.adapter = adapter
                   adapter.setNewInstance(workOrderData.appendix as MutableList<FileInfo>?)
                    adapter.setOnItemClickListener(this@WorkOrderDetailActivity)
               }
           }
            divHeader
            tvOrderId.text = "工单ID: ${workOrderData.work_id}"
            srlRefresh.setOnLoadMoreListener(this@WorkOrderDetailActivity)
        }
        changeStatus(workOrderData.visitor_state)
        getViewModel().sendEvent(WorkOrderDetailUiEvent.InitData(workOrderData.work_id!!))
    }


    private fun changeStatus(status:Int){
        requireBinding().apply {
            var statusText = ""
            var textColors = 0
            var bgColor = 0
            when(status){
                1->{
                    statusText = "处理中"
                    textColors = R.color.color_FF0083FF
                    bgColor = R.color.color_FFE6F4FF
                }
                2->{
                    statusText = "待反馈"
                    textColors = R.color.color_FFED7B2F
                    bgColor = R.color.color_FFFEF3E6
                    mWork.text = "结单"
                    mWork.visibility = View.VISIBLE
                }
                3->{
                    statusText = "已结束"
                    textColors = R.color.color_FFC5C5C5
                    bgColor = R.color.color_FFF5F5F5
                    llInput.visibility = View.GONE
                    mWork.visibility = View.GONE
                }
                4->{
                    statusText = "已撤销"
                    textColors = R.color.color_FFC5C5C5
                    bgColor = R.color.color_FFF5F5F5
                    llInput.visibility = View.GONE
                    mWork.visibility = View.GONE
                }
                5 -> {
                    statusText = "已失效"
                    textColors = R.color.color_FFC5C5C5
                    bgColor = R.color.color_FFF5F5F5
                    llInput.visibility = View.GONE
                    mWork.visibility = View.GONE
                }
                6-> {
                    mWork.visibility = View.VISIBLE
                    statusText = "未处理"
                    textColors = R.color.color_FFE34D59
                    bgColor = R.color.color_FFFDECEE
                }

            }
            val bgDrawable = stvStatus.background as GradientDrawable
            bgDrawable.setColor(resources.getColor(bgColor))
            stvStatus.text = statusText
            stvStatus.setTextColor(resources.getColor(textColors))
        }
    }

    override fun observer() {

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(WorkOrderDetailUiState::pageStatus){
                requireBinding().apply {
                    when(it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING
                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint)
                                .text = it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            srlRefresh.setEnableLoadMore(true)
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(WorkOrderDetailUiState::loadMoreStatus) {
                requireBinding().srlRefresh.apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                finishLoadMore()
                            } else {
                                finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!isLoading) {
                                autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(WorkOrderDetailUiState::refreshStatus) {
                requireBinding().srlRefresh.apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!isRefreshing) {
                                autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(WorkOrderDetailUiState::chatData){
                if(!it.isNullOrEmpty()){
                    mAdapter.setNewInstance(it as MutableList<OrderDetailMsgInfo>?)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect{
                when(it){
                    is WorkOrderDetailUiEffect.Toast -> ToastUtils.show(it.msg)
                    is WorkOrderDetailUiEffect.UploadFileSuccess -> workResponsePopup.addData(it.fileInfo)
                    is WorkOrderDetailUiEffect.EndOrderSuccess ->{
                        ToastUtils.show("结单成功")
                        changeStatus(3)
                    }
                    is WorkOrderDetailUiEffect.RevokeSuccess ->{

                        ToastUtils.show("撤销成功")
                        changeStatus(3)
                    }
                    WorkOrderDetailUiEffect.TakePhoto->takingPictures()
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(WorkOrderDetailUiState::sendData){
                if(it!=null){
                    mAdapter.addData(it)
                }
            }
        }
    }

    override fun createViewBinding(): ServiceWorkOrderDetailBinding = ServiceWorkOrderDetailBinding.inflate(layoutInflater)

    override fun getViewModelClass(): Class<WorkOrderDetailViewModel> = WorkOrderDetailViewModel::class.java
    override fun onClick(v: View) {
        when(v.id){
            android.R.id.edit->{
                if(mWork.text == "撤销"){
                    getViewModel().sendEvent(WorkOrderDetailUiEvent.RevokeOrder)
                }else{
                    getViewModel().sendEvent(WorkOrderDetailUiEvent.EndOrder)
                }
            }
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(WorkOrderDetailUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(WorkOrderDetailUiEvent.LoadDataMore)
    }




    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
      val data = adapter.getItem(position) as FileInfo
        if(data.name!!.indexOf("jpg")!=-1){
            ImageViewer.load(
                Constants.SERVICE_URL + (adapter.getItem(position) as FileInfo).path) //要加载的图片数据，单张或多张
                .selection(position) //当前选中位置
                .indicator(true) //是否显示指示器，默认不显示
                .imageLoader(CoilLoader()) //加载器，*必须配置，目前内置的有GlideImageLoader或PicassoImageLoader，也可以自己实现
                .theme(R.style.ImageViewerTheme) //设置主题风格，默认：R.style.ImageViewerTheme
                .orientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT) //设置屏幕方向,默认：ActivityInfo.SCREEN_ORIENTATION_BEHIND
                .start(this@WorkOrderDetailActivity, view)
        }
    }
    private lateinit var mPermissions: XXPermissions

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.getItem(position) as FileInfo
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.R && !Environment.isExternalStorageManager()){
            val uri = Uri.parse("package:" + AppUtils.getPackageName(this))
            startActivity(Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION, uri))
        }else{
            getViewModel().sendEvent(WorkOrderDetailUiEvent.DownLoadFile(data.path!!,data.name!!))
        }
    }
}