package com.at.video.service.ui.service.utils

import android.annotation.SuppressLint
import android.content.ContentResolver
import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.provider.OpenableColumns
import android.util.Base64
import android.webkit.MimeTypeMap
import androidx.annotation.RequiresApi
import androidx.core.net.toFile
import com.at.video.service.entity.FileData
import com.melancholy.utils.CipherUtils
import com.melancholy.utils.FileUtils
import com.melancholy.utils.uuid.Codec
import java.io.*
import java.math.RoundingMode
import java.text.DecimalFormat
import java.util.*

object UriUtils {

     fun getMimeType(context: Context, uri: Uri): String? {
        var mimeType: String? = null
        mimeType = if (ContentResolver.SCHEME_CONTENT == uri.scheme) {
            val cr: ContentResolver = context.contentResolver
            cr.getType(uri)
        } else {
            val fileExtension = MimeTypeMap.getFileExtensionFromUrl(
                uri
                    .toString()
            )
            MimeTypeMap.getSingleton().getMimeTypeFromExtension(
                fileExtension.lowercase(Locale.getDefault())
            )
        }
        return mimeType
    }

     @SuppressLint("Range")
     fun uriToFileName(uri: Uri, context: Context, type:Int): FileData {
        var size:String = ""
        val displayName = when(uri.scheme){
            ContentResolver.SCHEME_FILE -> uri.toFile().name
            ContentResolver.SCHEME_CONTENT->{
                val cursor = context.contentResolver.query(uri, null, null, null, null, null)
                cursor?.let {
                    it.moveToFirst()
                    val displayName = it.getString(it.getColumnIndex(OpenableColumns.DISPLAY_NAME))
                    size =  it.getString(it.getColumnIndex(OpenableColumns.SIZE))
                    cursor.close()
                    displayName
                }?:"${System.currentTimeMillis()}.${MimeTypeMap.getSingleton().getExtensionFromMimeType(context.contentResolver.getType(uri))}}"

            }
            else -> "${System.currentTimeMillis()}.${MimeTypeMap.getSingleton().getExtensionFromMimeType(context.contentResolver.getType(uri))}}"
        }
        val resSize = (size.toLong()/1024.0).toFloat()
        val sizeStr = if(resSize < 100){
            "${getFloatNoMoreThanTwoDigits(resSize)}KB"
        }else "${getFloatNoMoreThanTwoDigits((resSize/1024.0).toFloat())}M"
        return FileData( name = displayName, type = type, size = sizeStr, status = 1, img = uri)
    }


    fun uriToFile(uri: Uri, context: Context,fileName:String = "${System.currentTimeMillis()}.jpg"): File? {
        var file:File?  = when(uri.scheme){
            ContentResolver.SCHEME_FILE -> uri.toFile()
            ContentResolver.SCHEME_CONTENT->{

                val  temp = File(
                    FileUtils.getExternalCacheDirs(context)?.get(0) ?: context.cacheDir,
                    fileName
                )
                if (!temp.exists()) {
                    temp.createNewFile()
                }
                val fd = context.contentResolver.openFileDescriptor(uri,"r")
                try {
                    fd?.fileDescriptor?.let {
                        val fi = FileInputStream(it)
                        val fo = temp.outputStream()
                        fi.copyTo(fo)
                        fi.close()
                        fo.close()
                    }
                }catch (e:Exception){
                    return null
                }
                return  temp
            }
            else -> null
        }
     return file
    }


      fun fileToBase64(file:File):String{
         var res = ""
         val path = file.absolutePath
         if(path.isEmpty()){
             return  ""
         }

         var input:InputStream? = null
         var data:ByteArray? = null

         try {
             input = FileInputStream(path)
             data = ByteArray(input.available())
             input.read(data)
//             CipherUtils.fileToBase64(file)
//             res = Base64.getEncoder().encodeToString(data)
             Base64.encodeToString(data,Base64.DEFAULT)
         }catch (e:Exception){

         }
         return  res
     }


     fun base64ToFile(base64Str:String, context: Context):Uri{
         val bas64 = base64Str.split(",")[1]
        val  temp = File(
            FileUtils.getExternalCacheDirs(context)?.get(0) ?: context.cacheDir,
            "${System.currentTimeMillis()}.jpg")
        if (!temp.exists()) {
            temp.createNewFile()
        }
        var data = Base64.decode(bas64,Base64.NO_WRAP)
        var os:OutputStream? = null
        try {
            os = FileOutputStream(temp)
            os.write(data)
            os.flush()
            os.close()
        }catch (e:Exception){

        }
        return Uri.fromFile(temp)
    }

    fun base64ToBitmap(base64Str: String): Bitmap? {
        val bas64 = base64Str.split(",")[1]
        val data = Base64.decode(bas64, Base64.DEFAULT)
        var bitmap:Bitmap? = null
        try {
            bitmap = BitmapFactory.decodeByteArray(data, 0, data.size)
        }catch (e:Exception){

        }
        return bitmap
    }


      fun getFloatNoMoreThanTwoDigits(number: Float): String {
        val format = DecimalFormat("#.##")
        //舍弃规则，RoundingMode.FLOOR表示直接舍弃。
        format.roundingMode = RoundingMode.FLOOR
        return format.format(number)
    }
}