package com.at.video.update.dialog

import android.annotation.SuppressLint
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.at.common.utils.ToastUtils
import com.at.video.update.R
import com.at.video.update.databinding.UpdateDialogBinding
import com.at.video.update.utils.ApkInstallUtils
import com.melancholy.utils.FileUtils
import kotlinx.coroutines.flow.collect
import java.io.File

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/5 上午9:34
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/5 上午9:34
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

class UpdateDialog: BaseBindingDialogFragment<UpdateDialogBinding>(), View.OnClickListener {


    private val viewModel by lazy {
        val provider = ViewModelProvider(this)
        provider[UpdateViewModel::class.java]
    }

    private var url:String = ""

    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, url: String,size:String,title:String,content:String,isForceUpdate:Boolean) {
            val dialog = UpdateDialog()
            dialog.arguments = bundleOf(Constants.KEY_DATA to url,Constants.KEY_DATA_1 to size,Constants.KEY_DATA_2 to title,Constants.KEY_DATA_3 to content,Constants.KEY_DATA_4 to isForceUpdate)
            dialog.show(manager, null)
        }
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(inflater: LayoutInflater, container: ViewGroup?,
                                   savedInstanceState: Bundle?): UpdateDialogBinding {
        return UpdateDialogBinding.inflate(inflater, container, false)
    }

    @SuppressLint("SetTextI18n")
    override fun initialize(view: View, savedInstanceState: Bundle?) {
        url = requireArguments().getString(Constants.KEY_DATA)!!
        requireBinding().apply {
            ivClose.setOnClickListener(this@UpdateDialog)
            tvTitle.text =requireArguments().getString(Constants.KEY_DATA_2)!!
            tvUpdateInfo.text = "新版本大小：${requireArguments().getString(Constants.KEY_DATA_1)!!}\n${requireArguments().getString(Constants.KEY_DATA_3)!!}"
            tvIgnore.setOnClickListener(this@UpdateDialog)
            tvIgnore.visibility = if(requireArguments().getBoolean(Constants.KEY_DATA_4)) View.GONE else View.VISIBLE
            ivClose.visibility = if(requireArguments().getBoolean(Constants.KEY_DATA_4)) View.GONE else View.VISIBLE
            btnUpdate.setBackgroundColor(resources.getColor(R.color.color_FF2167FF))
            btnUpdate.setOnClickListener(this@UpdateDialog)
        }
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_400)
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {

    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            viewModel.effect.collect{
                when(it){
                    is UpdateUiEffect.Progress ->{
                        requireBinding().npbProgress.progress = it.progress
                    }
                    is UpdateUiEffect.DownloadSuccess ->{
                        if(it.result){
                            dismissAllowingStateLoss()
                            ToastUtils.show("下载完成!")
                            ApkInstallUtils.install(requireContext(),file)
                        }else{
                            ToastUtils.show("下载失败!")
                        }
                    }
                }
            }
        }
    }
    var file:File? =null
    override fun onClick(view: View) {
       when(view.id){
           R.id.tv_ignore,
           R.id.iv_close -> {
               dismissAllowingStateLoss()
           }
           R.id.btn_update ->{
               val fileName = url.substring(url.lastIndexOf("/") +1,url.length)
               val path = FileUtils.getExternalCacheDirs(requireContext())?.get(0) ?: requireContext().cacheDir
                 file = File(
                   FileUtils.getExternalCacheDirs(requireContext())?.get(0) ?: requireContext().cacheDir,
                   fileName
               )
//               if(!file!!.exists()){
//                   file!!.createNewFile()
//               }
            viewModel.sendEvent(UpdateUiEvent.DownloadApk(url, path.absolutePath ,fileName))
               requireBinding().btnUpdate.visibility = View.GONE
               requireBinding().npbProgress.visibility = View.VISIBLE
               requireBinding().tvIgnore.visibility = View.GONE
           }
       }
    }
}