package com.at.video.update.dialog

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.update.http.ApiService
import com.melancholy.network.ApiFactory
import com.melancholy.utils.FileUtils
import com.zhy.http.okhttp.OkHttpUtils
import com.zhy.http.okhttp.callback.FileCallBack
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.Call
import java.io.File
import java.io.FileOutputStream
import javax.inject.Inject

/**
 *
 * @Description: UpdateViewModel
 * @CreateDate: 2022/7/5 上午10:26
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/5 上午10:26
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class UpdateUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
) : IUiState

sealed class UpdateUiEvent : IUiEvent {
    data class DownloadApk(val url:String,val path:String,val fileName: String):UpdateUiEvent()
}
sealed class UpdateUiEffect : IUiEffect {
    data class Progress(val progress:Int):UpdateUiEffect()
    data class DownloadSuccess(val result:Boolean,val file:File?):UpdateUiEffect()
}

@HiltViewModel
class UpdateViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<UpdateUiState, UpdateUiEvent, UpdateUiEffect>(application) {

    override fun providerInitialState(): UpdateUiState {
        return UpdateUiState()
    }

    override fun handleEvent(event: UpdateUiEvent) {
        when(event){
            is UpdateUiEvent.DownloadApk ->download(event.url,event.path,event.fileName)
        }
    }

    private fun download(url:String,path: String,fileName:String){

      viewModelScope.launch {
          val  file = File(path)
          if(!file.exists()){
              file.createNewFile()
          }
          OkHttpUtils.get().url(url).addHeader("Accept-Encoding","identity")
              .tag(url)
              .build()
              .execute(object :FileCallBack(path,fileName){
                  override fun inProgress(progress: Float, total: Long, id: Int) {
                      setEffect { UpdateUiEffect.Progress((progress*100).toInt())}
                  }
                  override fun onError(call: Call?, e: java.lang.Exception?, id: Int) {
                       setEffect { UpdateUiEffect.DownloadSuccess(false,null) }
                  }

                  override fun onResponse(response: File?, id: Int) {
                      setEffect { UpdateUiEffect.DownloadSuccess(true,response) }
                  }
              })
      }
    }
}