package com.melancholy.webview

import android.annotation.SuppressLint
import android.content.Context
import android.content.MutableContextWrapper
import android.os.Looper
import com.melancholy.webview.widget.WebView
import java.util.*

/**
 * @Create_time: 2019/10/16 11:52
 * @Description: 预加载
 */
internal class PreloadWebView private constructor(private val context: Context) {

    private val mCacheMaxNumber = 2
    private val mCachedWebViewStack: Stack<WebView> = Stack()

    companion object {
        @Volatile
        @SuppressLint("StaticFieldLeak")
        private lateinit var sInstance: PreloadWebView

        @JvmStatic
        fun init(context: Context) {
            if(!::sInstance.isInitialized) {
                synchronized(PreloadWebView::class.java) {
                    if(!::sInstance.isInitialized) {
                        sInstance = PreloadWebView(context.applicationContext)
                    }
                }
            }
        }

        @JvmStatic
        fun getInstance(): PreloadWebView {
            if(!::sInstance.isInitialized) {
                throw NullPointerException("")
            }
            return sInstance
        }
    }

    init {
        preload()
    }

    /**
     * 创建WebView实例
     * 用了applicationContext
     * 当前线程空闲的时候去创建WebView
     */
    private fun preload() {
        Looper.myQueue().addIdleHandler {
            if (mCachedWebViewStack.size < mCacheMaxNumber) {
                mCachedWebViewStack.push(createWebView())
            }
            false
        }
    }

    @SuppressLint("SetJavaScriptEnabled")
    private fun createWebView(): WebView {
        return WebView(MutableContextWrapper(context))
    }

    /**
     * 从缓存池中获取合适的WebView
     *
     * @param context activity context
     * @return WebView
     */
    fun getWebView(context: Context): WebView {
        // 为空，直接返回新实例
        if (mCachedWebViewStack.isEmpty()) {
            val web = createWebView()
            val contextWrapper = web.context as MutableContextWrapper
            contextWrapper.baseContext = context
            return web
        }
        val webView = mCachedWebViewStack.pop()
        // webView不为空，则开始使用预创建的WebView,并且替换Context
        val contextWrapper = webView.context as MutableContextWrapper
        contextWrapper.baseContext = context
        return webView
    }
}