package com.melancholy.webview.mainprocess

import android.annotation.SuppressLint
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.IBinder
import com.melancholy.webview.ICallbackFromMainToWeb
import com.melancholy.webview.IWebToMain
import com.melancholy.webview.ToastCommand
import com.melancholy.webview.command.Command
import com.melancholy.webview.webviewprocess.OptimizationService
import java.util.ServiceLoader

/**
 * @Create_time: 2019/10/16 11:52
 * @Description:
 */
internal class MainProcessCommandsManager private constructor(private val context: Context):
    IWebToMain.Stub(), ServiceConnection {

    private val mCommands: MutableMap<String, Command> = mutableMapOf()

    companion object {
        @Volatile
        @SuppressLint("StaticFieldLeak")
        private lateinit var sInstance: MainProcessCommandsManager

        //初始化
        fun init(context: Context) {
            if(!::sInstance.isInitialized) {
                synchronized(MainProcessCommandsManager::class.java){
                    if(!::sInstance.isInitialized) {
                        sInstance = MainProcessCommandsManager(context.applicationContext)
                    }
                }
            }
        }

        fun getInstance(): MainProcessCommandsManager {
            if(!::sInstance.isInitialized) {
               throw NullPointerException("")
            }
            return sInstance
        }
    }

    init {
        initAidlConnect()
        //加载命令
        val serviceLoader: ServiceLoader<Command> = ServiceLoader.load(Command::class.java)
        for (command: Command in serviceLoader) {
            if(!mCommands.containsKey(command.name())) {
                mCommands[command.name()] = command
            }
        }
        mCommands["toast"] = ToastCommand()
    }

    //在application初始化的时候去调此方法
    private fun initAidlConnect() {
        val service = Intent(context, OptimizationService::class.java)
        service.action = "${context.packageName}.OptimizationService"
        context.bindService(service, this, Context.BIND_AUTO_CREATE)
    }

    private fun executeCommand(command: String, params: Map<Any?, Any?>?,
                               resultBack: ICallbackFromMainToWeb) {
        mCommands[command]?.exec(params, resultBack)
    }

    //注意这里是远端进程传递过来的，也是在子线程中，没有在主线程中
    override fun handleWebAction(command: String, params: Map<Any?, Any?>?,
                                 callback: ICallbackFromMainToWeb) {
        executeCommand(command, params, callback)
    }

    override fun onServiceConnected(name: ComponentName?, service: IBinder?) {

    }

    override fun onServiceDisconnected(name: ComponentName?) {
        initAidlConnect()
    }

    override fun onBindingDied(name: ComponentName?) {
        initAidlConnect()
    }
}