package com.melancholy.webview.ui.webview

import android.content.res.Configuration
import android.graphics.Color
import android.os.Bundle
import android.text.TextUtils
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.view.WindowManager
import com.at.common.binding.BaseBindingActivity
import com.at.common.global.Constants
import com.at.common.setLightStatusBar
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.utils.ViewUtils
import com.melancholy.webview.PreloadWebView
import com.melancholy.webview.databinding.WebviewActivityWebViewBinding
import com.melancholy.webview.widget.WebView


/**
 * @Created_time: 2018/3/9 10:14
 * @Description: ${TODO}(通用WebView)
 * WebView在SDK19 20两个版本中，由于安全问题谷歌在这两个版本中去掉了回调，因此要自己去实现js调用java
 * 解决WebView在19和20上不回调openFile选项https://github.com/cdibened/filechooser
 * WebView调用服务端页面如何访问本地文件
 * 然而由于安全原因，WebView限制了远程url页面访问本地文件
 * 如果我们加载的url是服务端的页面，那我们没有任何办法直接通过文件地址来访问客户端本地的文件
 * 找到了一种解决方法 重写shouldInterceptRequest方法 重写构造一个WebResourceResponse 以文件流的方法
 * 进行适配
 *  if(url.startsWith("http://android", true)){
 *  val str = "http://android"
 *  val path = url.substring(str.length)
 *  return  WebResourceResponse( "image/jpeg", "UTF-8", FileInputStream(File(path)))
 *  }
 *
 */

@Route(path = "/webview/WebViewActivity")
class WebViewActivity: BaseBindingActivity<WebviewActivityWebViewBinding>() {
    private lateinit var mWebView: WebView

    override fun createViewBinding(): WebviewActivityWebViewBinding {
        return WebviewActivityWebViewBinding.inflate(layoutInflater)
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        ViewUtils.setDrawDuringWindowsAnimating(window.decorView)
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setBackgroundColor(Color.WHITE)
        }
        requireBinding().apply {
            mWebView = PreloadWebView.getInstance().getWebView(this@WebViewActivity)
            mWebView.webChromeClient = WebChromeClientImpl(requireTitleBar(),
                requireBinding().pbProgress)
            val params = ViewGroup.LayoutParams(MATCH_PARENT, MATCH_PARENT)
            llContent.addView(mWebView, params)
        }
    }

    override fun loadData() {
        val url: String? = intent.getStringExtra(Constants.KEY_DATA)
        val html: String? = intent.getStringExtra(Constants.KEY_DATA_1)
        if(intent.hasExtra(Constants.KEY_DATA)){
            mWebView.loadUrl(url ?: "")
        } else if(intent.hasExtra(Constants.KEY_DATA_1)){
            mWebView.loadData(html ?: "", "text/html;charset=UTF-8", null)
        } else{
            throw IllegalArgumentException("url and html cannot be empty at the same time")
        }
    }

    override fun onResume() {
        mWebView.onResume()
        super.onResume()
    }

    override fun onPause() {
        mWebView.onPause()
        super.onPause()
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        //控制横竖屏的状态栏隐藏和显示
        if(newConfig.orientation==Configuration.ORIENTATION_PORTRAIT){
            window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
        } else if(newConfig.orientation==Configuration.ORIENTATION_LANDSCAPE){
            window.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
        }
        super.onConfigurationChanged(newConfig)
    }

    //WebView的返回逻辑
    override fun onBackPressedSupport(): Boolean {
        return if(mWebView.canGoBack()) {
            mWebView.goBack()
            true
        } else {
            false
        }
    }

    override fun onDestroy() {
        mWebView.destroy()
        super.onDestroy()
    }

}