package com.melancholy.webview.webviewprocess

import android.annotation.SuppressLint
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.Handler
import android.os.IBinder
import android.os.Looper
import com.at.common.utils.JsonUtils

import com.melancholy.webview.ICallbackFromMainToWeb
import com.melancholy.webview.IWebToMain
import com.melancholy.webview.mainprocess.MainProcessCommandService
import com.melancholy.webview.widget.WebView
import com.squareup.moshi.Types

/**
 * @Create_time: 2019/10/16 11:52
 * @Description:
 */
internal class WebViewProcessCommandDispatcher private constructor(private val context: Context):
    ServiceConnection {

    private var mIWebToMain: IWebToMain? = null
    private val mType = Types.newParameterizedType(MutableMap::class.java, Any::class.java,
        Any::class.java)
    private val mMainHandler: Handler = Handler(Looper.getMainLooper())

    companion object{
        @Volatile
        @SuppressLint("StaticFieldLeak")
        private lateinit var sInstance: WebViewProcessCommandDispatcher

        fun init(context: Context) {
            if(!::sInstance.isInitialized) {
                synchronized(WebViewProcessCommandDispatcher::class.java) {
                    if(!::sInstance.isInitialized) {
                        sInstance = WebViewProcessCommandDispatcher(context.applicationContext)
                    }
                }
            }
        }

        fun getInstance(): WebViewProcessCommandDispatcher {
            if(!::sInstance.isInitialized) {
                throw NullPointerException("")
            }
            return sInstance
        }
    }

    init {
        initAidlConnect()
    }

    private fun initAidlConnect() {
        val service = Intent(context, MainProcessCommandService::class.java)
        service.action = "${context.packageName}.MainProcessCommandService"
        context.bindService(service, this, Context.BIND_AUTO_CREATE)
    }

    //注意这里是webView传递过来的，也是在子线程中，没有在主线程中
    fun exec(param: String, webView: WebView) {
        try {
            val params: MutableMap<Any, Any> = JsonUtils.fromJson(mType, param)!!
            //获取命令
            val command: String = params["command"] as String
            //获取回调的key
            val callbackKey: String? = params["callback"] as String?
            //获取js回调过来的参数
            val args: String? = params["args"] as String?
            val argsMap: MutableMap<Any?, Any?>? = if(args.isNullOrEmpty()) null
                else JsonUtils.fromJson(mType, param)!!
            mIWebToMain?.handleWebAction(command, argsMap, object: ICallbackFromMainToWeb.Stub() {

                    override fun onResult(response: MutableMap<Any?, Any?>?) {
                        //回调callbackKey不为null才证明js是需要回调的
                        if (!callbackKey.isNullOrEmpty()) {
                            val responseMap: MutableMap<Any?, Any?> = mutableMapOf()
                            //将回调key添加到参数中
                            responseMap["callback"] = callbackKey
                            //将参数添加到map中
                            if (!response.isNullOrEmpty()) {
                                responseMap["args"] = JsonUtils.toJson(response)
                            }
                            handleCallback(responseMap, webView)
                        }
                    }

                })
        } catch (e: Exception) {
        }
    }

    fun handleCallback(response: MutableMap<Any?, Any?>, webView: WebView) {
        val callback = "nativeObjectJs.callback(${JsonUtils.toJson(response)})"
        mMainHandler.post {
            webView.callJavascript(callback, null)
        }
    }

    override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
        mIWebToMain = IWebToMain.Stub.asInterface(service)
    }

    override fun onServiceDisconnected(name: ComponentName?) {
        mIWebToMain = null
        initAidlConnect()
    }

    override fun onBindingDied(name: ComponentName?) {
        mIWebToMain = null
        initAidlConnect()
    }

}