package com.melancholy.webview.widget

import android.graphics.Bitmap
import android.net.Uri
import android.os.Build
import android.os.Message
import android.view.View
import android.webkit.*
import androidx.annotation.RequiresApi

/**
 * @Create_time: 2020/8/31 13:06
 * @Description:
 */
internal class WebChromeClientDelegate: WebChromeClient {

    private val mWebViewInterface: WebViewInterface
    private val mValueCallback: MutableMap<String, ValueCallback<String>>

    constructor(webViewInterface: WebViewInterface,
                valueCallback: MutableMap<String, ValueCallback<String>>) {
        mWebViewInterface = webViewInterface
        mValueCallback = valueCallback
    }

    private var mWebChromeClient: WebChromeClient? = null

    fun setWebChromeClient(client: WebChromeClient?) {
        mWebChromeClient = client
    }

    override fun onProgressChanged(view: android.webkit.WebView?, newProgress: Int) {
        mWebChromeClient?.onProgressChanged(view, newProgress)
    }

    override fun onReceivedTitle(view: android.webkit.WebView?, title: String?) {
        mWebChromeClient?.onReceivedTitle(view, title)
    }

    override fun onReceivedIcon(view: android.webkit.WebView?, icon: Bitmap?) {
        mWebChromeClient?.onReceivedIcon(view, icon)
    }

    override fun onReceivedTouchIconUrl(view: android.webkit.WebView?, url: String?,
                                        precomposed: Boolean) {
        mWebChromeClient?.onReceivedTouchIconUrl(view, url, precomposed)
    }

    override fun onShowCustomView(view: View?, callback: CustomViewCallback?) {
        mWebChromeClient?.onShowCustomView(view, callback)
    }

    override fun onShowCustomView(view: View?, requestedOrientation: Int,
                                  callback: CustomViewCallback?) {
        mWebChromeClient?.onShowCustomView(view, requestedOrientation, callback)
    }

    override fun onHideCustomView() {
        mWebChromeClient?.onHideCustomView()
    }

    override fun onCreateWindow(view: android.webkit.WebView?, isDialog: Boolean,
                                isUserGesture: Boolean, resultMsg: Message?): Boolean {
        return mWebChromeClient?.onCreateWindow(view, isDialog, isUserGesture, resultMsg) ?: false
    }

    override fun onRequestFocus(view: android.webkit.WebView?) {
        mWebChromeClient?.onRequestFocus(view)
    }

    override fun onCloseWindow(window: android.webkit.WebView?) {
        mWebChromeClient?.onCloseWindow(window)
    }

    override fun onJsAlert(view: android.webkit.WebView?, url: String?, message: String?,
                           result: JsResult?): Boolean {
        return mWebChromeClient?.onJsAlert(view, url, message, result) ?: false
    }

    override fun onJsConfirm(view: android.webkit.WebView?, url: String?, message: String?,
                             result: JsResult?): Boolean {
        return mWebChromeClient?.onJsConfirm(view, url, message, result) ?: false
    }

    override fun onJsPrompt(view: android.webkit.WebView?, url: String?, message: String?,
                            defaultValue: String?, result: JsPromptResult?): Boolean {
        if(onJsPromptIntercept(message, defaultValue)) {
            result?.cancel()
            return true
        }
        return mWebChromeClient?.onJsPrompt(view, url, message, defaultValue, result) ?: false
    }

    //所以采用显示可提示用户输入的对话框
    private fun onJsPromptIntercept(message: String?, defaultValue: String?): Boolean {
        if(!defaultValue.isNullOrEmpty()) {
            return when {
                defaultValue == "jsCallJava" -> {
                    //由于在AndroidApi17以下使用JavaInterface出现安全问题
                    if(!message.isNullOrEmpty()) {
                        mWebViewInterface.postWithCallback(message)
                    }
                    true
                }
                defaultValue.startsWith("resultCallback") -> {
                    //由于在AndroidApi19以下js调用java无法获取返回值，所以采用兼容模式
                    //获取key
                    val split: List<String> = defaultValue.split(":")
                    val key: String = split[1]
                    val valueCallback: ValueCallback<String>? = mValueCallback.remove(key)
                    if(!message.isNullOrEmpty() && valueCallback != null) {
                        valueCallback.onReceiveValue(message)
                    }
                    true
                }
                else -> {
                    false
                }
            }
        }
        return false
    }

    override fun onJsBeforeUnload(view: android.webkit.WebView?, url: String?, message: String?,
                                  result: JsResult?): Boolean {
        return mWebChromeClient?.onJsBeforeUnload(view, url, message, result) ?: false
    }

    override fun onExceededDatabaseQuota(url: String?, databaseIdentifier: String?, quota: Long,
                                         estimatedDatabaseSize: Long, totalQuota: Long,
                                         quotaUpdater: WebStorage.QuotaUpdater?) {
        mWebChromeClient?.onExceededDatabaseQuota(url, databaseIdentifier, quota,
            estimatedDatabaseSize, totalQuota, quotaUpdater)
    }

    override fun onReachedMaxAppCacheSize(requiredStorage: Long, quota: Long,
                                          quotaUpdater: WebStorage.QuotaUpdater?) {
        mWebChromeClient?.onReachedMaxAppCacheSize(requiredStorage, quota, quotaUpdater)
    }

    override fun onGeolocationPermissionsShowPrompt(origin: String?,
                                                    callback: GeolocationPermissions.Callback?) {
        mWebChromeClient?.onGeolocationPermissionsShowPrompt(origin, callback)
    }

    override fun onGeolocationPermissionsHidePrompt() {
        mWebChromeClient?.onGeolocationPermissionsHidePrompt()
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun onPermissionRequest(request: PermissionRequest?) {
        mWebChromeClient?.onPermissionRequest(request)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun onPermissionRequestCanceled(request: PermissionRequest?) {
        mWebChromeClient?.onPermissionRequestCanceled(request)
    }

    override fun onJsTimeout(): Boolean {
        return mWebChromeClient?.onJsTimeout() ?: true
    }

    override fun onConsoleMessage(message: String?, lineNumber: Int, sourceID: String?) {
        mWebChromeClient?.onConsoleMessage(message, lineNumber, sourceID)
    }

    override fun onConsoleMessage(consoleMessage: ConsoleMessage?): Boolean {
        return mWebChromeClient?.onConsoleMessage(consoleMessage) ?: false
    }

    override fun getDefaultVideoPoster(): Bitmap? {
        return mWebChromeClient?.defaultVideoPoster
    }

    override fun getVideoLoadingProgressView(): View? {
        return mWebChromeClient?.videoLoadingProgressView
    }

    override fun getVisitedHistory(callback: ValueCallback<Array<String>>?) {
        mWebChromeClient?.getVisitedHistory(callback)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun onShowFileChooser(webView: android.webkit.WebView?,
                                   filePathCallback: ValueCallback<Array<Uri>>?,
                                   fileChooserParams: FileChooserParams?): Boolean {
        return mWebChromeClient?.onShowFileChooser(webView, filePathCallback, fileChooserParams) ?: false
    }
}