package com.melancholy.webview.widget

import android.R.attr.webViewStyle
import android.annotation.SuppressLint
import android.content.Context
import android.os.Build
import android.util.AttributeSet
import android.webkit.ValueCallback
import android.webkit.WebChromeClient
import android.webkit.WebViewClient
import androidx.annotation.RequiresApi
import com.melancholy.widget.FixWebView

/**
 * @Create_time: 2021/8/6 22:55
 * @Description:
 */
class WebView: FixWebView {

    private lateinit var mWebViewInterface: WebViewInterface
    private lateinit var mWebViewClientDelegate: WebViewClientDelegate
    private lateinit var mWebChromeClientDelegate: WebChromeClientDelegate
    //在api19以下 由于没有方法去获取返回值
    private val mValueCallback: MutableMap<String, ValueCallback<String>> = mutableMapOf()

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = webViewStyle):
            super(context, attrs, defStyleAttr) {
        initialize()
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int):
            super(context, attrs, defStyleAttr, defStyleRes) {
        initialize()
    }

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int,
                privateBrowsing: Boolean) : super(context, attrs, defStyleAttr, privateBrowsing) {
        initialize()
    }

    @SuppressLint("ObsoleteSdkInt", "AddJavascriptInterface")
    private fun initialize() {
        mWebViewInterface = WebViewInterface(this)
        mWebViewClientDelegate = WebViewClientDelegate()
        mWebChromeClientDelegate = WebChromeClientDelegate(mWebViewInterface, mValueCallback)
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1){
            addJavascriptInterface(mWebViewInterface, "nativeObject")
        }else if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB){
            try {
                //在Android17一下移除该js对象 WebView安全问题
                removeJavascriptInterface("accessibility")
                removeJavascriptInterface("accessibilityTraversal")
                removeJavascriptInterface("searchBoxJavaBridge_")
            } catch (e: Exception){
            }
        }
        //是HTTP请求头部用来标识客户端信息的字符串， 包括操作系统， 浏览器等信息。
        // 为了建立手机客户端的信息数据库，需要从手机的http请求中取到这一字符串
        //mWebSettings.userAgentString = "pc"
        super.setWebViewClient(mWebViewClientDelegate)
        super.setWebChromeClient(mWebChromeClientDelegate)
    }

    override fun setWebViewClient(client: WebViewClient) {
        mWebViewClientDelegate.setWebViewClient(client)
    }

    override fun setWebChromeClient(client: WebChromeClient?) {
        mWebChromeClientDelegate.setWebChromeClient(client)
    }

    /**
     * 支持低版本兼容获取返回值
     */
    fun callJavascript(script: String?, resultCallback: ValueCallback<String>?) {
        when {
            Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT -> {
                val scriptCommand = "javascript:$script"
                super.evaluateJavascript(scriptCommand, resultCallback)
            }
            resultCallback != null -> {
                //需要带返回值
                //生成一个key
                val key = "callback${System.currentTimeMillis()}"
                val scriptCommand = "javascript:prompt($script, \"resultCallback:$key\")"
                mValueCallback[key] = resultCallback
                loadUrl(scriptCommand)
            }
            else -> {
                val scriptCommand = "javascript:$script"
                loadUrl(scriptCommand)
            }
        }
    }


    /**
     * 该方法无效了
     * 改用callJavascript方法
     */
    @Deprecated(message = "该方法被废弃掉了")
    override fun evaluateJavascript(script: String, resultCallback: ValueCallback<String>?) {

    }
}