package com.melancholy.webview.widget

import android.app.Application
import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.net.http.SslError
import android.os.Build
import android.os.Message
import android.view.KeyEvent
import android.webkit.*
import androidx.annotation.RequiresApi
import com.melancholy.utils.AppInstance

/**
 * @Create_time: 2021/8/6 22:55
 * @Description: 代理模式
 */
internal class WebViewClientDelegate: WebViewClient() {
    private var mWebViewClient: WebViewClient? = null

    fun setWebViewClient(webViewClient: WebViewClient?) {
        mWebViewClient = webViewClient
    }


    /**
     * 该接口，主要是给WebView提供时机，让其选择是否对UrlLoading进行拦截。
     * 关于该接口的返回值，True（拦截WebView加载Url），False（允许WebView加载Url）
     */
    @Suppress("DEPRECATION")
    override fun shouldOverrideUrlLoading(view: android.webkit.WebView?, url: String?): Boolean {
        return if(mWebViewClient?.shouldOverrideUrlLoading(view, url) == true) {
            true
        } else {
            shouldOverrideUrlLoading(url)
        }
    }

    @RequiresApi(Build.VERSION_CODES.N)
    override fun shouldOverrideUrlLoading(view: android.webkit.WebView?,
                                          request: WebResourceRequest?): Boolean {
        return if (mWebViewClient?.shouldOverrideUrlLoading(view, request) == true) {
            true
        } else {
            shouldOverrideUrlLoading(request?.url?.toString())
        }
    }

    //处理重定向的问题
    private fun shouldOverrideUrlLoading(url: String?): Boolean {
        return if(url?.startsWith("file") == true || url?.startsWith("http") == true
            || url?.startsWith("https") == true) {
            false
        } else {
            //微信支付也在此拦截
            try {
                val application: Application = AppInstance.getApplication()
                var intent = Intent(Intent.ACTION_VIEW)
                intent.data = Uri.parse(url)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                application.startActivity(intent)
            } catch (e: Exception) {
            }
            true
        }
    }

    override fun onPageStarted(view: android.webkit.WebView?, url: String?, favicon: Bitmap?) {
        mWebViewClient?.onPageStarted(view, url, favicon)
    }

    override fun onPageFinished(view: android.webkit.WebView?, url: String?) {
        mWebViewClient?.onPageFinished(view, url)
    }

    override fun onLoadResource(view: android.webkit.WebView?, url: String?) {
        mWebViewClient?.onLoadResource(view, url)
    }

    @RequiresApi(Build.VERSION_CODES.M)
    override fun onPageCommitVisible(view: android.webkit.WebView?, url: String?) {
        mWebViewClient?.onPageCommitVisible(view, url)
    }

    //在此方法进行拦截
    @Suppress("DEPRECATION")
    override fun shouldInterceptRequest(view: android.webkit.WebView?,
                                        url: String?): WebResourceResponse? {
        val response: WebResourceResponse? = mWebViewClient?.shouldInterceptRequest(view, url)
        return if(response != null) {
            response
        } else {
            //在此进行网络请求拦截进行拦截 可以使用OkHttp来请求资源
            var ve = 0
            null
        }
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun shouldInterceptRequest(view: android.webkit.WebView?,
                                        request: WebResourceRequest?): WebResourceResponse? {
        val response: WebResourceResponse? = mWebViewClient?.shouldInterceptRequest(view, request)
        return if(response != null) {
            response
        } else {
            //在此进行网络请求拦截进行拦截 可以使用OkHttp来请求资源
                val ver = 0
            null
        }
    }

    override fun onTooManyRedirects(view: android.webkit.WebView?, cancelMsg: Message?,
                                    continueMsg: Message?) {
        mWebViewClient?.onTooManyRedirects(view, cancelMsg, continueMsg)
    }

    @Suppress("DEPRECATION")
    override fun onReceivedError(view: android.webkit.WebView?, errorCode: Int,
                                 description: String?, failingUrl: String?) {
        mWebViewClient?.onReceivedError(view, errorCode, description, failingUrl)
    }

    @RequiresApi(Build.VERSION_CODES.M)
    override fun onReceivedError(view: android.webkit.WebView?, request: WebResourceRequest?,
                                 error: WebResourceError?) {
        mWebViewClient?.onReceivedError(view, request, error)
    }

    @RequiresApi(Build.VERSION_CODES.M)
    override fun onReceivedHttpError(view: android.webkit.WebView?, request: WebResourceRequest?,
                                     errorResponse: WebResourceResponse?) {
        mWebViewClient?.onReceivedHttpError(view, request, errorResponse)
    }

    override fun onFormResubmission(view: android.webkit.WebView?, dontResend: Message?,
                                    resend: Message?) {
        mWebViewClient?.onFormResubmission(view, dontResend, resend)
    }

    override fun doUpdateVisitedHistory(view: android.webkit.WebView?, url: String?,
                                        isReload: Boolean) {
        mWebViewClient?.doUpdateVisitedHistory(view, url, isReload)
    }

    override fun onReceivedSslError(view: android.webkit.WebView?, handler: SslErrorHandler?,
                                    error: SslError?) {
        mWebViewClient?.onReceivedSslError(view, handler, error)
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun onReceivedClientCertRequest(view: android.webkit.WebView?,
                                             request: ClientCertRequest?) {
        mWebViewClient?.onReceivedClientCertRequest(view, request)
    }

    override fun onReceivedHttpAuthRequest(view: android.webkit.WebView?, handler: HttpAuthHandler?,
                                           host: String?, realm: String?) {
        mWebViewClient?.onReceivedHttpAuthRequest(view, handler, host, realm)
    }

    override fun shouldOverrideKeyEvent(view: android.webkit.WebView?, event: KeyEvent?): Boolean {
        return mWebViewClient?.shouldOverrideKeyEvent(view, event) ?: false
    }

    override fun onUnhandledKeyEvent(view: android.webkit.WebView?, event: KeyEvent?) {
        mWebViewClient?.onUnhandledKeyEvent(view, event)
    }

    override fun onScaleChanged(view: android.webkit.WebView?, oldScale: Float, newScale: Float) {
        mWebViewClient?.onScaleChanged(view, oldScale, newScale)
    }

    override fun onReceivedLoginRequest(view: android.webkit.WebView?,
                                        realm: String?, account: String?, args: String?) {
        mWebViewClient?.onReceivedLoginRequest(view, realm, account, args)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onRenderProcessGone(view: android.webkit.WebView?,
                                     detail: RenderProcessGoneDetail?): Boolean {
        return mWebViewClient?.onRenderProcessGone(view, detail) ?: false
    }

    @RequiresApi(Build.VERSION_CODES.O_MR1)
    override fun onSafeBrowsingHit(view: android.webkit.WebView?, request: WebResourceRequest?,
                                   threatType: Int, callback: SafeBrowsingResponse?) {
        mWebViewClient?.onSafeBrowsingHit(view, request, threatType, callback)
    }
}