package com.video.module_widget

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/11/7 下午5:04
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/11/7 下午5:04
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
import android.annotation.SuppressLint
import com.at.common.global.Constants
import com.video.module_widget.constants.GlobalConsts
import java.math.RoundingMode
import java.text.DecimalFormat
import java.text.SimpleDateFormat
import java.util.*

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/30 下午8:28
 */
fun String.parseServerTime(format: String? = null): Date? {
    val serverTime: String = this
    var format = format
    if (format == null || format.isEmpty()) {
        format = "yyyy-MM-dd"
    }
    val sdf = SimpleDateFormat(format, Locale.CHINESE)
    sdf.timeZone = TimeZone.getTimeZone("GMT+8:00")
    var date: Date? = null
    try {
        date = sdf.parse(serverTime)
    } catch (e: Exception) {
        // Timber.e(e, "")
    }
    return date
}

fun String.fileUrl(): String = Constants.BASE_URL + "/v1/file/download?address=$this"

fun String.formatTimeMills(format: String? = null): Date? {
    val serverTime: String = this
    var format = format
    if (format == null || format.isEmpty()) {
        format = "yyyy-MM-dd"
    }
    val sdf = SimpleDateFormat(format, Locale.CHINESE)
    var date: Date? = Date(serverTime.toLong())
    try {
        date = sdf.parse(sdf.format(date))
    } catch (e: Exception) {
        // Timber.e(e, "")
    }
    return date
}

@SuppressLint("SimpleDateFormat")
fun Date.parseString(format: String? = null): String {
    var format = format
    if (format == null || format.isEmpty()) {
        format = "MM/dd"
    }
    val formatter = SimpleDateFormat(format)
    return formatter.format(this) + "/发布"
}

fun String.strToInt(): Int {
    if (this.isNullOrEmpty()) return 0
    return this.toInt()
}

fun String.strToFloat(): Float {
    if (this.isNullOrEmpty()) return 0f
    return this.toFloat()
}

// 保留一位小数
fun Float.getFloatDigits(digits: Int = 1): String {
    val format = if (digits == 1) DecimalFormat("#.#") else DecimalFormat("#.##")
    // 舍弃规则，RoundingMode.FLOOR表示直接舍弃。
    format.roundingMode = RoundingMode.FLOOR
    return format.format(this)
}

fun String?.imageUrl(): String? {
    return if (this?.startsWith("http") == true) {
        this
    } else if (this.isNullOrEmpty()) {
        this
    } else {
        if (this.startsWith("/")) {
            "${GlobalConsts.CMS_URL}${this.substring(1)}"
        } else {
            "${GlobalConsts.CMS_URL}$this"
        }
    }
}

/**
 * 获取文件大小
 */
fun getFileSize(size: Long): String {
    if (size <= 0) return "0"
    val units = arrayOf("b", "kb", "M", "G", "T")
    // 计算单位的，原理是利用lg,公式是 lg(1024^n) = nlg(1024)，最后 nlg(1024)/lg(1024) = n。
    val digitGroups =
        (Math.log10(size.toDouble()) / Math.log10(1024.0)).toInt()
    // 计算原理是，size/单位值。单位值指的是:比如说b = 1024,KB = 1024^2
    return DecimalFormat("#,##0.##")
        .format(size / Math.pow(1024.0, digitGroups.toDouble())) + " " + units[digitGroups]
}

fun Int.hot2String(hint: String = ""): String {
    return if (this < 10000) {
        "${this}$hint"
    } else { // 保留两位小数
        "${(this / 10000f).getFloatNoMoreThanTwoDigits()}万$hint"
    }
}

// 保留一位小数
fun Float.getFloatNoMoreThanTwoDigits(): String {
    val format = DecimalFormat("#.#")
    // 舍弃规则，RoundingMode.FLOOR表示直接舍弃。
    format.roundingMode = RoundingMode.HALF_UP
    return format.format(this)
}
