package com.video.module_widget.binder // ktlint-disable package-name

import android.util.Log
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.global.Constants
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.R
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.Content
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.*
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/6/23 19:44
 * @Description:
 */

class AdImageBinder(private val showMarginVertical: Boolean = false, private val showMarginHorizontal: Boolean = true, val style: Content? = null, val bg: String? = null) : BaseItemBinder<AdImageBinder.Data, BaseViewHolder>() {

    private val mPadding: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_16)
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.common_ad_image_view, parent, false)
        )
    }

    override fun convert(holder: BaseViewHolder, data: Data) {
        val frameLayout = holder.getView<FrameLayout>(R.id.ad_root)
        val param = frameLayout.layoutParams as GridLayoutManager.LayoutParams
        if (showMarginVertical && showMarginHorizontal) {
            param.setMargins(mPadding, mPadding, mPadding, 0)
        }
        if (!showMarginHorizontal && !showMarginVertical) {
            param.setMargins(0, 0, 0, 0)
        }

        if (!showMarginHorizontal && showMarginVertical) {
            param.setMargins(0, mPadding, 0, 0)
        }
        bg?.let {
            frameLayout.setBackgroundColor(it.toArgb())
        }
        val imgUrl = data.data.image_url.imageUrl()
        holder.getView<DiversityImageView>(R.id.div_ad)
            .loadByGlide(
                GlideImageConfig(
                    url = imgUrl,
                    placeholderId = R.drawable.common_ic_ad_default,
                    errorId = R.drawable.common_ic_ad_default
                ),
                WeakReference(context)
            )
        style?.let {
            it.config?.apply {
                param.height = context.getPx(height)
                holder.getView<DiversityImageView>(R.id.div_ad).setFullRadius(context.getPx(radius).toFloat())
            }
        }
        Log.e("Banner", imgUrl!!)
        holder.itemView.setOnClickListener {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, data.data.jump_addr!!)
                .navigation()
        }
    }

    data class Data(val data: AdvertisingBean)
}
