package com.video.module_widget.binder

import android.content.Context
import android.graphics.Color
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.widget.FrameLayout
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.chad.library.adapter.base.binder.BaseItemBinder
import com.chad.library.adapter.base.viewholder.BaseViewHolder
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppInstance
import com.melancholy.widget.DiversityImageView
import com.video.module_widget.R
import com.video.module_widget.databinding.CommonBannerViewBinding
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.Content
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.*
import com.video.module_widget.widget.CustomBanner
import com.video.module_widget.widget.ShapeTextViewExt
import com.youth.banner.Banner
import com.youth.banner.adapter.BannerAdapter
import com.youth.banner.listener.OnBannerListener
import com.youth.banner.listener.OnPageChangeListener
import java.lang.ref.WeakReference

/**
 * @Create_time: 2022/4/27 16:50
 * @Description: 轮播图
 */

class BannerBinder(val style:Content? = null,val showMarginVertical: Boolean = false): BaseItemBinder<BannerBinder.BannerInfo, BaseViewHolder>(),
    OnPageChangeListener, OnBannerListener<AdvertisingBean> {

    private var mBinding: CommonBannerViewBinding? = null
    private var mAdapter: Adapter? = null
    private val mPadding: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_16)
    private val mPadding1: Int = AppInstance.getApplication().resources
        .getDimensionPixelSize(com.at.common.R.dimen.dp_24)

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        val inflater: LayoutInflater = LayoutInflater.from(parent.context)
        val banner = CustomBanner(parent.context)
//        val duration = IMainService.getMainService().getSysConfig().getConfigAdRollingTime() * 1000L
        var duration = 3 * 1000L

        mBinding = CommonBannerViewBinding.inflate(inflater, banner, true)
        banner.addOnPageChangeListener(this)
        style?.let {
            it.config?.let { it1 ->
              val  height =  context.getPx(it1.height)
                val params = FrameLayout.LayoutParams(MATCH_PARENT, height)
                if(showMarginVertical){
                    params.topMargin = mPadding
                }
                duration = it1.duration.toLong()
                if(it1.style == 2 ){
                    params.leftMargin = mPadding
                    params.rightMargin = mPadding
                    banner.setBannerRound(parent.context.getDp(it1.radius))
                }
                if(it1.style == 3){
                    banner.setBannerGalleryEffect(10,6)
                    mBinding!!.rivIndicator.visibility = View.GONE
                    mBinding!!.flRoot.setPadding(0,0,mPadding,0)
                    mBinding!!.tvText.setPadding(mPadding1,0,0,0)
                }
                if(it1.style == 1){
                    mBinding!!.tvText.setPadding(mPadding,0,0,0)
                }
                banner.layoutParams = params
            }
        }
        banner.setLoopTime(duration)
        mAdapter = Adapter(context,style)
        mAdapter!!.setStyle(style)
        banner.setAdapter(mAdapter)
//        banner.setBannerGalleryEffect(10,6)
        //mBinding!!.rivIndicator.setupWithViewPager(banner.viewPager2, true)
        banner.setOnBannerListener(this)
        return BaseViewHolder(banner)
    }

    override fun convert(holder: BaseViewHolder, data: BannerInfo) {
        (holder.itemView as Banner<AdvertisingBean, Adapter>)
            .setDatas(data.list)
        mBinding?.rivIndicator?.setCount(data.list.size)
        mBinding?.tvText?.text = data.list[0].name
        changeTagStyle(holder.getView(R.id.stv_tag))
    }


    private fun changeTagStyle(shapeTextViewExt: ShapeTextViewExt){
        val params = shapeTextViewExt.layoutParams as FrameLayout.LayoutParams
        style?.let {
            when(it.config?.tagStyle){
                1 ->{
                    shapeTextViewExt.apply {
                        setBorderRadius(20)
                        params.gravity = Gravity.BOTTOM or Gravity.LEFT
                        val type = it.config.style
                        if(type == 2){
                            params.leftMargin = mPadding1
                        }
                        if(type == 3){
                            params.leftMargin = mPadding
                        }
                        reBuilder()
                    }
                }
                3 ->{
                    shapeTextViewExt.apply {
                        setBorderRadius(4)
                        params.gravity = Gravity.LEFT or Gravity.TOP
                        val type = it.config?.style
                        if(type == 2){
                            params.leftMargin = mPadding1
                        }
                        if(type == 3){
                            params.leftMargin = mPadding
                        }
                        reBuilder()
                    }
                }
                else ->{
                    shapeTextViewExt.apply {
                        setBorderRadius(leftTop = context.getDp(com.at.common.R.dimen.dp_20).toInt(), leftBottom = context.getDp(com.at.common.R.dimen.dp_20).toInt())
                        params.gravity = Gravity.RIGHT or Gravity.TOP
                        reBuilder()
                    }
                }
            }
        }

    }

    override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {

    }

    override fun onPageSelected(position: Int) {
        mBinding?.rivIndicator?.setIndex(position)
        mBinding?.tvText?.text = mAdapter?.getData(position)?.name
    }

    override fun onPageScrollStateChanged(state: Int) {

    }

    data class BannerInfo(val list: List<AdvertisingBean>)

    class Adapter(val context: Context, val style:Content? = null): BannerRadiusImageAdapter<AdvertisingBean>(mutableListOf()) {


        override fun onBindView(
            holder: BannerRadiusImageViewHolder,
            data: AdvertisingBean,
            position: Int,
            size: Int
        ) {
//            holder.imageView.load(ImageConfig(data.image_url.imageUrl(),
//                placeholderId = R.drawable.common_ic_ad_default,
//                errorId = R.drawable.common_ic_ad_default))

            holder.imageView.loadByGlide(
                GlideImageConfig(data.image_url.imageUrl(),
                placeholderId = R.drawable.common_ic_ad_default,
                errorId = R.drawable.common_ic_ad_default),
                WeakReference(context)
            )
        }
    }

     class BannerRadiusImageViewHolder(itemView: View):RecyclerView.ViewHolder(itemView){
          var imageView: ImageView
         init {
             imageView = itemView as ImageView
         }
     }


    abstract class BannerRadiusImageAdapter<T>(data:List<T>):BannerAdapter<T,BannerRadiusImageViewHolder>(data){

        private var style:Content? = null

        fun setStyle(mStyle:Content?){
            style = mStyle
        }

        override fun onCreateHolder(
            parent: ViewGroup?,
            viewType: Int
        ): BannerRadiusImageViewHolder {
            val imageView = DiversityImageView(parent!!.context)
            val params = ViewGroup.LayoutParams(
                MATCH_PARENT,
                MATCH_PARENT
            )
            imageView.layoutParams = params
           style?.let {
               it.config?.let { it1 ->
                   imageView.setFullRadius(
                       AppInstance.getApplication().getPx(it1.radius).toFloat()
                   )
               }
           }
            imageView.scaleType = ImageView.ScaleType.CENTER_CROP
            return  BannerRadiusImageViewHolder(imageView)
        }


    }

    override fun OnBannerClick(data: AdvertisingBean, position: Int) {
        Router.getInstance().build("/webview/WebViewActivity")
            .withString(Constants.KEY_DATA, data.jump_addr!!)
            .navigation()
    }
}