package com.video.module_widget.dialog

import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.video.module_widget.R
import com.video.module_widget.databinding.CommonTipDialogBinding

/**
 *
 * @Description: 通用提示弹出
 * @CreateDate: 2023年02月01日14:56:08
 * @UpdateUser: 更新者 ly
 * @UpdateDate: 2023年02月01日14:56:17
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class CommonTipDialog : BaseBindingDialogFragment<CommonTipDialogBinding>(), View.OnClickListener {
    private lateinit var messages: String
    private var closeCallback: CloseCallback? = null

    companion object {
        @JvmStatic
        fun showDialog(
            manager: FragmentManager,
            msg: String,
            closeLabel: String = "确定",
            callback: CloseCallback? = null
        ) {
            val dialog = CommonTipDialog()
            dialog.arguments =
                bundleOf(Constants.KEY_DATA to msg, Constants.KEY_DATA_1 to closeLabel)
            dialog.setCloseCallback(callback)
            dialog.show(manager, null)
        }
    }

    fun setCloseCallback(callback: CloseCallback?) {
        closeCallback = callback
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): CommonTipDialogBinding {
        return CommonTipDialogBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvConfirm.setOnClickListener(this@CommonTipDialog)
            messages = requireArguments().getString(Constants.KEY_DATA)!!
            tvTipsSub.text = messages
            tvConfirm.text = requireArguments().getString(Constants.KEY_DATA_1)!!
            tvTipsSub.movementMethod = ScrollingMovementMethod.getInstance()
        }
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int = WindowManager.LayoutParams.WRAP_CONTENT

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {
    }

    override fun onClick(view: View) {
        if (view.id == R.id.tv_confirm) {
            closeCallback?.let {
                closeCallback!!.close()
            }
            dismissAllowingStateLoss()
        }
    }

    interface CloseCallback {
        fun close()
    }
}
