package com.video.module_widget.dialog

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.video.module_widget.R
import com.video.module_widget.databinding.AlterDialogBinding

/**
 * @Create_time: 2022/6/22 16:25
 * @Description:
 */

class HintDialog : BaseBindingDialogFragment<AlterDialogBinding>(), View.OnClickListener {

    private var mCallback: ((Boolean) -> Unit)? = null

    companion object {
        @JvmStatic
        fun showDialog(manager: FragmentManager, title: String = "提示", leftLabel: String = "退出", rightLabel: String = "重试", content: String = "获取配置信息失败", callback: ((Boolean) -> Unit)) {
            val dialog = HintDialog()
            dialog.setCallback(callback)
            dialog.setTitle(title)
            dialog.setContent(content)
            dialog.setLeftLabel(leftLabel)
            dialog.setRightLabel(rightLabel)
            dialog.show(manager, null)
        }
    }

    fun setCallback(callback: ((Boolean) -> Unit)) {
        mCallback = callback
    }

    override fun getCancelable(): Boolean = false

    override fun getCustomTheme(): Int = R.style.HintDialog

    override fun getWidth(): Int = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_300)

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): AlterDialogBinding {
        return AlterDialogBinding.inflate(layoutInflater, container, false)
    }

    fun setTitle(title: String) {
        titleTv = title
    }

    fun setContent(content: String) {
        contentTv = content
    }

    fun setLeftLabel(left: String) {
        tvLeftTv = left
    }

    fun setRightLabel(right: String) {
        tvRightTv = right
    }

    private lateinit var tvLeftTv: String
    private lateinit var tvRightTv: String
    private lateinit var titleTv: String
    private lateinit var contentTv: String
    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvLeft.setOnClickListener(this@HintDialog)
            tvRight.setOnClickListener(this@HintDialog)
            tvLeft.text = tvLeftTv
            tvRight.text = tvRightTv
            tvTitle.text = titleTv
            tvTipsSub.text = contentTv
        }
    }

    override fun loadData() {
    }

    override fun onClick(v: View) {
        if (v.id == R.id.tv_left) {
            mCallback?.invoke(true)
        } else {
            mCallback?.invoke(false)
        }
        dismiss()
    }
}
