package com.video.module_widget.dialog

import android.os.Bundle
import android.text.method.ScrollingMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import com.at.common.binding.BaseBindingDialogFragment
import com.at.common.global.Constants
import com.video.module_widget.R
import com.video.module_widget.databinding.CommonMessageDialogBinding
import com.video.module_widget.utils.StyleUtils

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/12/5 上午10:53
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/12/5 上午10:53
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class MessageDialog : BaseBindingDialogFragment<CommonMessageDialogBinding>(), View.OnClickListener {
    private lateinit var messages: String
    private var closeCallback: CloseCallback? = null
    companion object {
        @JvmStatic
        fun showDialog(
            manager: FragmentManager,
            msg: String,
            closeLabel: String = "确定",
            callback: CloseCallback? = null
        ) {
            val dialog = MessageDialog()
            dialog.arguments = bundleOf(Constants.KEY_DATA to msg, Constants.KEY_DATA_1 to closeLabel)
            dialog.setCloseCallback(callback)
            dialog.show(manager, null)
        }
    }

    fun setCloseCallback(callback: CloseCallback?) {
        closeCallback = callback
    }

    override fun getCustomTheme(): Int {
        return com.at.common.R.style.CommonDialogStyle
    }

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): CommonMessageDialogBinding {
        return CommonMessageDialogBinding.inflate(inflater, container, false)
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireBinding().apply {
            tvButton.setOnClickListener(this@MessageDialog)
            messages = requireArguments().getString(Constants.KEY_DATA)!!
            tvContent.text = messages
            tvButton.text = requireArguments().getString(Constants.KEY_DATA_1)!!
            tvButton.setTextColor(StyleUtils.primaryColor)
            tvContent.movementMethod = ScrollingMovementMethod.getInstance()
        }
    }

    override fun getWidth(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_343)
    }

    override fun getHeight(): Int {
        return requireContext().resources.getDimensionPixelSize(com.at.common.R.dimen.dp_222)
    }

    override fun getCancelable(): Boolean {
        return false
    }

    override fun loadData() {
    }

    override fun onClick(view: View) {
        if (view.id == R.id.tv_button) {
            closeCallback?.let {
                closeCallback!!.close()
            }
            dismissAllowingStateLoss()
        }
    }

    interface CloseCallback {
        fun close()
    }
}
