package com.video.module_widget.utils

import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.melancholy.utils.AppInstance
import com.melancholy.utils.AppUtils
import com.squareup.moshi.KotlinJsonAdapterFactory
import com.squareup.moshi.Moshi
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.*
import com.video.module_widget.entity.adapter.DataReportAdapter
import java.net.URLEncoder

object DataUtils {

    fun getTestAdv(datas: List<AdvertisingInfo>, location: Int, limit: Int): List<AdvertisingInfo>? {
        var res = mutableListOf<AdvertisingInfo>()
        var tempLimit = limit
        for (adv in datas) {
            if (adv.location == location && limit != 0) {
                res.add(adv)
                tempLimit--
            }
            if (limit == 0) {
                break
            }
        }
        return res
    }

    // 过滤广告根据位置
    fun filterAdvPosition(datas: List<AdvertisingBean>, pos: String): List<AdvertisingBean>? {
        var res = mutableListOf<AdvertisingBean>()
        for (adv in datas) {
            if (adv.show_position?.indexOf(pos) != -1) {
                res.add(adv)
            }
        }
        return if (res.isEmpty()) null else res
    }

    fun dataReport(): String {
//        Log.e("dataInfo", "dataReport: ${PhoneUtils.toPrintDeviceInfo()}" )
        var deviceId = CacheHelper.getInstance().getString("deviceId", null)
        var isFirst = 1
        if (deviceId == null) {
            deviceId = PhoneUtils.getIMEI()
            CacheHelper.getInstance().putString("deviceId", deviceId)
            isFirst = 1
        } else {
            isFirst = 2
        }
        val simCount = PhoneUtils.getSimUsedCount()
        var sim1: String? = null
        var sim2: String? = null
        if (simCount > 0) {
            val sims = PhoneUtils.getSimMultiInfo()
            if (sims != null) {
                for (i in sims.indices) {
                    if (i == 0) {
                        sim1 = sims[i].mNumber as String
                    } else {
                        sim2 = sims[i].mNumber as String
                    }
                }
            }
        }

        val deviceInfo = DataReport(
            device_id = deviceId,
//            webmaster_id = GlobalConsts.WEB_MASTER_ID,
            app_version = AppUtils.getVersionName(AppInstance.getApplication()),
            report_time = System.currentTimeMillis(),
//            app_id = Constants.APP_ID,
            device_system = "Android",
            system_version = PhoneUtils.getDeviceAndroidVersion(),
            device_brand = PhoneUtils.getDeviceBrand(),
            device_model = PhoneUtils.getDeviceModel(),
            network_status = PhoneUtils.getNetworkType(),
            network_operator = PhoneUtils.getSimOperatorByMnc(),
            screen_resolution = PhoneUtils.getAppSize(),
            sim1 = sim1,
            sim2 = sim2,
            province = if (LocationManagerUtils.provice != null) URLEncoder.encode(LocationManagerUtils.provice, "UTF-8") else null,
            city = if (LocationManagerUtils.city != null) URLEncoder.encode(LocationManagerUtils.city, "UTF-8") else null,
            district = if (LocationManagerUtils.address_details != null) URLEncoder.encode(LocationManagerUtils.address_details, "UTF-8") else null,
            longitude = if (LocationManagerUtils.longitude != null) URLEncoder.encode(LocationManagerUtils.longitude, "UTF-8") else null,
            latitude = if (LocationManagerUtils.latitude != null) URLEncoder.encode(LocationManagerUtils.latitude, "UTF-8") else null,
            country = if (LocationManagerUtils.country != null) URLEncoder.encode(LocationManagerUtils.country, "UTF-8") else null,
            is_first = isFirst
        )

        val moshi = Moshi.Builder()
            .add(DataReportAdapter())
            .add(KotlinJsonAdapterFactory())
            .build()
        val jsonAdapter = moshi.adapter(DataReport::class.java)

//        Log.e("data_report", "dataReport: ${ jsonAdapter.toJson(deviceInfo)}")
        return jsonAdapter.toJson(deviceInfo)
    }

    // 生成上传实体类
    fun dataReport(type: Int = 0): DataRecordBean {
        var deviceId = CacheHelper.getInstance().getString("deviceId", null)
        var isFirst = 1
        if (deviceId == null) {
            deviceId = PhoneUtils.getIMEI()
            CacheHelper.getInstance().putString("deviceId", deviceId)
            isFirst = 1
        } else {
            isFirst = 2
        }
        val simCount = PhoneUtils.getSimUsedCount()
        var sim1: String? = null
        var sim2: String? = null
        if (simCount > 0) {
            val sims = PhoneUtils.getSimMultiInfo()
            if (sims != null) {
                for (i in sims.indices) {
                    if (i == 0) {
                        sim1 = sims[i].mNumber as String
                    } else {
                        sim2 = sims[i].mNumber as String
                    }
                }
            }
        }

        return when (type) {
            0 -> { // 启动数据
                val startUpRecord = StartUpRecord(
                    device_id = deviceId,
                    webmaster_id = GlobalConsts.WEB_MASTER_ID,
                    app_version = AppUtils.getVersionName(AppInstance.getApplication()),
                    app_id = Constants.APP_ID,
                    device_system = "Android",
                    system_version = PhoneUtils.getDeviceAndroidVersion(),
                    device_brand = PhoneUtils.getDeviceBrand(),
                    device_model = PhoneUtils.getDeviceModel(),
                    network_status = PhoneUtils.getNetworkType(),
                    network_operator = PhoneUtils.getSimOperatorByMnc(),
                    screen_resolution = PhoneUtils.getAppSize(),
                    sim1 = sim1,
                    sim2 = sim2,
                    province = LocationManagerUtils.provice,
                    city = LocationManagerUtils.city,
                    district = LocationManagerUtils.address_details,
                    longitude = LocationManagerUtils.longitude,
                    latitude = LocationManagerUtils.latitude,
                    country = LocationManagerUtils.country,
                    is_first = isFirst,
                    ip = PhoneUtils.getLocalIPAddress()
                )
                DataRecordBean(startup_record = startUpRecord)
            }
            1 -> { // 日志上报数据
                val logRecord = LogRecord(
                    device_id = deviceId,
                    webmaster_id = GlobalConsts.WEB_MASTER_ID,
                    app_version = AppUtils.getVersionName(AppInstance.getApplication()),
                    app_id = Constants.APP_ID,
                    device_system = "Android",
                    system_version = PhoneUtils.getDeviceAndroidVersion(),
                    device_brand = PhoneUtils.getDeviceBrand(),
                    device_model = PhoneUtils.getDeviceModel(),
                    sdk_version = PhoneUtils.getDeviceSDK().toString()
                )
                DataRecordBean(log_record = logRecord)
            }
            2 -> { // 活跃上报
                val activeRecord = ActiveRecord(
                    device_id = deviceId,
                    webmaster_id = GlobalConsts.WEB_MASTER_ID,
                    app_version = AppUtils.getVersionName(AppInstance.getApplication()),
                    app_id = Constants.APP_ID,
                    device_system = "Android",
                    system_version = PhoneUtils.getDeviceAndroidVersion(),
                    device_brand = PhoneUtils.getDeviceBrand(),
                    device_model = PhoneUtils.getDeviceModel(),
                    network_status = PhoneUtils.getNetworkType(),
                    network_operator = PhoneUtils.getSimOperatorByMnc(),
                    screen_resolution = PhoneUtils.getAppSize(),
                    sim1 = sim1,
                    sim2 = sim2,
                    province = LocationManagerUtils.provice,
                    city = LocationManagerUtils.city,
                    district = LocationManagerUtils.address_details,
                    longitude = LocationManagerUtils.longitude,
                    latitude = LocationManagerUtils.latitude,
                    country = LocationManagerUtils.country,
                    ip = PhoneUtils.getLocalIPAddress()
                )
                DataRecordBean(active_record = activeRecord)
            }
            3 -> { // 播放阅读等上报
                val activeRecord = PlayRecord(
                    device_id = deviceId,
                    webmaster_id = GlobalConsts.WEB_MASTER_ID,
                    app_version = AppUtils.getVersionName(AppInstance.getApplication()),
                    app_id = Constants.APP_ID,
                    device_system = "Android",
                    system_version = PhoneUtils.getDeviceAndroidVersion(),
                    device_brand = PhoneUtils.getDeviceBrand(),
                    device_model = PhoneUtils.getDeviceModel(),
                    network_status = PhoneUtils.getNetworkType(),
                    network_operator = PhoneUtils.getSimOperatorByMnc(),
                    screen_resolution = PhoneUtils.getAppSize(),
                    sim1 = sim1,
                    sim2 = sim2,
                    province = LocationManagerUtils.provice,
                    city = LocationManagerUtils.city,
                    district = LocationManagerUtils.address_details,
                    longitude = LocationManagerUtils.longitude,
                    latitude = LocationManagerUtils.latitude,
                    country = LocationManagerUtils.country,
                    ip = PhoneUtils.getLocalIPAddress()
                )
                DataRecordBean(play_record = activeRecord)
            }
            else -> DataRecordBean()
        }
    }

    fun dataReportTest(): String = "test"
}
