package com.video.module_widget.utils

import android.R.attr.radius
import android.content.Context
import android.graphics.drawable.Drawable
import android.widget.ImageView
import androidx.annotation.DrawableRes
import androidx.annotation.IntRange
import com.bumptech.glide.Glide
import com.bumptech.glide.Priority
import com.bumptech.glide.load.MultiTransformation
import com.bumptech.glide.load.resource.bitmap.CenterCrop
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import java.lang.ref.WeakReference


/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/10/19 下午1:28
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/10/19 下午1:28
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
data class GlideImageConfig(
    //加载图片
    val url: Any?,
    //加载图片的宽高
    @IntRange(from = -1L)
    val width: Int = -1,
    @IntRange(from = -1L)
    val height: Int = -1,
    //占位图
    val placeholder: Drawable? = null,
    @DrawableRes val placeholderId: Int = 0,
    //错误占位图
    val error: Drawable? = null,
    @DrawableRes val errorId: Int = 0,
    val priority: Priority = Priority.NORMAL,
    val isSkipMemoryCache:Boolean = false,
    val isCircle:Boolean = false,
    val radius:Float = -1f,
    val blur:Int = -1
)

fun ImageView.loadByGlide(config:GlideImageConfig,context:WeakReference<Context>){
    var options = RequestOptions()
    if(config.radius!=-1f){
        options = RequestOptions.bitmapTransform(
            MultiTransformation(
                CenterCrop(),
                RoundedCorners(radius)
            )
        )
    }
    if(config.isCircle){
        options.centerCrop().circleCrop()
    }
    if(config.placeholder!= null){
        options.placeholder(config.placeholder)
    }
    if(config.placeholderId!=0){
        options.placeholder(config.placeholderId)
    }
    if(config.error !=null){
        options.error(config.error)
    }
    if(config.errorId != 0){
        options.error(config.errorId)
    }
    if(width !=-1 && height !=-1){
        options.override(width,height)
    }

        options.priority(config.priority)
        options.skipMemoryCache(config.isSkipMemoryCache)
    val contexts: Context? = context.get()
    if(contexts !=null){
        Glide.with(contexts).load(config.url).apply(options).fitCenter().into(this)
    }
}