package com.video.module_widget.utils

import android.content.Context
import com.at.common.global.CacheHelper
import com.squareup.moshi.JsonAdapter
import com.squareup.moshi.Moshi
import com.squareup.moshi.Types
import com.squareup.moshi.adapter
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import org.json.JSONObject
import java.io.BufferedInputStream
import java.io.IOException
import java.io.InputStream
import java.lang.reflect.Type

object JsonUtils {

    val moshi: Moshi = Moshi.Builder().build()

    fun getHomeContentStyle(homeStyle: HomeStyle?, key:String): Content?{
        val res = homeStyle?.content!!.filter { if(it !=null)it.type == key else false }
        return if(res.isNotEmpty()) res[0] else null
    }

    fun getHomePageStyle(key: String): HomeStyle {
        val homeStr = CacheHelper.getInstance().getString(key, "")
        val jsonAdapter = moshi.adapter<HomeStyle>(HomeStyle::class.java)
        return jsonAdapter.fromJson(homeStr)!!
    }

    inline fun <reified T> getStyleBean(key:String):T?{
       try {
           val jsonStr = CacheHelper.getInstance().getString(key, "")
           val jsonAdapter = moshi.adapter<T>(T::class.java)
           return jsonAdapter.fromJson(jsonStr)!!
       }catch (e:Exception){

       }
        return null
    }

    inline fun <reified T> getObjectData(jsonStr:String):T{
        val jsonAdapter = moshi.adapter<T>(T::class.java)
        return jsonAdapter.fromJson(jsonStr)!!
    }

    inline fun <reified T> getObjectByJson(key:String,subKey:String):T?{
        try {
            val jsonStr = CacheHelper.getInstance().getString(key, "")
            val jsonObj = JSONObject(jsonStr)
            val subStr = jsonObj.getString(subKey)
            val jsonAdapter = moshi.adapter(T::class.java)
            return jsonAdapter.fromJson(subStr)!!
        }catch (e:Exception){

        }
        return null
    }

    inline fun <reified T> getListData(jsonObject: String):List<T>?{
        val type:Type = Types.newParameterizedType(List::class.java,T::class.java)
        val adapter:JsonAdapter<List<T>> = moshi.adapter(type)
        return adapter.fromJson(jsonObject)
    }

    inline fun <reified T> getAdvListData(key: String):List<T>?{
        val jsonStr = CacheHelper.getInstance().getString(key,"")
        val type:Type = Types.newParameterizedType(List::class.java,T::class.java)
        val adapter:JsonAdapter<List<T>> = moshi.adapter(type)
        return adapter.fromJson(jsonStr)
    }


    fun getDataFromAssetFile(context: Context,fileName: String?):JSONObject?{
          var jsonStr:String? = null
        jsonStr =  getAssertsFile(context,fileName)?.let { String(it) }
        var json:JSONObject? = null
        if(!jsonStr.isNullOrEmpty()){
             json = JSONObject(jsonStr)
        }
        return  json
    }

     private fun getAssertsFile(context: Context, fileName: String?): ByteArray? {
        var inputStream: InputStream? = null
        val assetManager = context.assets
        try {
            inputStream = assetManager.open(fileName!!)
            var bis: BufferedInputStream? = null
            val length: Int
            try {
                bis = BufferedInputStream(inputStream)
                length = bis.available()
                val data = ByteArray(length)
                bis.read(data)
                return data
            } catch (e: IOException) {
            } finally {
                if (bis != null) {
                    try {
                        bis.close()
                    } catch (e: Exception) {
                    }
                }
            }
            return null
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return null
    }
}