package com.video.module_widget.utils

import android.content.res.ColorStateList
import com.at.common.global.CacheHelper
import com.melancholy.utils.AppInstance
import com.video.module_widget.R
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.*
import com.video.module_widget.widget.LodeMoreView
import org.json.JSONObject

object StyleUtils {
    var primaryColor: Int = -1
    var primaryStr: String = ""
    var tagStyle = 1
    var tagEnable = false
    var loadIcon = 1
    var footer: Footer? = null
    var searchIcon = 1
    var change: Change? = null
    var homeMenuEnable = false

    fun getDpByDimen(resId: Int): Int {
        return AppInstance.getApplication().resources
            .getDimensionPixelSize(resId)
    }

    fun getHomeContentStyle(homeStyle: HomeStyle, key: String): Content? = JsonUtils.getHomeContentStyle(homeStyle, key)

    fun getHomePageStyle(key: String): HomeStyle = JsonUtils.getHomePageStyle(key)

    fun getPageBackground(key: String): String {
        val homeRecommend =
            JSONObject(CacheHelper.getInstance().getString(key, ""))
        return homeRecommend.get(StyleConsts.BACKGROUND).toString()
    }

    inline fun <reified T> getStyleBean(key: String): T? = JsonUtils.getStyleBean(key)

    // 获取主题色
    fun getPrimaryColor(): String {
        return primaryStr
    }

    fun getGlobalSearchIcon(): Int = when (searchIcon) {
        1 -> R.drawable.vector_drawable_icon_search1
        2 -> R.drawable.vector_drawable_icon_search2
        else -> R.drawable.vector_drawable_icon_search3
    }

    /**
     * 初始化全局变量
     */
    fun initGlobalConst() {
        val global = getStyleBean<Global>(StyleConsts.GLOBAL)
        global?.let {
            tagStyle = it.tag.style
            tagEnable = it.tag.enable
            primaryStr = it.primaryColor
            primaryColor = primaryStr.toArgb()
            loadIcon = it.loadIcon
            footer = it.footer
            searchIcon = it.searchIcon
            change = it.change
            homeMenuEnable = it.homeEntryMenuEnable
        }
    }

    fun createColorStateList(defaultColor: Int, selectedColor: Int): ColorStateList? {
        val states = arrayOfNulls<IntArray>(2)
        val colors = IntArray(2)
        var i = 0
        states[i] = intArrayOf(android.R.attr.state_checked)
        colors[i] = selectedColor
        i++
        // Default enabled state
        states[i] = intArrayOf(android.R.attr.state_enabled)
        colors[i] = defaultColor
        i++
        return ColorStateList(states, colors)
    }

    fun changeLoadMore(footerView: LodeMoreView) {
        footer?.let {
            footerView.setTextColor(it.color.toArgb())
            footerView.setTextNoData(it.text)
        }
        val img = if (loadIcon == 1) com.video.module_widget.R.drawable.vector_drawable_load_more_1 else com.video.module_widget.R.drawable.vector_drawable_load_more_2
        footerView.changeLoadIcon(img)
    }

    /**
     * 根据类型返回宫格显示数量
     * type
     * 1 两列平铺  8条
     * 2 三列平铺 12条
     * 3 横向滚动 10条
     * 其他 单张卡片 10条
     */
    fun getGridCountByType(type: Int) = when (type) {
        1 -> {
            8
        }
        2 -> {
            12
        }
        else -> {
            10
        }
    }

    fun getSpanCount(type: Int) = when (type) {
        1 -> {
            5
        }
        2 -> {
            4
        }
        3 -> {
            3
        }
        4 -> {
            2
        }
        else -> {
            4
        }
    }

    fun getCategoryColor(index: Int): List<String> {
        val colors = listOf(
            listOf("#DCE0F0", "#B9C2E7"),
            listOf("#D8E9E6", "#AABFBA"),
            listOf("#E5CCBB", "#C7A894"),
            listOf("#B1CAE5", "#B1CAE5"),
            listOf("#F7DBD6", "#F7DBD6"),
            listOf("#BDDFF1", "#91C2D3"),
            listOf("#CED3E0", "#AFB6C4"),
            listOf("#DBDBDB", "#A0A0A0")
        )
        if (index == -1) return colors[colors.size - 1]
        return colors[index]
    }
}
