package com.video.module_widget.widget

import android.content.res.ColorStateList
import android.graphics.Color
import android.util.Log
import android.util.StateSet
import android.view.Gravity
import android.view.View
import android.widget.TextView
import com.google.android.material.tabs.TabLayout
import com.loper7.tab_expand.indicator.BaseIndicator
import com.loper7.tab_expand.text.BaseText
import com.video.module_widget.utils.toColor

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/6 上午10:21
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/6 上午10:21
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class CustomTabText:BaseText() {


    private var tabColors = arrayListOf<ColorStateList>()
    private var tabNormalSizes:MutableList<Float> = mutableListOf()
    private var tabSelectedSizes:MutableList<Float> = mutableListOf()
    private var tabWidth = BaseIndicator.MATCH

    fun setTabColors(normalColors: MutableList<Int>, selectedColors: MutableList<Int>):CustomTabText{
        for (i in normalColors.indices){
            tabColors.add(createColorStateList(normalColors[i],selectedColors[i]))
        }
        return this
    }

    fun setTabNormalSizes(normalSizes: MutableList<Float>):CustomTabText{
        tabNormalSizes = normalSizes
        return this
    }

    fun setTabSelectedSizes(selectedSizes: MutableList<Float>):CustomTabText{
        tabSelectedSizes = selectedSizes
        return this
    }


    fun setWidth(mWidth:Int):CustomTabText{
        tabWidth = mWidth
        return this
    }

    private fun createColorStateList(defaultColor: Int, selectedColor: Int): ColorStateList {
        val states = arrayOfNulls<IntArray>(2)
        val colors = IntArray(2)
        var i = 0
        states[i] = intArrayOf(android.R.attr.state_selected)
        colors[i] = selectedColor
        i++
        // Default enabled state
        states[i] =intArrayOf(android.R.attr.state_enabled)
        colors[i] = defaultColor
        i++
        return ColorStateList(states, colors)
    }

    override fun bind() {
        tabLayout?.post {
            tabLayout?.apply {
                for (i in 0 until tabCount) {
                    getTabAt(i)?.let {
                        it.customView = TextView(context).apply {
                            if(width != BaseIndicator.MATCH){
                                width = tabWidth
                            }
                            text = it.text
                            val tabSelectSize = if(tabSelectedSizes.size > 1) tabSelectedSizes[i] else tabNormalSizes[0]
                            val tabNormalSize = if(tabNormalSizes.size > 1) tabNormalSizes[i] else tabNormalSizes[0]
                            textSize = if (selectedTabPosition==i) tabSelectSize else tabNormalSize
                            if (selectedTabPosition==i){
                                paint?.isFakeBoldText = selectTextBold
                                Log.e("tag","$selectTextBold")
                            }
                            else
                                paint?.isFakeBoldText = normalTextBold
                            gravity = Gravity.CENTER
                            setTextColor(if(tabColors.size >1)tabColors[i]else tabColors[0])
                        }
                    }
                }

                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabReselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    if(width != BaseIndicator.MATCH){
                                        width = tabWidth
                                    }
                                    text = it.text
                                    val tabSelectSize = if(tabSelectedSizes.size > 1) tabSelectedSizes[selectedTabPosition] else tabSelectedSizes[0]
                                    textSize = tabSelectSize
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(if(tabColors.size >1)tabColors[selectedTabPosition]else tabColors[0])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                val tabSelectSize = if(tabSelectedSizes.size > 1) tabSelectedSizes[selectedTabPosition] else tabSelectedSizes[0]
                                textSize = tabSelectSize
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                    override fun onTabUnselected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    if(width != BaseIndicator.MATCH){
                                        width = tabWidth
                                    }
                                    text = it.text
                                    val tabNormalSize = if(tabNormalSizes.size > 1) tabNormalSizes[selectedTabPosition] else tabNormalSizes[0]
                                    textSize =tabNormalSize
                                    paint?.isFakeBoldText = normalTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(if(tabColors.size >1)tabColors[selectedTabPosition]else tabColors[0])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                val tabNormalSize = if(tabNormalSizes.size > 1) tabNormalSizes[selectedTabPosition] else tabNormalSizes[0]
                                textSize =tabNormalSize
                                paint?.isFakeBoldText = normalTextBold
                            }
                        }
                    }

                    override fun onTabSelected(tab: TabLayout.Tab?) {
                        val view = tab?.customView
                        if(null ==view){
                            tab?.let {
                                it.customView = TextView(context).apply {
                                    if(width != BaseIndicator.MATCH){
                                        width = tabWidth
                                    }
                                    text = it.text
                                    val tabSelectSize = if(tabSelectedSizes.size > 1) tabSelectedSizes[selectedTabPosition] else tabSelectedSizes[0]
                                    textSize = tabSelectSize
                                    paint?.isFakeBoldText = selectTextBold
                                    gravity = Gravity.CENTER
                                    setTextColor(if(tabColors.size >1)tabColors[selectedTabPosition]else tabColors[0])
                                }
                            }
                        }else{
                            (view as? TextView)?.apply {
                                val tabSelectSize = if(tabSelectedSizes.size > 1) tabSelectedSizes[selectedTabPosition] else tabSelectedSizes[0]
                                textSize = tabSelectSize
                                paint?.isFakeBoldText = selectTextBold
                            }
                        }
                    }

                })
            }
        }
    }
}