package com.video.module_widget.widget

import android.annotation.TargetApi
import android.os.Build
import android.graphics.RectF
import android.animation.ValueAnimator
import android.animation.ValueAnimator.AnimatorUpdateListener
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.util.AttributeSet
import android.view.View

class MaterialCheckBox : View {
    private var paintBlue: Paint? = null
    private var paintWithe: Paint? = null
    private var borderColor = Color.parseColor("#C4C4C5") //边框颜色
    private var backgroundColor = Color.RED //填充颜色
    private var doneShapeColor = Color.WHITE //对号颜色
    private var baseWidth //checkbox 边框宽度
            = 0
    private var borderWidth = 0
    private var mWidth = 0
    private var mHeight //控件宽高
            = 0
    private val points = FloatArray(8) //对号的4个点的坐标
    private val DURATION = 200 //动画时长
    private var checked //选择状态
            = false
    private var correctProgress //划对号的进度
            = 0f
    private var drawRecting = false
    private var isAnim = false
    private var listener: OnCheckedChangeListener? = null
    private var padding //内切圆的边据边框的距离
            = 0f

    @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        init(context)
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(
        context: Context?,
        attrs: AttributeSet?,
        defStyleAttr: Int,
        defStyleRes: Int
    ) : super(context, attrs, defStyleAttr, defStyleRes) {
    }

    /**
     * init
     *
     * @param context
     */
    private fun init(context: Context) {
        baseWidth = dp2px(1.5f)
        borderWidth = baseWidth
        paintBlue = Paint(Paint.ANTI_ALIAS_FLAG)
        paintBlue!!.color = borderColor
        paintBlue!!.strokeWidth = borderWidth.toFloat()
        paintWithe = Paint(Paint.ANTI_ALIAS_FLAG)
        paintWithe!!.color = doneShapeColor
        paintWithe!!.strokeWidth = dp2px(2f).toFloat()
        paintWithe!!.strokeCap = Paint.Cap.ROUND
        setOnClickListener { setChecked(!isChecked()) }
        drawRecting = true
    }

    var tempWidth = 0f
    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        mWidth = Math.max(w, h)
        mHeight = mWidth
        tempWidth = width - padding
        points[0] = 0.25f * tempWidth
        points[1] = 0.5f * tempWidth
        points[2] = 0.48f * tempWidth
        points[3] = 0.71f * tempWidth
        points[4] = 0.51f * tempWidth
        points[5] = 0.68f * tempWidth
        points[6] = 0.76f * tempWidth
        points[7] = 0.33f * tempWidth
        padding = 0f
    }

    /**
     * draw checkbox
     *
     * @param canvas
     */
    override fun onDraw(canvas: Canvas) {
        val rect = RectF(padding, padding, width - padding, height - padding)
        val radius = (height - padding - padding) / 2
        //        canvas.drawRoundRect(rect, baseWidth, baseWidth, paintBlue);
        canvas.drawCircle(radius, radius, radius, paintBlue!!)
        if (drawRecting) {
            val innerRadius = radius - borderWidth
            canvas.drawCircle(radius, radius, innerRadius, paintWithe!!)
            //            canvas.drawRect(padding + borderWidth, padding + borderWidth, width - padding - borderWidth, height - padding - borderWidth, paintWithe);
        } else {
            //画对号
            if (correctProgress > 0) {
                if (correctProgress < 1 / 3f) {
                    val x = points[0] + (points[2] - points[0]) * correctProgress
                    val y = points[1] + (points[3] - points[1]) * correctProgress
                    canvas.drawLine(points[0], points[1], x, y, paintWithe!!)
                } else {
                    val x = points[4] + (points[6] - points[4]) * correctProgress
                    val y = points[5] + (points[7] - points[5]) * correctProgress
                    canvas.drawLine(points[0], points[1], points[2], points[3], paintWithe!!)
                    canvas.drawLine(points[4], points[5], x, y, paintWithe!!)
                }
            }
        }
    }

    override fun setBackgroundColor(backgroundColor: Int) {
        this.backgroundColor = backgroundColor
    }

    fun setDoneShapeColor(doneShapeColor: Int) {
        this.doneShapeColor = doneShapeColor
        paintWithe!!.color = doneShapeColor
    }

    fun setBorderColor(borderColor: Int) {
        this.borderColor = borderColor
        paintBlue!!.color = borderColor
        invalidate()
    }

    fun setBorderWidth(baseWidth: Int) {
        this.baseWidth = baseWidth
    }

    fun isChecked(): Boolean {
        return checked
    }

    fun setChecked(checked: Boolean) {
        this.checked = checked
        if (checked) {
            showRect()
        } else {
            hideCorrect()
        }
    }

    fun toggle(){
        setChecked(!isChecked())
    }

    private fun hideRect() {
        if (isAnim) {
            return
        }
        isAnim = true
        drawRecting = true
        val va = ValueAnimator.ofFloat(0f, 1f).setDuration(DURATION.toLong())
        va.addUpdateListener { animation ->
            val p = animation.animatedValue as Float
            val c = 1f - p
            borderWidth = (baseWidth + c * (mWidth - baseWidth)).toInt()
            paintBlue!!.color = evaluate(c, borderColor, backgroundColor)
            invalidate()
            if (p >= 1) {
                isAnim = false
                if (listener != null) {
                    checked = false
                    listener!!.onCheckedChanged(this@MaterialCheckBox, checked)
                }
            }
        }
        va.start()
    }

    private fun showRect() {
        if (isAnim) {
            return
        }
        isAnim = true
        drawRecting = true
        val va = ValueAnimator.ofFloat(0f, 1f).setDuration(DURATION.toLong())
        va.addUpdateListener { animation ->
            val p = animation.animatedValue as Float
            borderWidth = (10 + p * (width - 10)).toInt()
            paintBlue!!.color = evaluate(p, borderColor, backgroundColor)
            invalidate()
            if (p >= 1) {
                isAnim = false
                drawRecting = false
                showCorrect()
            }
        }
        va.start()
    }

    private fun showCorrect() {
        if (isAnim) {
            return
        }
        isAnim = true
        correctProgress = 0f
        drawRecting = false
        val va = ValueAnimator.ofFloat(0f, 1f).setDuration(DURATION.toLong())
        va.addUpdateListener { animation ->
            correctProgress = animation.animatedValue as Float
            invalidate()
            if (correctProgress >= 1) {
                isAnim = false
                if (listener != null) {
                    checked = true
                    listener!!.onCheckedChanged(this@MaterialCheckBox, checked)
                }
            }
        }
        va.start()
    }

    private fun hideCorrect() {
        if (isAnim) {
            return
        }
        isAnim = true
        correctProgress = 1f
        drawRecting = false
        val va = ValueAnimator.ofFloat(0f, 1f).setDuration(DURATION.toLong())
        va.addUpdateListener { animation ->
            val p = animation.animatedValue as Float
            correctProgress = 1f - p
            invalidate()
            if (p >= 1) {
                isAnim = false
                hideRect()
            }
        }
        va.start()
    }

    fun setOnCheckedChangedListener(listener: OnCheckedChangeListener?) {
        this.listener = listener
    }

    interface OnCheckedChangeListener {
        fun onCheckedChanged(view: View?, isChecked: Boolean)
    }

    private fun evaluate(fraction: Float, startValue: Int, endValue: Int): Int {
        val startA = startValue shr 24 and 0xff
        val startR = startValue shr 16 and 0xff
        val startG = startValue shr 8 and 0xff
        val startB = startValue and 0xff
        val endA = endValue shr 24 and 0xff
        val endR = endValue shr 16 and 0xff
        val endG = endValue shr 8 and 0xff
        val endB = endValue and 0xff
        return (startA + (fraction * (endA - startA)).toInt() shl 24
                or (startR + (fraction * (endR - startR)).toInt() shl 16)
                or (startG + (fraction * (endG - startG)).toInt() shl 8)
                or startB + (fraction * (endB - startB)).toInt())
    }

    fun dp2px(value: Float): Int {
        val scale = context.resources.displayMetrics.densityDpi.toFloat()
        return (value * (scale / 160) + 0.5f).toInt()
    }
}