package com.video.module_widget.widget

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.Gravity
import androidx.appcompat.widget.AppCompatTextView
import com.video.module_widget.R
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.utils.*

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/22 上午10:59
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/22 上午10:59
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class ShapeTextViewExt  @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null,
                                                  defStyleAttr: Int = 0): AppCompatTextView(context, attrs, defStyleAttr) {
    var helper:ViewBackgroundHelperExt
    init {
          helper =  ViewBackgroundHelperExt(this,context,attrs,defStyleAttr,0)
        }

    fun setBackgroundColor(color:String){
        helper.setShapeSolidColor(color.toArgb())
        helper.setShapeStartColor(color.toArgb())
        helper.setShapeEndColor(color.toArgb())
    }

    fun setBackgroundColorWithRGBA(color:Int){
        helper.setShapeSolidColor(color)
        helper.setShapeStartColor(color)
        helper.setShapeEndColor(color)
    }

    fun setBorderRadius(radius:Int){
        helper.setShapeRadius(context.getPx(radius))
    }

    fun setBorderRadius(leftTop:Int = 0,leftBottom:Int =0,rightTop:Int =0,rightBottom:Int = 0){
        helper.setShapeTopLeftRadius(leftTop)
        helper.setShapeTopRightRadius(rightTop)
        helper.setShapeBottomLeftRadius(leftBottom)
        helper.setShapeBottomRightRadius(rightBottom)
    }

    fun setBorderColor(color:String){
        helper.setShapeStrokeColor(color.toArgb())
    }

    fun setBorderColor(color:Int){
        helper.setShapeStrokeColor(color)
    }

    fun setBorderWidth(width:Int){
        helper.setShapeStrokeWidth(context.getDp(width).toInt())
    }


    var alignStr:String = StyleConsts.LEFT

    fun setAlign(align:String){
         alignStr = align
         postInvalidate()
    }

    fun reBuilder(){
        helper.createDrawable(this)
    }

    override fun onDraw(canvas: Canvas) {
        val drawables = compoundDrawables
        val drawableLeft = drawables[0]
        if(drawableLeft!=null){
            val textWidth = paint.measureText(text.toString())
            val contentWidth = textWidth + compoundDrawablePadding +drawableLeft.intrinsicWidth
            if(width - contentWidth > 0){
                when(alignStr){
                    StyleConsts.CENTER ->{
                        gravity = Gravity.CENTER_VERTICAL or Gravity.LEFT
                        canvas.translate((width - contentWidth - paddingRight - paddingLeft)/2f,0f)
//                        if(alignStr == "center"){
//
//                        }else{
//                            canvas.translate(-(width - contentWidth - paddingLeft - paddingRight),0f)
//                        }
                    }
                    StyleConsts.LEFT ->{
                        gravity = Gravity.CENTER_VERTICAL or Gravity.LEFT
                    }
                    StyleConsts.RIGHT ->{
                        gravity = Gravity.CENTER_VERTICAL or Gravity.LEFT
                        canvas.translate((width - contentWidth - paddingLeft - paddingRight),0f)
                    }
                    else ->{
                        gravity = Gravity.CENTER_VERTICAL or Gravity.RIGHT
                    }
                }
            }

        }
        super.onDraw(canvas)
    }
}