package com.video.module_widget.widget

import android.content.Context
import android.content.res.TypedArray
import android.graphics.Color
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.util.Log
import android.view.View
import androidx.annotation.ColorInt
import com.melancholy.widget.R
import com.melancholy.widget.helper.IViewBackgroundHelper
import com.melancholy.widget.helper.Shape

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/7/22 上午11:33
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/7/22 上午11:33
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class ViewBackgroundHelperExt constructor(view: View, context: Context, attrs: AttributeSet?,
                                          defStyleAttr: Int, defStyleRes: Int):
    IViewBackgroundHelper {

    @Shape.ShapeType
    private var mShape: Int = Shape.RECTANGLE
    private var mShapeWidth: Int = -1
    private var mShapeHeight: Int = -1

    @ColorInt
    private var mShapeSolidColor: Int = Color.TRANSPARENT
    @ColorInt
    private var mShapeSolidPressedColor: Int = 0
    @ColorInt
    private var mShapeSolidDisabledColor: Int = 0
    @ColorInt
    private var mShapeSolidFocusedColor: Int = 0
    @ColorInt
    private var mShapeSolidSelectedColor: Int = 0

    private var mShapeTopLeftRadius: Int = 0
    private var mShapeTopRightRadius: Int = 0
    private var mShapeBottomLeftRadius: Int = 0
    private var mShapeBottomRightRadius: Int = 0

    @ColorInt
    private var mShapeStartColor: Int = 0
    @ColorInt
    private var mShapeCenterColor: Int = 0
    @ColorInt
    private var mShapeEndColor: Int = 0
    private var mShapeUseLevel: Boolean = false
    private var mShapeAngle: Int = 0
    @Shape.GradientType
    private var mShapeGradientType: Int = Shape.LINEAR_GRADIENT
    private var mShapeCenterX: Float = 0f
    private var mShapeCenterY: Float = 0f
    private var mShapeGradientRadius: Int = 0

    @ColorInt
    private var mShapeStrokeColor: Int = 0
    @ColorInt
    private var mShapeStrokePressedColor: Int = 0
    @ColorInt
    private var mShapeStrokeDisabledColor: Int = 0
    @ColorInt
    private var mShapeStrokeFocusedColor: Int = 0
    @ColorInt
    private var mShapeStrokeSelectedColor: Int = 0

    private var mShapeStrokeWidth: Int = 0
    private var mShapeDashWidth: Int = 0
    private var mShapeDashGap: Int = 0

    private var mShapeInnerRadius: Int = 0
    private var mShapeInnerRadiusRatio: Float = 0f
    private var mShapeThickness: Int = 0
    private var mShapeThicknessRatio: Float = 0f

    private var mShapeShadowSize: Int = 0
    private var mShapeShadowColor: Int = 0
    private var mShapeShadowOffsetX: Int = 0
    private var mShapeShadowOffsetY: Int = 0

    init {
        val array: TypedArray = context.obtainStyledAttributes(attrs, R.styleable.ShapeBackground,
            defStyleAttr, defStyleRes)
        mShape = array.getInt(R.styleable.ShapeBackground_shape, mShape)
        mShapeWidth = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeWidth,
            mShapeWidth)
        mShapeHeight = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeHeight,
            mShapeHeight)
        mShapeSolidColor = array.getColor(
            R.styleable.ShapeBackground_shapeSolidColor,
            mShapeSolidColor)
        mShapeSolidPressedColor = array.getColor(
            R.styleable.ShapeBackground_shapeSolidPressedColor,
            mShapeSolidColor)
        mShapeSolidDisabledColor = array.getColor(
            R.styleable.ShapeBackground_shapeSolidDisabledColor, mShapeSolidColor)
        mShapeSolidFocusedColor = array.getColor(
            R.styleable.ShapeBackground_shapeSolidFocusedColor,
            mShapeSolidColor)
        mShapeSolidSelectedColor = array.getColor(
            R.styleable.ShapeBackground_shapeSolidSelectedColor, mShapeSolidColor)
        val radius: Int = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeRadius,
            0)
        mShapeTopLeftRadius = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeTopLeftRadius, radius)
        mShapeTopRightRadius = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeTopRightRadius, radius)
        mShapeBottomLeftRadius = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeBottomLeftRadius, radius)
        mShapeBottomRightRadius = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeBottomRightRadius, radius)
        mShapeStartColor = array.getColor(
            R.styleable.ShapeBackground_shapeStartColor,
            mShapeSolidColor)
        mShapeCenterColor = array.getColor(
            R.styleable.ShapeBackground_shapeCenterColor,
            mShapeSolidColor)
        mShapeEndColor = array.getColor(R.styleable.ShapeBackground_shapeEndColor, mShapeSolidColor)
        mShapeUseLevel = array.getBoolean(R.styleable.ShapeBackground_shapeUseLevel, false)
        mShapeAngle = array.getInt(R.styleable.ShapeBackground_shapeAngle, 0)
        mShapeGradientType = array.getInt(
            R.styleable.ShapeBackground_shapeGradientType,
            mShapeGradientType)
        mShapeCenterX = array.getFloat(R.styleable.ShapeBackground_shapeCenterX, 0.5f)
        mShapeCenterY = array.getFloat(R.styleable.ShapeBackground_shapeCenterY, 0.5f)
        mShapeGradientRadius = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeGradientRadius, radius)
        mShapeStrokeColor = array.getColor(
            R.styleable.ShapeBackground_shapeStrokeColor,
            Color.TRANSPARENT)
        mShapeStrokePressedColor = array.getColor(
            R.styleable.ShapeBackground_shapeStrokePressedColor, mShapeStrokeColor)
        mShapeStrokeDisabledColor = array.getColor(
            R.styleable.ShapeBackground_shapeStrokeDisabledColor, mShapeStrokeColor)
        mShapeStrokeFocusedColor = array.getColor(
            R.styleable.ShapeBackground_shapeStrokeFocusedColor, mShapeStrokeColor)
        mShapeStrokeSelectedColor = array.getColor(
            R.styleable.ShapeBackground_shapeStrokeSelectedColor, mShapeStrokeColor)
        mShapeStrokeWidth = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeStrokeWidth, 0)
        mShapeDashWidth = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeDashWidth,
            0)
        mShapeDashGap = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeDashGap,
            0)
        mShapeInnerRadius = array.getDimensionPixelOffset(
            R.styleable.ShapeBackground_shapeInnerRadius, -1)
        mShapeInnerRadiusRatio = array.getFloat(
            R.styleable.ShapeBackground_shapeInnerRadiusRatio, 3.0f)
        mShapeThickness = array.getDimensionPixelOffset(
            R.styleable.ShapeBackground_shapeThickness,
            -1)
        mShapeThicknessRatio = array.getFloat(
            R.styleable.ShapeBackground_shapeThicknessRatio,
            9.0f)
        mShapeShadowSize = array.getDimensionPixelSize(
            R.styleable.ShapeBackground_shapeShadowSize,
            0)
        mShapeShadowColor = array.getColor(R.styleable.ShapeBackground_shapeShadowColor, Color.RED)
        mShapeShadowOffsetX = array.getDimensionPixelOffset(
            R.styleable.ShapeBackground_shapeShadowOffsetX, 0)
        mShapeShadowOffsetY = array.getDimensionPixelOffset(
            R.styleable.ShapeBackground_shapeShadowOffsetY, 0)
        array.recycle()
        createDrawable(view)
    }

    override fun setShape(shape: Int) {
        mShape = shape
    }

    override fun getShape(): Int = mShape

    override fun setShapeWidth(width: Int) {
        mShapeWidth = width
    }

    override fun getShapeWidth(): Int = mShapeWidth

    override fun setShapeHeight(height: Int) {
        mShapeHeight = height
    }

    override fun getShapeHeight(): Int = mShapeHeight

    override fun setShapeSolidColor(color: Int) {
        mShapeSolidColor = color
    }

    override fun getShapeSolidColor(): Int = mShapeSolidColor

    override fun setShapeSolidPressedColor(color: Int) {
        mShapeSolidPressedColor = color
    }

    override fun getShapeSolidPressedColor(): Int = mShapeSolidPressedColor

    override fun setShapeSolidDisabledColor(color: Int) {
        mShapeSolidDisabledColor = color
    }

    override fun getShapeSolidDisabledColor(): Int = mShapeSolidDisabledColor

    override fun setShapeSolidFocusedColor(color: Int) {
        mShapeSolidFocusedColor = color
    }

    override fun getShapeSolidFocusedColor(): Int = mShapeSolidFocusedColor

    override fun setShapeSolidSelectedColor(color: Int) {
        mShapeSolidSelectedColor = color
    }

    override fun getShapeSolidSelectedColor(): Int = mShapeSolidSelectedColor

    override fun setShapeTopLeftRadius(radius: Int) {
        mShapeTopLeftRadius = radius
    }

    override fun getShapeTopLeftRadius(): Int = mShapeTopLeftRadius

    override fun setShapeTopRightRadius(radius: Int) {
        mShapeTopRightRadius = radius
    }

    override fun getShapeTopRightRadius(): Int = mShapeTopRightRadius

    override fun setShapeBottomLeftRadius(radius: Int) {
        mShapeBottomLeftRadius = radius
    }

    override fun getShapeBottomLeftRadius(): Int = mShapeBottomLeftRadius

    override fun setShapeBottomRightRadius(radius: Int) {
        mShapeBottomRightRadius = radius
    }

    override fun getShapeBottomRightRadius(): Int = mShapeBottomRightRadius

    override fun setShapeStartColor(color: Int) {
        mShapeStartColor = color
    }

    override fun getShapeStartColor(): Int = mShapeStartColor

    override fun setShapeCenterColor(color: Int) {
        mShapeCenterColor = color
    }

    override fun getShapeCenterColor(): Int = mShapeCenterColor

    override fun setShapeEndColor(color: Int) {
        mShapeEndColor = color
    }

    override fun getShapeEndColor(): Int = mShapeEndColor

    override fun setShapeUseLevel(useLevel: Boolean) {
        mShapeUseLevel = useLevel
    }

    override fun isShapeUseLevel(): Boolean = mShapeUseLevel

    override fun setShapeAngle(angle: Int) {
        mShapeAngle = angle
    }

    override fun getShapeAngle(): Int = mShapeAngle

    override fun setShapeGradientType(type: Int) {
        mShapeGradientType = type
    }

    override fun getShapeGradientType(): Int = mShapeGradientType

    override fun setShapeCenterX(x: Float) {
        mShapeCenterX = x
    }

    override fun getShapeCenterX(): Float = mShapeCenterX

    override fun setShapeCenterY(y: Float) {
        mShapeCenterY = y
    }

    override fun getShapeCenterY(): Float = mShapeCenterY

    override fun setShapeGradientRadius(radius: Int) {
        mShapeGradientRadius = radius
    }

    override fun getShapeGradientRadius(): Int = mShapeGradientRadius

    override fun setShapeStrokeColor(color: Int) {
        mShapeStrokeColor = color
    }

    override fun getShapeStrokeColor(): Int = mShapeStrokeColor

    override fun setShapeStrokePressedColor(color: Int) {
        mShapeStrokePressedColor = color
    }

    override fun getShapeStrokePressedColor(): Int = mShapeStrokePressedColor

    override fun setShapeStrokeDisabledColor(color: Int) {
        mShapeStrokeDisabledColor = color
    }

    override fun getShapeStrokeDisabledColor(): Int = mShapeStrokeDisabledColor

    override fun setShapeStrokeFocusedColor(color: Int) {
        mShapeStrokeFocusedColor = color
    }

    override fun getShapeStrokeFocusedColor(): Int = mShapeStrokeFocusedColor

    override fun setShapeStrokeSelectedColor(color: Int) {
        mShapeStrokeSelectedColor = color
    }

    override fun getShapeStrokeSelectedColor(): Int = mShapeStrokeSelectedColor

    override fun setShapeStrokeWidth(width: Int) {
        mShapeStrokeWidth = width
    }

    override fun getShapeStrokeWidth(): Int = mShapeStrokeWidth

    override fun setShapeDashWidth(width: Int) {
        mShapeDashWidth = width
    }

    override fun getShapeDashWidth(): Int = mShapeDashWidth

    override fun setShapeDashGap(gap: Int) {
        mShapeDashGap = gap
    }

    override fun getShapeDashGap(): Int = mShapeDashGap

    override fun setShapeInnerRadius(radius: Int) {
        mShapeInnerRadius = radius
    }

    override fun getShapeInnerRadius(): Int = mShapeInnerRadius

    override fun setShapeInnerRadiusRatio(ratio: Float) {
        mShapeInnerRadiusRatio = ratio
    }

    override fun getShapeInnerRadiusRatio(): Float = mShapeInnerRadiusRatio

    override fun setShapeThickness(size: Int) {
        mShapeThickness = size
    }

    override fun getShapeThickness(): Int = mShapeThickness

    override fun setShapeThicknessRatio(ratio: Float) {
        mShapeThicknessRatio = ratio
    }

    override fun getShapeThicknessRatio(): Float = mShapeThicknessRatio

    override fun setShapeShadowSize(size: Int) {
        mShapeShadowSize = size
    }

    override fun getShapeShadowSize(): Int = mShapeShadowSize

    override fun setShapeShadowColor(color: Int) {
        mShapeShadowColor = color
    }

    override fun getShapeShadowColor(): Int = mShapeShadowColor

    override fun setShapeShadowOffsetX(offsetX: Int) {
        mShapeShadowOffsetX = offsetX
    }

    override fun getShapeShadowOffsetX(): Int = mShapeShadowOffsetX

    override fun setShapeShadowOffsetY(offsetY: Int) {
        mShapeShadowOffsetY = offsetY
    }

    override fun getShapeShadowOffsetY(): Int = mShapeShadowOffsetY

    /**
     * 创建Drawable
     * @param view
     */
     fun createDrawable(view: View) {
        val drawable: Drawable = buildBackgroundDrawable() ?: return
        if (isShapeShadowEnable()) {
            // 需要关闭硬件加速，否则阴影无法生效
            view.setLayerType(View.LAYER_TYPE_SOFTWARE, null)
        }
        view.background = drawable
    }
}